﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Repräsentiert die Anwendungsklasse der Konsolenanwendung
    /// </summary>
    class Program
    {
        /// <summary>
        /// Repräsentiert den Einsprungpunkt in die Anwendung.
        /// </summary>
        /// <param name="astrArgument">Argumente die beim Start des Programms vom Betriebssystem übergeben werden.</param>
        static int Main(string[] astrArgument)
        {
            // Lokale Daten
            CwlbBehaviour objBehave = new CwlbBehaviour();
            int iReturn = 0;

            #region Verarbeitung
            // Exceptionhandling
            try
            {
                // Fenstertitel definieren
                Console.Title = "Demoprogramm Konsolensteurung";

                // Prüfe die Programmargumente
                CheckParams(astrArgument, objBehave);

                // Prüfe die Anforderung der Anzeige der Benutzungshinweise
                if (objBehave.ShowHelp)
                {
                    Help();
                    return 1;
                }

                // Effektive Verarbeitung 
                try
                {
                    // Prüfe Modus
                    if (objBehave.WorkInteractive)
                    {
                        // Interaktiver Modus
                        objBehave.DoWorkInteractive();
                        Console.ResetColor();
                        Console.Clear();
                    }
                    else
                    {
                        // Batchmodus
                        objBehave.DoWork();
                    }
                }
                catch (Exception exc)
                {
                    // Laufzeitfehler in der Verarbeitung
                    Console.ForegroundColor = ConsoleColor.Red;
                    Console.WriteLine();
                    Console.WriteLine(exc.Message);
                    iReturn = 3;
                }
            }
            catch (Exception excObject)
            {
                // Fehler in der Parameterprüfung
                iReturn = 2;
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine();
                Console.WriteLine(excObject.Message);
                Help();
            }
            finally
            {
                Console.ResetColor();
            }
            #endregion

            // Wertrückgabe
            return iReturn;
        }

        #region Ausgabe Gebrauchsanweisung
        /// <summary>
        /// Ausgabe einer Gebrauchsanweisung 
        /// </summary>
        static void Help()
        {

            // Allgemeine Beschreibung
            Console.ForegroundColor = ConsoleColor.DarkGreen;
            Console.WriteLine();
            Console.WriteLine("Konsolen Beispielprogramm");
            Console.WriteLine("-------------------------");
            Console.WriteLine("Copyright by weroSoft GmbH 2010");
            Console.WriteLine();
            Console.WriteLine("Benützung des Programms:");
            Console.WriteLine("ConsoleSample [-?]|[-i]|[[[-m][u]]|[[-p]|[-p:nnn]]]");
            Console.WriteLine("");
            Console.WriteLine("wobei:");
            Console.WriteLine("   -m     Liefert Informationen über den Rechner");
            Console.WriteLine("   -u     Liefert Informationen über den angemeldeten Benutzer");
            Console.WriteLine("   -p     Liefert eine Prozessliste");
            Console.WriteLine("   -p:nnn Liefert Angaben über den Prozess mit der Nummer nnn");
            Console.WriteLine("   -i     Interaktiver Modus");
            Console.WriteLine("   -?     Ziegt diese Gebrauchsanweisung");
            Console.WriteLine("");
            Console.WriteLine("Rückgabewerte (für Batchverarbeitung):");
            Console.WriteLine("    0     Kein Fehler");
            Console.WriteLine("    1     Kein Fehler, Onlinehilfe wurde angefordert");
            Console.WriteLine("    2     Fehler in der Parameterprüfung");
            Console.WriteLine("    3     Fehler in der Verarbeitung der angeforderten Funktionen");
            Console.WriteLine("");
        }
        #endregion

        #region Überprüfung der Argumente
        /// <summary>
        /// Überprüft die Argumente und definiert die Steuerstruktur.
        /// </summary>
        /// <param name="astrArgument">Übergebene Argumente.</param>
        /// <param name="objBehave">Steuerobjekt für das Programm.</param>
        static void CheckParams(string[] astrArgument, CwlbBehaviour objBehave)
        {

            // Anzahl Argumente prüfen
            if (astrArgument.Length == 0)
            {
                objBehave.ShowHelp = true;
                return;
            }

            // Schleife über alle Eingabeparameter
            foreach (string strArgument in astrArgument)
            {

                if (strArgument.Length < 2)
                {

                    // Kann kein gültiges Argument sein
                    throw new Exception("Ungültiges Argument: " + strArgument);
                }
                else
                {

                    // Evaluiere Argument
                    switch (strArgument.ToLower()[1])
                    {
                        case 'm':
                            objBehave.SetShowMachineInfo();
                            break;
                        case 'u':
                            objBehave.SetShowUserInfo();
                            break;
                        case 'i':
                            objBehave.SetWorkInteractive();
                            break;
                        case 'p':
                            if (strArgument.Length > 3)
                            {
                                if (strArgument[2] == ':')
                                {
                                    objBehave.SetShowProcessNumber(
                                        Convert.ToInt32(strArgument.Substring(3)));
                                }
                                else
                                {
                                    throw new Exception("Die Prozessnummer konnte nicht erkannt werden.");
                                }
                            }
                            else
                            {
                                objBehave.SetShowProcessInfo();
                            }
                            break;
                        case '?':
                            objBehave.ShowHelp = true;
                            break;
                        default:
                            objBehave.ShowHelp = true;
                            throw new Exception("Parameter nicht erkannt");
                    }
                }
            }
        }
        #endregion
    }
}
