using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Klasse zur Demonstration und zum Kennenlernen der Einstellungen der Steuerelemente.
    /// </summary>
    public partial class CDlgControlExplorer : Form
    {
        // Definiert den WIN32 Code fr das Windwos Ereignis linke Maustaste
        private const int WM_LBUTTONDOWN = 0x0201;
        // Definiert ein Ereignisfilter fr die dynamische Behandlung des Dialogs
        private CwlbMessageFilter _objMessageFilter;

        /// <summary>
        /// Standardkonstruktor
        /// </summary>
        public CDlgControlExplorer()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignishandler Laden des Dialogs.
        /// </summary>
        private void CDlgControlProperties_Load(object sender, EventArgs e)
        {
            // Erstelle ein MessageFilter fr Click Events von Subelementen der
            _objMessageFilter = new CwlbMessageFilter(_fltPanel, WM_LBUTTONDOWN);
            _objMessageFilter.MessageReceived += new EventHandler<CwlbMessageFilterEventArgs>(objMessageFilter_Click);
            Application.AddMessageFilter(_objMessageFilter);
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen des Fensters.
        /// </summary>
        private void CDlgControlExplorer_FormClosing(object sender, FormClosingEventArgs e)
        {
            Application.RemoveMessageFilter(_objMessageFilter);
            _objMessageFilter.MessageReceived -= new EventHandler<CwlbMessageFilterEventArgs>(objMessageFilter_Click);
        }

        /// <summary>
        /// Ereignisfilter hat ein Click Event ausgelst.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void objMessageFilter_Click(object sender, CwlbMessageFilterEventArgs e)
        {
            this._pgrLabelProperies.SelectedObject = sender;
        }

        /// <summary>
        /// Ereignishandler Knopf Hierarchie wurde bettigt
        /// </summary>
        private void _btnHierarchy_Click(object sender, EventArgs e)
        {
            CdlgControlsHierarchy dlgHierarchy = new CdlgControlsHierarchy(this);
            dlgHierarchy.ShowDialog();
        }
    }
}