using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{

    /// <summary>
    /// Demonstriert die Positionierung von Dialogen
    /// </summary>
    public partial class CdlgWindow : Form
    {
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CdlgWindow.
        /// </summary>
        public CdlgWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignishandler Laden des Fensters
        /// </summary>
        private void CDlgScreens_Load(object sender, EventArgs e)
        {
            // Lokale Daten
            TreeNode nodeTree = null;
            Screen[] aobjScreen = Screen.AllScreens;

            // Bildschirmdaten in Treeview einfllen
            foreach (Screen objScreen in aobjScreen)
            {
                nodeTree = _tvScreens.Nodes.Add("Name = " + objScreen.DeviceName);
                nodeTree.Nodes.Add("Bit per Pixel" + objScreen.BitsPerPixel.ToString());
                nodeTree.Nodes.Add("Bounds" + objScreen.Bounds.ToString());
                nodeTree.Nodes.Add("Primary" + objScreen.Primary.ToString());
                nodeTree.Nodes.Add("WorkingArea" + objScreen.WorkingArea.ToString());
            }

            // Liste fr Fenstersil initialisieren
            foreach (FormBorderStyle enmStyle in Enum.GetValues(typeof(FormBorderStyle)))
            {
                _cboStyle.Items.Add(enmStyle.ToString());
            }

            // Aktuelle Einstellung vorwhlen
            _cboStyle.SelectedItem = this.FormBorderStyle.ToString();
        }

        /// <summary>
        /// Ereignishandler Knopf oben links gedrckt
        /// </summary>
        private void ButtonTopLeft_Click(object sender, EventArgs e)
        {
            Rectangle rectWorkingArea;
            if (_chkVirtualScreen.Checked)
            {
                rectWorkingArea = SystemInformation.VirtualScreen;
            }
            else
            {
                rectWorkingArea = Screen.FromPoint(this.Location).WorkingArea;
            }
            if (_chkWorkspace.Checked)
            {
                this.SetDesktopLocation(0, 0);
            }
            else
            {
                this.Location = new Point(0, 0);
            }
        }

        /// <summary>
        /// Ereignishandler Knopf oben rechts gedrckt
        /// </summary>
        private void ButtonTopRight_Click(object sender, EventArgs e)
        {
            Rectangle rectWorkingArea;
            if (_chkVirtualScreen.Checked)
            {
                rectWorkingArea = SystemInformation.VirtualScreen;
            }
            else
            {
                rectWorkingArea = Screen.FromPoint(this.Location).WorkingArea;
            }
            if (_chkWorkspace.Checked)
            {
                this.SetDesktopLocation(rectWorkingArea.Width - this.Width, 0);
            }
            else
            {
                this.Location = new Point(rectWorkingArea.Width - this.Width, 0);
            }
        }

        /// <summary>
        /// Ereignishandler Knopf unten links gedrckt
        /// </summary>
        private void ButtonBottomLeft_Click(object sender, EventArgs e)
        {
            Rectangle rectWorkingArea;
            if (_chkVirtualScreen.Checked)
            {
                rectWorkingArea = SystemInformation.VirtualScreen;
            }
            else
            {
                rectWorkingArea = Screen.FromPoint(this.Location).WorkingArea;
            }
            if (_chkWorkspace.Checked)
            {
                this.SetDesktopLocation(0, rectWorkingArea.Height - this.Height);
            }
            else
            {
                this.Location = new Point(0, rectWorkingArea.Height - this.Height);
            }
        }

        /// <summary>
        /// Ereignishandler Knopf unten rechts gedrckt
        /// </summary>
        private void ButtonBottomRight_Click(object sender, EventArgs e)
        {
            Rectangle rectWorkingArea;
            if (_chkVirtualScreen.Checked)
            {
                rectWorkingArea = SystemInformation.VirtualScreen;
            }
            else
            {
                rectWorkingArea = Screen.PrimaryScreen.WorkingArea;
            }
            if (_chkWorkspace.Checked)
            {
                this.SetDesktopLocation(rectWorkingArea.Width - this.Width,
                    rectWorkingArea.Height - this.Height);
            }
            else
            {
                this.Location = new Point(rectWorkingArea.Width - this.Width,
                    rectWorkingArea.Height - this.Height);
            }
        }

        /// <summary>
        /// Ereignishandler Knopf Bildschirmmitte gedrckt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCenterScreen_Click(object sender, EventArgs e)
        {
            Rectangle rectWorkingArea;
            if (_chkVirtualScreen.Checked)
            {
                rectWorkingArea = SystemInformation.VirtualScreen;
            }
            else
            {
                rectWorkingArea = Screen.FromPoint(this.Location).WorkingArea;
            }
            if (_chkWorkspace.Checked)
            {
                this.SetDesktopLocation(rectWorkingArea.Width / 2 - this.Width / 2,
                    rectWorkingArea.Height / 2 - this.Height / 2);
            }
            else
            {
                this.Location = new Point(rectWorkingArea.Width / 2 - this.Width / 2,
                    rectWorkingArea.Height / 2 - this.Height / 2);
            }
        }

        /// <summary>
        /// Ereignishandler Topmost wurde bettigt
        /// </summary>
        private void Top_CheckedChanged(object sender, EventArgs e)
        {
            if (_chkTop.Checked)
            {
                this.TopMost = true;
            }
            else
            {
                this.TopMost = false;
            }
        }

        /// <summary>
        /// Ereignisbehandlung fr nderung des Fensterstils
        /// </summary>
        private void Style_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.FormBorderStyle = (FormBorderStyle)Enum.Parse(typeof(FormBorderStyle), _cboStyle.Text);
        }
    }
}