using System;
using System.ComponentModel;
using System.Reflection;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Eigene PropertyGrid-Klasse. Die Klasse implementiert die 
    /// Kontrolle der Tab oder Shift-Tab Taste
    /// </summary>
    public class CctrlPropertyGrid : System.Windows.Forms.PropertyGrid
    {
        /// <summary>
        /// Deaktivieren der Eigenschaft LargeButton.
        /// </summary>
        [Browsable(false)]
        public new bool LargeButtons
        {
            get
            {
                return false;
            }
            set
            {
                value = false;
            }
        }

        /// <summary>
        /// Ereignishandler Befehlstaste bettigt
        /// </summary>
        /// <param name="msg">WIN32-Nachricht</param>
        /// <param name="keyData">Transformierte Tasteninformation</param>
        /// <returns>True, wenn die Nachricht verarbeitet wurde, andernfalls false.</returns>
        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            // Prfe Tastenbettigung auf <Tab> oder <Shift><Tab>
            if ((keyData == Keys.Tab) || (keyData == (Keys.Tab | Keys.Shift)))
            {
                // Lese selektiertes Element
                GridItem selectedItem = SelectedGridItem;
                if (selectedItem != null)
                {
                    // Lese die Auflistung des bergeordneten Elements
                    GridItemCollection gridItems = selectedItem.Parent.GridItems;

                    // Finde das selektierte Element
                    int foundIndex = -1;
                    for (int i = 0; i < gridItems.Count; i++)
                    {
                        if (gridItems[i] == selectedItem)
                        {
                            foundIndex = i;
                            break;
                        }
                    }

                    // Position auf nchstes Element
                    if ((keyData & Keys.Shift) == Keys.Shift)
                    {
                        foundIndex--;
                        if (foundIndex < 0)
                        {
                            foundIndex = gridItems.Count - 1;
                        }
                    }
                    else
                    {
                        foundIndex++;
                        if (foundIndex >= gridItems.Count)
                        {
                            foundIndex = 0;
                        }
                    }

                    // Selektiere Element
                    SelectedGridItem = gridItems[foundIndex];

                    // Nachricht wurde bearbeitet
                    return true;
                }
            }

            // Aufruf der Methode in der Bassiklasse
            return base.ProcessCmdKey(ref msg, keyData);
        }
    }
}

