﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die Funktionalität für das Dialogfeld der Einstellungen.
    /// </summary>
    public partial class CdlgSettings : Form
    {
        /// <summary>
        /// Liefert oder definiert das Standardverzeichnis der Anwendung.
        /// </summary>
        public string DefaultDirectory { get; set; }

        /// <summary>
        /// Liefert oder definiert eine Liste von anwendungsspezifischen Farben.
        /// </summary>
        public List<int> CustomBGRColorCollection { get; set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CdlgSettings.
        /// </summary>
        public CdlgSettings()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung Laden des Formulars.
        /// </summary>
        private void CdlgSettings_Load(object sender, EventArgs e)
        {
            // Inhalt initialisieren
            _tbDefaultDirectory.Text = DefaultDirectory;

            // Anwendungsspezifische Farben anzeigen
            if (CustomBGRColorCollection == null)
            {
                CustomBGRColorCollection = new List<int>();
            }
            RefreshColor();
        }

        /// <summary>
        /// Erneuert die Wiedergabe der Farben im Dialogfeld.
        /// </summary>
        private void RefreshColor()
        {
            // Lokale Elemente
            Control[] objControl;

            // Schleife durch die Liste der Farben
            for (int iIndex = 0; iIndex < CustomBGRColorCollection.Count; iIndex++)
            {
                // Steuerelement für die Anzeige der Farbe suchen und prüfen, ob gefunden.
                objControl = Controls.Find(string.Format("_pnlCustomColor{0}", iIndex.ToString("X")), true);
                if (objControl.Length == 1)
                {
                    // Farbe aus der Liste auslesen
                    // Achtung: Das ColorDialogfeld liefert Farben im BGR Modus. Das heisst umrechnen nach RGB,
                    // weil .NET mit RGB-Farben arbeitet.
                    int iBGR = CustomBGRColorCollection[iIndex];
                    objControl[0].BackColor = Color.FromArgb((iBGR & 0x000000FF), (iBGR & 0x0000FF00) >> 8, (iBGR & 0x00FF0000) >> 16);
                }
            }
        }


        /// <summary>
        /// Ereignisbehandlung der Schaltfläche Verzeichnis suchen.
        /// </summary>
        private void ButtonBrowseDefaultDirectory_Click(object sender, EventArgs e)
        {
            // Dialogfeld erstellen
            FolderBrowserDialog dlgFolder = new FolderBrowserDialog();

            // Dialogfeld konfigurieren
            dlgFolder.Description = "Wählen Sie das Standardverzeichnis der Anwendung aus.";
            dlgFolder.SelectedPath = DefaultDirectory;
            dlgFolder.ShowNewFolderButton = true;
            dlgFolder.RootFolder = Environment.SpecialFolder.Desktop;

            // Dialogfeld anzeigen und Rückgabewert auswerten
            if (dlgFolder.ShowDialog() == DialogResult.OK)
            {
                // Übernehmen des ausgewählten Pfades
                _tbDefaultDirectory.Text = dlgFolder.SelectedPath;
            }
        }

        /// <summary>
        /// Ereignisbehandlung der Schaltfläche 'Speichern'.
        /// </summary>
        private void ButtonSave_Click(object sender, EventArgs e)
        {
            DefaultDirectory = _tbDefaultDirectory.Text;
            DialogResult = DialogResult.OK;
        }

        /// <summary>
        /// Ereignisbehandlung Farbfläche doppelclick.
        /// </summary>
        private void CustomColor_DoubleClick(object sender, EventArgs e)
        {
            ButtonBrowseColor_Click(this, e);
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche 'Farben definieren'.
        /// </summary>
        private void ButtonBrowseColor_Click(object sender, EventArgs e)
        {
            // Dialogfeld erstellen
            ColorDialog dlgColor = new ColorDialog();

            // Dialogfeld konfigurieren
            dlgColor.AllowFullOpen = true;
            dlgColor.AnyColor = true;
            dlgColor.CustomColors = CustomBGRColorCollection.ToArray();
            dlgColor.FullOpen = true;

            // Dialogfeld anzeigen und Rückgabewert auswerten
            if (dlgColor.ShowDialog() == DialogResult.OK)
            {
                // Übernehmen des ausgewählten Pfades
                CustomBGRColorCollection = dlgColor.CustomColors.ToList();
                RefreshColor();
            }
        }
    }
}
