﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Printing;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert ein MDI-Clientfenster für die Verarbeitung von einfachen Standardtextdateien.
    /// </summary>
    public partial class CmdiStandardText : Form, ImdiBase
    {
        /// <summary>
        /// Initialisiert eine Instanz der Klasse CmdiStandardText.
        /// </summary>
        public CmdiStandardText()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Laden einer Datei vom Dateisystem.
        /// </summary>
        /// <param name="strFileName">Dateiname aus dem die Daten geladen werden sollen.</param>
        public void LoadFile(string strFileName)
        {
            _tbEdit.Text = File.ReadAllText(strFileName);
        }

        /// <summary>
        /// Ereignisbehandlung des Menübefehls 'Speichern unter ...'
        /// </summary>
        private void MenuFileSaveAs_Click(object sender, EventArgs e)
        {
            // Lese den Dateinamen über den Standarddialogfeld ein und prüfe die Gültigkeit 
            string strFileName = CwlbMainWindow.SaveFile("Textdatei speichern", "Textdatei speichern|*.txt", "*.txt");
            if (!string.IsNullOrEmpty(strFileName))
            {
                try
                {
                    // Speichern des Inhalts als Textdatei
                    using (TextWriter objTextFile = File.CreateText(strFileName))
                    {
                        objTextFile.Write(_tbEdit.Text);
                        this.Text = strFileName;
                    }
                }
                catch (Exception excObject)
                {
                    MessageBox.Show(
                        string.Format(
                            "Wärend dem Speichern der Datei ist folgender Fehler aufgetreten:{0}{1}",
                            Environment.NewLine,
                            excObject.Message),
                        CwlbMainWindow.TITLE,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Beginnen des Druckvorgangs.
        /// </summary>
        public void PrintBegin(object sender, PrintEventArgs e)
        {
            // Start at the beginning of the text
        }

        /// <summary>
        /// Ereignisbehandlung für das Drucken der Seite
        /// </summary>
        public void PrintPage(object sender, PrintPageEventArgs e)
        {
        }

        /// <summary>
        /// Ereignisbehandlung für das Beenden des Druckvorgangs.
        /// </summary>
        public void PrintEnd(object sender, PrintEventArgs e)
        {
        }
    }
}
