﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Printing;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert die Funktioanlität des Hauptfensters.
    /// </summary>
    public partial class CwlbMainWindow : Form
    {
        /// <summary>
        /// Definiert den Titel der Anwendung.
        /// </summary>
        public static string TITLE = "Learning Windows Forms 03";

        /// <summary>
        /// Liefert oder definiert ein Dokumentobjekt für das Drucken.
        /// </summary>
        private PrintDocument _objDocument;

        /// <summary>
        /// Liefert oder definiert das Standardverzeichnis der Anwendung.
        /// </summary>
        public string DefaultDirectory { get; set; }

        /// <summary>
        /// Ereignis für die Betätigung der Schaltfläche fett in der Symbolleiste.
        /// </summary>
        public event EventHandler<EventArgs> Bold;

        /// <summary>
        /// Ereignis für die Betätigung der Schaltfläche unterstrichen in der Symbolleiste.
        /// </summary>
        public event EventHandler<EventArgs> Underscore;

        /// <summary>
        /// Ereignis für die Betätigung der Schaltfläche auflisten in der Symbolleiste.
        /// </summary>
        public event EventHandler<EventArgs> List;

        /// <summary>
        /// Liefert oder definiert ein Dokumentobjekt für das Drucken.
        /// </summary>
        public PrintDocument PrintDocument
        {
            get
            {
                if (_objDocument == null)
                {
                    _objDocument = new PrintDocument();
                }
                return _objDocument;
            }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbMainWindow
        /// </summary>
        public CwlbMainWindow()
        {
            // Initialisierung des generierten Code
            InitializeComponent();

            // Registrieren der Ereignisbehandlung des Hauptfensters
            Application.Idle += new EventHandler(Application_Idle);
        }

        /// <summary>
        /// Behandlung des Leerlauf Ereignisses.
        /// </summary>
        void Application_Idle(object sender, EventArgs e)
        {
            // CAPS LOCK und NUM LOCK kontrollieren
            _statusCaps.Text = Console.CapsLock ? "CAPS" : string.Empty;
            _statusNum.Text = Console.NumberLock ? "NUM" : string.Empty;

            // Drucken kontrollieren
            if (MdiChildren.Length != 0)
            {
                _mnuWindowHorizontal.Enabled = true;
                _mnuWindowVertical.Enabled = true;
                _mnuWindowDiagonal.Enabled = true;
            }
            else
            {
                _mnuWindowHorizontal.Enabled = false;
                _mnuWindowVertical.Enabled = false;
                _mnuWindowDiagonal.Enabled = false;
            }
            if (ActiveMdiChild is CmdiRichTextFormat)
            {
                _mnuFilePageSetup.Enabled = true;
                _mnuFilePrint.Enabled = true;
                _mnuFilePrintPreview.Enabled = true;
                _tbtnPrint.Enabled = true;
            }
            else
            {
                _mnuFilePageSetup.Enabled = false;
                _mnuFilePrint.Enabled = false;
                _mnuFilePrintPreview.Enabled = false;
                _tbtnPrint.Enabled = false;
            }
        }

        /// <summary>
        /// Stellt die zentrale Handhabung des Dialogs Speichern zur Verfügung.
        /// </summary>
        /// <param name="strTitle">Anzuzeigender Dialogtitel.</param>
        /// <param name="strFilter">Zu verwendender Filterstring.</param>
        /// <param name="strExtension">Zu verwendende Erweiterung.</param>
        /// <returns>Liefert den Dateinamen oder eine Null-Referenz, wenn die datei nicht gespeichert werden soll.</returns>
        internal static string SaveFile(string strTitle, string strFilter, string strExtension)
        {
            // Dialogfeld zum Speichern der Datei
            SaveFileDialog dlgSave = new SaveFileDialog();

            // Initialisieren des Dialogs
            dlgSave.Filter = strFilter;
            dlgSave.FilterIndex = 0;
            dlgSave.AddExtension = true;
            dlgSave.OverwritePrompt = true;
            dlgSave.Title = strTitle;

            // Wenn Sie untenstehende Option einschalten, erscheint ein modifiziertes Dialogfeld
            // dlgSave.ShowHelp = true;

            // Dialogfeld anzeigen und Rückgabe auswerten
            if (dlgSave.ShowDialog() == DialogResult.OK)
            {
                return dlgSave.FileName;
            }
            else
            {
                return null;
            }
        }


        #region Ereignisbehandlung Stufe Formular
        /// <summary>
        /// Behandlung des Ereignisses Fenster laden.
        /// </summary>
        private void CwlbMainWindow_Load(object sender, EventArgs e)
        {
            _statusTime.Text = DateTime.Now.ToLongTimeString();
            ToolStripManager.LoadSettings(this);
        }

        /// <summary>
        /// Behandlung des Ereignisses Formular schliessen.
        /// </summary>
        private void CwlbMainWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            ToolStripManager.SaveSettings(this);
        }

        /// <summary>
        /// Ereignisbehandlung für MDI Child Fenster wurde aktiviert.
        /// </summary>
        private void CwlbMainWindow_MdiChildActivate(object sender, EventArgs e)
        {
            if (ActiveMdiChild is CmdiRichTextFormat)
            {
                _tbtnFormatBold.Enabled = true;
                _tbtnFormatUnderscore.Enabled = true;
                _tbtnFormatList.Enabled = true;
            }
            else
            {
                _tbtnFormatBold.Enabled = false;
                _tbtnFormatUnderscore.Enabled = false;
                _tbtnFormatList.Enabled = false;
            }
        }

        /// <summary>
        /// Behandlung des Ereignisses Haupttimer wurde ausgelöst.
        /// </summary>
        private void TimerMain_Tick(object sender, EventArgs e)
        {
            _statusTime.Text = DateTime.Now.ToLongTimeString();
        }
        #endregion

        #region Ereignisbehandlung Stufe Menü
        /// <summary>
        /// Ereignisbehandlung für Menübefehl Neue .rtf-Datei
        /// </summary>
        private void MenuFileNewRtf_Click(object sender, EventArgs e)
        {
            CmdiRichTextFormat objRtf = new CmdiRichTextFormat();
            objRtf.MdiParent = this;
            objRtf.Show();
        }

        /// <summary>
        /// Ereignisbehandlung für Menübefehl Neue .txt-Datei
        /// </summary>
        private void MenuFileNewTxt_Click(object sender, EventArgs e)
        {
            CmdiStandardText objText = new CmdiStandardText();
            objText.MdiParent = this;
            objText.Show();
        }

        /// <summary>
        /// Ereignisbehandlung für Menübefehl Datei öffnen ...
        /// </summary>
        private void MenuFileOpen_Click(object sender, EventArgs e)
        {
            // Erstellen des Dialogobjekts
            OpenFileDialog dlgOpen = new OpenFileDialog();

            // Initialisieren des Dialogs
            dlgOpen.Filter = "Textdateien|*.rtf;*.txt|RTF-Dateien|*.rtf|Textdateien|*.txt|Alle Dateien|*.*";
            dlgOpen.FilterIndex = 1;
            dlgOpen.Title = "Öffnen einer Textdatei";
            dlgOpen.RestoreDirectory = true;

            // Wenn Sie eine der untenstehenen Optionen einschalten, erscheint ein modifiziertes Dialogfeld
            //dlgOpen.ShowReadOnly = true;
            //dlgOpen.ShowHelp = true;

            if (dlgOpen.ShowDialog() == DialogResult.OK)
            {
                // Öffne Datei
                ImdiBase objMdi = null;
                try
                {
                    // Datei soll geöffnet werden
                    if (Path.GetExtension(dlgOpen.FileName).ToLower() == ".rtf")
                    {
                        // Erstelle ein RTF MDI Fenster
                        objMdi = new CmdiRichTextFormat();
                        objMdi.LoadFile(dlgOpen.FileName);
                    }
                    else if (Path.GetExtension(dlgOpen.FileName).ToLower() == ".txt")
                    {
                        // Erstelle ein TXT MDI Fenster
                        objMdi = new CmdiStandardText();
                        objMdi.LoadFile(dlgOpen.FileName);
                    }
                    else
                    {
                        throw new NotSupportedException();
                    }
                    ((Form)objMdi).MdiParent = this;
                    ((Form)objMdi).Show();
                    ((Form)objMdi).Text = dlgOpen.FileName;
                }
                catch (NotSupportedException)
                {
                    MessageBox.Show(
                        "Das ausgewählte Format wird nicht unterstützt",
                        TITLE,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
                catch (Exception excObject)
                {
                    MessageBox.Show(
                        string.Format(
                            "Wärend dem Laden der Datei ist folgender Fehler aufgetreten:{0}{1}",
                            Environment.NewLine,
                            excObject.Message),
                        TITLE,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für Menübefehl Seite einrichten ...
        /// </summary>
        private void MenuFilePageSetup_Click(object sender, EventArgs e)
        {
            // Dialogobjekt erstellen
            PageSetupDialog dlgPageSetup = new PageSetupDialog();

            // Druckereinstellungen initialisieren
            dlgPageSetup.PrinterSettings = new PrinterSettings();
            dlgPageSetup.AllowMargins = true;
            dlgPageSetup.AllowOrientation = true;
            dlgPageSetup.AllowPaper = true;
            dlgPageSetup.PageSettings = PrintDocument.DefaultPageSettings;

            // Folgende beiden Einstellungen bleiben ab Windows Vista wirkungslos
            //dlgPageSetup.AllowPrinter = true;
            //dlgPageSetup.ShowNetwork = true;

            // Anzeigen des Dialogfeldes und Daten übernehmen wenn OK
            if (dlgPageSetup.ShowDialog() == DialogResult.OK)
            {
                // Daten in das Datenmodell der Anwendung übernehmen
                PrintDocument.DefaultPageSettings = dlgPageSetup.PageSettings;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für Menübefehl Seitenansicht ...
        /// </summary>
        private void MenuPagePreview_Click(object sender, EventArgs e)
        {
            // Dailogfeld für die Druckvorschau erstellen
            PrintPreviewDialog dlgPrintPreview = new PrintPreviewDialog();

            // Ereignisbehandlungen zuordnen
            if (ActiveMdiChild is CmdiRichTextFormat)
            {
                PrintDocument.BeginPrint += new PrintEventHandler(((CmdiRichTextFormat)ActiveMdiChild).PrintBegin);
                PrintDocument.PrintPage += new PrintPageEventHandler(((CmdiRichTextFormat)ActiveMdiChild).PrintPage);
                PrintDocument.EndPrint += new PrintEventHandler(((CmdiRichTextFormat)ActiveMdiChild).PrintEnd);
            }
            else if (ActiveMdiChild is CmdiStandardText)
            {
                PrintDocument.BeginPrint += new PrintEventHandler(((CmdiStandardText)ActiveMdiChild).PrintBegin);
                PrintDocument.PrintPage += new PrintPageEventHandler(((CmdiStandardText)ActiveMdiChild).PrintPage);
                PrintDocument.EndPrint += new PrintEventHandler(((CmdiStandardText)ActiveMdiChild).PrintEnd);
            }

            // Definiere den Dokumentnamen
            PrintDocument.DocumentName = "RTF Ausgabe";

            // Definiere die kleinste Ausdehnung des Dialogs
            dlgPrintPreview.MinimumSize = new Size(400, 250);
            dlgPrintPreview.StartPosition = FormStartPosition.CenterParent;
            dlgPrintPreview.Size = this.Size;
            dlgPrintPreview.UseAntiAlias = true;            // Einschalten der Kantenglättung
            dlgPrintPreview.Document = PrintDocument;       // Definiere das Ausgabedokument

            // Das PrintPreviewControl-Steuerelement kann separat konfiguriert werden
            dlgPrintPreview.PrintPreviewControl.BackColor = Color.Wheat;
            dlgPrintPreview.PrintPreviewControl.Zoom = 1;

            // Anzeigen des Druckdialogs
            dlgPrintPreview.ShowDialog();

            // Zuordnen Ereignisbehandlungen aufheben
            if (ActiveMdiChild is CmdiRichTextFormat)
            {
                PrintDocument.BeginPrint -= new PrintEventHandler(((CmdiRichTextFormat)ActiveMdiChild).PrintBegin);
                PrintDocument.PrintPage -= new PrintPageEventHandler(((CmdiRichTextFormat)ActiveMdiChild).PrintPage);
                PrintDocument.EndPrint -= new PrintEventHandler(((CmdiRichTextFormat)ActiveMdiChild).PrintEnd);
            }
            else if (ActiveMdiChild is CmdiStandardText)
            {
                PrintDocument.BeginPrint -= new PrintEventHandler(((CmdiStandardText)ActiveMdiChild).PrintBegin);
                PrintDocument.PrintPage -= new PrintPageEventHandler(((CmdiStandardText)ActiveMdiChild).PrintPage);
                PrintDocument.EndPrint -= new PrintEventHandler(((CmdiStandardText)ActiveMdiChild).PrintEnd);
            }
        }

        /// <summary>
        /// Ereignisbehandlung für Menübefehl Drucken.
        /// </summary>
        private void MenuPrint_Click(object sender, EventArgs e)
        {
            // Instanziiere das Druckdialogfeld
            PrintDialog dlgPrint = new PrintDialog();

            // Ereignisbehandlungen zuordnen
            if (ActiveMdiChild is CmdiRichTextFormat)
            {
                PrintDocument.BeginPrint += new PrintEventHandler(((CmdiRichTextFormat)ActiveMdiChild).PrintBegin);
                PrintDocument.PrintPage += new PrintPageEventHandler(((CmdiRichTextFormat)ActiveMdiChild).PrintPage);
                PrintDocument.EndPrint += new PrintEventHandler(((CmdiRichTextFormat)ActiveMdiChild).PrintEnd);
            }
            else if (ActiveMdiChild is CmdiStandardText)
            {
                PrintDocument.BeginPrint += new PrintEventHandler(((CmdiStandardText)ActiveMdiChild).PrintBegin);
                PrintDocument.PrintPage += new PrintPageEventHandler(((CmdiStandardText)ActiveMdiChild).PrintPage);
                PrintDocument.EndPrint += new PrintEventHandler(((CmdiStandardText)ActiveMdiChild).PrintEnd);
            }

            // Initialisiere das Druckdialogfeld
            dlgPrint.AllowCurrentPage = false;
            dlgPrint.AllowSomePages = true;
            dlgPrint.AllowSelection = false;
            dlgPrint.UseEXDialog = true;

            // Definiere das Ausgabedokument
            dlgPrint.Document = PrintDocument;

            // Dialogfeld anzeigen und Rückgabe auswerten
            if (dlgPrint.ShowDialog() == DialogResult.OK)
            {
                PrintDocument.Print();
            }

            // Zuordnen Ereignisbehandlungen aufheben
            if (ActiveMdiChild is CmdiRichTextFormat)
            {
                PrintDocument.BeginPrint -= new PrintEventHandler(((CmdiRichTextFormat)ActiveMdiChild).PrintBegin);
                PrintDocument.PrintPage -= new PrintPageEventHandler(((CmdiRichTextFormat)ActiveMdiChild).PrintPage);
                PrintDocument.EndPrint -= new PrintEventHandler(((CmdiRichTextFormat)ActiveMdiChild).PrintEnd);
            }
            else if (ActiveMdiChild is CmdiStandardText)
            {
                PrintDocument.BeginPrint -= new PrintEventHandler(((CmdiStandardText)ActiveMdiChild).PrintBegin);
                PrintDocument.PrintPage -= new PrintPageEventHandler(((CmdiStandardText)ActiveMdiChild).PrintPage);
                PrintDocument.EndPrint -= new PrintEventHandler(((CmdiStandardText)ActiveMdiChild).PrintEnd);
            }
        }

        /// <summary>
        /// Ereignisbehandlung für Menübefehl Einstellungen vornehmen.
        /// </summary>
        private void MenuOptionsSettings_Click(object sender, EventArgs e)
        {
            // Erstellen des Dialogsfür Einstellungen
            CdlgSettings dlgSettings = new CdlgSettings();

            // Initialisieren des Dialogs
            dlgSettings.DefaultDirectory = DefaultDirectory;

            // Anzeigen des Dialogs und Auswerten der Rückgabe
            if (dlgSettings.ShowDialog() == DialogResult.OK)
            {
                // Daten aus dem Dailog übernehmen
                DefaultDirectory = dlgSettings.DefaultDirectory;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für Menübefehl Fenster horizontal anordnen.
        /// </summary>
        private void MenuWindowHorizontal_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.TileHorizontal);
        }

        /// <summary>
        /// Ereignisbehandlung für Menübefehl Fenster vertikal anordnen.
        /// </summary>
        private void MenuWindowVertical_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.TileVertical);
        }

        /// <summary>
        /// Ereignisbehandlung für Menübefehl Fenster diagonal anordnen.
        /// </summary>
        private void MenuWindowDiagonal_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.Cascade);
        }

        /// <summary>
        /// Ereignisbehandlung für Menübefehl Infodialogfeld anzeigen.
        /// </summary>
        private void MenuInfo_Click(object sender, EventArgs e)
        {
            CdlgInfo objInfo = new CdlgInfo();
            objInfo.ShowDialog();
        }
        #endregion  

        /// <summary>
        /// Ereignisbehandlung für Toolbar Schaltfläche fett wurde betätigt.
        /// </summary>
        private void _tbtnFormatBold_Click(object sender, EventArgs e)
        {
            if (Bold != null)
            {
                Bold(this, new EventArgs());
            }
        }

        /// <summary>
        /// Ereignisbehandlung für Toolbar Schaltfläche unterstrichen wurde betätigt.
        /// </summary>
        private void _tbtnFormatUnderscore_Click(object sender, EventArgs e)
        {
            if (Underscore != null)
            {
                Underscore(this, new EventArgs());
            }
        }

        /// <summary>
        /// Ereignisbehandlung für Toolbar Schaltfläche listenformat wurde betätigt.
        /// </summary>
        private void _tbtnFormatList_Click(object sender, EventArgs e)
        {
            if (List != null)
            {
                List(this, new EventArgs());
            }
        }
    }
}
