﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert den Beispielcode der Masked Textbox
    /// </summary>
    public partial class CpnlMaskedTextBox : UserControl
    {
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CpnlMaskedTextBox
        /// </summary>
        public CpnlMaskedTextBox()
        {
            InitializeComponent();

            // Automatische Validierung für Wert einschalten
            _tbValue.ValidatingType = typeof(uint);
        }

        /// <summary>
        /// Ereignisbehandlung für die Validierung des Inhalts.
        /// </summary>
        private void DateTime_Validating(object sender, CancelEventArgs e)
        {
            // Prüfen, ob das Feld Eingaben hat, und wenn nein die Validierung verlassen
            // Diese Maßnahme ist nur notwendig, wenn die Validierung abgebrochen werden
            // können muss
            if (_tbDateTime.MaskedTextProvider.AssignedEditPositionCount == 0)
            {
                return;
            }

            // Prüfen, ob die Maske komplett ausgefüllt wurde, und wenn nein Fehler anzeigen
            // sowie Steuerung so definieren, dass das Feld nicht verlassen wird.
            if (!_tbDateTime.MaskCompleted)
            {
                _errProvider.SetError(_tbDateTime, "Die Eingabe ist unvollständig.");
                e.Cancel = true;
                return;
            }

            // Eingabe ist komplett, Datum prüfen und Fehler anzeigen wenn nicht gültig,
            // respektive Fehler löschen wenn gültig
            DateTime dtCheck;
            if (!DateTime.TryParse(_tbDateTime.Text, out dtCheck))
            {
                _errProvider.SetError(_tbDateTime, "Die Eingabe definiert keinen gültigen Zeitpunkt.");
                e.Cancel = true;
            }
            else
            {
                _errProvider.Clear();
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die erweiterte Validierung der Datumseingabe.
        /// </summary>
        private void Date_TypeValidationCompleted(object sender, TypeValidationEventArgs e)
        {
            // Prüfen, ob das Feld Eingaben hat, und wenn nein die Validierung verlassen
            // Diese Maßnahme ist nur notwendig, wenn die Validierung abgebrochen werden
            // können muss
            if (_tbDate.MaskedTextProvider.AssignedEditPositionCount == 0)
            {
                _errProvider.Clear();
                return;
            }

            // Prüfen, ob die Eingabe gültig ist
            if (!e.IsValidInput)
            {
                _errProvider.SetError(_tbDate, "Die Eingabe definiert keinen gültigen Zeitpunkt.");
                e.Cancel = true;
            }
            else
            {
                if (!((DateTime)e.ReturnValue >= DateTime.Now.AddYears(-100) &&
                      (DateTime)e.ReturnValue < DateTime.Now.AddYears(100)))
                {
                    _errProvider.SetError(
                        _tbDate, 
                        string.Format( 
                            "Die Eingabe muss zwischen {0} und {1} liegen.",
                            DateTime.Now.AddYears(-100),
                            DateTime.Now.AddYears(100)));
                    e.Cancel = true;
                }
                else
                {
                    _errProvider.Clear();
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die erweiterte Validierung der numerischen Eingabe.
        /// </summary>
        private void Value_TypeValidationCompleted(object sender, TypeValidationEventArgs e)
        {
            // Prüfen, ob das Feld Eingaben hat, und wenn nein die Validierung verlassen
            // Diese Maßnahme ist nur notwendig, wenn die Validierung abgebrochen werden
            // können muss
            if (_tbValue.MaskedTextProvider.AssignedEditPositionCount == 0)
            {
                _errProvider.Clear();
                return;
            }

            // Prüfen, ob die Eingabe gültig ist
            if (!e.IsValidInput)
            {
                _errProvider.SetError(_tbValue, "Das Feld ist nicht korrekt ausgefüllt.");
                e.Cancel = true;
            }
            else
            {
                if (!((uint)e.ReturnValue >= 8000 &&
                      (uint)e.ReturnValue < 80000))
                {
                    _errProvider.SetError( _tbValue, 
                        string.Format( "Die Eingabe muss zwischen {0} und {1} liegen.", 8000, 80000));
                    e.Cancel = true;
                }
                else
                {
                    _errProvider.Clear();
                }
            }
        }
    }
}
