﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;
using System.IO;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Funktionalität des Dialogfeldes für die Demos des Chart-Steuerelements.
    /// </summary>
    public partial class CpnlChart : UserControl
    {
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CpnlChart.
        /// </summary>
        public CpnlChart()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung für das Laden des Elements.
        /// </summary>
        private void CpnlChart_Load(object sender, EventArgs e)
        {
            // Grafiktypen initialisieren
            _cbChartType.Items.AddRange(Enum.GetNames(typeof(SeriesChartType)));
            _cbChartType.SelectedIndex = 10;
            _chk3D.Checked = true;

            // Daten initialisieren
            _chartData.Series[0].Points.Add(new DataPoint(1, 60));
            _chartData.Series[0].Points.Add(new DataPoint(2, 70));
            _chartData.Series[0].Points.Add(new DataPoint(3, 80));
            _chartData.Series[0].Points.Add(new DataPoint(4, 40));
            _chartData.Series[0].Points.Add(new DataPoint(5, 50));
            _chartData.Series[0].Points.Add(new DataPoint(6, 65));
            _chartData.Series[0].Points.Add(new DataPoint(7, 90));
            _chartData.Series[0].Points.Add(new DataPoint(8, 75));
            _chartData.Series[0].Points.Add(new DataPoint(9, 35));
            _chartData.Series[0].Points.Add(new DataPoint(10, 55));
            _chartData.Series[0].Points.Add(new DataPoint(11, 75));
            _chartData.Series[0].Points.Add(new DataPoint(12, 95));
            _chartData.Series.Add(string.Empty);
            _chartData.Series[1].Points.Add(new DataPoint(1, 10));
            _chartData.Series[1].Points.Add(new DataPoint(2, 50));
            _chartData.Series[1].Points.Add(new DataPoint(3, 20));
            _chartData.Series[1].Points.Add(new DataPoint(4, 50));
            _chartData.Series[1].Points.Add(new DataPoint(5, 30));
            _chartData.Series[1].Points.Add(new DataPoint(6, 45));
            _chartData.Series[1].Points.Add(new DataPoint(7, 30));
            _chartData.Series[1].Points.Add(new DataPoint(8, 45));
            _chartData.Series[1].Points.Add(new DataPoint(9, 35));
            _chartData.Series[1].Points.Add(new DataPoint(10, 35));
            _chartData.Series[1].Points.Add(new DataPoint(11, 45));
            _chartData.Series[1].Points.Add(new DataPoint(12, 25));
            _chartData.Series.Add(string.Empty);
            _chartData.Series[2].Points.Add(new DataPoint(1, 5));
            _chartData.Series[2].Points.Add(new DataPoint(2, 25));
            _chartData.Series[2].Points.Add(new DataPoint(3, 10));
            _chartData.Series[2].Points.Add(new DataPoint(4, 30));
            _chartData.Series[2].Points.Add(new DataPoint(5, 20));
            _chartData.Series[2].Points.Add(new DataPoint(6, 35));
            _chartData.Series[2].Points.Add(new DataPoint(7, 10));
            _chartData.Series[2].Points.Add(new DataPoint(8, 0));
            _chartData.Series[2].Points.Add(new DataPoint(9, 15));
            _chartData.Series[2].Points.Add(new DataPoint(10, 15));
            _chartData.Series[2].Points.Add(new DataPoint(11, 20));
            _chartData.Series[2].Points.Add(new DataPoint(12, 20));

            // Charteinstellungen vornehmen
            // Titel
            _chartData.Titles.Add(new Title("Demo Chart-Steuerelement", Docking.Top));
            _chartData.Titles[0].Font = new Font("Calibra", 18.0f);
            _chartData.Titles[0].ForeColor = Color.Red;
            _chartData.Series[0].Color = Color.OrangeRed;
            // Datenserien benennen
            _chartData.Series[0].LegendText = "DemoDaten Serie 0";
            _chartData.Series[1].Color = Color.DodgerBlue;
            _chartData.Series[1].LegendText = "Demodaten Serie 1";
            _chartData.Series[2].Color = Color.DeepPink;
            _chartData.Series[2].LegendText = "Demodaten Serie 2";
            // Legende positionieren
            _chartData.Legends[0].Docking = Docking.Bottom;
            _chartData.Legends[0].LegendStyle = LegendStyle.Table;
            _chartData.Legends[0].Font = new Font("Calibra", 12.0f);
            // Achsen beschriften
            _chartData.ChartAreas[0].AxisX.Title = "Monate";
            _chartData.ChartAreas[0].AxisX.TitleFont = new Font("Calibra", 10.0f, FontStyle.Bold);
            _chartData.ChartAreas[0].AxisX.TitleAlignment = StringAlignment.Far;
            _chartData.ChartAreas[0].AxisY.Title = "Anz. Verkäufe";
            _chartData.ChartAreas[0].AxisY.TitleFont = new Font("Calibra", 10.0f, FontStyle.Bold);
            _chartData.ChartAreas[0].AxisY.TitleAlignment = StringAlignment.Far;
            _chartData.ChartAreas[0].AxisX.Interval = 3;
            _chartData.ChartAreas[0].AxisY.Interval = 20;
            _chartData.ChartAreas[0].AxisY.MajorGrid.Interval = 20;
        }

        /// <summary>
        /// Ereignisbehandlung für die Auswahl des Grafiktyps.
        /// </summary>
        private void ChartType_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Grafiktyp aus dem ComboBox-Steuerelement auslesen
            SeriesChartType enmChartType =
                (SeriesChartType)Enum.Parse(typeof(SeriesChartType), 
                _cbChartType.SelectedItem.ToString());

            foreach (Series objSerie in _chartData.Series)
            {
                objSerie.ChartType = enmChartType;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die Aktivierung und Deaktivierung des *D-Stils.
        /// </summary>
        private void _chk3D_CheckedChanged(object sender, EventArgs e)
        {
            if (_chk3D.Checked)
            {
                _chartData.ChartAreas[0].Area3DStyle.Enable3D = true;
            }
            else
            {
                _chartData.ChartAreas[0].Area3DStyle.Enable3D = false;
            }
        }

        /// <summary>
        /// Ereignisbehandlung des Kontextmenübefehls zum Öffnen der Daten des Chart-Steuerelements.
        /// </summary>
        private void ChartOpen_Click(object sender, EventArgs e)
        {
            // Instanz des Dialogfeldes erstellen
            OpenFileDialog dlgOpen = new OpenFileDialog();
            // Initialisierung der Filter
            dlgOpen.Filter = "Binäre Daten|*.chart|XML Daten|*.xml";
            dlgOpen.FilterIndex = 2;

            // Dialog anzeigen und verarbeiten
            if (dlgOpen.ShowDialog() == DialogResult.OK)
            {
                if (dlgOpen.FilterIndex == 1)
                {
                    _chartData.Serializer.Format = System.Windows.Forms.DataVisualization.Charting.SerializationFormat.Binary;
                }
                else
                {
                    _chartData.Serializer.Format = System.Windows.Forms.DataVisualization.Charting.SerializationFormat.Xml;
                }
                _chartData.Serializer.Load(dlgOpen.FileName);
            }
        }

        /// <summary>
        /// Ereignisbehandlung des Kontextmenübefehls zum Speichern der Daten des Chart-Steuerelements.
        /// </summary>
        private void ChartSave_Click(object sender, EventArgs e)
        {
            // Instanz des Dialogfeldes erstellen
            SaveFileDialog dlgSave = new SaveFileDialog();
            // Initialisierung der Filter
            dlgSave.Filter = "Binäre Daten|*.chart|XML Daten|*.xml";
            dlgSave.FilterIndex = 2;

            // Dialog anzeigen und verarbeiten
            if (dlgSave.ShowDialog() == DialogResult.OK)
            {
                if (dlgSave.FilterIndex == 1)
                {
                    _chartData.Serializer.Format = System.Windows.Forms.DataVisualization.Charting.SerializationFormat.Binary;
                }
                else
                {
                    _chartData.Serializer.Format = System.Windows.Forms.DataVisualization.Charting.SerializationFormat.Xml;
                }
                _chartData.Serializer.Save(dlgSave.FileName);
            }
        }

        /// <summary>
        /// Ereignisbehandlung des Kontextmenübefehls Speichern der Grafik als Bild.
        /// </summary>
        private void ChartSaveImage_Click(object sender, EventArgs e)
        {
            // Instanz des Dialogfeldes erstellen
            SaveFileDialog dlgSave = new SaveFileDialog();
            // Initialisierung der Filter
            dlgSave.Filter = "File interchange format|*.jpg";
            dlgSave.Filter += "|Bitmap format|*.bmp";
            dlgSave.Filter += "|Graphics interchange format|*.gif";
            dlgSave.Filter += "|Enhanced meta file|*.emf";
            dlgSave.Filter += "|Portable network grafik|*.png";
            dlgSave.Filter += "|Tagged image file format|*.tiff";
            dlgSave.FilterIndex = 5;

            // Dialog anzeigen und verarbeiten
            if (dlgSave.ShowDialog() == DialogResult.OK)
            {
                // Definieren des Bildformats
                ChartImageFormat enmFormat;
                switch(dlgSave.FilterIndex)
                {
                    case 1:
                        enmFormat = ChartImageFormat.Jpeg;
                        break;
                    case 2:
                        enmFormat = ChartImageFormat.Bmp;
                        break;
                    case 3:
                        enmFormat = ChartImageFormat.Gif;
                        break;
                    case 4:
                        enmFormat = ChartImageFormat.Emf;
                        break;
                    case 5:
                        enmFormat = ChartImageFormat.Png;
                        break;
                    case 6:
                        enmFormat = ChartImageFormat.Tiff;
                        break;
                    default:
                        enmFormat = ChartImageFormat.Png;
                        break;
                }
                // Speichern in der definierten Datei und dem gewünschten Format
                _chartData.SaveImage(dlgSave.FileName, enmFormat);

                // Prüfen, ob der Benutzer die Grafik auch in der Zwischenablage wünscht
                if (MessageBox.Show(
                    "Wünschen Sie die Übertragung der Grafik in die Zwischenablege?",
                    CwlbMainWindow.TITLE, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    // Gleichzeitig in die Zwischenablage stellen
                    MemoryStream streamGraphics = new MemoryStream();
                    _chartData.SaveImage(streamGraphics, enmFormat);
                    Image imgGraphics = Bitmap.FromStream(streamGraphics);
                    Clipboard.Clear();
                    Clipboard.SetImage(imgGraphics);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung des Kontextmenübefehls Daten importieren.
        /// </summary>
        private void ChartImportData_Click(object sender, EventArgs e)
        {
            // Instanz des Dialogfeldes erstellen
            OpenFileDialog dlgOpen = new OpenFileDialog();
            // Initialisierung der Filter
            dlgOpen.Filter = "Xml Datei|*.xml";
            dlgOpen.FilterIndex = 1;

            // Dialog anzeigen und verarbeiten
            if (dlgOpen.ShowDialog() == DialogResult.OK)
            {
                // Lese die Daten als Datenset ein
                DataSet dataSet = new DataSet();
                dataSet.ReadXml(dlgOpen.FileName);
                _chartData.DataSource = dataSet;
                // Konfigurieren der Bindungen
                _chartData.Series[0].XValueMember = "X";
                _chartData.Series[0].YValueMembers = "Y0";
                _chartData.Series[1].YValueMembers = "Y1";
                _chartData.Series[2].YValueMembers = "Y2";
                _chartData.DataBind();
            }
        }

        /// <summary>
        /// Ereignisbehandlung des Kontextmenübefehls Daten exportieren.
        /// </summary>
        private void ChartExportData_Click(object sender, EventArgs e)
        {
            // Instanz des Dialogfeldes erstellen
            SaveFileDialog dlgSave = new SaveFileDialog();
            // Initialisierung der Filter
            dlgSave.Filter = "Xml Datei|*.xml";
            dlgSave.FilterIndex = 1;
            dlgSave.AddExtension = true;

            // Dialog anzeigen und verarbeiten
            if (dlgSave.ShowDialog() == DialogResult.OK)
            {
                DataSet dataSet = _chartData.DataManipulator.ExportSeriesValues();
                dataSet.WriteXml(dlgSave.FileName);
            }
        }
    }
}
