﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections.ObjectModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Funktionalität des Dialogfeldes für die Demos des CpnlDataGridView-Steuerelements.
    /// </summary>
    public partial class CpnlDataGridView : UserControl
    {
        // Konstanten für den Zugriff auf die Elemente des Dataset
        private const string PERSON_TABLE = "Person";
        private const string COLUMN_PERSON_ID = "PersonId";
        private const string COLUMN_PERSON_NAME = "Name";
        private const string COLUMN_PERSON_FIRST_NAME = "FirstName";
        private const string COLUMN_PERSON_CITY = "City";
        private const string COLUMN_ORDER_ID = "OrderId";
        private const string COLUMN_ORDER_PERSON_ID = COLUMN_PERSON_ID;
        private const string COLUMN_ORDER_DATE = "OrderDate";
        private const string COLUMN_ORDER_COSTS = "Costs";
        private const string COLUMN_ORDER_STATE = "State";
        private const string PERSON_ORDER_RELATION = "PersonOrders";

        // Dataset und Bindungselemente für den Zugriff auf die XML Daten
        private DataSet _dataSet = new DataSet();
        private BindingSource _objPersonsBindingSource = new BindingSource();
        private BindingSource _objDetailsBindingSource = new BindingSource();

        // Konstanten für die Schaltfläche
        private const string BUTTON_TEXT_LOCKED = "Locked";
        private const string BUTTON_TEXT_UNLOCKED = "Unlocked";

        // Eigenes Steuerelement
        private DataGridViewButtonColumn _colButton;

        /// <summary>
        /// Initialisiert eine Instanz der Klasse CpnlDataGridView.
        /// </summary>
        public CpnlDataGridView()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung für das Laden des Formulars
        /// </summary>
        private void CpnlDataGridView_Load(object sender, EventArgs e)
        {
            // Ergänzen einer neuen Spalte
            _colButton = new DataGridViewButtonColumn();
            _colButton.HeaderText = "Status";
            _dgrDemo.CellClick += new DataGridViewCellEventHandler(ListView_CellClick);
            _dgrDemo.RowsAdded += new DataGridViewRowsAddedEventHandler(ListView_RowsAdded);
            _dgrDemo.Columns.Add(_colButton);

            // Konfigurieren des Datentyps für die Spalte von Zahlenwerten
            _colDouble.ValueType = typeof(double);

            // Konfigurieren der Spalte für den Kalender zur Nutzung eines eigenen Zellen-Steuerelements
            _dgrDemo.Columns["_colDateTime"].CellTemplate = new CwlbDataGridViewCalendarCell();
            _dgrDemo.Columns[3].HeaderText = "Datum";
            _dgrDemo.Columns[3].ValueType = typeof(DateTime);

            // Konfigurieren der Spalte für Aufzählung
            DataGridViewComboBoxColumn col = _dgrDemo.Columns[4] as DataGridViewComboBoxColumn;
            col.Name = "Auflistung";
            col.DataSource = Enum.GetValues(typeof(EwlbPropertyGridDemo));
            col.ValueType = typeof(EwlbPropertyGridDemo);

            // Erste generierte Zeile korrekt erstellen. Dies ist notwendig weil die Zellenvorlage der
            // Datumsspalte auf die erste generierte Zeile nicht korrekt angewendet wird.
            _dgrDemo.Rows.Add();
            _dgrDemo.AllowUserToAddRows = true;
            _dgrDemo.Rows.RemoveAt(0);
        }

        /// <summary>
        /// Ereignisbehandlung für den Kontextmenübefehl Öffnen.
        /// </summary>
        private void MenuOpen_Click(object sender, EventArgs e)
        {
            // Standarddialog für Dateizugriff verwenden
            OpenFileDialog dlgOpen = new OpenFileDialog();
            dlgOpen.Filter = "XML Daten|*.xml";
            dlgOpen.FilterIndex = 1;

            // Dialog anzeigen und Abschluss prüfen
            if (dlgOpen.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    // Auschalten des Lables
                    _lblInroduction.Visible = false;
                    // Herstellen der Datenbindung für beide Listen
                    _dgrPerson.DataSource = _objPersonsBindingSource;
                    _dgrDetail.DataSource = _objDetailsBindingSource;

                    // Daten einlesen
                    _dataSet.ReadXml(dlgOpen.FileName);

                    // Personentabelle binden
                    _objPersonsBindingSource.DataSource = _dataSet;
                    _objPersonsBindingSource.DataMember = PERSON_TABLE;
                    // Detailtabelle Binden
                    _objDetailsBindingSource.DataSource = _objPersonsBindingSource;
                    _objDetailsBindingSource.DataMember = PERSON_ORDER_RELATION;
                    // Einstellungen der Spaltenbreite automatisieren
                    _dgrPerson.AutoResizeColumns();
                    _dgrDetail.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.AllCells;

                    // Auswahlspalte ausschalten und Einzelauswahl einschalten
                    _dgrPerson.RowHeadersVisible = false;
                    _dgrPerson.SelectionMode = DataGridViewSelectionMode.FullRowSelect;
                    _dgrPerson.MultiSelect = false;
                    _dgrDetail.RowHeadersVisible = false;
                    _dgrDetail.SelectionMode = DataGridViewSelectionMode.FullRowSelect;

                    // Ausschalten der Möglichkeit zum Erfassen von neuen Daten
                    _dgrPerson.AllowUserToAddRows = false;
                    // Spalten mit Datenschlüssel unsichtbar schalten
                    _dgrPerson.Columns[COLUMN_PERSON_ID].Visible = false;
                    _dgrDetail.Columns[COLUMN_ORDER_PERSON_ID].Visible = false;
                    _dgrDetail.Columns[COLUMN_ORDER_ID].Visible = false;
                    // Formatierungen festlegen
                    _dgrDetail.Columns[COLUMN_ORDER_DATE].DefaultCellStyle.Format = "d";
                    _dgrDetail.Columns[COLUMN_ORDER_COSTS].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
                    _dgrDetail.Columns[COLUMN_ORDER_COSTS].DefaultCellStyle.Format = "N2";

                    // Spaltenbeschriftungen anpassen
                    _dgrPerson.Columns[COLUMN_PERSON_NAME].HeaderText = "Name";
                    _dgrPerson.Columns[COLUMN_PERSON_FIRST_NAME].HeaderText = "Vorname";
                    _dgrPerson.Columns[COLUMN_PERSON_CITY].HeaderText = "Ort";
                    _dgrDetail.Columns[COLUMN_ORDER_COSTS].HeaderText = "Kosten";
                    _dgrDetail.Columns[COLUMN_ORDER_DATE].HeaderText = "Datum";
                    _dgrDetail.Columns[COLUMN_ORDER_STATE].HeaderText = "Status";
                }
                catch(Exception excObject)
                {
                    MessageBox.Show(string.Format("Fehler beim Einlesen der Daten:{0}{1}", Environment.NewLine, excObject.Message),
                        CwlbMainWindow.TITLE, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die Validierung einer Zelle in der ersten Tabelle.
        /// </summary>
        private void ListView_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            if (e.ColumnIndex == 1 && e.RowIndex >= 0)
            {
                double dValue = 0.0;
                if ( double.TryParse(e.FormattedValue.ToString(), out dValue) && !(10 <= dValue && dValue < 100))
                {
                    MessageBox.Show("Sie müssen einen Wert zwischen 10 und 100 eingeben", CwlbMainWindow.TITLE, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    e.Cancel = true;
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Einfügen einer Zeile.
        /// </summary>
        void ListView_RowsAdded(object sender, DataGridViewRowsAddedEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                _dgrDemo[5, e.RowIndex].Value = BUTTON_TEXT_UNLOCKED;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Verändern einer Zelle.
        /// </summary>
        void ListView_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            // Das Ereignis ist nur zu beachten, wenn die Veränderung in einer gültigen Zeile
            // und für die Spalte erfolgt, auf die sich die Schaltfläche bezieht
            if (e.ColumnIndex == _colButton.Index && e.RowIndex >= 0)
            {
                // Prüfen, ob die Schaltfläche den Text Locked beinhaltet
                if (_dgrDemo[e.ColumnIndex, e.RowIndex].Value.ToString() == BUTTON_TEXT_LOCKED)
                {
                    // Die Zeile wird für Eingaben gesperrt und eingefärbt
                    _dgrDemo[e.ColumnIndex, e.RowIndex].Value = BUTTON_TEXT_UNLOCKED;
                    for (int iIndex = 4; iIndex >= 0; iIndex--)
                    {
                        _dgrDemo[iIndex, e.RowIndex].ReadOnly = false;
                        _dgrDemo[iIndex, e.RowIndex].Style.BackColor = Color.FromKnownColor(KnownColor.Control);
                    }
                }
                else
                {
                    _dgrDemo[e.ColumnIndex, e.RowIndex].Value = BUTTON_TEXT_LOCKED;
                    for( int iIndex = 4; iIndex >= 0; iIndex--)
                    {
                        _dgrDemo[iIndex, e.RowIndex].ReadOnly = true;
                        _dgrDemo[iIndex, e.RowIndex].Style.BackColor = Color.Salmon;
                    }
                }
            }
        }
    }
}
