﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Funktionalität des Dialogfeldes für die Demos der ListView-Steuerelemente.
    /// </summary>
    public partial class CpnlListView : UserControl
    {
        // Verwendete Konstanten für die Gruppierung
        const string GROUP_DIR = "dir";
        const string GROUP_EXE = "exe";
        const string GROUP_DLL = "dll";
        const string GROUP_ALL = "all";

        /// <summary>
        /// Initialisiert ein neue Instanz der Klasse CpnlListView.
        /// </summary>
        public CpnlListView()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung für das Laden des Formulars
        /// </summary>
        private void CpnlListView_Load(object sender, EventArgs e)
        {
            // Initialisieren der Stilauswahl
            _cbStyle.Items.AddRange(Enum.GetNames(typeof(View)));
            _cbStyle.SelectedIndex = 1;

            // Initialisiern der Gruppierung
            _lvMain.Groups.Add(GROUP_DIR, "Verzeichnisse");
            _lvMain.Groups.Add(GROUP_EXE, "Ausführbare Programme");
            _lvMain.Groups.Add(GROUP_DLL, "Bibliotheken");
            _lvMain.Groups.Add(GROUP_ALL, "Übrige Dateien");

            // Initialisieren des angezeigten Verzeichnisses
            _tbDirectory.Text = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);

            // Initialisieren der Anzeige der Gruppierung
            _chkGroupEnable.Checked = true;
        }

        /// <summary>
        /// Ereignisbehandlung für die Veränderung der Darstellung.
        /// </summary>
        private void _cbStyle_SelectedIndexChanged(object sender, EventArgs e)
        {
            _lvMain.View = (View)Enum.Parse(typeof(View), _cbStyle.SelectedItem.ToString());
        }

        /// <summary>
        /// Ereignisbehandlung für die Veränderung des Verzeichnisnamens
        /// </summary>
        private void Directory_TextChanged(object sender, EventArgs e)
        {
            // Lokale Daten
            DirectoryInfo objDirInfo;
            FileInfo objFileInfo;
            ListViewItem objItem;

            // Prüfen, ob das Verzeichnis existiert
            if (Directory.Exists(_tbDirectory.Text))
            {
                // Löschen des Inhalts der Liste 
                _lvMain.Items.Clear();
                _lvMain.ShowGroups = true;
                try
                {

                    // Vrzeichnisse einfüllen
                    foreach (string strDirectory in Directory.GetDirectories(_tbDirectory.Text))
                    {
                        // Verzeichnis informationen abfragen
                        objDirInfo = new DirectoryInfo(strDirectory);

                        // Item für die ListView bereitstellen
                        objItem = new ListViewItem();
                        objItem.Text = objDirInfo.Name;
                        objItem.ImageIndex = 0;
                        objItem.Group = _lvMain.Groups[GROUP_DIR];

                        // Item in Liste ergänzen
                        _lvMain.Items.Add(objItem);
                    }
                    foreach (string strFile in Directory.GetFiles(_tbDirectory.Text))
                    {
                        // Verzeichnis informationen abfragen
                        objFileInfo = new FileInfo(strFile);

                        // Item für die ListView bereitstellen
                        objItem = new ListViewItem();
                        objItem.Text = objFileInfo.Name;
                        switch (objFileInfo.Extension)
                        {
                            case ".exe":
                                objItem.ImageIndex = 1;
                                objItem.Group = _lvMain.Groups[GROUP_EXE];
                                break;
                            case ".dll":
                                objItem.ImageIndex = 2;
                                objItem.Group = _lvMain.Groups[GROUP_DLL];
                                break;
                            default:
                                objItem.ImageIndex = 3;
                                objItem.Group = _lvMain.Groups[GROUP_ALL];
                                break;
                        }
                        // Erstelle die Zusatzinformationen
                        objItem.SubItems.Add(objFileInfo.LastAccessTime.ToString());
                        objItem.SubItems.Add(string.Format("{0:N0} KB", objFileInfo.Length / 1024));

                        // Item in Liste ergänzen
                        _lvMain.Items.Add(objItem);
                    }
                }
                catch
                {
                    // Alle Fehler schlucken
                }
                finally
                {
                    GroupEnable_CheckedChanged(sender, e);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die handhabung der Einbelndung der Gruppen.
        /// </summary>
        private void GroupEnable_CheckedChanged(object sender, EventArgs e)
        {
            if (_chkGroupEnable.Checked)
            {
                _lvMain.ShowGroups = true;
            }
            else
            {
                _lvMain.ShowGroups = false;
            }
        }
    }
}
