﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Funktionalität des Dialogfeldes für die Demos des PropertyGrid-Steuerelements.
    /// </summary>
    public partial class CpnlPropertyGrid : UserControl
    {
        // Objekt für die automatische (statische) Nutzung des PropertyGrid-Steuerelements.
        private CwlbPropertyGridTestData _objStaticData = new CwlbPropertyGridTestData();
        // Obejkte für die selber gesteuerte Nutzung des PropertyGrid-Steuerelements
        private CwlbPropertyGridItemCollection _objDynamicData = new CwlbPropertyGridItemCollection();

        /// <summary>
        /// Initialisiert eine Instanz der Klasse CpnlPropertyGrid.
        /// </summary>
        public CpnlPropertyGrid()
        {
            InitializeComponent();

            _cbSelectData.SelectedIndex = 1;
            _pgrData.SelectedObject = _objStaticData;
            InitializeDynamicData();
        }

        /// <summary>
        /// Diese Methode fügt ein Liste von Eigenschaften zu einer Liste
        /// zusammen, die das Property-Grid Steuerelement interpretieren kann.
        /// </summary>
        private void InitializeDynamicData()
        {
            _objDynamicData.Add("Property01", new CwlbPropertyGridItem
            {
                DisplayName = "String Readonly",
                PropertyTypeName = "System.String",
                Description = "Dieser String kann nicht editiert werden.",
                Category = "Gruppe 1",
                Value = "Test 1",
                ReadOnly = true
            });

            _objDynamicData.Add("Property02", new CwlbPropertyGridItem
            {
                DisplayName = "String Edit",
                PropertyTypeName = "System.String",
                Description = "Dieser String kann editiert werden.",
                Category = "Gruppe 1",
                Value = "Test 2",
                ReadOnly = false
            });

            _objDynamicData.Add("Property03", new CwlbPropertyGridItem
            {
                DisplayName = "Datum",
                PropertyTypeName = "System.DateTime",
                Description = "Dieses Datum kann editiert werden",
                Category = "Gruppe 1",
                Value = DateTime.Now,
                ReadOnly = false
            });

            _objDynamicData.Add("Property04", new CwlbPropertyGridItem
            {
                DisplayName = "Wahr oder Falsch",
                PropertyTypeName = "System.Boolean",
                Description = "Doppelklick auf dem Label toggelt den Wert.",
                Category = "Gruppe 2",
                Value = true,
                ReadOnly = false
            });

            _objDynamicData.Add("Property05", new CwlbPropertyGridItem
            {
                DisplayName = "Kurzer Int",
                PropertyTypeName = "System.Int16",
                Description = "Das ist ein System.Int16.",
                Category = "Gruppe 2",
                Value = 42,
                ReadOnly = false
            });

            _objDynamicData.Add("Property06", new CwlbPropertyGridItem
            {
                DisplayName = "Normaler Int",
                PropertyTypeName = "System.Int32",
                Description = "Das ist ein System.Int32.",
                Category = "Gruppe 2",
                Value = 456,
                DefaultValue = 0,
                ReadOnly = false
            });

            _objDynamicData.Add("Property07", new CwlbPropertyGridItem
            {
                DisplayName = "Langer Int",
                PropertyTypeName = "System.Int64",
                Description = "Das ist ein System.Int64.",
                Category = "Gruppe 2",
                Value = 123456789,
                ReadOnly = false
            });

            _objDynamicData.Add("Property08", new CwlbPropertyGridItem
            {
                DisplayName = "Float",
                PropertyTypeName = "System.Single",
                Description = "Das ist ein System.Single.",
                Category = "Gruppe 3",
                Value = 34.45f,
                ReadOnly = false
            });

            _objDynamicData.Add("Property09", new CwlbPropertyGridItem
            {
                DisplayName = "Double",
                PropertyTypeName = "System.Double",
                Description = "Das ist ein System.Double.",
                Category = "Gruppe 3",
                Value = 1234.56789,
                ReadOnly = false
            });

            _objDynamicData.Add("Property10", new CwlbPropertyGridItem
            {
                DisplayName = "Decimal",
                PropertyTypeName = "System.Decimal",
                Description = "Das ist ein System.Decimal.",
                Category = "Gruppe 3",
                Value = 123456789.987654321m,
                ReadOnly = false
            });

            _objDynamicData.Add("Property11", new CwlbPropertyGridItem
            {
                DisplayName = "Farbe",
                PropertyTypeName = "System.Drawing.Color",
                Description = "Das ist ein System.Drawing.Color.",
                Category = "Gruppe 4",
                Value = Color.Azure,
                DefaultValue = Color.Red,
                ReadOnly = false,
                ConverterTypeName = "System.Drawing.ColorConverter, System.Drawing, Version=4.0.0.0," +  
                                     "Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a",
                EditorTypeName = "System.Drawing.Design.ColorEditor, System.Drawing.Design, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
            });

            _objDynamicData.Add("Property12", new CwlbPropertyGridItem
            {
                DisplayName = "String Array",
                PropertyTypeName = "System.String[]",
                Description = "Das ist ein String Array.",
                Category = "Gruppe 4",
                Value = new string[] { "Das ist ein Text", "2. Zeile" },
                ReadOnly = false,
                ConverterTypeName = "System.ComponentModel.StringConverter, System, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089",
            });

            _objDynamicData.Add("Property13", new CwlbPropertyGridItem
            {
                DisplayName = "The Font",
                PropertyTypeName = "System.Drawing.Font",
                Description = "Definition eines Font.",
                Category = "Gruppe 4",
                Value = new Font("Arial", 8.25f),
                ReadOnly = false,
                DefaultValue = new Font("Arial", 8.25f),
                ResetValue = new Font("Arial", 8.25f),
                ConverterTypeName = "System.Drawing.FontConverter, System.Drawing, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a",
                EditorTypeName = "System.Drawing.Design.FontEditor, System.Drawing.Design, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
            });

            _objDynamicData.Add("Property14", new CwlbPropertyGridItem
            {
                DisplayName = "The Cursor",
                PropertyTypeName = "System.Windows.Forms.Cursor, System.Windows.Forms, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089",
                Description = "Definition eines Cursors.",
                Category = "Gruppe 4",
                Value = Cursors.Cross,
                ReadOnly = false,
                ConverterTypeName = "System.Windows.Forms.CursorConverter, System.Windows.Forms, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089",
                EditorTypeName = "System.Drawing.Design.CursorEditor, System.Drawing.Design, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"
            });

            _objDynamicData.Add("Property15", new CwlbPropertyGridItem
            {
                DisplayName = "Eigene Enumeration",
                PropertyTypeName = "WeroSoft.Samples.EwlbPropertyGridDemo",
                Description = "Verwendung einer eigenen Enumeration.",
                Category = "Gruppe 4",
                Value = EwlbPropertyGridDemo.Colorful,
                ReadOnly = false,
            });
        }

        /// <summary>
        /// Ereignisbehandlung für die Auswahl der Demo
        /// </summary>
        private void SelectData_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Sicherheitsüberprüfung
            if (_cbSelectData.SelectedItem == null)
            {
                return;
            }
            else if (_cbSelectData.SelectedItem.ToString().ToLower().Contains("haupt"))
            {
                // Das Hauptfensterobjekt anzeigen
                _pgrData.SelectedObject = CwlbMainWindow.MainForm;
            }
            else if (_cbSelectData.SelectedItem.ToString().ToLower().Contains("statisch"))
            {
                // Die statischen Daten anzeigen
                _pgrData.SelectedObject = _objStaticData;
            }
            else if (_cbSelectData.SelectedItem.ToString().ToLower().Contains("dynamisch"))
            {
                // Die dynamischen Daten anzeigen
                _pgrData.SelectedObject = _objDynamicData;
            }
            else if (_cbSelectData.SelectedItem.ToString().ToLower().Contains("keine"))
            {
                // Keine Daten anzeigen
                _pgrData.SelectedObject = null;
            }
        }
    }
}
