﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Funktionalität des Dialogfeldes für die Demos des RichTextBox-Steuerelements.
    /// </summary>
    public partial class CpnlRichTextBox : UserControl
    {
        /// <summary>
        /// Initialisiert eine Instanz der Klasse CpnlRichTextBox.
        /// </summary>
        public CpnlRichTextBox()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Erzeugt eine Stildefinition aus einer bestehenden Schriftart.
        /// </summary>
        /// <param name="objFont">Schriftart aus der der Stil übernommen werden soll.</param>
        /// <returns>Definierte Schriftart.</returns>
        private FontStyle MakeFontStyle(Font objFont)
        {
            // Lokale Daten
            FontStyle enmStyle = FontStyle.Regular;

            // Prüfen, ob eine Schrift als Parameter übergeben wurde
            if (objFont == null)
                return FontStyle.Regular;

            // Schriftattribute definieren
            if (objFont.Bold)
                enmStyle = FontStyle.Bold;
            if (objFont.Underline)
                enmStyle |= FontStyle.Underline;
            if (objFont.Strikeout)
                enmStyle |= FontStyle.Strikeout;
            if (objFont.Italic)
                enmStyle |= FontStyle.Italic;
            return enmStyle;
        }

        /// <summary>
        /// Wendet eine definierte Schriftart auf den aktuell ausgewählten Text an
        /// </summary>
        /// <param name="enmStyle">Schriftstil, der angewendet werden soll.</param>
        private void SetFont(FontStyle enmStyle)
        {
            if (_rtfEdit.SelectedText.Length == 0)
            {
                return;
            }
            Font fontSelecttion = new Font(_rtfEdit.SelectionFont, enmStyle);
            _rtfEdit.SelectionFont = fontSelecttion;
        }

        /// <summary>
        /// Ereignisbehandlung Kontextmenü Fettschrift.
        /// </summary>
        private void ContextMenuBold_Click(object sender, EventArgs e)
        {
            FontStyle enmStyle = MakeFontStyle(_rtfEdit.SelectionFont) ^ FontStyle.Bold;
            SetFont(enmStyle);
        }

        /// <summary>
        /// Ereignisbehandlung Kontextmenü unterstrichene Schrift.
        /// </summary>
        private void ContextMenuUnderscore_Click(object sender, EventArgs e)
        {
            FontStyle enmStyle = MakeFontStyle(_rtfEdit.SelectionFont) ^ FontStyle.Underline;
            SetFont(enmStyle);
        }

        /// <summary>
        /// Ereignisbehandlung Kontextmenü kurive Schrift.
        /// </summary>
        private void ContextFormatItalic_Click(object sender, EventArgs e)
        {
            FontStyle enmStyle = MakeFontStyle(_rtfEdit.SelectionFont) ^ FontStyle.Italic;
            SetFont(enmStyle);
        }

        /// <summary>
        /// Ereignisbehandlung Kontextmenü durchgestrichene Schrift.
        /// </summary>
        private void ContextFormatStrikeout_Click(object sender, EventArgs e)
        {
            FontStyle enmStyle = MakeFontStyle(_rtfEdit.SelectionFont) ^ FontStyle.Strikeout;
            SetFont(enmStyle);
        }

        /// <summary>
        /// Ereignisbehandlung Kontextmenü Aufzählung.
        /// </summary>
        private void ContextMenuList_Click(object sender, EventArgs e)
        {
            _rtfEdit.SelectionBullet = true;
            _rtfEdit.SelectionHangingIndent = 10;
            _rtfEdit.SelectionIndent = 10;
        }

        /// <summary>
        /// Ereignisbehandlung Link wurde geklickt
        /// </summary>
        private void RichTextBoxEdit_LinkClicked(object sender, LinkClickedEventArgs e)
        {
            // Starte einen Prozess
            try
            {
                System.Diagnostics.Process.Start(e.LinkText);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, CwlbMainWindow.TITLE, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Betätigen der Schaltfläche Laden.
        /// </summary>
        private void ButtonLoad_Click(object sender, EventArgs e)
        {
            // Dialogfeld instanziieren
            OpenFileDialog dlgOpen = new OpenFileDialog();
            // Dialogfeld initialisieren
            dlgOpen.Filter = "Rich Text Format|*.rtf|Alle Dateien|*.*";
            dlgOpen.FilterIndex = 0;
            // Dialogfeld anzeigen und Resultat abfragen
            if (dlgOpen.ShowDialog() == DialogResult.OK)
            {
                // Das RichTextBox-Steuerelement unterstützt das Laden
                _rtfEdit.LoadFile(dlgOpen.FileName);
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Betätigen der Schaltfläche Speichern.
        /// </summary>
        private void ButtonSave_Click(object sender, EventArgs e)
        {
            // Dialogfeld instanziieren
            SaveFileDialog dlgSave = new SaveFileDialog();
            // Dialogfeld initialisieren
            dlgSave.Filter = "Rich Text Format|*.rtf|Alle Dateien|*.*";
            dlgSave.FilterIndex = 0;
            // Dialogfeld anzeigen und Resultat abfragen
            if (dlgSave.ShowDialog() == DialogResult.OK)
            {
                // Das RichTextBox-Steuerelement unterstützt das Speichern
                _rtfEdit.SaveFile(dlgSave.FileName);
            }
        }

        /// <summary>
        /// Ereignisbehandlung für den Kontextmenübefehl 'Auswahl der Schriftart'.
        /// </summary>
        private void ContextMenuFont_Click(object sender, EventArgs e)
        {
            // Dialogfeld für die Auswahl einer Schriftart
            FontDialog dlgFont = new FontDialog();

            // Initialisieren
            dlgFont.AllowScriptChange = true;
            dlgFont.AllowSimulations = true;
            dlgFont.AllowVectorFonts = true;
            dlgFont.AllowVerticalFonts = true;
            dlgFont.ShowEffects = true;
            dlgFont.ShowColor = true;
            dlgFont.Font = _rtfEdit.SelectionFont;
            dlgFont.ScriptsOnly = false;
            dlgFont.ShowApply = true;

            // Dialogfeld anzeigen und Rückgabe auswerten
            if (dlgFont.ShowDialog() == DialogResult.OK)
            {
                // Auf Auswahl anwenden
                _rtfEdit.SelectionFont = dlgFont.Font;
            }
        }

        /// <summary>
        /// Ereignisbehandlung Kontextmenü Farbe
        /// </summary>
        private void ContextMenuColor_Click(object sender, EventArgs e)
        {
            // Dialogfeld für die Auswahl einer Farbe erstellen
            ColorDialog dlgColor = new ColorDialog();

            // Dialogfeld initialisieren
            dlgColor.AllowFullOpen = true;
            dlgColor.AnyColor = true;
            dlgColor.CustomColors = new int[] { 6579300, 13158600 };
            dlgColor.Color = _rtfEdit.SelectionColor;

            // Dialogfeld anzeigen und Rückgabe auswerten
            if (dlgColor.ShowDialog() == DialogResult.OK)
            {
                // Farbe übernehmen
                _rtfEdit.SelectionColor = dlgColor.Color;
            }
        }
    }
}
