﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Funktionalität des Dialogfeldes für die Demos des TreeView-Steuerelements.
    /// </summary>
    public partial class CpnlTreeView : UserControl
    {
        /// <summary>
        /// Initialisiert eine Instanz der Klasse CpnlTreeView.
        /// </summary>
        public CpnlTreeView()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Fügt den Inhalt eines Verzeichnisses in den Baum.
        /// </summary>
        void AddDirectory(TreeNodeCollection cobjNodes, string strPath)
        {
            // Lokale Daten
            TreeNode objNode;
            DirectoryInfo objDirectory;
            FileInfo objFile;

            // Fehlerhandhabung
            try
            {
                // Alle Verzeichnisse ergänzen
                foreach (string strDirectory in Directory.GetDirectories(strPath))
                {
                    // Verzeichnisinformationen erzeugen
                    objDirectory = new DirectoryInfo(strDirectory);

                    // Versteckte Verzeichnisse überspringen
                    if (objDirectory.Attributes.HasFlag(FileAttributes.Hidden) ||
                        objDirectory.Attributes.HasFlag(FileAttributes.System))
                    {
                        continue;
                    }

                    // Anzeigeobjekt herstellen
                    objNode = new TreeNode();
                    objNode.Text = objDirectory.Name;
                    objNode.Tag = objDirectory.FullName;
                    objNode.ImageIndex = objNode.SelectedImageIndex = 7;
                    cobjNodes.Add(objNode);
                }

                // Alle Dateien anzeigen
                foreach (string strFileName in Directory.GetFiles(strPath))
                {
                    // Dateiinformationen lesen
                    objFile = new FileInfo(strFileName);

                    // Versteckte Dateien überspringen
                    if (objFile.Attributes.HasFlag(FileAttributes.Hidden) ||
                        objFile.Attributes.HasFlag(FileAttributes.System))
                    {
                        continue;
                    }

                    // Anzeigeobjekt erzeugen
                    objNode = new TreeNode();
                    objNode.Text = objFile.Name;
                    objNode.Tag = objFile.FullName;
                    // Abbildung ergänzen
                    switch (objFile.Extension.ToLower())
                    {
                        case ".exe":
                            objNode.ImageIndex = objNode.SelectedImageIndex = 8;
                            break;
                        case ".dll":
                            objNode.ImageIndex = objNode.SelectedImageIndex = 9;
                            break;
                        default:
                            objNode.ImageIndex = objNode.SelectedImageIndex = 10;
                            break;
                    }

                    // Tooltop erzeugen
                    objNode.ToolTipText = string.Format(
                        "{0}{4}Größe: {1:N0} {2}{4}Letzter Zugriff: {3}",
                        objFile.FullName,
                        objFile.Length > 10240 ? objFile.Length / 1024 : objFile.Length,
                        objFile.Length > 10240 ? "KB" : "Bytes",
                        objFile.LastAccessTime,
                        Environment.NewLine);
                    cobjNodes.Add(objNode);
                }
            }
            catch
            {
                // Alle Fehler schlucken
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Laden des Formulars.
        /// </summary>
        private void CpnlTreeView_Load(object sender, EventArgs e)
        {
            // Lokale Daten
            TreeNode objNode;

            // Alle Laufwerke auflisten und als sichtbare Objekte darstellen
            foreach (DriveInfo objDrive in DriveInfo.GetDrives())
            {
                objNode = new TreeNode();
                objNode.Text = objDrive.Name;
                // Abbildung ergänzen
                switch (objDrive.DriveType)
                {
                    case DriveType.CDRom:
                        objNode.ImageIndex = objNode.SelectedImageIndex = 2;
                        break;
                    case DriveType.Fixed:
                        objNode.ImageIndex = objNode.SelectedImageIndex = 3;
                        break;
                    case DriveType.Network:
                        objNode.ImageIndex = objNode.SelectedImageIndex = 4;
                        break;
                    case DriveType.Removable:
                        objNode.ImageIndex = objNode.SelectedImageIndex = 5;
                        break;
                    case DriveType.NoRootDirectory:
                    case DriveType.Ram:
                    case DriveType.Unknown:
                        objNode.ImageIndex = objNode.SelectedImageIndex = 6;
                        break;
                }

                // Knoten in Baum einfügen und prüfen, ob das Laufwerk gelesen werden kann
                _tvMain.Nodes.Add(objNode);
                if (objDrive.IsReady)
                {
                    // Laufwerk kann gelesen werden. Erste Ebene der Daten einlesen
                    AddDirectory(objNode.Nodes, objDrive.RootDirectory.FullName);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Erweitern eines Knotens
        /// </summary>
        private void TreeViewMain_BeforeExpand(object sender, TreeViewCancelEventArgs e)
        {
            // Für jeden bereits enthaltenen Knoten die nächste Ebene lesen.
            // Diese Massnahme ist notwendig, damit das TreeView-Steuerelement
            // für die Knoten die richtige Darstellung wählt (erweiterbar oder nicht).
            foreach (TreeNode objNode in e.Node.Nodes)
            {
                AddDirectory(objNode.Nodes, objNode.Tag.ToString());
            }
        }
    }
}
