﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Funktionalität des Dialogfeldes für die Demos des WebBrowser-Steuerelements.
    /// </summary>
    public partial class CpnlWebBrowser : UserControl
    {
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CpnlWebBrowser.
        /// </summary>
        public CpnlWebBrowser()
        {
            InitializeComponent();

            // Load Event simulieren
            CpnlWebBrowser_Load(this, new EventArgs());
        }

        /// <summary>
        /// Ereignisbehandlung für das Laden des Elements.
        /// </summary>
        private void CpnlWebBrowser_Load(object sender, EventArgs e)
        {
            // Registrieren zusätzlicher Eventhandler
            _webView.CanGoBackChanged += new EventHandler(Browser_CanGoBackChanged);
            _webView.CanGoForwardChanged += new EventHandler(Browser_CanGoForwardChanged);
            _webView.EncryptionLevelChanged += new EventHandler(Browser_EncryptionLevelChanged);
            _webView.StatusTextChanged += new EventHandler(Browser_StatusTextChanged);
            _webView.DocumentTitleChanged += new EventHandler(Browser_DocumentTitleChanged);

            // Initialisierung der Schaltflächen
            _btnGoForward.Enabled = _webView.CanGoForward ? true : false;
            _btnGoBack.Enabled = _webView.CanGoBack ? true : false;

        }

        /// <summary>
        /// Ereignisbehandlung beim Verlassen der Adresszeile.
        /// </summary>
        private void AddressLine_Leave(object sender, EventArgs e)
        {
            _webView.Navigate(_tbAddress.Text);
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Eingabetaste in der Adresszeile.
        /// </summary>
        private void AddressLine_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Return)
            {
                _webView.Navigate(_tbAddress.Text);
            }
        }

        /// <summary>
        /// Ereignisbehandlung wenn der Titel änderte
        /// </summary>
        private void Browser_DocumentTitleChanged(object sender, EventArgs e)
        {
            CwlbMainWindow.MainForm.Text = _webView.DocumentTitle;
        }

        /// <summary>
        /// Ereignisbehandlung bei Veränderung der Vorwärtsnavigation.
        /// </summary>
        private void Browser_CanGoForwardChanged(object sender, EventArgs e)
        {
            _btnGoForward.Enabled = _webView.CanGoForward ? true : false;
        }

        /// <summary>
        /// Ereignisbehandlung bei Veränderung der Rückwärtsnavigation.
        /// </summary>
        private void Browser_CanGoBackChanged(object sender, EventArgs e)
        {
            _btnGoBack.Enabled = _webView.CanGoBack ? true : false;
        }

        /// <summary>
        /// Ereignisbehandlung bei Veränderung der Verschlüsselung.
        /// </summary>
        private void Browser_EncryptionLevelChanged(object sender, EventArgs e)
        {
            if (_webView.EncryptionLevel == WebBrowserEncryptionLevel.Insecure)
            {
                _statusImage.Image = null;
            }
            else
            {
                _statusImage.Image = Properties.Resources.Locked;
            }
        }

        /// <summary>
        /// Ereignisbehandlung bei Veränderung des Statustexts.
        /// </summary>
        private void Browser_StatusTextChanged(object sender, EventArgs e)
        {
            _statusText.Text = _webView.StatusText;
        }

        /// <summary>
        /// Ereignisbehandlung nachdem das Dokument fertig geladen ist.
        /// </summary>
        private void BrowserView_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
        {
            _statusProgress.Visible = false;
        }

        /// <summary>
        /// Ereignisbehandlung bei Veränderung des Fortschritts während dem Ladevorgang.
        /// </summary>
        private void Browser_ProgressChanged(object sender, WebBrowserProgressChangedEventArgs e)
        {
            _statusProgress.Maximum = (int)e.MaximumProgress;
            if (e.CurrentProgress < 0)
            {
                _statusProgress.Value = 0;
            }
            else
            {
                _statusProgress.Value = (int)e.CurrentProgress;
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Starten eines Navigationsvorganges.
        /// </summary>
        private void BrowserView_Navigating(object sender, WebBrowserNavigatingEventArgs e)
        {
            _statusProgress.Visible = true;
        }

        /// <summary>
        /// Ereignisbehandlung bei Betätigung der Rückwärtsnavigation.
        /// </summary>
        private void GoBack_Click(object sender, EventArgs e)
        {
            _webView.GoBack();
        }

        /// <summary>
        /// Ereignisbehandlung bei Betätigung der Vorwärtsnavigation.
        /// </summary>
        private void GoForward_Click(object sender, EventArgs e)
        {
            _webView.GoForward();
        }

        /// <summary>
        /// Ereignisbehandlung bei Abbrechen der laufenden Navigation.
        /// </summary>
        private void Stop_Click(object sender, EventArgs e)
        {
            _webView.Stop();
            _statusProgress.Value = 0;
        }

        /// <summary>
        /// Ereignisbehandlung beim Click auf die Basisfläche des Controls, zum Defineren des im Eigenschaftseditor angezeigten Steuerelements.
        /// </summary>
        private void CpnlWebBrowser_Click(object sender, EventArgs e)
        {
            CwlbMainWindow.MainForm.SetPropertyControl(_webView);
        }
    }
}
