﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Funktionalität des Dialogfeldes für die Demos des ElementHost-Steuerelements.
    /// </summary>
    public partial class CpnlElementHost : UserControl
    {
        // Konstante für die Drag&Drop Operation
        private const string FILE_DROP = "FileDrop";
        // Hilfsvariable für die Kontrolle der Trackerpositionierung
        private bool _bMouseInTrackerDown;
        // Hilfsvariable für die Steuerung der Stopfunktion
        private bool _bIgnoreStop = false;
        // WPF MediaElement-Steuerelement
        private System.Windows.Controls.MediaElement _mediaPlayer = new System.Windows.Controls.MediaElement();

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CpnlElementHost.
        /// </summary>
        public CpnlElementHost()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung beim Laden des Formulars.
        /// </summary>
        private void CpnlElementHost_Load(object sender, EventArgs e)
        {
            // MediaElement-Steuerelement konfigurieren
            _mediaPlayer.LoadedBehavior = System.Windows.Controls.MediaState.Manual;
            _mediaPlayer.UnloadedBehavior = System.Windows.Controls.MediaState.Stop;
            _mediaPlayer.Stretch = System.Windows.Media.Stretch.Fill;
            _mediaPlayer.MediaOpened += new System.Windows.RoutedEventHandler(MediaPlayer_MediaOpened);
            _mediaPlayer.MediaEnded += new System.Windows.RoutedEventHandler(MediaPlayer_MediaEnded);
            _mediaPlayer.MediaFailed += new EventHandler<System.Windows.ExceptionRoutedEventArgs>(MediaElement_MediaFailed);
            _mediaPlayer.AllowDrop = true;
            _mediaPlayer.DragOver += new System.Windows.DragEventHandler(MediaPlayer_DragOver);
            _mediaPlayer.Drop += new System.Windows.DragEventHandler(MediaPlayer_Drop);
            // MediaElement-Steuerelement mit dem WPF Host verknüpfen.
            _hostWpf.Child = _mediaPlayer;
            // Weitere Steuerelemente konfigurieren konfigurieren
            ResetControls();
        }

        /// <summary>
        /// Schaltet alle weiteren Bedienelemente aus.
        /// </summary>
        private void ResetControls()
        {
            _btnStart.Enabled = false;
            _btnPause.Enabled = false;
            _btnStop.Enabled = false;
            _trkTimeline.Enabled = false;
            _lblDuration.Text = string.Empty;
            _lblCurrent.Text = string.Empty;
            _trkTimeline.Value = 0;
            _lblMusic.Visible = false;
            _hostWpf.Visible = false;
        }

        /// <summary>
        /// Ereignisbehandlung beim Eintreten eines Fehlers auf dem MediaElement-Steuerelement
        /// </summary>
        void MediaElement_MediaFailed(object sender, System.Windows.ExceptionRoutedEventArgs e)
        {
            MessageBox.Show(string.Format(
                "{0}{1}{2}",
                "Fehler bei der Verarbeitung der Datei. Ev. ist das Medienformat nicht unterstützt.",
                Environment.NewLine, 
                e.ErrorException.Message));
            _timerMedia.Stop();
            ResetControls();
        }

        /// <summary>
        /// Ereignisbehandlung nach dem Öffnen des Mediums.
        /// </summary>
        void MediaPlayer_MediaOpened(object sender, System.Windows.RoutedEventArgs e)
        {
            // Prüfen, ob das Medium ein Zeit Information hat
            if (_mediaPlayer.NaturalDuration.HasTimeSpan)
            {
                // Zeitachsenanzeige konfigurieren
                _trkTimeline.Maximum = (int)_mediaPlayer.NaturalDuration.TimeSpan.TotalMilliseconds;
                // Dauer anzeigen
                TimeSpan tsDurataion = _mediaPlayer.NaturalDuration.TimeSpan;
                _lblDuration.Text = string.Format("{0:00}:{1:00}:{2:00}", tsDurataion.Hours, tsDurataion.Minutes, tsDurataion.Seconds);
                // Anzeigen des Ersatztexts oder des Medienelements
                if (_mediaPlayer.HasVideo)
                {
                    _hostWpf.Visible = true;
                }
                else
                {
                    _lblMusic.Visible = true;
                }

                _bIgnoreStop = false;
            }
            else
            {
                _bIgnoreStop = true;
                _timerMedia.Stop();
                _hostWpf.Visible = true;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die Beendigung der Wiedergabe des Mediums.
        /// </summary>
        void MediaPlayer_MediaEnded(object sender, System.Windows.RoutedEventArgs e)
        {
            if (!_bIgnoreStop)
            {
                _timerMedia.Stop();
                _mediaPlayer.Stop();
                ResetControls();
            }
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche Start.
        /// </summary>
        private void Start_Click(object sender, EventArgs e)
        {
            _mediaPlayer.Play();
            _btnStart.Enabled = false;
            _btnPause.Enabled = true;
            _btnStop.Enabled = true;
            _trkTimeline.Enabled = true;
            _timerMedia.Start();
            if (_mediaPlayer.HasVideo)
            {
                _hostWpf.Visible = true;
            }
            else if (_mediaPlayer.HasAudio)
            {
                _lblMusic.Visible = true;
            }
        }


        /// <summary>
        /// Ereignisbehandlung Schaltfläche Pause.
        /// </summary>
        private void Pause_Click(object sender, EventArgs e)
        {
            _mediaPlayer.Pause();
            _btnStart.Enabled = true;
            _btnPause.Enabled = false;
            _btnStop.Enabled = false;
            _trkTimeline.Enabled = false;
            _lblMusic.Visible = false;
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche Stop.
        /// </summary>
        private void Stop_Click(object sender, EventArgs e)
        {
            _mediaPlayer.Stop();
            _btnStart.Enabled = true;
            _btnPause.Enabled = false;
            _btnStop.Enabled = false;
            _timerMedia.Stop();
            _hostWpf.Visible = false;
            _lblMusic.Visible = false;
        }

        /// <summary>
        /// Ereignisbehandlung Intervall des Timers erreicht.
        /// </summary>
        private void TimerMedia_Tick(object sender, EventArgs e)
        {
            // Die automatische Nachführung wird nur gemacht, 
            // wenn nicht mit dern Maus am Tracker geschoben wird.
            if (!_bMouseInTrackerDown)
            {
            // Prüfen, ob das Medium ein Zeit Information hat
                if (_mediaPlayer.NaturalDuration.HasTimeSpan)
                {
                    TimeSpan tsPosition = _mediaPlayer.Position;
                    _trkTimeline.Value = (int)tsPosition.TotalMilliseconds;
                    _lblCurrent.Text = string.Format("{0:00}:{1:00}:{2:00}", tsPosition.Hours, tsPosition.Minutes, tsPosition.Seconds);
                }
            }
        }

        /// <summary>
        /// Ereignisbhandlung beim Klicken mit der Maus auf den Tracker.
        /// </summary>
        private void Timeline_MouseDown(object sender, MouseEventArgs e)
        {
            _bMouseInTrackerDown = true;
            _mediaPlayer.Pause();
        }

        /// <summary>
        /// Ereignisbhandlung beim Beenden des Klickens mit der Maus auf den Tracker.
        /// </summary>
        private void Timeline_MouseUp(object sender, MouseEventArgs e)
        {
            _bMouseInTrackerDown = false;
            _mediaPlayer.Play();
        }

        /// <summary>
        /// Ereignisbehandlung beim Bewegen des Trackers.
        /// </summary>
        private void Timeline_Scroll(object sender, EventArgs e)
        {
            if (_bMouseInTrackerDown)
            {
            // Prüfen, ob das Medium ein Zeit Information hat
                if (_mediaPlayer.NaturalDuration.HasTimeSpan)
                {
                    TimeSpan tsPosition = TimeSpan.FromMilliseconds(_trkTimeline.Value);
                    _mediaPlayer.Position = tsPosition;
                    _lblCurrent.Text = string.Format("{0:00}:{1:00}:{2:00}", tsPosition.Hours, tsPosition.Minutes, tsPosition.Seconds);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die Aktivierung des MediaElements-Steuerelements im Eigenschaftseditor.
        /// </summary>
        private void CpnlElementHost_Click(object sender, EventArgs e)
        {
            CwlbMainWindow.MainForm.SetPropertyControl(_mediaPlayer);
        }

        /// <summary>
        /// Ereignisbehandlung für Drag Over für Forms.
        /// </summary>
        private void Panel_DragOver(object sender, DragEventArgs e)
        {
            string[] astrFileData = (string[])e.Data.GetData("FileDrop");
            if (astrFileData.Length == 1)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für Drag Drop für Forms.
        /// </summary>
        private void Panel_DragDrop(object sender, DragEventArgs e)
        {
            // Dateinamen aus den DD-Informationen übernehmen
            string[] astrFileData = (string[])e.Data.GetData("FileDrop");
            if (astrFileData.Length == 1)
            {
                _mediaPlayer.Source = new Uri(astrFileData[0]);
            }
            _btnStart.Enabled = true;
        }

        /// <summary>
        /// Ereignisbehandlung für Drag Over für WPF.
        /// </summary>
        void MediaPlayer_DragOver(object sender, System.Windows.DragEventArgs e)
        {
            string[] astrFileData = (string[])e.Data.GetData("FileDrop");
            if (astrFileData.Length == 1)
            {
                e.Effects = System.Windows.DragDropEffects.Copy;
            }
            else
            {
                e.Effects = System.Windows.DragDropEffects.None;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für Drag Drop der WPF.
        /// </summary>
        void MediaPlayer_Drop(object sender, System.Windows.DragEventArgs e)
        {
            // Dateinamen aus den DD-Informationen übernehmen
            string[] astrFileData = (string[])e.Data.GetData("FileDrop");
            if (astrFileData.Length == 1)
            {
                _mediaPlayer.Source = new Uri(astrFileData[0]);
            }
            _btnStart.Enabled = true;
        }

        /// <summary>
        /// Ereignisbehandlung für Klick auf die Panel-Fläche.
        /// </summary>
        private void _panelDemo_Click(object sender, EventArgs e)
        {
            CwlbMainWindow.MainForm.SetPropertyControl(_hostWpf);
        }
    }
}
