﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Repräsentiert eine Vorlage für eine Zelle in einem DataGridView, das einen DatTimePciker verwendet. 
    /// </summary>
    public class CwlbDataGridViewCalendarCell : DataGridViewTextBoxCell
    {
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbDataGridViewCalendarCell.
        /// </summary>
        public CwlbDataGridViewCalendarCell()
            : base()
        {
            this.Style.Format = "d";
        }

        /// <summary>
        /// Verbindet und initialisiert das Steuerelement für die zu editierende Zelle.
        /// </summary>
        /// <param name="iRowIndex"></param>
        /// <param name="initialFormattedValue"></param>
        /// <param name="dataGridViewCellStyle"></param>
        public override void InitializeEditingControl(int iRowIndex, object objInitialFormattedValue, DataGridViewCellStyle dataGridViewCellStyle)
        {
            // Das Steuerelement mit der Zelle verbinden.
            base.InitializeEditingControl(iRowIndex, objInitialFormattedValue, dataGridViewCellStyle);
            // Erzeugt das zu verwendende Steuerelement
            CwlbDataGridDateTimePicker objControl = DataGridView.EditingControl as CwlbDataGridDateTimePicker;
            // Benütze den Standardwert der Spalte, wenn die Zelle null ist
            if (this.Value == null)
            {
                objControl.Value = (DateTime)this.DefaultNewRowValue;
            }
            else
            {
                objControl.Value = (DateTime)this.Value;
            }
        }

        /// <summary>
        /// Liefert den Typ des Steuerelements das von dieser Vorlage verwendet wird.
        /// </summary>
        public override Type EditType
        {
            get
            {
                return typeof(CwlbDataGridDateTimePicker);
            }
        }

        /// <summary>
        /// Liefert den Typ des Resultats, das das Steuerelement herstellt.
        /// </summary>
        public override Type ValueType
        {
            get
            {
                return typeof(DateTime);
            }
        }

        /// <summary>
        /// Liefert den Standardwert, der im Fall einer neuen Zeile verwendet wird.
        /// </summary>
        public override object DefaultNewRowValue
        {
            get
            {
                return DateTime.Now;
            }
        }
    }
}
