﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Windows.Forms.Design;
using System.Drawing;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using System.Runtime.Serialization;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Security.Permissions;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert eine zusätzliche Seite für das angezeigte Objekt.
    /// </summary>
    [PermissionSet(SecurityAction.Demand, Name = "FullTrust")]
    public class CwlbFieldGridTab : PropertyTab
    {
        /// <summary>
        /// Liefert eine Liste der Beschreibungen der darzustellenden Elemente der Seite.
        /// </summary>
        /// <param name="objData">Datenobjekt für angezeigt werden soll.</param>
        /// <param name="aobjAttributes">Liste der Attribute, die erkannt wurde.</param>
        /// <returns>Liste der Beschreibungen der Items.</returns>
        public override PropertyDescriptorCollection GetProperties(object objData, Attribute[] aobjAttributes)
        {
            // Lokale Daten
            List<CwlbFieldDescriptor> cobjData = new List<CwlbFieldDescriptor>();
            List<Attribute> cobjAttribute = new List<Attribute>();
            object[] aobjAttribute;

            // Auflisten alle Felder des Datenobjekts
            foreach (FieldInfo objInfo in objData.GetType().GetFields(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance))
            {
                // Attributliste des Items löschen
                cobjAttribute.Clear();

                // Prüfen, ob es sich um ein generiertes oder ein programmiertes Feld handelt
                aobjAttribute = objInfo.GetCustomAttributes(typeof(CompilerGeneratedAttribute), true);
                if (aobjAttribute.Length > 0)
                {
                    // Gruppe für generierte Felder mit Beschreibung als Attribut erzeugen.
                    cobjAttribute.Add(new CategoryAttribute("Generierte Felder"));
                    cobjAttribute.Add(new DescriptionAttribute("Dieses Feld wurde vom Compiler erzeugt."));
                }
                else
                {
                    // Gruppe für programmierte Felder mit Beschreibung als Attribut erzeugen.
                    cobjAttribute.Add(new CategoryAttribute("Programmierte Felder"));
                    cobjAttribute.Add(new DescriptionAttribute("Dieses Feld wurde durch den Entwickler definiert."));
                }

                // Die Beschreibung zur Liste hinzufügen
                cobjAttribute.Add(new DisplayNameAttribute(objInfo.Name));
                cobjData.Add(new CwlbFieldDescriptor(objInfo, objData, cobjAttribute.ToArray()));
            }

            return new PropertyDescriptorCollection(cobjData.ToArray<CwlbFieldDescriptor>());
        }

        /// <summary>
        /// Liest alle Eigenschaften des Objekts.
        /// </summary>
        /// <param name="objData">Datenobjekt für das die Methode ausgeführt wird.</param>
        /// <returns>Liefert eine Liste der anzuzeigenden Eigenschaften.</returns>
        public override System.ComponentModel.PropertyDescriptorCollection GetProperties(object objData)
        {
            return this.GetProperties(objData, null);
        }

        /// <summary>
        /// Liefert den Namen der Tabseite.
        /// </summary>
        public override string TabName
        {
            get
            {
                return "Felder des Objekts anzeigen";
            }
        }

        /// <summary>
        /// Liefert die Abbildung, die für den Zugriff auf die Tabseite benutzt wird.
        /// </summary>
        public override System.Drawing.Bitmap Bitmap
        {
            get
            {
                return Properties.Resources.Properties_Private;
            }
        }
    }
}
