using System;
using System.Collections.Generic;
using System.Drawing.Design;
using System.Windows.Forms;
using System.ComponentModel;
using System.Security.Permissions;
using System.Windows.Forms.Design;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Demoklasse fr die Erzeugung eines eigenen Editors zu einem PropertyGrid-Steuerelement.
    /// </summary>
    [PermissionSet(SecurityAction.Demand, Name = "FullTrust")]
    class CwlbPropertyEditorIPAddress : UITypeEditor
    {
        /// <summary>
        /// Die Methode teilt dem PropertyGrid mit, welche Art von Dialogfeld Sie whlen. Der Typ
        /// UITypeEditorEditStyle.Modal beschreibt, dass Sie ein eigenes, vollstndiges Dialogfeld
        /// erstellen. Der Typ UITypeEditorEditStyle.DropDown hingegen definiert, dass Sie
        /// ein Steuerelement im Rahmen eines Dropdown-Dialogfelds zeichnen.
        /// </summary>
        /// <param name="context">Liefert Kontextinformationen wie Referenz auf das
        /// Objekt im PropertyGrid beziehungsweise die zu editierende Eigenschaft.</param>
        /// <returns>Liefert den Typ des Editor GUI</returns>
        public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
        {
            return UITypeEditorEditStyle.Modal;
        }

        /// <summary>
        /// Methode fr das effektive Editieren des Wertes. In dieser Methode 
        /// erstellen Sie den Dialog und bergeben diesem die Kontrolle.
        /// </summary>
        /// <param name="objContext">Liefert Kontextinformationen wie Referenz auf das
        /// Objekt im PropertyGrid beziehungsweise die zu editierende Eigenschaft.</param>
        /// <param name="objProvider">Angaben zum Aufrufer und der zu editierenden Eigenschaft</param>
        /// <param name="objValue">
        /// Wert der aktuell in der Eigenschaft eingestellt ist. 
        /// Beachten Sie dass hier je nach Typ eine null-Referenz eingebracht werden kann.
        /// </param>
        /// <returns>
        /// Wert der an das PropertyGrid als Eingabe des Benutzers zurckgegeben werden soll.
        /// Beachten Sie, dass der Typ des Objekts dem erwarteten Typ der Eigenschaft entspricht.
        /// </returns>
        public override object EditValue(ITypeDescriptorContext objContext, IServiceProvider objProvider, object objValue)
        {
            // Hole das zustndige Verwaltungsobjekt um einen Dialog darzustellen
            IWindowsFormsEditorService objWindowsFormsEditorService =
               objProvider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;

            // Prfen, ob das Verwaltungsobjekt hergestellt werden konnte.
            if (objWindowsFormsEditorService != null)
            {
                // Erzeugen und initialisieren das Dialogfelds
                CdlgPropertyGridEditorIpAddress dlgEdit = new CdlgPropertyGridEditorIpAddress();
                dlgEdit.WindowsFormsEditorService = objWindowsFormsEditorService;
                // Anzeigen des Dialogfeldes und Auswerten des Resultats
                if (objWindowsFormsEditorService.ShowDialog(dlgEdit) == DialogResult.OK)
                {
                    // Zurckschreiben der Daten
                    return dlgEdit._tbIPAddress.Text;
                }
            }
            return null;
        }
    }
}
