﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace WeroSoft.Samples {

    /// <summary>
    /// Klasse für die Beschreibung eines Elements das im PropertyGrid-Steuerelement angezeigt werden soll.
    /// </summary>
    /// <remarks>
    /// Sie sollten die Loaklisierungen vor dem Laden der Objekte durchführen.
    /// </remarks>
    public class CwlbPropertyGridItem {

        // Instanzdaten
        private bool _bFirstAssignment = true;
        private object _objValue;
        private object _objResetValue;

        /// <summary>
        /// Liefert oder definiert den Anzeigenamen des Elements.
        /// </summary>
        public string DisplayName { get; set; }

        /// <summary>
        /// Liefert oder definiert den Namen des Elements.
        /// </summary>
        public string Name { get; internal set; }

        /// <summary>
        /// Liefert oder definiert den Namen der Kategorie zu der das Element gehört.
        /// </summary>
        public string Category { get; set; }

        /// <summary>
        /// Liefert oder definiert die Beschreibung des Elements.
        /// </summary>
        public string Description { get; set; }

        /// <summary>
        /// Liefert oder definiert den Typnamen der Eigenschaft des Elements.
        /// </summary>
        public string PropertyTypeName { get; set; }

        /// <summary>
        /// Liefert oder definiert den Wert der Eigenschaft des Elements.
        /// </summary>
        public object Value {
            get {
                return _objValue;
            }
            set {
                _objValue = value;
                ResetValue = value;
            }
        }

        /// <summary>
        /// Liefert oder definiert den Originalwert der Eigenschaft.
        /// </summary>
        public object ResetValue {
            get {
                return _objResetValue;
            }
            set {

                // Sets the value only the first time
                if (_bFirstAssignment) {
                    _bFirstAssignment = false;
                    _objResetValue = value;
                }
            }
        }

        /// <summary>
        /// Liefert oder definiert den Standardwert des Elements.
        /// </summary>
        public object DefaultValue { get; set; }

        /// <summary>
        /// Liefert oder definiert einen WErt der definiert ob das Element schreibgeschützt ist.
        /// </summary>
        public bool ReadOnly { get; set; }

        /// <summary>
        /// Liefert oder definiert den Typenamen des Editors der zu verwenden ist.
        /// </summary>
        public string EditorTypeName { get; set; }

        /// <summary>
        /// Liefert oder definiert den Typnamen des zu verwendenden Konverters.
        /// </summary>
        public string ConverterTypeName { get; set; }

        /// <summary>
        /// Liefert oder definiert eine Referenz für benutzerspezifische Daten.
        /// </summary>
        public object Tag { get; set; }
    }
}
