﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections;

namespace WeroSoft.Samples
{

    /// <summary>
    /// Implementiert eine eigene Auflistung zur Beschreibung der Daten, das 
    /// PropertyGrid-Steuerelement anzeigen soll.
    /// </summary>
    public class CwlbPropertyGridItemCollection : IDictionary<string, CwlbPropertyGridItem>, ICustomTypeDescriptor
    {
        // Instanzdaten
        private Dictionary<string, CwlbPropertyGridItem> _cobData = new Dictionary<string, CwlbPropertyGridItem>();

        #region IDictionary<string,CwlbPropertyDataItem> Members
        /// <summary>
        /// Liefert eine Wert der anzeigt obe ine bestimmter Schlüssel in der Auflistung vorhanden ist.
        /// </summary>
        /// <param name="strKey">Zu prüfendender Schlüssel.</param>
        /// <returns>True wenn der Schlüssel in der Auflistung enthalten ist; anderfalls false.</returns>
        public bool ContainsKey(string strKey)
        {
            return _cobData.ContainsKey(strKey);
        }

        /// <summary>
        /// Liefert eine Auflistung der enthaltenen Schlüssel.
        /// </summary>
        public ICollection<string> Keys
        {
            get
            {
                return _cobData.Keys;
            }
        }

        /// <summary>
        /// Löscht das definierte Schlüssel-Wert-Paar aus der Auflistung.
        /// </summary>
        /// <param name="strKey">Schlüssel des zu entfernenden Schlüssel-Wert-Paares.</param>
        /// <returns>True, wenn das definierte Schlüssel-Wert-Paar erfolgreich entfernt werden konnte; anderfalls false.</returns>
        bool IDictionary<string, CwlbPropertyGridItem>.Remove(string strKey)
        {
            return _cobData.Remove(strKey);
        }

        /// <summary>
        /// Versucht einen dfinierten Wert aus der Auflistung zu lesen.
        /// </summary>
        /// <param name="strKey">Schlüssel des zu lesenden Schlüssel-Wert-Paares.</param>
        /// <param name="objValue">Wert der ausgelesen wurde.</param>
        /// <returns>True, wenn das definierte Schlüssel-Wert-Paar erfolgreich gelesen werden konnte; anderfalls false.</returns>
        public bool TryGetValue(string strKey, out CwlbPropertyGridItem objValue)
        {
            return _cobData.TryGetValue(strKey, out objValue);
        }

        /// <summary>
        /// Liefert eine Auflistung der enthaltenen Werte.
        /// </summary>
        public ICollection<CwlbPropertyGridItem> Values
        {
            get
            {
                return _cobData.Values;
            }
        }

        /// <summary>
        /// Indexer für das Auslesen eines Wertes aufgrund eines definierten Schlüssels.
        /// </summary>
        /// <param name="strKey">Schlüssel des zu lesenden Schlüssel-Wert-Paares.</param>
        /// <returns>Wert der ausgelesen wurde.</returns>
        public CwlbPropertyGridItem this[string strKey]
        {
            get
            {
                return _cobData[strKey];
            }
            set
            {
                _cobData[strKey] = value;
            }
        }

        /// <summary>
        /// Fügt der Auflistung ein Schlüssel-Wert-Paar hinzu.
        /// </summary>
        /// <param name="strKey">Schlüssel des hinzuzufügenden Schlüssel-Wert-Paares.</param>
        /// <param name="objValue">Wert des hinzuzufügenden Schlüssel-Wert-Paares.</param>
        public void Add(string strKey, CwlbPropertyGridItem objValue)
        {
            objValue.Name = strKey;
            _cobData.Add(strKey, objValue);
        }
        #endregion

        #region ICollection<KeyValuePair<string,CwlbPropertyDataItem>> Members
        /// <summary>
        /// Fügt der Auflistung ein Schlüssel-Wert-Paar hinzu.
        /// </summary>
        /// <param name="objItem">Schlüssel-WErt-Paar, das hinzuzufügen ist.</param>
        public void Add(KeyValuePair<string, CwlbPropertyGridItem> objItem)
        {
            objItem.Value.Name = objItem.Key;
            _cobData.Add(objItem.Key, objItem.Value);
        }

        /// <summary>
        /// Löscht den Inhalt der Auflistung.
        /// </summary>
        public void Clear()
        {
            _cobData.Clear();
        }

        /// <summary>
        /// Prüft das Vorhandensein eines definierten Schlüssel-Wert-Paares.
        /// </summary>
        /// <param name="objItem">Schlüssel-Wert-Paar, das zu prüfen ist.</param>
        /// <returns>True, wenn das SChlüssel-Wert-Paar in der Auflistung enthalten ist; andernfalls false.</returns>
        public bool Contains(KeyValuePair<string, CwlbPropertyGridItem> objItem)
        {
            return _cobData.Contains(objItem);
        }

        /// <summary>
        /// Diese Methode ist nicht implementiert und löst in jedem Fall eine Ausnahme aus.
        /// </summary>
        /// <param name="aobjData">Zielarray für die Aufnahme der Daten.</param>
        /// <param name="iArrayIndex">Index für das Kopieren.</param>
        public void CopyTo(KeyValuePair<string, CwlbPropertyGridItem>[] aobjData, int iArrayIndex)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Liefert die Anzahl der Elemente in der Auflistung.
        /// </summary>
        public int Count
        {
            get
            {
                return _cobData.Count;
            }
        }

        /// <summary>
        /// Liefert einen Wert der darauf hinweis ob die Auflistung schreibgeschützt ist oder nicht.
        /// </summary>
        /// <value>True, wenn die Auflistung schreibgeschützt ist; andernfalls false.</value>
        public bool IsReadOnly
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Löscht ein definiertes Schlüssel-Wert-Paar.
        /// </summary>
        /// <param name="objItem">Schlüssel des Schlüssel-Wert-Paares das zu löschen ist.</param>
        /// <returns>True, wenn das Schlüssel-Wert-Paar erfolgreich gelöscht wurde; andernfalls false.</returns>
        public bool Remove(KeyValuePair<string, CwlbPropertyGridItem> objItem)
        {
            return _cobData.Remove(objItem.Key);
        }

        #endregion

        #region IEnumerable<KeyValuePair<string,CwlbPropertyDataItem>> Members
        /// <summary>
        /// Listet alle Schlüssel-Wert-Paare der Auflistung auf.
        /// </summary>
        /// <returns>Schlüssel-Wert-Paar der laufenden Auflistung.</returns>
        public IEnumerator<KeyValuePair<string, CwlbPropertyGridItem>> GetEnumerator()
        {
            foreach (KeyValuePair<string, CwlbPropertyGridItem> objItem in _cobData)
            {
                yield return objItem;
            }
        }
        #endregion

        #region IEnumerable Members
        /// <summary>
        /// Listet alle Schlüssel-Wert-Paare der Auflistung auf.
        /// </summary>
        /// <returns>Schlüssel-Wert-Paar der laufenden Auflistung.</returns>
        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }
        #endregion

        #region ICustomTypeDescriptor Members
        /// <summary>
        /// Liefert den Klassennamen einer Instanz von dieser Komponente.
        /// </summary>
        /// <returns>
        /// Vollständiger Klassenname.
        /// </returns>
        public string GetClassName()
        {
            return TypeDescriptor.GetClassName(this, true);
        }

        /// <summary>
        /// Liefert eine Auflistung der benutzerdefinierten Attribute für eine Instanz dieser Komponente.
        /// </summary>
        /// <returns>
        /// AttributeCollection mit den benutzerdefinierten Attributen.
        /// </returns>
        public AttributeCollection GetAttributes()
        {
            return TypeDescriptor.GetAttributes(this, true);
        }

        /// <summary>
        /// Liefert den Namen einer Instanz dieser Komponente.
        /// </summary>
        /// <returns>
        /// Namen der Kompenente.
        /// </returns>
        public string GetComponentName()
        {
            return TypeDescriptor.GetComponentName(this, true);
        }

        /// <summary>
        /// Liefert einen TypConverter für eine Instanz dieser Komponente.
        /// </summary>
        /// <returns>
        /// TypConverter.
        /// </returns>
        public TypeConverter GetConverter()
        {
            return TypeDescriptor.GetConverter(this, true);
        }

        /// <summary>
        /// Liefert das Standardereignis für eine Instanz dieser Komponente.
        /// </summary>
        /// <returns>
        /// Beschreibung des Standardereignisses.
        /// </returns>
        public EventDescriptor GetDefaultEvent()
        {
            return TypeDescriptor.GetDefaultEvent(this, true);
        }

        /// <summary>
        /// Liefert die Standardeigenschaft für eine Instanz dieser Komponente.
        /// </summary>
        /// <returns>
        /// Standardeigenschaft.
        /// </returns>
        public PropertyDescriptor GetDefaultProperty()
        {
            return TypeDescriptor.GetDefaultProperty(this, true);
        }

        /// <summary>
        /// Liefert den Editor für eine Instanz dieser Komponente.
        /// </summary>
        /// <param name="editorBaseType">Typ, der die Basis des Editors definiert.</param>
        /// <returns>
        /// Instanz des zu verwendenden Editors.
        /// </returns>
        public object GetEditor(Type editorBaseType)
        {
            return TypeDescriptor.GetEditor(this, editorBaseType, true);
        }

        /// <summary>
        /// Liefert eine Auflistung der Ereignisse für eine Instanz dieser Komponente.
        /// </summary>
        /// <param name="aobjAttributes">Array der Attribute, die als Filter verwendet werden.</param>
        /// <returns>Auflistung der Ereignisse.</returns>
        public EventDescriptorCollection GetEvents(Attribute[] aobjAttributes)
        {
            return TypeDescriptor.GetEvents(this, aobjAttributes, true);
        }

        /// <summary>
        /// Liefert eine Auflistung der Ereignisse für eine Instanz dieser Komponente.
        /// </summary>
        /// <returns>Auflistung der Ereignisse.</returns>
        public EventDescriptorCollection GetEvents()
        {
            return TypeDescriptor.GetEvents(this, true);
        }

        /// <summary>
        /// Liefert den Besitzer einer Eigenschaft aufgrund einer definierte Eigenschaftsbeschreibung.
        /// </summary>
        /// <param name="objPropertyDescriptor">Eigenschaftsbeschreibung.</param>
        /// <returns>Besitzer der Eigenschaft.</returns>
        public object GetPropertyOwner(PropertyDescriptor objPropertyDescriptor)
        {
            return this;
        }

        /// <summary>
        /// Liefert eine Auflistung von Eigenschaftsbeschreibungen.
        /// </summary>
        /// <param name="aobjAttributes">Array von Attributen, die als Filter verwendet werden.</param>
        /// <returns>Auflistung der Eigenschaftsbeschreibungen.</returns>
        public PropertyDescriptorCollection GetProperties(Attribute[] aobjAttributes)
        {
            return GetProperties();
        }

        /// <summary>
        /// erstellt eine Auflistung von Eigenschaftsbeschreibungen.
        /// </summary>
        /// <returns>Auflistung der Eigenschaftsbeschreibungen.</returns>
        public PropertyDescriptorCollection GetProperties()
        {
            // Locale daten
            CwlbPropertyGridItemDescriptor objDataPropertyDescriptor;
            PropertyDescriptorCollection cobjPropertyDescriptorCollection = new PropertyDescriptorCollection(null);

            // Die gesamte Auflistung durchlaufen
            foreach (KeyValuePair<string, CwlbPropertyGridItem> objItem in _cobData)
            {
                // beschreibung herstellen
                objDataPropertyDescriptor = new CwlbPropertyGridItemDescriptor(this, objItem.Key);
                cobjPropertyDescriptorCollection.Add(objDataPropertyDescriptor);
            }
            return cobjPropertyDescriptorCollection;
        }
        #endregion
    }
}
