﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.ComponentModel;
using System.Drawing.Design;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Testklasse für die Demo des PropertyGrid-Steuerelements.
    /// </summary>
    [PropertyTabAttribute(typeof(CwlbFieldGridTab), PropertyTabScope.Component)]
    class CwlbPropertyGridTestData
    {
        // Definition eigener Kategorien
        private const string CATEGORY_SIMPLE = "Typen mit einfacher Eingabe";
        private const string CATEGORY_EDITOR = "Typen mit Editor";
        private const string CATEGORY_OWN_EDITOR = "Typen mit eigenem Editor";
        private const string CATEGORY_COMPOSED = "Zusammengesetzte Typen";

        // Diese Felder werden im PropertyGrid-Steuerelement nicht angezeigt.
        private int _iPrivateValue = 20;
        protected int _iProtectedValue = 30;
        public int _iPublicValue = 40;
        private readonly int _iReadonlyPrivateValue = 99;

        // Die folgenden Eigenschaften werden im PropertyGrid-Steuerelement NICHT angezeigt.
        protected int ProtectedValue { get; set; }

        [Browsable(false)]
        public int InvisibleValue { get; set; }

        // Die folgenden Eigenschaften werden im PropertyGrid-Steuerelement angezeigt.
        [Description("Doppelklick erlaubt das Wechseln des booleschen Werts.")]
        [Category(CATEGORY_SIMPLE)]
        public bool BoolValue { get; set; }

        [Description("Hier handelt es sich um einen einfachen short Wert.")]
        [Category(CATEGORY_SIMPLE)]
        [ReadOnly(true)]
        public short ShortValue { get; set; }

        [Description("Dies ist ein Wert mit einem selber definierten Anzeigenamen.")]
        [Category(CATEGORY_SIMPLE)]
        [DisplayName("Ganzzahliger Wert")]
        public int IntValue { get; set; }

        [Description("Hier handelt es sich um einen einfachen unsigned long Wert.")]
        [Category(CATEGORY_SIMPLE)]
        public ulong ULongValue { get; set; }

        [Description("Diese Eigenschaft ist mit dem Attribut ParenthesizePropertyName gekennzeichnet.")]
        [Category(CATEGORY_SIMPLE)]
        [ParenthesizePropertyName(true)]
        public string Name { get; set; }

        [Description("Eigene Auflistungen werden automatisch aufgelöst.")]
        [Category(CATEGORY_SIMPLE)]
        public EwlbPropertyGridDemo EnumValue { get; set; }

        [Description("Hier handelt es sich um eine einfache Zeichenfolge mit Platzhalterzeichen.")]
        [Category(CATEGORY_SIMPLE)]
        [PasswordPropertyText(true)]
        public string StringValueSecured { get; set; }

        [Description("Für Datums-/Zeit-Werte wird das DateTimePicker-Steuerelement verwendet.")]
        [Category(CATEGORY_EDITOR)]
        public DateTime DateTimeValue { get; set; }

        [Description("Für die Auswahl eines Farbwertes wird ein spezielles Auswahldialogfeld angezeigt.")]
        [Category(CATEGORY_EDITOR)]
        public Color ColorValue { get; set; }

        [EditorAttribute(typeof(CwlbPropertyEditorIPAddress), typeof(UITypeEditor))]
        [DisplayName("IP-Adresse")]
        [Description("IP-Adresse des Servers.")]
        [Category(CATEGORY_OWN_EDITOR)]
        public string IP_Address { get; set; }

        [EditorAttribute(typeof(CwlbPropertyEditorValuePicker), typeof(UITypeEditor))]
        [DisplayName("Volumen")]
        [Description("Diese Eigenschaft heißt in Wirklichkeit PackedValue.")]
        [Category(CATEGORY_OWN_EDITOR)]
        public int PackedValue { get; set; }

        [Description("Textfelder können direkt editiert werden.")]
        [Category(CATEGORY_COMPOSED)]
        public string[] TextValue { get; set; }

        [Description("Zusammengesetzte Typen werden in einer eigenen Struktur dargestellt.")]
        [Category(CATEGORY_COMPOSED)]
        public Size SizeValue { get; set; }

        [Description("Diese Eigenschaft verwendet eine Standardkategorie.")]
        [Category("Behavior")]
        public string ShowData { get; set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbPropertyGridTestData.
        /// </summary>
        public CwlbPropertyGridTestData()
        {
            ShortValue = 4;
            IntValue = 64;
            ULongValue = 37463;
            Name = "Testobjekt";
            StringValueSecured = "Passwort";
            DateTimeValue = DateTime.Now;
            SizeValue = new Size(20, 40);
            TextValue = new string[] { "Zeile 1", "Zeile 2", "Zeile 3" };
            EnumValue = EwlbPropertyGridDemo.Colorful;
        }
    }
}
