﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;
using System.Diagnostics;
using System.Drawing.Text;
using System.Drawing.Imaging;
using System.IO;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert die Funktionalität der Demo 'Zeichnen Basisfunktionen'.
    /// </summary>
    public partial class CmdiBase : Form
    {
        /// <summary>
        /// Definition der möglichen Zustände für das Fenster.
        /// </summary>
        private enum EwlbDemoType
        {
            Welcome,
            DrawingArea,
            CoordinateSystem,
            Quality,
            LineStyles,
            BrushSolid,
            BrushHatch,
            BrushTexture,
            BrushLinearGradient,
            BrushPathGradient,
            OpenFigure,
            ClosedFigures,
            FilledFigure,
            PathFigures,
            TextFontMeasurement,
            TextOutput,
            TextFonts,
            TextRotate,
            TextDetails,
            BitmapCreating,
            BitmapShow,
            BitmapColored,
            BitmapScreenCopy,
            Regions,
            Combination
        }

        /// <summary>
        /// Aktueller Zustand für das Zeichnen des Fensterinhalts
        /// </summary>
        private EwlbDemoType _enmDemoType;

        /// <summary>
        /// Zustand der wiederhergestellt werden soll.
        /// </summary>
        private EwlbDemoType _enmKeep;

        /// <summary>
        /// Initialisiert eine neue Instanz des Typs CmdiBase.
        /// </summary>
        public CmdiBase()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung für die Veränderung der Eigenschaft Bild.
        /// </summary>
        public void MainWindow_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "DemoImage")
            {
                Invalidate();
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Ändern der Grösse des Fensters.
        /// </summary>
        private void CmdiBase_SizeChanged(object sender, EventArgs e)
        {
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung bei Betätigung einer Taste.
        /// </summary>
        private void CmdiBase_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.PageDown)
            {
                _enmDemoType++;
            }
            else if (e.KeyCode == Keys.PageUp)
            {
                if (_enmDemoType > EwlbDemoType.Welcome)
                {
                    _enmDemoType--;
                }
            }
            else
            {
                return;
            }
            e.Handled = true;
            Invalidate();
        }

        #region Eventhandler Menü Zeichnen
        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Willkommen.
        /// </summary>
        private void MenuDrawingBaseWelcom_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.Welcome;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Zeichnungsfläche.
        /// </summary>
        private void MenuDrawBaseDrawingArea_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.DrawingArea;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Koordinaten.
        /// </summary>
        private void MenuDrawBaseCoordinates_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.CoordinateSystem;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Ausgabequalität.
        /// </summary>
        private void MenuDrawBaseQuality_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.Quality;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Stift.
        /// </summary>
        private void MenuDrawPen_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.LineStyles;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Pinsel (Solid).
        /// </summary>
        private void MenuDrawPenBrushSolid_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.BrushSolid;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Pinsel (Hatch).
        /// </summary>
        private void MenuDrawPenBrushHatch_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.BrushHatch;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Pinsel (Texture).
        /// </summary>
        private void MenuDrawPenBrushTexture_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.BrushTexture;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Pinsel (LinearGradient).
        /// </summary>
        private void MenuDrawPenBrushLinearGradient_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.BrushLinearGradient;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Pinsel (PathGradient).
        /// </summary>
        private void MenuDrawPenBrushSolidPathGradient_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.BrushPathGradient;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Figuren zeichnen.
        /// </summary>
        private void MenuDrawFiguresOpen_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.OpenFigure;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl geschlossene Figuren zeichnen.
        /// </summary>
        private void MenuDrawFiguresClosed_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.ClosedFigures;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl gefüllte Figuren zeichnen.
        /// </summary>
        private void MenuDrawFiguresFilled_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.FilledFigure;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Figuren zeichnen mit der Pfad-Technik.
        /// </summary>
        private void MenuDrawFiguresPath_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.PathFigures;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Schriftdetails.
        /// </summary>
        private void MenuDrawTextFontMeasurement_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.TextFontMeasurement;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Texte ausgeben.
        /// </summary>
        private void MenuDrawTextSimple_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.TextOutput;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Schriftarten.
        /// </summary>
        private void MenuDrawTextFonts_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.TextFonts;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Texte rotieren.
        /// </summary>
        private void MenuDrawTextRotate_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.TextRotate;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Textdetails.
        /// </summary>
        private void MenuDrawTextDetails_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.TextDetails;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Bitmap erstellen.
        /// </summary>
        private void _mnuDrawBitmapCreating_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.BitmapCreating;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Bitmap anzeigen.
        /// </summary>
        private void MenuDrawBitmapShow_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.BitmapShow;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Bitmap einfärben.
        /// </summary>
        private void MenuDrawBitmapColoring_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.BitmapColored;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Bildschirmkopie herstellen.
        /// </summary>
        private void MenuDrawBitmapScreenCopy_Click(object sender, EventArgs e)
        {
            _enmKeep = _enmDemoType;
            _enmDemoType = EwlbDemoType.BitmapScreenCopy;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Regionen darstellen.
        /// </summary>
        private void MenuDrawCombiRegions_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.Regions;
            Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Kombination herstellen.
        /// </summary>
        private void MenuDrawCombiCombi_Click(object sender, EventArgs e)
        {
            _enmDemoType = EwlbDemoType.Combination;
            Invalidate();
        }
        #endregion

        /// <summary>
        /// Ereignisbehandlung für das PaintEvent
        /// </summary>
        private void CmdiBase_Paint(object sender, PaintEventArgs e)
        {
            // Ausgabequalität steuern
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
            e.Graphics.CompositingQuality = CompositingQuality.HighQuality;
            e.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;

            // Fallverzweigung für die Demo
            switch (_enmDemoType)
            {
                default:
                case EwlbDemoType.Welcome:
                    ShowWelcome(e.Graphics);
                    break;
                case EwlbDemoType.DrawingArea:
                    ShowDrawingArea(e.Graphics);
                    break;
                case EwlbDemoType.CoordinateSystem:
                    ShowStandardCoordinates(e.Graphics);
                    break;
                case EwlbDemoType.Quality:
                    ShowQuality(e.Graphics);
                    break;
                case EwlbDemoType.LineStyles:
                    ShowLineStyles(e.Graphics);
                    break;
                case EwlbDemoType.BrushSolid:
                    ShowSolidBrush(e.Graphics);
                    break;
                case EwlbDemoType.BrushHatch:
                    ShowHatchBrush(e.Graphics);
                    break;
                case EwlbDemoType.BrushTexture:
                    ShowTextureBrush(e.Graphics);
                    break;
                case EwlbDemoType.BrushLinearGradient:
                    ShowLinearGradientBrush(e.Graphics);
                    break;
                case EwlbDemoType.BrushPathGradient:
                    ShowPathGradientBrush(e.Graphics);
                    break;
                case EwlbDemoType.OpenFigure:
                    ShowOpenLines(e.Graphics);
                    break;
                case EwlbDemoType.ClosedFigures:
                    ShowClosedLines(e.Graphics);
                    break;
                case EwlbDemoType.FilledFigure:
                    ShowFilledFigures(e.Graphics);
                    break;
                case EwlbDemoType.PathFigures:
                    ShowPath(e.Graphics);
                    break;
                case EwlbDemoType.TextFontMeasurement:
                    ShowFontMeasurement(e.Graphics);
                    break;
                case EwlbDemoType.TextOutput:
                    ShowText(e.Graphics);
                    break;
                case EwlbDemoType.TextFonts:
                    ShowFonts(e.Graphics);
                    break;
                case EwlbDemoType.TextRotate:
                    ShowTextRotation(e.Graphics);
                    break;
                case EwlbDemoType.TextDetails:
                    ShowTextQuality(e.Graphics);
                    break;
                case EwlbDemoType.BitmapCreating:
                    ShowCreatingImage(e.Graphics);
                    break;
                case EwlbDemoType.BitmapShow:
                    ShowImage(e.Graphics);
                    break;
                case EwlbDemoType.BitmapColored:
                    ShowImageColored(e.Graphics);
                    break;
                case EwlbDemoType.BitmapScreenCopy:
                    ShowScreenCopy(e.Graphics);
                    break;
                case EwlbDemoType.Regions:
                    ShowRegionComplement(e.Graphics);
                    break;
                case EwlbDemoType.Combination:
                    ShowCombination(e.Graphics);
                    break;
            }
        }

        /// <summary>
        /// Zeichnet die Grundfigur der Demo.
        /// </summary>
        private void ShowWelcome(Graphics objGraphics)
        {
            // Ressourcen für Kugel bereitstellen
            GraphicsPath pathBall = new GraphicsPath();
            pathBall.AddEllipse(50, 50, 180, 180);
            PathGradientBrush brushBall = new PathGradientBrush(pathBall);
            brushBall.CenterPoint = new Point(170, 100);
            brushBall.CenterColor = Color.White;
            brushBall.SurroundColors = new Color[] { Color.Blue };
            // Kugel anzeigen
            objGraphics.FillEllipse(brushBall, 50, 50, 180, 180);
            objGraphics.DrawEllipse(Pens.Blue, new Rectangle(50, 50, 180, 180));

            // Ressource für Linie bereitstellen
            LinearGradientBrush brushLine = new LinearGradientBrush(
                new Point(0, 100), 
                new Point(this.ClientSize.Width, 100), 
                Color.Red, Color.Blue);
            Pen penLine = new Pen(brushLine, 20.0f);
            // Linie anzeigen
            objGraphics.DrawLine( 
                penLine,
                50, this.ClientSize.Height / 3 * 2 + 40,
                this.ClientSize.Width - 50, this.ClientSize.Height / 3 * 2 + 40);

            // Format und Ressource für Titeltext bereitstellen
            StringFormat formatTitle = new StringFormat();
            formatTitle.Alignment = StringAlignment.Far;
            formatTitle.LineAlignment = StringAlignment.Center;
            Font fontTitle = new Font("Arial", 28.0F, FontStyle.Regular);
            // Titeltext anzeigen
            objGraphics.DrawString(
                "Zeichnen mit GDI+",
                fontTitle,
                Brushes.Red,
                this.ClientSize.Width - 50, this.ClientSize.Height / 3 * 2,
                formatTitle);

            // Ressource für Hinweistext bereitstellen
            Font fontAnnotation = new Font("Arial", 12.0F, FontStyle.Regular);
            // Hinweistext ausgeben
            objGraphics.DrawString(
                "Wählen Sie im Menü Zeichnen eine Demo aus!",
                fontAnnotation,
                Brushes.Red,
                10, 20);

            // Freigeben der lokalen Ressourcen
            fontAnnotation.Dispose();
            fontTitle.Dispose();
            formatTitle.Dispose();
            penLine.Dispose();
            brushLine.Dispose();
            pathBall.Dispose();
            brushBall.Dispose();
        }

        /// <summary>
        /// Demo der Standardzeichenfläche.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowDrawingArea(Graphics objGraphics)
        {
            // Lokale Ressourcen erstellen
            Pen pen = new Pen(Color.Red, 2);
            Brush brush = new HatchBrush(HatchStyle.WideDownwardDiagonal, Color.White);
            Trace.WriteLine("ShowingDrawingArea");

            objGraphics.FillRectangle(brush, this.ClientRectangle);
            objGraphics.DrawRectangle(pen, 1, 1, this.ClientSize.Width - 2, this.ClientSize.Height - 2);

            // Lokale Ressourcen frei geben
            pen.Dispose();
            brush.Dispose();
        }

        /// <summary>
        /// Demo der Standardkoordinaten.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowStandardCoordinates(Graphics objGraphics)
        {
            // Lokale Ressourcen
            Font fontBig = new Font("Arial", 20.0F, FontStyle.Regular);
            Font fontSmall = new Font("Arial", 10.0F, FontStyle.Regular);
            Pen pen = new Pen(Color.Red);
            AdjustableArrowCap objArrowCap = new AdjustableArrowCap(10, 12);
            pen.CustomEndCap = objArrowCap;

            // Zeichne die Linien
            objGraphics.DrawLine(pen, 0, 10, this.ClientSize.Width - 1, 10);
            objGraphics.DrawLine(pen, 10, 0, 10, this.ClientSize.Height - 1);
            objGraphics.DrawEllipse(pen, 0, 0, 20, 20);

            // Beschriftung einbringen
            objGraphics.DrawString("+X", fontBig, Brushes.Red, this.ClientSize.Width - 50, 20);
            objGraphics.DrawString("+Y", fontBig, Brushes.Red, 20, this.ClientSize.Height - 50);
            objGraphics.DrawString("Point( 10, 10 )", fontSmall, Brushes.Red, 20, 20);

            // Linie 96 Punkte
            objGraphics.DrawLine(Pens.Black, 100, 95, 100, 130);
            objGraphics.DrawLine(Pens.Black, 100, 100, 196, 100);
            objGraphics.DrawLine(Pens.Black, 196, 95, 196, 130);
            pen.Color = Color.Black;
            AdjustableArrowCap objSmallArrowCap = new AdjustableArrowCap(3, 4);
            pen.CustomEndCap = objSmallArrowCap;
            pen.CustomStartCap = objSmallArrowCap;
            objGraphics.DrawLine(pen, 100, 127, 196, 127);
            objGraphics.DrawString("96 Einheiten", fontSmall, Brushes.Black, 110, 110);

            // Rechteck wegen Pixelberechnung Linienbreite 3
            Pen penHelper = new Pen(Color.Blue, 3);
            objGraphics.SmoothingMode = SmoothingMode.Default;
            Rectangle rectDemo = new Rectangle(300, 100, 10, 10);
            objGraphics.FillRectangle(Brushes.Red, rectDemo);
            objGraphics.DrawRectangle(penHelper, rectDemo);
            objGraphics.FillRectangle(Brushes.White, 300, 100, 1, 1);

            // Rechteck wegen Pixelberechnung Linienbreite 4
            rectDemo = new Rectangle(320, 100, 10, 10);
            penHelper.Width = 4;
            objGraphics.FillRectangle(Brushes.Red, rectDemo);
            objGraphics.DrawRectangle(penHelper, rectDemo);
            objGraphics.FillRectangle(Brushes.White, 320, 100, 1, 1);


            // Lokale Ressourcen frei geben
            penHelper.Dispose();
            pen.Dispose();
            objArrowCap.Dispose();
        }

        /// <summary>
        /// Demo der Ausgabequalität
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowQuality(Graphics objGraphics)
        {
            // Lokale Daten
            float fLength = 150.0f;
            float fOffset = 200.0f;
            float fDelta = (float)(Math.PI / 60);
            Stopwatch objWatch = Stopwatch.StartNew();

            // Lokale Ressourcen
            using( StringFormat format = new StringFormat())
            using (Pen pen = new Pen(Color.DarkBlue, 1.0f))
            {
                // Einstellungen vornehmen
                objGraphics.SmoothingMode = SmoothingMode.AntiAlias;
                objGraphics.CompositingQuality = CompositingQuality.HighQuality;
                objGraphics.PixelOffsetMode = PixelOffsetMode.HighQuality;
                objGraphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;
                format.Alignment = StringAlignment.Center;

                // Grafik erstellen 
                objWatch.Restart();
                for (float fX = 0; fX <= Math.PI; fX += fDelta)
                {
                    objGraphics.DrawLine(pen,
                        (float)Math.Sin(fX) * fLength + fOffset, (float)Math.Cos(fX) * fLength + fOffset,
                        -(float)Math.Sin(fX) * fLength + fOffset, -(float)Math.Cos(fX) * fLength + fOffset);
                }
                for (float fPos = 10; fPos <= fLength; fPos+=10)
                {
                    objGraphics.DrawEllipse(pen,
                        fOffset - fPos, fOffset - fPos,
                        fPos * 2, fPos * 2);
                }
                objWatch.Stop();
                // Gemessene Zeit ausgeben
                string strOut = objWatch.Elapsed.ToString();
                objGraphics.DrawString(strOut, SystemFonts.CaptionFont, Brushes.Navy, fOffset, 390, format);


                // Einstellungen vornehmen
                objGraphics.SmoothingMode = SmoothingMode.HighSpeed;
                objGraphics.CompositingQuality = CompositingQuality.HighSpeed;
                objGraphics.PixelOffsetMode = PixelOffsetMode.Default;
                objGraphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.SystemDefault;

                // Grafik abermals erstellen (nach rechts verschoben)
                objWatch.Restart();
                for (float fX = 0; fX <= Math.PI; fX += fDelta)
                {
                    objGraphics.DrawLine(pen,
                        (float)Math.Sin(fX) * fLength + fOffset * 3, (float)Math.Cos(fX) * fLength + fOffset,
                        -(float)Math.Sin(fX) * fLength + fOffset * 3, -(float)Math.Cos(fX) * fLength + fOffset);
                }
                for (float fPos = 10; fPos <= fLength; fPos+=10)
                {
                    objGraphics.DrawEllipse(pen,
                        fOffset * 3 - fPos, fOffset - fPos,
                        fPos * 2, fPos * 2);
                }
                objWatch.Stop();
                // Gemessene Zeit ausgeben
                strOut = objWatch.Elapsed.ToString();
                objGraphics.DrawString(strOut, SystemFonts.CaptionFont, Brushes.Navy, fOffset * 3, 390, format);
            }
        }

        /// <summary>
        /// Demo der verschiedenen Stiftarten anhand von Linien.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowLineStyles(Graphics objGraphics)
        {
            // Lokale Ressourcen
            Pen pen = new Pen(Color.Navy, 0.0f);
            Font font = new System.Drawing.Font("Arial", 8.0f);

            // Antialiasing ausschalten
            SmoothingMode enmSmoothingMode = objGraphics.SmoothingMode;
            objGraphics.SmoothingMode = SmoothingMode.Default;

            // Demo für unterschiedlich breite Linien
            for (int iX = 25; iX < 100; iX += 10)
            {
                // Erzeuge Stift
                objGraphics.DrawLine(pen, iX, 50, iX, this.ClientSize.Height - 50);
                objGraphics.DrawString(pen.Width.ToString(), font, Brushes.Black, new PointF(iX - 4, 35));
                pen.Width += 1.0f;
            }

            // Farbe und Linienbreite neu einstellen
            pen.Color = Color.OrangeRed;
            pen.Width = 3;

            // Setze Stil: Strich
            pen.DashStyle = DashStyle.Dash;
            objGraphics.DrawLine(pen, 160, 50, 160, this.ClientSize.Height - 50);

            // Setze Stil: Strich-Punkt
            pen.DashStyle = DashStyle.DashDot;
            objGraphics.DrawLine(pen, 180, 50, 180, this.ClientSize.Height - 50);

            // Setze Stil: Strich-Punkt-Punkt
            pen.DashStyle = DashStyle.DashDotDot;
            objGraphics.DrawLine(pen, 200, 50, 200, this.ClientSize.Height - 50);

            // Setze Stil: Punkt
            pen.DashStyle = DashStyle.Dot;
            objGraphics.DrawLine(pen, 220, 50, 220, this.ClientSize.Height - 50);

            // Setze Stil: Linie
            pen.DashStyle = DashStyle.Solid;
            objGraphics.DrawLine(pen, 240, 50, 240, this.ClientSize.Height - 50);

            // Farbe und Linienbreite neu einstellen
            pen.Color = Color.Green;
            pen.Width = 5;

            // Setze Stil: Benutzerdefiniert
            float[] afDash = { 15.0F, 3.0F, 10.0F, 3.0F, 3.0F, 3.0F, 3.0F, 3.0F, 10.0F, 3.0F };
            pen.DashStyle = DashStyle.Custom;
            pen.DashPattern = afDash;
            objGraphics.DrawLine(pen, 320, 50, 320, this.ClientSize.Height - 50);

            // Definiere eine eigene Linienzusammensetzung
            // Farbe und Linienbreite neu einstellen
            pen.Color = Color.Black;
            pen.Width = 10;
            // Definere das Muster
            float[] afCompoud = { 
                0.0F, // Lücke bis zum Beginn der ersten Linie (minimal 0)
                0.1F, // Breite der ersten Linie (in % der Linienbreite)
                0.2F, // Lücke bis zur zweiten Linie
                0.3F, // Breite der zweiten Linie
                0.4F, // Lücke bis zur dritten Linie
                1.0F }; // Ende des letzten Elements (maximal 1)
            // Registriere das Muster
            pen.CompoundArray = afCompoud;
            // Linie ausgeben
            objGraphics.DrawLine(pen, 340, 50, 340, this.ClientSize.Height - 50);
            // Antialiasing zurückstellen
            objGraphics.SmoothingMode = enmSmoothingMode;

            // Neues Linienobjekt erstellen
            pen.Dispose();
            pen = new Pen(Color.Black, 7);

            // Linienenden zeichnen
            // Pfeilenden setzen
            pen.SetLineCap(LineCap.ArrowAnchor, LineCap.DiamondAnchor, DashCap.Flat);
            objGraphics.DrawLine(pen, 420, 50, 420, this.ClientSize.Height - 50);
            // Pfeilenden setzen
            pen.SetLineCap(LineCap.Round, LineCap.RoundAnchor, DashCap.Flat);
            objGraphics.DrawLine(pen, 440, 50, 440, this.ClientSize.Height - 50);
            // Pfeilenden setzen
            pen.SetLineCap(LineCap.Square, LineCap.SquareAnchor, DashCap.Flat);
            objGraphics.DrawLine(pen, 460, 50, 460, this.ClientSize.Height - 50);
            // Pfeilenden setzen
            pen.SetLineCap(LineCap.Triangle, LineCap.NoAnchor, DashCap.Flat);
            objGraphics.DrawLine(pen, 480, 50, 480, this.ClientSize.Height - 50);

            // Pfeilenden setzen
            pen.StartCap = LineCap.Flat;
            pen.EndCap = LineCap.Custom;
            pen.Width = 1;
            AdjustableArrowCap objArrowCap = new AdjustableArrowCap(6, 8);
            pen.CustomEndCap = objArrowCap;
            objGraphics.DrawLine(pen, 500, 50, 500, this.ClientSize.Height - 50);

            // Lokale Ressourcen frei geben
            pen.Dispose();
            font.Dispose();
            objArrowCap.Dispose();
        }

        /// <summary>
        /// Demo der Pinsel mit Vollfarben.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowSolidBrush(Graphics objGraphics)
        {
            // Lokale Ressourcen
            int iX = (this.ClientSize.Width - 100) / 6;
            int iY = (this.ClientSize.Height - 100) / 6;
            SolidBrush brush = new SolidBrush(Color.Black);
            byte byteColorDelta = 255 / 5;
            byte byteRed = 255;
            byte byteLineRed = 0;
            byte byteGreen = 0;
            byte byteBlue = 0;
            byte byteBlueLine;

            // Schleife Zeilen
            for (int iCountRow = 0; iCountRow < 6; iCountRow++)
            {
                // Farbe für Zeile initialisieren
                byteGreen = 0;
                byteBlueLine = 0;
                byteLineRed = byteRed;

                // Schleife Spalten
                for (int iCountCol = 0; iCountCol < 6; iCountCol++)
                {
                    // Farbe definieren
                    brush.Color = Color.FromArgb(byteLineRed, byteGreen, byteBlueLine);

                    // Rechteck zeichnen
                    objGraphics.FillRectangle(
                        brush,
                        iCountCol * iX + 50, iCountRow * iY + 50,
                        iX, iY);

                    // Farbe für Spalte inkrementieren
                    byteGreen += byteColorDelta;
                    byteBlueLine += (byte)(byteBlue / 5);
                    byteLineRed -= (byte)(byteRed / 5);
                }

                // Farbe für Zeile inkrementieren
                byteRed -= byteColorDelta;
                byteBlue += byteColorDelta;
            }

            // Lokale Ressourcen freigeben
            brush.Dispose();
        }

        /// <summary>
        /// Demo der Pinsel für Schraffuren.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowHatchBrush(Graphics objGraphics)
        {
            // Lokale Ressourcen
            int iX = (this.ClientSize.Width - 100) / 8;
            int iY = (this.ClientSize.Height - 100) / 7;
            Pen pen = new Pen(Color.Black, 2);
            int iCountCol = 0;
            int iCountRow = 0;

            // Antialiasing ausschalten
            SmoothingMode enmSmoothingMode = objGraphics.SmoothingMode;
            objGraphics.SmoothingMode = SmoothingMode.Default;

            // Schleife durch alle Typen
            foreach (HatchStyle enmHatch in Enum.GetValues(typeof(HatchStyle)))
            {
                // Verändere Brush
                using (Brush brush = new HatchBrush(enmHatch, Color.Black, Color.White))
                {
                    // Zeichne gefülltes Rechteck mit Pinsel
                    objGraphics.FillRectangle(
                        brush,
                        iCountCol * iX + 50, iCountRow * iY + 50,
                        iX, iY);
                    objGraphics.DrawRectangle(
                        pen,
                        iCountCol * iX + 50, iCountRow * iY + 50,
                        iX, iY);

                    // Positionen inkrementieren
                    if (iCountCol >= 7)
                    {
                        iCountCol = 0;
                        iCountRow++;
                    }
                    else
                    {
                        iCountCol++;
                    }
                }
            }

            // Pen-Ressource freigeben
            pen.Dispose();
        }

        /// <summary>
        /// Demo eines Pinsels für eigene Muster.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowTextureBrush(Graphics objGraphics)
        {
            using (TextureBrush brush = new TextureBrush(Properties.Resources.Texture))
            {
                // Drehe die Textur um 45 Grad
                brush.RotateTransform(45);

                // Zeiche Rechteck
                objGraphics.FillEllipse(
                    brush,
                    50, 50,
                    this.ClientSize.Width - 100, this.ClientSize.Height - 100);
            }
        }

        /// <summary>
        /// Demo der Pinsel für lineare Farbverläufe.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowLinearGradientBrush(Graphics objGraphics)
        {
            // oben links: Einfacher LinearGradientBrush --------------------------------
            using (LinearGradientBrush brush = new LinearGradientBrush(
                new Point(50, 50), new Point(this.ClientSize.Width / 2, 50),
                Color.White, Color.Black))
            {
                // Zeichne Rechteck
                objGraphics.FillRectangle(
                    brush,
                    50, 50,
                    (this.ClientSize.Width - 100) / 2, (this.ClientSize.Height - 100) / 2);
            }

            // oben rechts: LineargradientBrush mit Gammakorrektur ----------------------
            using (LinearGradientBrush brush = new LinearGradientBrush(
                new Point(50, 50), new Point(this.ClientSize.Width / 2, 50),
                Color.White, Color.Black))
            {
                // Gammakorrektur einschalten
                brush.GammaCorrection = true;

                // Zeichne Rechteck
                objGraphics.FillRectangle(
                    brush,
                    this.ClientSize.Width / 2, 50,
                    (this.ClientSize.Width - 100) / 2, (this.ClientSize.Height - 100) / 2);
            }

            // unten links: LiniearGradientBrush mit Blend -------------------------------
            using (LinearGradientBrush brush = new LinearGradientBrush(
                new Point(50, 50), new Point(this.ClientSize.Width / 2, 50),
                Color.White, Color.Black))
            {
                // Blend Definition mit 9 Stufen
                Blend objBlend = new Blend(9);
                // Definiere die Farbfaktoren an den Positionen
                // 0 für erste Farbe, 1 für zweite Farbe
                objBlend.Factors = new float[]{
                0.0F, 0.3F, 0.6F, 0.9F, 1.0F, 0.9F, 0.6F, 0.3F, 0.0F};
                // Definiere die Stufen relativ zur Breite des Brush-Rechtecks
                objBlend.Positions = new float[]{
                0.0F, 0.1F, 0.3F, 0.4F, 0.5F, 0.6F, 0.7F, 0.8F, 1.0F};
                brush.Blend = objBlend;

                // Zeichen Rechteck
                objGraphics.FillRectangle(
                    brush,
                    50, this.ClientSize.Height / 2,
                    (this.ClientSize.Width - 100) / 2, (this.ClientSize.Height - 100) / 2);
            }

            // unten rechts: Mehrfarbiger LinearGradientBrush ---------------------------
            using (LinearGradientBrush brush = new LinearGradientBrush(
                new Point(50, 50), new Point(this.ClientSize.Width / 2, 50),
                Color.White, Color.Black))
            {
                // Farbdefinitionen vornehmen
                // Erzeuge ein Objekt zur Farbanpassung
                ColorBlend objColorBlend = new ColorBlend();
                // Definiere die Positionen der Farben relativ zur Breite des Brush
                objColorBlend.Positions = new float[] {
                0.0F, 0.6F, 1.0F };
                // Definiere die Farben an den jeweiligen Positionen
                objColorBlend.Colors = new Color[] {
                Color.Red, Color.Yellow, Color.Blue};
                brush.InterpolationColors = objColorBlend;

                // Zeichne Rechteck
                objGraphics.FillRectangle(
                    brush,
                    this.ClientSize.Width / 2, this.ClientSize.Height / 2,
                    (this.ClientSize.Width - 100) / 2, (this.ClientSize.Height - 100) / 2);
            }
        }

        /// <summary>
        /// Demo der Pinsel für pfadspezifische Farbverläufe.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowPathGradientBrush(Graphics objGraphics)
        {

            // linke Figur -------------------------------------------
            using (GraphicsPath graphPath = new GraphicsPath())
            {
                // Definiere einen Kreis im Pathobjekt
                graphPath.AddEllipse(50, 50, 180, 180);

                // Erzeuge ein PathGradientBrush-Objekt
                using (PathGradientBrush brush = new PathGradientBrush(graphPath))
                {
                    // Definiere den Zentrumspunkt und seine Farbe
                    brush.CenterPoint = new Point(170, 100);
                    brush.CenterColor = Color.White;
                    // Definiere seine umfassende Farbe
                    brush.SurroundColors = new Color[] { Color.Blue };

                    // Zeichne ein Rechteck mit Hilfe des Pinsels
                    objGraphics.FillRectangle(brush, 50, 50, 180, 180);
                    objGraphics.DrawEllipse(Pens.Blue, 50, 50, 180, 180);
                }
            }

            // mittlere Figur ----------------------------------------
            using (GraphicsPath graphPath = new GraphicsPath())
            {
                // Definiere einen Kreis im Path-Objekt
                graphPath.AddEllipse(250, 50, 180, 180);

                // Erzeuge ein PathGradientBrush-Objekt
                using (PathGradientBrush brush = new PathGradientBrush(graphPath))
                {
                    // Definiere den Zentrumspunkt und seine Farbe
                    brush.CenterPoint = new Point(370, 100);
                    brush.CenterColor = Color.White;
                    // Definiere seine Farbringe
                    ColorBlend objColorBlend = new ColorBlend(3);
                    objColorBlend.Colors = new Color[] { 
                        Color.Blue, Color.Red, Color.Yellow };
                    objColorBlend.Positions = new float[] {
                        0.0F, 0.5F, 1.0F };
                    brush.InterpolationColors = objColorBlend;

                    // Zeichne ein Rechteck mit Hilfe des Pinsels
                    objGraphics.FillRectangle(brush, 250, 50, 180, 180);
                    objGraphics.DrawEllipse(Pens.Blue, 250, 50, 180, 180);
                }
            }

            // rechte Figur ----------------------------------------
            using (GraphicsPath graphPath = new GraphicsPath())
            {
                // Definiere ein Sechseck im Path-Objekt
                graphPath.StartFigure();
                graphPath.AddLine(535, 50, 460, 100);
                graphPath.AddLine(460, 180, 535, 230);
                graphPath.AddLine(610, 180, 610, 100);
                graphPath.CloseFigure();

                // Erzeuge ein PathGradientBrush-Objekt
                using (PathGradientBrush brush = new PathGradientBrush(graphPath))
                {
                    // Definiere den Zentrumspunkt und seine Farbe
                    brush.CenterPoint = new Point(535, 140);
                    brush.CenterColor = Color.White;
                    // Definere seine umfassende Farbe
                    brush.SurroundColors = new Color[] { 
                        Color.Blue, Color.Red, Color.Yellow,
                        Color.Blue, Color.Red, Color.Yellow };

                    // Zeichne ein Rechteck mit Hilfe des Pinsels
                    objGraphics.FillRectangle(brush, 460, 50, 635, 230);
                    objGraphics.DrawPath(Pens.Blue, graphPath);
                }
            }
        }

        /// <summary>
        /// Demo offene Figuren.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowOpenLines(Graphics objGraphics)
        {
            // Lokale Ressourcen
            using( Pen pen = new Pen(Color.Green, 4))
            using (Pen penBase = new Pen(Color.Red, 1))
            {
                // Gerade Linie zeichnen
                objGraphics.DrawLine(pen, 40, 40, 250, 70);

                // Kreissegment zeichnen
                objGraphics.DrawArc(
                    pen,        // Stift
                    100, 100,   // X-, Y-Koordinate Startpunkt
                    100, 100,   // Breite, Höhe des umfassenden Rechtecks
                    225,        // Winkel des Beginns der Kurve
                    180);       // Öffnungswinkel der Kurve

                // Bezier
                Point ptP1 = new Point(50, 350);
                Point ptP1Base = new Point(100, 200);
                Point ptP2 = new Point(250, 250);
                Point ptP2Base = new Point(375, 380);

                // Hilfslinien zeichnen
                objGraphics.DrawLine(penBase, ptP1, ptP1Base);
                objGraphics.DrawLine(penBase, ptP2, ptP2Base);
                objGraphics.DrawBezier(pen, ptP1, ptP1Base, ptP2Base, ptP2);

                // Kurve
                Point[] pt = new Point[] { 
                new Point( 350, 100 ), new Point( 400, 150 ), new Point( 350, 250 ),
                new Point( 400, 200 ), new Point( 450, 250 ), new Point( 550, 150 ),
                new Point( 550, 100 ), new Point( 400, 100 ), new Point( 500,  50 )};
                objGraphics.DrawCurve(pen, pt);
            }
        }

        /// <summary>
        /// Demo geschlossene Figuren.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowClosedLines(Graphics objGraphics)
        {
            // Lokale Ressourcen
            using (Pen pen = new Pen(Color.Green, 2))
            {
                // Rechteck
                objGraphics.DrawRectangle(
                    pen,        // Stift
                    50, 50,     // X-, Y-Koordinate Ecke oben links
                    75, 100);   // Breite, Höhe des Rechtecks

                // Ellipse
                objGraphics.DrawEllipse(
                    pen,        // Stift
                    50, 200,    // X-, Y-Koordinate Ecke oben links des umfassendes Rechtecks
                    75, 75);    // Breite, Höhe des umfassenden Rechtecks

                // Kreisabschnitt
                objGraphics.DrawPie(
                    pen,        // Stift
                    200, 50,    // X-, Y-Koordinate Startpunkt
                    100, 100,   // Breite, Höhe des umfassenden Rechtecks
                    225,        // Winkel des Beginns der Kurve
                    180);       // Öffnungswinkel der Kurve

                // Polygon
                Point[] pt = new Point[] {
                new Point( 250, 200 ), new Point( 200, 300 ), new Point( 300, 300 )};
                objGraphics.DrawPolygon(pen, pt);

                // Kurve
                pt = new Point[] { 
                new Point( 350, 100 ), new Point( 400, 150 ), new Point( 350, 250 ),
                new Point( 400, 200 ), new Point( 450, 250 ), new Point( 550, 150 ),
                new Point( 550, 100 ), new Point( 400, 100 ), new Point( 500,  50 )};
                objGraphics.DrawClosedCurve(pen, pt);
            }
        }

        /// <summary>
        /// Demo gefüllte Figuren.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowFilledFigures(Graphics objGraphics)
        {
            // Rechteck ohne Umrandung
            objGraphics.FillRectangle(Brushes.Orange, 50, 50, 75, 75);

            // Ellipse mit Umrandung
            using (Pen pen = new Pen(Color.Red, 1))
            {
                objGraphics.FillEllipse(Brushes.Orange, 50, 200, 75, 75);
                objGraphics.DrawEllipse(pen, 50, 200, 75, 75);
            }

            // Kreissegment mit Umrandung
            using (Pen pen = new Pen(Color.DarkMagenta, 10))
            {
                objGraphics.FillPie(Brushes.Orange, 200, 50, 100, 100, 225, 180);
                objGraphics.DrawPie(pen, 200, 50, 100, 100, 225, 180);
            }

            // Polygon mit Umrandung
            Point[] pt = new Point[] {
                new Point( 250, 200 ), new Point( 200, 300 ), new Point( 300, 300 )};
            using (Pen pen = new Pen(Color.Blue, 2))
            {
                pen.DashStyle = DashStyle.Dot;
                objGraphics.FillPolygon(Brushes.Orange, pt);
                objGraphics.DrawPolygon(pen, pt);
            }

            // Kurve mit Umrandung
            pt = new Point[] { 
                new Point( 350, 100 ), new Point( 400, 150 ), new Point( 350, 250 ),
                new Point( 400, 200 ), new Point( 450, 250 ), new Point( 550, 150 ),
                new Point( 550, 100 ), new Point( 400, 100 ), new Point( 500,  50 )};
            using (Pen pen = new Pen(Color.DeepPink, 5))
            {
                pen.DashStyle = DashStyle.DashDotDot;
                objGraphics.FillClosedCurve(Brushes.Orange, pt);
                objGraphics.DrawClosedCurve(pen, pt);
            }
        }

        /// <summary>
        /// Demo für die Erstellung einer Pfadfigur.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowPath(Graphics objGraphics)
        {
            // Lokale Ressourcen
            using( TextureBrush brush = new TextureBrush(Properties.Resources.Texture))
            using (Pen pen = new Pen(Color.Red, 2))
            {
                // Drehe die Textur um 45 Grad
                brush.RotateTransform(45);

                // Einfache offene Figur
                using (GraphicsPath objPath = new GraphicsPath())
                {
                    objPath.AddArc(50, 50, 50, 50, 90, 180);
                    objPath.AddArc(200, 50, 50, 50, 270, 180);
                    objGraphics.DrawPath(pen, objPath);
                }

                // Einfache geschlossene Figur
                using (GraphicsPath objPath = new GraphicsPath())
                {
                    objPath.StartFigure();
                    objPath.AddArc(350, 50, 50, 50, 90, 180);
                    objPath.AddArc(500, 50, 50, 50, 270, 180);
                    objPath.CloseFigure();
                    objGraphics.FillPath(Brushes.DarkCyan, objPath);
                    objGraphics.DrawPath(pen, objPath);
                }

                // Stern links
                Point[] aptLines = new Point[] {
                    new Point(175, 200), new Point(275, 400),
                    new Point(50, 275), new Point(300, 275),
                    new Point(75, 400), new Point(175, 200)
                };
                using (GraphicsPath objPath = new GraphicsPath())
                {
                    objPath.StartFigure();
                    objPath.AddLines(aptLines);
                    objPath.CloseFigure();
                    objPath.FillMode = FillMode.Winding;
                    objGraphics.FillPath(brush, objPath);
                    objGraphics.DrawPath(pen, objPath);
                }

                // Stern rechts
                aptLines = aptLines.Select(ptData =>
                    {
                        ptData.Offset(300, 0); // Verschiebt die Koordinaten um
                        return ptData;         // 300 pixel nach rechts
                    }).ToArray();
                using (GraphicsPath objPath = new GraphicsPath())
                {
                    objPath.StartFigure();
                    objPath.AddLines(aptLines);
                    objPath.CloseFigure();
                    objPath.FillMode = FillMode.Alternate;
                    objGraphics.FillPath(brush, objPath);
                    objGraphics.DrawPath(pen, objPath);
                }
            }
        }

        /// <summary>
        /// Demo der Grunddaten einer Schrift
        /// </summary>
        /// <param name="objGraphics"></param>
        private void ShowFontMeasurement(Graphics objGraphics)
        {
            // Lokale Daten
            string strOut = "ABFG abfg Ää";

            // Lokale Ressourcen
            using(Font font = new Font("Century Gothic", 50))
            using(StringFormat format = new StringFormat())
            using(Pen pen = new Pen(Color.Red)) 
            {
                // Stil und Forma einstellen
                pen.DashStyle = DashStyle.Dash;
                format.Alignment = StringAlignment.Near;        // Standardwert verwenden
                format.LineAlignment = StringAlignment.Near;    // Standardwert verwenden

                // Basispunkt definieren und Ausgabegröße berechnen
                Point ptBase = new Point(50, 50);
                SizeF sizeString = objGraphics.MeasureString(strOut, font, int.MaxValue, format);

                // Text in Ausgaberechteck zeichnen
                objGraphics.DrawString(strOut, font, Brushes.Blue, ptBase, format);
                objGraphics.DrawRectangle(pen, new Rectangle(ptBase, sizeString.ToSize()));

                // Zeichnen der Basispunkte
                // Oben
                objGraphics.FillEllipse(Brushes.Red, ptBase.X - 4, ptBase.Y - 4, 8, 8);
                objGraphics.FillEllipse(Brushes.Red, ptBase.X - 4 + sizeString.Width / 2, ptBase.Y - 4, 8, 8);
                objGraphics.FillEllipse(Brushes.Red, ptBase.X - 4 + sizeString.Width, ptBase.Y - 4, 8, 8);

                // Mitte
                objGraphics.FillEllipse(Brushes.Red, ptBase.X - 4, ptBase.Y - 4 + sizeString.Height / 2, 8, 8);
                objGraphics.FillEllipse(Brushes.Red, ptBase.X - 4 + sizeString.Width / 2, ptBase.Y - 4 + sizeString.Height / 2, 8, 8);
                objGraphics.FillEllipse(Brushes.Red, ptBase.X - 4 + sizeString.Width, ptBase.Y - 4 + sizeString.Height / 2, 8, 8);

                // Unten
                objGraphics.FillEllipse(Brushes.Red, ptBase.X - 4, ptBase.Y - 4 + sizeString.Height, 8, 8);
                objGraphics.FillEllipse(Brushes.Red, ptBase.X - 4 + sizeString.Width / 2, ptBase.Y - 4 + sizeString.Height, 8, 8);
                objGraphics.FillEllipse(Brushes.Red, ptBase.X - 4 + sizeString.Width, ptBase.Y - 4 + sizeString.Height, 8, 8);

                // Text mit Zeilenhöhe zeichnen
                ptBase.Offset(0, font.Height);
                objGraphics.DrawLine(
                    pen,
                    ptBase + new Size(0, font.Height), 
                    ptBase + new Size((int)sizeString.Width, 0) + new Size(0, font.Height));
                ptBase.Offset(0, font.Height);
                objGraphics.DrawString(strOut, font, Brushes.Blue, ptBase, format);
                objGraphics.DrawLine(
                    pen, ptBase + new Size(0, font.Height),
                    ptBase + new Size((int)sizeString.Width, 0) + new Size(0, font.Height));
                ptBase.Offset(0, font.Height);
                objGraphics.DrawString(strOut, font, Brushes.Blue, ptBase, format);
                objGraphics.DrawLine(
                    pen,
                    ptBase + new Size(0, font.Height),
                    ptBase + new Size((int)sizeString.Width, 0) + new Size(0, font.Height));
            }
        }

        /// <summary>
        /// Demo Textausgabe mit einem definierten Ausgabebereich.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowText(Graphics objGraphics)
        {
            // Zeige einen Text innerhalb eines definierten Rechtecks
            // Formatierungsangabe Standardeinstellung
            using (StringFormat format = new StringFormat())
            {
                format.Trimming = StringTrimming.EllipsisWord;

                // Rechteck definieren
                int iHeight = ClientSize.Height / 3 - 20;
                int iWidth = ClientSize.Width - 50;
                Rectangle rectText = new Rectangle(25, 20, iWidth, iHeight);
                // Text definieren
                string strOut = "Das ist ein Text, der mit GDI+ und seinen Methoden ";
                strOut += "formatiert wird. Dazu werden die Methode DrawString() und ";
                strOut += "die dazugehörenden weiteren Klassen verwendet. Mittels einer ";
                strOut += "speziellen Formatangabe, kann der Text einfach formatiert ";
                strOut += "werden. Diese Schriftprobe endet hier.";

                // Ausgabebereich und Text ausgeben
                objGraphics.DrawRectangle(Pens.Red, rectText);
                objGraphics.DrawString(
                    strOut,                               // Auszugebender Text
                    CwlbMainWindow.MainWindow.DemoFont,   // Zu verwendender Font
                    Brushes.DarkBlue,                     // Zu verwendender Brush
                    rectText,                             // Ausgabebereich
                    format);                              // Zu verwendendes Format

                // Formatierungsangaben zentrieren und Ausgaberechteck verschieben
                format.Alignment = StringAlignment.Center;
                format.LineAlignment = StringAlignment.Center;
                rectText.Offset(0, iHeight + 10);
                // Ausgabebereich und Text ausgeben
                objGraphics.DrawRectangle(Pens.Red, rectText);
                objGraphics.DrawString(
                    strOut, CwlbMainWindow.MainWindow.DemoFont, Brushes.DarkBlue, rectText, format);

                // Formatierungsangaben rechtsbündig und unten, Ausgaberechteck verschieben
                format.Alignment = StringAlignment.Far;
                format.LineAlignment = StringAlignment.Far;
                rectText.Offset(0, iHeight + 10);
                // Ausgabebereich und Text ausgeben
                objGraphics.DrawRectangle(Pens.Red, rectText);
                objGraphics.DrawString(
                    strOut, CwlbMainWindow.MainWindow.DemoFont, Brushes.DarkBlue, rectText, format);
            }
        }

        /// <summary>
        /// Demo installierte Schriften.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowFonts(Graphics objGraphics)
        {
            // Lokale Daten
            int iY = 20;

            // Schriftoptimierung umstellen
            objGraphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.SystemDefault;

            // Zeige alle Schriftfamilien
            foreach (FontFamily fontFamily in FontFamily.Families)
            {
                // Benutze Ausnahmebehandlung, für den Fall, dass eine
                // Schrift sich nicht mit Standardwerten bilden lässt
                try
                {
                    using (Font font = new Font(fontFamily, 12))
                    {
                        // Vertikale Schriften und Schriften, die höher sind als 30 Einheiten, überspringen
                        if (font.GdiVerticalFont || font.Height > 30)
                        {
                            continue;
                        }

                        // Ausgabe durchführen
                        objGraphics.DrawString(
                            fontFamily.ToString(),  // Textausgabe
                            font,                   // Zu verwendende Schrift
                            Brushes.Blue,           // Farbe
                            new Point(10, iY));     // Position

                        // Inkrement der Position und prüfen, ob noch etwas zu sehen ist
                        iY += font.Height;
                        if (iY > ClientSize.Height)
                        {
                            break;
                        }
                    }
                }
                catch
                {
                    // Alle Fehler fangen
                }
            }
        }

        /// <summary>
        /// Demo Schriften drehen.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowTextRotation(Graphics objGraphics)
        {
            // Schriftoptimierung umstellen
            objGraphics.TextRenderingHint = TextRenderingHint.ClearTypeGridFit;

            // Mittelpunkt für die Ausgabe berechenen
            Point ptCenter = new Point((int)(CwlbMainWindow.MainWindow.DemoFont.Height * 2.0F), 0);
            objGraphics.TranslateTransform(this.ClientSize.Width / 2, this.ClientSize.Height / 2);
            using (StringFormat format = new StringFormat())
            {
                // Winkel und Format bestimmen
                float fDegree = 22.5F;
                format.Alignment = StringAlignment.Near;
                format.LineAlignment = StringAlignment.Center;

                // Rotation durchführen
                for (int iCount = 0; iCount < 16; iCount++)
                {
                    // Rotation durchführen
                    objGraphics.RotateTransform(fDegree);
                    objGraphics.DrawString(
                        "Das ist eine Textprobe",               // Ausgabetext
                        CwlbMainWindow.MainWindow.DemoFont,     // Schrift
                        Brushes.Navy,                           // Farbe
                        ptCenter,                               // Basispunkt
                        format);                                // Format
                }
            }
        }

        /// <summary>
        /// Demo Schriften drehen.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowTextQuality(Graphics objGraphics)
        {
            // Lokale Daten
            using (Font font = new System.Drawing.Font(CwlbMainWindow.MainWindow.DemoFont.FontFamily, 24.0f))
            {
                // Textrendering verändern
                objGraphics.TextRenderingHint = TextRenderingHint.SingleBitPerPixel;
                objGraphics.DrawString("ABCdfg123 mit TextRenderingHint.SingleBitPerPixel",
                    font,
                    Brushes.Black,
                    new PointF(30, 90));
                objGraphics.TextRenderingHint = TextRenderingHint.ClearTypeGridFit;
                objGraphics.DrawString("ABCdfg123 mit TextRenderingHint.ClearTypeGridFit",
                    font,
                    Brushes.Black,
                    new PointF(30, 140));
                objGraphics.TextRenderingHint = TextRenderingHint.AntiAliasGridFit;
                objGraphics.DrawString("ABCdfg123 mit TextRenderingHint.AntiAliasGridFit",
                    font,
                    Brushes.Black,
                    new PointF(30, 190));
                objGraphics.TextContrast = 0;
                objGraphics.DrawString("ABCdfg123 mit TextContrast = 0",
                    font,
                    Brushes.DodgerBlue,
                    new PointF(30, 240));
                objGraphics.TextContrast = 4;
                objGraphics.DrawString("ABCdfg123 mit TextContrast = 4 (Standard)",
                    font,
                    Brushes.DodgerBlue,
                    new PointF(30, 290));
                objGraphics.TextContrast = 12;
                objGraphics.DrawString("ABCdfg123  mit TextContrast = 12",
                    font,
                    Brushes.DodgerBlue,
                    new PointF(30, 340));
            }
        }

        /// <summary>
        /// Demo Bitmap erstellen.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowCreatingImage(Graphics objGraphics)
        {
            // Lokale Daten
            int iHeight = 300;
            int iWidth = 400;

            // Bitmap-Objekt und Ausgabeobjekt dafür herstellen
            using (Font font = new Font("Comic Sans MS", 18.0f, FontStyle.Bold))
            using (Pen pen = new Pen(Color.RoyalBlue, 3))
            using (Bitmap objBitmap = new Bitmap(iWidth, iHeight))
            using (Graphics objGraphicsBitmap = Graphics.FromImage(objBitmap))
            {
                // Grafikkontext konfigurieren
                objGraphicsBitmap.TextRenderingHint = TextRenderingHint.AntiAlias;
                objGraphicsBitmap.SmoothingMode = SmoothingMode.AntiAlias;

                // Bitmap einfärben
                objGraphicsBitmap.Clear(Color.Yellow);

                // Rahmen zeichnen
                objGraphicsBitmap.DrawRectangle(Pens.Red, 0, 0, iWidth - 1, iHeight - 1);
                // Beschriften
                objGraphicsBitmap.DrawString("Selber hergestelltes Bitmap", font, Brushes.Blue, 20, 20);
                // Figuren zeichnen
                objGraphicsBitmap.FillRectangle(Brushes.Magenta, 50, 75, 200, 200);
                objGraphicsBitmap.DrawRectangle(Pens.Gold, 50, 75, 200, 200);
                objGraphicsBitmap.FillEllipse(Brushes.LightBlue, 75, 100, 150, 150);
                objGraphicsBitmap.DrawEllipse(pen, 75, 100, 150, 150);
                // Ein Bitmap in das Bitmap zeichnen
                objGraphicsBitmap.DrawImage(Properties.Resources.Texture, 130, 150);

                // Transparentbereich einstellen
                objBitmap.MakeTransparent(Color.Magenta);

                // Das Bitmap auf den Bildschirm projizieren
                GraphicsUnit enmGraphicsUnit = GraphicsUnit.Pixel;
                objGraphics.DrawImage(
                    objBitmap,                                    // Zu zeichnendes Bitmap
                    ClientRectangle,                              // Örtliche Definition des Ausgabeziels
                    objBitmap.GetBounds(ref enmGraphicsUnit),     // Definition des Ausschnitts des Bitmaps
                    enmGraphicsUnit);                             // Verwendete Grafikeinheit
            }
        }

        /// <summary>
        /// Demo Bitmapbild anzeigen.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowImage(Graphics objGraphics)
        {
            if (CwlbMainWindow.MainWindow.DemoImage == null)
            {
                MessageBox.Show(
                    "Sie müssen zuerst ein Bild laden. Benutzen Sie den Menübefehl Extras/Bild für Demo auswählen.", 
                    CwlbMainWindow.TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            // Bild anzeigen
            objGraphics.DrawImage(CwlbMainWindow.MainWindow.DemoImage, 0, 0, ClientSize.Width, ClientSize.Height);
        }

        /// <summary>
        /// Demo Erstellung von Farbveränderungen an Bildern
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowImageColored(Graphics objGraphics)
        {
            // Lokale Daten
            int iWidth = ClientSize.Width / 3;
            int iHeight = ClientSize.Height / 3;
            Rectangle rectTarget = new Rectangle(
                0, 0,
                iWidth, iHeight);
            Image imgColored = CwlbMainWindow.MainWindow.DemoImage;
            ImageAttributes imgAttribute = new ImageAttributes();

            // Farbmatrix für Graustufenbild
            float[][] aafGrayScale = { 
                new float[]{0.333f, 0.333f, 0.333f, 0.0f, 0.0f},
                new float[]{0.333f, 0.333f, 0.333f, 0.0f, 0.0f},
                new float[]{0.333f, 0.333f, 0.333f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 1.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 0.0f, 1.0f}};

            // Farbmatrix für Bild mit Rotanteil
            float[][] aafRed = { 
                new float[]{1.0f, 0.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 1.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 0.0f, 1.0f}};

            // Farbmatrix für Bild mit Grünanteil
            float[][] aafGreen = { 
                new float[]{0.0f, 0.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 1.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 1.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 0.0f, 1.0f}};

            // Farbmatrix für Bild mit Blauanteil
            float[][] aafBlue = { 
                new float[]{0.0f, 0.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 1.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 1.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 0.0f, 1.0f}};

            // Farbmatrix für unverändertes Bild
            float[][] aafNormal = { 
                new float[]{1.0f, 0.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 1.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 1.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 1.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 0.0f, 1.0f}};

            // Farbmatrix für Farbinversion
            float[][] aafInversion = { 
                new float[]{-0.99f, 0.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, -0.99f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, -0.99f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 1.0f, 0.0f},
                new float[]{0.99f, 0.99f, 0.99f, 0.0f, 1.0f}};

            // Farbmatrix für Farbinversion
            float[][] aafSepia = { 
                new float[] {0.393f, 0.349f, 0.272f, 0, 0},
                new float[] {0.769f, 0.686f, 0.534f, 0, 0},
                new float[] {0.189f, 0.168f, 0.131f, 0, 0},
                new float[] {     0,      0,      0, 1, 0},
                new float[] {     0,      0,      0, 0, 1}};

            // Farbmatrix für helleres Bild
            float[][] aafMoreBrightness = { 
                new float[]{1.0f, 0.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 1.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 1.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 1.0f, 0.0f},
                new float[]{0.3f, 0.3f, 0.3f, 0.0f, 1.0f}};

            // Farbmatrix für dunkleres Bild
            float[][] aafLessBrightness = { 
                new float[]{1.0f, 0.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 1.0f, 0.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 1.0f, 0.0f, 0.0f},
                new float[]{0.0f, 0.0f, 0.0f, 1.0f, 0.0f},
                new float[]{-0.1f, -0.1f, -0.1f, 0.0f, 1.0f}};

            // Rotes Bild anzeigen
            ColorMatrix colorMatrix = new ColorMatrix(aafRed);
            imgAttribute.SetColorMatrix(colorMatrix, ColorMatrixFlag.Default, ColorAdjustType.Bitmap);
            objGraphics.DrawImage(imgColored, rectTarget,
                0, 0, imgColored.Width, imgColored.Height,
                GraphicsUnit.Pixel, imgAttribute);

            // Grünes Bild anzeigen
            colorMatrix = new ColorMatrix(aafGreen);
            imgAttribute.SetColorMatrix(colorMatrix, ColorMatrixFlag.Default, ColorAdjustType.Bitmap);
            rectTarget.Offset(iWidth, 0);
            objGraphics.DrawImage(imgColored, rectTarget,
                0, 0, imgColored.Width, imgColored.Height,
                GraphicsUnit.Pixel, imgAttribute);

            // Blaues Bild anzeigen
            colorMatrix = new ColorMatrix(aafBlue);
            imgAttribute.SetColorMatrix(colorMatrix, ColorMatrixFlag.Default, ColorAdjustType.Bitmap);
            rectTarget.Offset(iWidth, 0);
            objGraphics.DrawImage(imgColored, rectTarget,
                0, 0, imgColored.Width, imgColored.Height,
                GraphicsUnit.Pixel, imgAttribute);

            // Graustufenbild anzeigen
            colorMatrix = new ColorMatrix(aafGrayScale);
            rectTarget.Offset(iWidth * -2, iHeight);
            imgAttribute.SetColorMatrix(colorMatrix, ColorMatrixFlag.Default, ColorAdjustType.Bitmap);
            objGraphics.DrawImage(imgColored, rectTarget,
                0, 0, imgColored.Width, imgColored.Height,
                GraphicsUnit.Pixel, imgAttribute);

            // Normales Bild anzeigen
            colorMatrix = new ColorMatrix(aafNormal);
            rectTarget.Offset(iWidth, 0);
            imgAttribute.SetColorMatrix(colorMatrix, ColorMatrixFlag.Default, ColorAdjustType.Bitmap);
            objGraphics.DrawImage(imgColored, rectTarget,
                0, 0, imgColored.Width, imgColored.Height,
                GraphicsUnit.Pixel, imgAttribute);

            // Bild mit Farbinversion anzeigen
            colorMatrix = new ColorMatrix(aafInversion);
            rectTarget.Offset(iWidth, 0);
            imgAttribute.SetColorMatrix(colorMatrix, ColorMatrixFlag.Default, ColorAdjustType.Bitmap);
            objGraphics.DrawImage(imgColored, rectTarget,
                0, 0, imgColored.Width, imgColored.Height,
                GraphicsUnit.Pixel, imgAttribute);

            // Graustufenbild anzeigen
            colorMatrix = new ColorMatrix(aafSepia);
            rectTarget.Offset(iWidth * -2, iHeight);
            imgAttribute.SetColorMatrix(colorMatrix, ColorMatrixFlag.Default, ColorAdjustType.Bitmap);
            objGraphics.DrawImage(imgColored, rectTarget,
                0, 0, imgColored.Width, imgColored.Height,
                GraphicsUnit.Pixel, imgAttribute);

            // Normales Bild mit mehr Helligeit anzeigen
            colorMatrix = new ColorMatrix(aafMoreBrightness);
            rectTarget.Offset(iWidth, 0);
            imgAttribute.SetColorMatrix(colorMatrix, ColorMatrixFlag.Default, ColorAdjustType.Bitmap);
            objGraphics.DrawImage(imgColored, rectTarget,
                0, 0, imgColored.Width, imgColored.Height,
                GraphicsUnit.Pixel, imgAttribute);

            // Normales Bild mit weniger Helligeit anzeigen
            colorMatrix = new ColorMatrix(aafLessBrightness);
            rectTarget.Offset(iWidth, 0);
            imgAttribute.SetColorMatrix(colorMatrix, ColorMatrixFlag.Default, ColorAdjustType.Bitmap);
            objGraphics.DrawImage(imgColored, rectTarget,
                0, 0, imgColored.Width, imgColored.Height,
                GraphicsUnit.Pixel, imgAttribute);
        }

        /// <summary>
        /// Demo Bildschirmkopie erstellen.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowScreenCopy(Graphics objGraphics)
        {
            // Lokale Daten
            ImageFormat enmFormat = ImageFormat.Jpeg;           // Bildformat für das Speichern
            Size objSize = Screen.PrimaryScreen.Bounds.Size;    // Größe des aktuellen Bildschirms erfragen

            // Bitmap-Objekt in der erforderlichen Größe herstellen
            Bitmap objBitmap = new Bitmap(objSize.Width, objSize.Height);
            // Graphics-Objekt für das Zeichnen in das Bitmap erstellen
            using (Graphics objBitmapGraphics = Graphics.FromImage(objBitmap))
            {
                // Bildschirminhalt kopieren
                objBitmapGraphics.CopyFromScreen(
                    new Point(0, 0), 
                    new Point(0, 0),
                    new Size(objSize.Width, objSize.Height));
            }

            // Dialogfeld zum Speichern des Bildes erstellen und konfigurieren
            SaveFileDialog dlgSave = new SaveFileDialog();
            dlgSave.Filter = "File interchange format|*.jpg";
            dlgSave.Filter += "|Bitmap format|*.bmp";
            dlgSave.Filter += "|Graphics interchange format|*.gif";
            dlgSave.Filter += "|Enhanced meta file|*.emf";
            dlgSave.Filter += "|Portable network grafik|*.png";
            dlgSave.Filter += "|Tagged image file format|*.tiff";
            dlgSave.FilterIndex = 3;

            // Dialogfeld darstellen und auswerten
            if (dlgSave.ShowDialog() == DialogResult.OK)
            {

                // Bild speichern
                switch (dlgSave.FilterIndex)
                {
                    case 1:
                        enmFormat = ImageFormat.Bmp;
                        break;
                    case 2:
                        enmFormat = ImageFormat.Gif;
                        break;
                    case 3:
                        enmFormat = ImageFormat.Jpeg;
                        break;
                    case 4:
                        enmFormat = ImageFormat.Png;
                        break;
                    default:
                        enmFormat = ImageFormat.Bmp;
                        break;
                }
                objBitmap.Save(dlgSave.FileName, enmFormat);

                // Erstellen des Vorschaubildes. Die Methode benötigt zwingend die Angabe einer
                // Delegatmethode, was hier mit einer anonymen Version erledigt ist.
                Image imgThumbnail = objBitmap.GetThumbnailImage(
                    75, 56, 
                    delegate() { return false; },
                    IntPtr.Zero);
                // Speichern der Abbildung
                string strFileName = string.Format(
                    "{0}\\{1}Thumbnail{2}",
                    Path.GetDirectoryName(dlgSave.FileName),
                    Path.GetFileNameWithoutExtension(dlgSave.FileName),
                    Path.GetExtension(dlgSave.FileName));
                imgThumbnail.Save(strFileName, enmFormat);
                imgThumbnail.Dispose();

                // Prüfen, ob das Bild auch angezeigt werden soll
                if (MessageBox.Show(
                    "Möchten Sie das Bild jetzt anzeigen?\r\nDadurch wird das Standardbild der Demo verändert.",
                    CwlbMainWindow.TITLE, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    // Bild als Standardbild übernehmen. Achtung: In diesem Fall darf die Bitmap
                    // nicht zerstört werden, da sie sonst nicht mehr angezeigt werden kann
                    CwlbMainWindow.MainWindow.DemoImage = objBitmap;
                    // Auswahl einstellen
                    _enmDemoType = EwlbDemoType.BitmapShow;
                    Invalidate();
                    return;
                }
            }

            // Bitmap freigeben, Auswahl zurückstellen und Neuzeichnen veranlassen
            objBitmap.Dispose();
            _enmDemoType = _enmKeep;
            Invalidate();
        }

        /// <summary>
        /// Demo Regionen erstellen.
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowRegionComplement(Graphics objGraphics)
        {
            // Lokale Daten
            int iFontOffsetY = 120;

            // Lokale Ressourcen erstellen 
            using (SolidBrush brush = new SolidBrush(Color.Blue))
            using (Pen penBlack = new Pen(Color.Black, 2))
            using (Pen penRed = new Pen(Color.Red, 2))
            using (Font font = new Font("Arial", 14))
            {
                // Complement von zwei Rechtecken
                // ------------------------------------------------------
                // Erstellen eines ersten Rechtecks in schwarzer Farbe
                Rectangle rectOrigin = new Rectangle(20, 20, 100, 100);
                objGraphics.DrawRectangle(penBlack, rectOrigin);
                // Erstellen eines zweiten Rechtecks in roter Farbe
                Rectangle rectCombination = new Rectangle(90, 30, 100, 100);
                objGraphics.DrawRectangle(penRed, rectCombination);

                // Erstellen der Region mit dem ursprünglichen Rechteck
                using (Region rgnRegion = new Region(rectOrigin))
                {
                    // Erstellen der komplementär Region mit dem Complementär-
                    // Rechteck zum Originalrechteck
                    rgnRegion.Complement(rectCombination);
                    // Einfärben der gebildeten Region
                    objGraphics.FillRegion(brush, rgnRegion);
                    // Beschriften der Region
                    objGraphics.DrawString("Complement()", font, brush, rectOrigin.X, rectOrigin.Y + iFontOffsetY);
                }

                // Exclude von zwei Rechtecken
                // ------------------------------------------------------
                // Erstellen eines ersten Rechtecks in schwarzer Farbe
                rectOrigin.Offset(200, 0); // = new Rectangle(20, 200, 100, 100);
                objGraphics.DrawRectangle(penBlack, rectOrigin);
                // Erstellen eines zweiten Rechtecks in roter Farbe
                rectCombination.Offset(200, 0);
                objGraphics.DrawRectangle(penRed, rectCombination);

                // Erstellen der Region mit dem ursprünglichen Rechteck
                using (Region rgnRegion = new Region(rectOrigin))
                {
                    // Erstellen der exklusiven Region mit dem 
                    // Rechteck zum Originalrechteck
                    rgnRegion.Exclude(rectCombination);
                    // Einfärben der gebildeten Region
                    objGraphics.FillRegion(brush, rgnRegion);
                    // Beschriften der Region
                    objGraphics.DrawString("Exclude()", font, brush, rectOrigin.X, rectOrigin.Y + iFontOffsetY);
                }

                // Intersect von zwei Rechtecken
                // ------------------------------------------------------
                // Erstellen eines ersten Rechtecks in schwarzer Farbe
                rectOrigin.Offset(200, 0);
                objGraphics.DrawRectangle(penBlack, rectOrigin);
                // Erstellen eines zweiten Rechtecks in roter Farbe
                rectCombination.Offset(200, 0);
                objGraphics.DrawRectangle(penRed, rectCombination);

                // Erstellen der Region mit dem ursprünglichen Rechteck
                using (Region rgnRegion = new Region(rectOrigin))
                {
                    // Erstellen der exklusiven Region mit dem 
                    // Rechteck zum Originalrechteck
                    rgnRegion.Intersect(rectCombination);
                    // Einfärben der gebildeten Region
                    objGraphics.FillRegion(brush, rgnRegion);
                    // Beschriften der Region
                    objGraphics.DrawString("Intersect()", font, brush, rectOrigin.X, rectOrigin.Y + iFontOffsetY);
                }

                // Union von zwei Rechtecken
                // ------------------------------------------------------
                // Erstellen eines ersten Rechtecks in schwarzer Farbe
                rectOrigin.Offset(-400, 200);
                objGraphics.DrawRectangle(penBlack, rectOrigin);
                // Erstellen eines zweiten Rechtecks in roter Farbe
                rectCombination.Offset(-400, 200);
                objGraphics.DrawRectangle(penRed, rectCombination);

                // Erstellen der Region mit dem ursprünglichen Rechteck
                using (Region rgnRegion = new Region(rectOrigin))
                {
                    // Erstellen der exklusiven Region mit dem 
                    // Rechteck zum Originalrechteck
                    rgnRegion.Union(rectCombination);
                    // Einfärben der gebildeten Region
                    objGraphics.FillRegion(brush, rgnRegion);
                    // Beschriften der Region
                    objGraphics.DrawString("Union()", font, brush, rectOrigin.X, rectOrigin.Y + iFontOffsetY);
                }

                // Xor von zwei Rechtecken
                // ------------------------------------------------------
                // Erstellen eines ersten Rechtecks in schwarzer Farbe
                rectOrigin.Offset(200, 0);
                objGraphics.DrawRectangle(penBlack, rectOrigin);
                // Erstellen eines zweiten Rechtecks in roter Farbe
                rectCombination.Offset(200, 0);
                objGraphics.DrawRectangle(penRed, rectCombination);

                // Erstellen der Region mit dem ursprünglichen Rechteck
                using (Region rgnRegion = new Region(rectOrigin))
                {
                    // Erstellen der exklusiven Region mit dem 
                    // Rechteck zum Originalrechteck
                    rgnRegion.Xor(rectCombination);
                    // Einfärben der gebildeten Region
                    objGraphics.FillRegion(brush, rgnRegion);
                    // Beschriften der Region
                    objGraphics.DrawString("Xor()", font, brush, rectOrigin.X, rectOrigin.Y + iFontOffsetY);
                }
            }
        }

        /// <summary>
        /// Demo Clipping und Regionen nutzen..
        /// </summary>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowCombination(Graphics objGraphics)
        {
            using (Font fontSigns = new Font("Arial", 48.0f, FontStyle.Bold))
            using (Font fontText = new Font("Arial", 12.0f))
            {
                // Bild zeigen
                objGraphics.DrawImage(Properties.Resources.Papagei, 20, 20, 200, 200);
                // Pluszeichen
                objGraphics.DrawString("+", fontSigns, Brushes.Black, 220, 80);
                // Region Zeichnen
                Rectangle rectClip = new Rectangle(280, 20, 200, 200);
                objGraphics.FillRectangle(Brushes.Red, rectClip);
                objGraphics.FillEllipse(SystemBrushes.Control, rectClip);
                // Gleichzeichen
                objGraphics.DrawString("=", fontSigns, Brushes.Black, 480, 80);

                // Erstellen der Region mit der notwendigen Ellipse
                using(GraphicsPath pathEllipse = new GraphicsPath())
                using (Region rgnRegion = new Region())
                {
                    // Pfad definieren
                    pathEllipse.StartFigure();
                    pathEllipse.AddEllipse(540, 20, 200, 200);
                    // Erstellen der Region
                    rgnRegion.Intersect(pathEllipse);

                    // Aktivieren der Region
                    objGraphics.Clip = rgnRegion;
                    objGraphics.DrawImage(Properties.Resources.Papagei, 540, 20, 200, 200);

                    // Region ausschalten und Umrandung zeichnen
                    objGraphics.ResetClip();
                    objGraphics.DrawEllipse(Pens.DarkGray, 540, 20, 200, 200);
                }

                // Text definieren und ausgeben
                string strOut = "Lorem ipsum putent accommodare vix ad, tempor similique cum ne," +
                                "tritani repudiandae vix eu. Ea sit inani percipitur eloquentiam," +
                                " vel cu iudico elaboraret accommodare. Possit eligendi ponderum has ut mea.";
                // Format definieren
                using (StringFormat format = new StringFormat())
                {
                    format.Alignment = StringAlignment.Center;
                    format.LineAlignment = StringAlignment.Center;

                    // Text ausgeben
                    objGraphics.DrawString(strOut, fontText, Brushes.Navy, new Rectangle(20, 260, 200, 200), format);
                    // Pluszeichen
                    objGraphics.DrawString("+", fontSigns, Brushes.Black, 220, 320);
                    // Region Zeichnen
                    rectClip = new Rectangle(280, 260, 200, 200);
                    objGraphics.FillRectangle(Brushes.Red, rectClip);
                    objGraphics.FillEllipse(SystemBrushes.Control, 300, 280, 160, 160);
                    // Gleichzeichen
                    objGraphics.DrawString("=", fontSigns, Brushes.Black, 480, 320);

                    // Erstellen der Region mit der notwendigen Ellipse
                    using (GraphicsPath pathEllipse = new GraphicsPath())
                    using (Region rgnRegion = new Region())
                    {
                        // Pfad definieren
                        pathEllipse.StartFigure();
                        pathEllipse.AddEllipse(560, 280, 160, 160);
                        // Erstellen der Region
                        rgnRegion.Intersect(pathEllipse);

                        // Aktivieren der Region
                        objGraphics.Clip = rgnRegion;
                        objGraphics.DrawString(strOut, fontText, Brushes.Navy, new Rectangle(560, 260, 200, 200), format);
                    }
                }
            }
        }
    }
}
