using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Printing;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Reprsentiert die Demo fr die Handhabung des Koordiantensystems mti GDI+.
    /// </summary>
    public partial class CmdiCoordinateSystem : Form
    {
        // Definiert eine Dreicksfigur
        GraphicsPath _objTriangle = new GraphicsPath();
        Pen _penSin = new Pen(Color.Green, 2.5f);

        /// <summary>
        /// Initialisiert ein neues Objekt der Klasse CmdiCoordinateSystem.
        /// </summary>
        public CmdiCoordinateSystem()
        {
            InitializeComponent();

            // Dreieck definieren
            _objTriangle.StartFigure();
            _objTriangle.AddLine(-25, -25, 0, 25);
            _objTriangle.AddLine(0, 25, 25, -25);
            _objTriangle.CloseFigure();
        }

        /// <summary>
        /// Einstellen der Tracker auf die Grsse des Zeichnungspanles
        /// </summary>
        private void SetTrackers()
        {
            // Einstellungen an den Trackern vornehmen
            _trkVertical.Maximum = _pnlDraw.ClientSize.Height;
            _trkHorizontal.Maximum = _pnlDraw.ClientSize.Width;
            _trkVertical.TickFrequency = (int)_trkVertical.Maximum / 100;
            _trkHorizontal.TickFrequency = (int)_trkHorizontal.Maximum / 100;
            _pnlDraw.Invalidate();
        }

        /// <summary>
        /// Demo fr die Transformationen von GDI+.
        /// </summary>
        /// <remarks>
        /// Die Methode wird von der Drucksteuerung und der Bildschirmsteuerung
        /// aufgerufen. deshalb ist sie separiert und nicht im Paint-Ereignishandler
        /// direkt implementiert.
        /// </remarks>
        /// <param name="objGraphics">Grafikkontext.</param>
        private void ShowGraphics(Graphics objGraphics)
        {
            // Lokale Daten
            GraphicsState gs = null;
            Matrix mtxDraw = null;
            Matrix mtxText = null;
            Matrix mtxClone = null;
            int iTextPos = 30;
            StringFormat format = new StringFormat();
            format.LineAlignment = StringAlignment.Center;
            format.Alignment = StringAlignment.Near;

            // Einstellungen des Grafikkontexts
            objGraphics.CompositingQuality = CompositingQuality.HighQuality;
            objGraphics.SmoothingMode = SmoothingMode.AntiAlias;

            // Schreibe den Titeltext
            this.Text = string.Format(
                "Ruler X: {0}, Ruler Y: {1}",
                _trkHorizontal.Value, _trkVertical.Value);

            // Prfe Einheiten
            if (_chkUnitMm.Checked)
            {
                objGraphics.PageUnit = GraphicsUnit.Millimeter;
            }
            // Spiegele Y Achse
            if (_chkMatrix.Checked)
            {
                mtxDraw = new Matrix(1, 0, 0, -1, 0, _pnlDraw.ClientRectangle.Height);
                objGraphics.Transform = mtxDraw;
            }

            // Nullpunktverschiebung
            if (_chkTranslateTransform.Checked)
            {
                objGraphics.TranslateTransform(_trkHorizontal.Value, _trkVertical.Value);
            }

            // Rotation
            objGraphics.RotateTransform((float)_nudDegree.Value);

            // Skalierung
            objGraphics.ScaleTransform(Convert.ToSingle(_nudScaleX.Value), Convert.ToSingle(_nudScaleY.Value));

            // Zeichne Koordinatenkreuz
            objGraphics.DrawLine(
                Pens.Red,
                0, -_pnlDraw.ClientRectangle.Height,
                0, _pnlDraw.ClientRectangle.Height);
            objGraphics.DrawLine(
                Pens.Red,
                -_pnlDraw.ClientSize.Width, 0,
                _pnlDraw.ClientSize.Width, 0);

            // Eigentliche Erstellung der Zeichnung
            // Rechteck zeichnen
            objGraphics.FillRectangle(Brushes.Red, -25, -25, 50, 50);
            // Dreieck zeichnen
            objGraphics.FillPath(Brushes.Gold, _objTriangle);
            // Text anzeigen
            if (_chkTextTransformation.Checked)
            {

                // Erstelle neue Matrix
                gs = objGraphics.Save();
                mtxText = new Matrix(1, 0, 0, -1, 0, CwlbMainWindow.MainWindow.DemoFont.Height);
                mtxClone = mtxDraw.Clone();
                mtxClone.Multiply(mtxText);
                objGraphics.Transform = mtxClone;


                if (_chkTranslateTransform.Checked)
                {
                    objGraphics.TranslateTransform(_trkHorizontal.Value, -_trkVertical.Value + CwlbMainWindow.MainWindow.DemoFont.Height);
                    objGraphics.RotateTransform((float)_nudDegree.Value * -1);
                }

                // Vorzeichen fr Textausgabe invertieren
                iTextPos *= -1;
            }
            objGraphics.DrawString("Quadrat mit 50 Einheiten Seitenlnge", CwlbMainWindow.MainWindow.DemoFont, Brushes.Red, 30, iTextPos, format);
            if (_chkTextTransformation.Checked)
            {
                objGraphics.Restore(gs);
            }

            // Sinus berechnen und anzeigen
            float fPrev = 0;
            float fFactor = 50;
            List<PointF> cptSinus = new List<PointF>();
            for (float f = 0.0F; f <= 2 * Math.PI; f += 0.01F)
            {
                cptSinus.Add(new PointF(fPrev * fFactor, Convert.ToSingle(Math.Sin(fPrev) * fFactor)));
                fPrev = f;
            }
            objGraphics.DrawCurve(_penSin, cptSinus.ToArray());
        }

        /// <summary>
        /// Ereignishandler Formular wird angezeigt
        /// </summary>
        private void CDlgCoordSystem_Load(object sender, EventArgs e)
        {
            // Tracker einstellen
            SetTrackers();
        }

        /// <summary>
        /// Ereignishandler Grsse des Formulars wurde verndert.
        /// </summary>
        private void CDlgCoordSystem_SizeChanged(object sender, EventArgs e)
        {
            // Tracker einstellen
            SetTrackers();
        }

        /// <summary>
        /// Ereignishandler Zeichnen des Panels
        /// </summary>
        private void _pnlDraw_Paint(object sender, PaintEventArgs e)
        {
            // Grafik anzeigen
            ShowGraphics(e.Graphics);
        }

        /// <summary>
        /// Ereignishandler horizontaler Tracker wurde bewegt.
        /// </summary>
        private void _trkHorizontal_ValueChanged(object sender, EventArgs e)
        {
            // Zeichungspanel neu zeichnen
            _pnlDraw.Invalidate();
        }

        /// <summary>
        /// Ereignishandler vertikaler Tracker wurde bewegt.
        /// </summary>
        private void _trkVertical_ValueChanged(object sender, EventArgs e)
        {
            // Zeichungspanel neu zeichnen
            _pnlDraw.Invalidate();
        }

        /// <summary>
        /// Ereignishandler Checkbox Translate Transform wurde bettigt
        /// </summary>
        private void _chkTranslateTransform_CheckedChanged(object sender, EventArgs e)
        {
            // Zeichungspanel neu zeichnen
            _pnlDraw.Invalidate();

            // Tracker einstellen
            if (_chkTranslateTransform.Checked)
            {
                _trkHorizontal.Enabled = true;
                _trkHorizontal.Minimum = 0;
                _trkHorizontal.Maximum = _pnlDraw.ClientSize.Width;
                _trkHorizontal.Value = _pnlDraw.ClientSize.Width / 2;
                _trkVertical.Enabled = true;
                _trkVertical.Minimum = 0;
                _trkVertical.Maximum = _pnlDraw.ClientSize.Height;
                _trkVertical.Value = _pnlDraw.ClientSize.Height / 2;
            }
            else
            {
                _trkHorizontal.Enabled = false;
                _trkHorizontal.Minimum = 0;
                _trkHorizontal.Maximum = 100;
                _trkHorizontal.Value = 0;
                _trkVertical.Enabled = false;
                _trkVertical.Minimum = 0;
                _trkVertical.Maximum = 100;
                _trkVertical.Value = 0;
            }
        }

        /// <summary>
        /// Ereignishandler Checkbox Benutze Matrix wurde bettigt
        /// </summary>
        private void _chkMatrix_CheckedChanged(object sender, EventArgs e)
        {
            // Prfe Status Text matrix
            if (_chkMatrix.Checked)
            {
                _chkTextTransformation.Enabled = true;
                _chkTextTransformation.Checked = true;
            }
            else
            {
                _chkTextTransformation.Checked = false;
                _chkTextTransformation.Enabled = false;
            }
            _pnlDraw.Invalidate();
        }

        /// <summary>
        /// Ereignishandler Checkbox Text Transformation wurde bettigt
        /// </summary>
        private void _chkTextTransformation_CheckedChanged(object sender, EventArgs e)
        {
            // Zeichungspanel neu zeichnen
            _pnlDraw.Invalidate();
        }

        /// <summary>
        /// Ereignishandler Verwende Einheiten mm wurde bettigt
        /// </summary>
        private void _chkUnitMm_CheckedChanged(object sender, EventArgs e)
        {
            // Zeichungspanel neu zeichnen
            _pnlDraw.Invalidate();
        }

        /// <summary>
        /// Ereignishandler Winkel wurde verndert
        /// </summary>
        private void _nudDegree_ValueChanged(object sender, EventArgs e)
        {
            // Zeichungspanel neu zeichnen
            _pnlDraw.Invalidate();
        }

        /// <summary>
        /// Ereignishandler Skalierung X wurde verndert
        /// </summary>
        private void _nudScaleX_ValueChanged(object sender, EventArgs e)
        {
            // Zeichungspanel neu zeichnen
            _pnlDraw.Invalidate();
        }

        /// <summary>
        /// Ereignishandler Skalierung Y wurde verndert
        /// </summary>
        private void _nudScaleY_ValueChanged(object sender, EventArgs e)
        {
            // Zeichungspanel neu zeichnen
            _pnlDraw.Invalidate();
        }

        /// <summary>
        /// Ereignishandler Knopf Skalierung zurcksetzen
        /// </summary>
        private void _btnScaleReset_Click(object sender, EventArgs e)
        {
            // Zurcksetzen der Skalierung
            _nudScaleX.Value = 1.0M;
            _nudScaleY.Value = 1.0M;
            _pnlDraw.Invalidate();
        }

        /// <summary>
        /// Ereignishandler Knopf Schliessen wurde bettigt
        /// </summary>
        private void _btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Ereignishandler Knopf Drucken
        /// </summary>
        private void _btnPrint_Click(object sender, EventArgs e)
        {

            // Erstellen des Druckdialogs
            PrintDialog dlgPrint = new PrintDialog();
            dlgPrint.PrinterSettings = new PrinterSettings();

            // Zeige den Druckdialog
            if (dlgPrint.ShowDialog() == DialogResult.OK)
            {

                // Erstelle das Dokument zum Drucken
                PrintDocument docPrint = new PrintDocument();
                // Ereignishandler zum Drucken regisitrieren
                docPrint.PrintPage += new PrintPageEventHandler(docPrint_PrintPage);
                // Einstellungen vornehmen
                docPrint.DocumentName = "Test Printing";
                docPrint.PrinterSettings = dlgPrint.PrinterSettings;

                // Drucken Starten
                docPrint.Print();
            }
        }

        /// <summary>
        /// Ereignishandler Drucken des Dokuments
        /// </summary>
        private void docPrint_PrintPage(object sender, PrintPageEventArgs e)
        {
            // Zeichnungsmethode aufrufen
            this.ShowGraphics(e.Graphics);
        }
    }
}