using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Diagnostics;

namespace WeroSoft.Samples
{
    public partial class CmdiDraw : Form
    {
        public enum EDrawFigure
        {
            Show,
            Line,
            Rectangle,
            RectFilled,
            Ellipse,
            EllipseFilled
        }

        // Datenmodell
        private CwlDataModel _objModel = new CwlDataModel();
        // Bitmap fr die Doublebuffering Methode
        private Bitmap _bmpBuffer = null;
        // Zeichnungsflag (Rubberbanding)
        private bool _bRubberband = false;
        // Startpunkt
        private Point _ptStart;
        // Linienart
        private DashStyle _enmDashStyle = DashStyle.Solid;
        // Liniendicke
        private int _iLineWidth = 1;
        // Linienfarbe
        private Color _colLine = Color.Black;
        // Flchenfarbe
        private Color _colArea = Color.LightGray;

        // Aktueller Zeichnstatus mit Optimierungsvariabel
        public EDrawFigure CurrentShape { get; set; }
        public EDrawFigure OldShape { get; set; }

        /// <summary>
        /// Standarddkonstruktor
        /// </summary>
        public CmdiDraw()
        {
            InitializeComponent();

            // Eigene Daten initialisieren
            CurrentShape = EDrawFigure.Show;
            OldShape = EDrawFigure.Line;
        }

        /// <summary>
        /// Ereignishandler: Laden des Formulars
        /// </summary>
        private void CDlgMenuDemo_Load(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Ereignisbehandlung fr das Zeichnen der Menuitems Liniendicke
        /// </summary>
        private void MenuLineWidth_Paint(object sender, PaintEventArgs e)
        {
            // Breite des Menbalkens bestimmen
            int iCheckSize = (int)(_mnuMain.ImageScalingSize.Width * 1.5);
            // Dicke der Linie aus Eigenschaft Tag bernehmen
            Pen pen = new Pen(Color.Blue,
                Convert.ToSingle(((ToolStripItem)sender).Tag));
            // Linie zeichnen
            e.Graphics.DrawLine(pen,
                iCheckSize + 4,
                e.ClipRectangle.Height / 2,
                e.ClipRectangle.Width - 6,
                e.ClipRectangle.Height / 2);
            // Ressource freigeben
            pen.Dispose();
        }

        /// <summary>
        /// Ereignisbehandlung Zeichnen des Menpunktes Strichart.
        /// </summary>
        private void MenuLineType_Paint(object sender, PaintEventArgs e)
        {
            // Breite des Menbalkens bestimmen
            int iCheckSize = (int)(_mnuMain.ImageScalingSize.Width * 1.5);
            // Pen definieren
            Pen pen = new Pen(Color.Blue, 2.0f);
            // Text aus Menitem bernehmen
            switch (((ToolStripMenuItem)sender).Text)
            {
                case "2":
                    pen.DashStyle = DashStyle.Dash;
                    _enmDashStyle = DashStyle.Dash;
                    break;
                case "3":
                    pen.DashStyle = DashStyle.Dot;
                    _enmDashStyle = DashStyle.Dot;
                    break;
                case "4":
                    pen.DashStyle = DashStyle.DashDot;
                    _enmDashStyle = DashStyle.DashDot;
                    break;
                case "5":
                    pen.DashStyle = DashStyle.DashDotDot;
                    _enmDashStyle = DashStyle.Dash;
                    break;
                case "1":
                default:
                    pen.DashStyle = DashStyle.Solid;
                    _enmDashStyle = DashStyle.Solid;
                    break;
            }

            // Linie zeichnen
            e.Graphics.DrawLine(pen,
                iCheckSize + 4,
                e.ClipRectangle.Height / 2,
                e.ClipRectangle.Width - 6,
                e.ClipRectangle.Height / 2);
            // Ressource freigeben
            pen.Dispose();
        }



        /// <summary>
        /// Ereignishandler Knopf Linienfarbe wurde bettigt
        /// </summary>
        private void _btnDrawLineColor_Click(object sender, EventArgs e)
        {
            // Farbdialog erzeugen
            ColorDialog dlgColor = new ColorDialog();

            // Dialog initialisieren
            dlgColor.Color = _colLine;
            dlgColor.FullOpen = false;
            dlgColor.AllowFullOpen = false;

            // Dialogfeld anzeigen
            if (dlgColor.ShowDialog() == DialogResult.OK)
            {
                // Farbe bernehmen
                _colLine = dlgColor.Color;
            }
        }

        /// <summary>
        /// Ereignishandler Knopf Flchenfarbe wurde bettigt
        /// </summary>
        private void _btnDrawAreaColor_Click(object sender, EventArgs e)
        {
            // Farbdialog erzeugen
            ColorDialog dlgColor = new ColorDialog();

            // Dialog initialisieren
            dlgColor.Color = _colArea;
            dlgColor.FullOpen = true;

            // Dialogfeld anzeigen
            if (dlgColor.ShowDialog() == DialogResult.OK)
            {
                // Farbe bernehmen
                _colArea = dlgColor.Color;
            }
        }

        /// <summary>
        /// Ereignishandler Knopf Linienfarbe zeichnen
        /// </summary>
        private void _btnDrawColor_Paint(object sender, PaintEventArgs e)
        {

            // lokale Daten
            SolidBrush objBrush = null;
            Rectangle rectColor = new Rectangle();

            // initialisierung
            rectColor.X = 4;
            rectColor.Y = 4;
            rectColor.Width = 14;
            rectColor.Height = 14;

            //// Prfe Knopf Linienfarbe
            //if (sender == _btnDrawLineColor)
            //{
            //    objBrush = new SolidBrush(_colLine);
            //}
            //else if (sender == _btnDrawAreaColor)
            //{
            //    // Prfe Knopf Flchenfarbe
            //    objBrush = new SolidBrush(_colArea);
            //}
            //else
            //{
            //    objBrush = new SolidBrush(Color.Black);
            //}

            //// Zeichne Knopffarbe
            //_btnDrawLineColor.DisplayStyle = ToolStripItemDisplayStyle.None;
            e.Graphics.FillRectangle(objBrush, rectColor);
            e.Graphics.DrawRectangle(Pens.Blue, rectColor);

            // Ressourcen freigeben
            objBrush.Dispose();
        }



        /// <summary>
        /// Ereignishandler fr Click-Ereignis auf dem Menpunkt Liniendicke.
        /// </summary>
        /// <param name="sender">Menpunkt der das Click auslste.</param>
        /// <param name="e">Nicht weiter bentigtes Ereignisargument.</param>
        private void _btnDrawLineWidth_Click(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Ereignishandler Menuitem Auswhlen ausgewhlt
        /// </summary>
        private void _mnuDrawShow_Click(object sender, EventArgs e)
        {
            // Zeichnungsmodus setzen
            CurrentShape = EDrawFigure.Show;
        }

        /// <summary>
        /// Ereignishandler Menuitem Linie ausgewhlt
        /// </summary>
        private void _mnuDrawLine_Click(object sender, EventArgs e)
        {
            // Zeichnungsmodus setzen
            CurrentShape = EDrawFigure.Line;
        }

        /// <summary>
        /// Ereignishandler Menuitem Rechteck ausgewhlt
        /// </summary>
        private void _mnuDrawRect_Click(object sender, EventArgs e)
        {
            // Zeichnungsmodus setzen
            CurrentShape = EDrawFigure.Rectangle;
        }

        /// <summary>
        /// Ereignishandler Menuitem geflltes Rechteck ausgewhlt
        /// </summary>
        private void _mnuDrawRectFilled_Click(object sender, EventArgs e)
        {
            // Zeichnungsmodus setzen
            CurrentShape = EDrawFigure.RectFilled;
        }

        /// <summary>
        /// Ereignishandler Menuitem Ellipse ausgewhlt
        /// </summary>
        private void _mnuDrawEllipse_Click(object sender, EventArgs e)
        {
            // Zeichnungsmodus setzen
            CurrentShape = EDrawFigure.Ellipse;
        }

        /// <summary>
        /// Ereignishandler Maustaste wurde bettigt
        /// </summary>
        private void _pnlDraw_MouseDown(object sender, MouseEventArgs e)
        {
            try
            {
                // Prfe Zeichnungsmodus
                if (CurrentShape != EDrawFigure.Show)
                {
                    // Erzeuge Bildbuffer
                    using (Graphics objGraphics = CreateGraphics())
                    {
                        _bmpBuffer = new Bitmap(Width, Height, objGraphics);
                        Graphics objBmpGraphics = Graphics.FromImage(_bmpBuffer);
                        objBmpGraphics.Clear(Color.White);
                        foreach (CwlbShape objFigure in _objModel.Figures)
                        {
                            objFigure.Draw(objBmpGraphics, false);
                        }
                        objBmpGraphics.Dispose();
                    }

                    // Setze Rubberband Flag
                    _bRubberband = true;

                    // Speicher Ursprung
                    _ptStart = new Point(e.X, e.Y);
                }
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Oops!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Ereignishandler Maustaste wurde losgelassen
        /// </summary>
        private void _pnlDraw_MouseUp(object sender, MouseEventArgs e)
        {
            // Lokale Daten
            int iX;
            int iY;
            int iWidth;
            int iHeight;

            try
            {
                // Ist der Zeichnungsmodus eingeschaltet
                if (_bRubberband)
                {
                    // Zeichne Bitmap
                    using (Graphics objGarphics = CreateGraphics())
                    {
                        objGarphics.DrawImage(_bmpBuffer, 0, 0);

                        // Erzeuge Figur
                        CwlbShape objFigure = null;
                        switch (CurrentShape)
                        {
                            case EDrawFigure.Line:
                                objFigure = new CwlbLineShape(_ptStart, new Point(e.X, e.Y), _iLineWidth, _colLine, _enmDashStyle);
                                break;
                            case EDrawFigure.Rectangle:
                                iX = Math.Min(_ptStart.X, e.X);
                                iY = Math.Min(_ptStart.Y, e.Y);
                                iWidth = Math.Abs(_ptStart.X - e.X);
                                iHeight = Math.Abs(_ptStart.Y - e.Y);
                                objFigure = new CwlbRectangleShape(iX, iY, iWidth, iHeight, _iLineWidth, _colLine, _enmDashStyle);
                                break;
                            case EDrawFigure.RectFilled:
                                iX = Math.Min(_ptStart.X, e.X);
                                iY = Math.Min(_ptStart.Y, e.Y);
                                iWidth = Math.Abs(_ptStart.X - e.X);
                                iHeight = Math.Abs(_ptStart.Y - e.Y);
                                objFigure = new CwlbRectangleShape(iX, iY, iWidth, iHeight, _iLineWidth, _colLine, _enmDashStyle, _colArea);
                                break;
                            default:
                                break;
                        }

                        // Registriere Figur
                        _objModel.AddFigure(objFigure);
                        // Letzte Figur neu Zeichnen
                        objFigure.Draw(objGarphics, false);
                    }

                    // Zeichnugsmodus ausschalten
                    _bRubberband = false;
                }
                else
                {
                    // Selektionsmodus
                    if (ModifierKeys != Keys.Control)
                    {
                        // Deselektion aller Figuren
                        _objModel.DeselectAll();
                    }
                    _objModel.Select(new Point(e.X, e.Y));
                    Invalidate();
                }
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Oops!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _bRubberband = false;
            }
        }

        /// <summary>
        /// Ereignishandler Maus wurde bewegt
        /// </summary>
        private void _pnlDraw_MouseMove(object sender, MouseEventArgs e)
        {
            try
            {
                // Ist der Zeichnungsmodus eingeschaltet
                if (_bRubberband)
                {
                    // Zeichne Bitmap
                    using (Graphics objGarphics = CreateGraphics())
                    {
                        objGarphics.DrawImage(_bmpBuffer, 0, 0);
                        switch (CurrentShape)
                        {
                            case EDrawFigure.Line:
                                objGarphics.DrawLine(Pens.Red, _ptStart, new Point(e.X, e.Y));
                                break;
                            case EDrawFigure.Rectangle:
                            case EDrawFigure.RectFilled:
                                int iX = Math.Min(_ptStart.X, e.X);
                                int iY = Math.Min(_ptStart.Y, e.Y);
                                int iWidth = Math.Abs(_ptStart.X - e.X);
                                int iHeight = Math.Abs(_ptStart.Y - e.Y);
                                objGarphics.DrawRectangle(Pens.Red, iX, iY, iWidth, iHeight);
                                break;
                            default:
                                break;
                        }
                    }
                }
                else
                {
                    foreach (CwlbShape objFigure in _objModel.SelectedFigures)
                    {
                        if (objFigure.IsMarker(e.Location))
                        {
                            Cursor = Cursors.SizeAll;
                            break;
                        }
                        else
                        {
                            SetCursor();
                        }
                    }
                }
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "Oops!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Ereignisbehandlung Zeichne die Daten.
        /// </summary>
        private void _pnlDraw_Paint(object sender, PaintEventArgs e)
        {
            // Zeichnen aller Figuren
            foreach (CwlbShape objFigure in _objModel.Figures)
            {
                objFigure.Draw(e.Graphics, false);
            }

            // Markieren der Figuren
            foreach (CwlbShape objFigure in _objModel.SelectedFigures)
            {
                objFigure.ShowMarker(e.Graphics);
            }
        }

        private void CmdiDraw_MouseEnter(object sender, EventArgs e)
        {
            SetCursor();
        }

        private void SetCursor()
        {
            // Ausgewhlte Elemente selektieren
            switch (CurrentShape)
            {
                default:
                    Cursor = Cursors.Arrow;
                    break;
                case EDrawFigure.Line:
                case EDrawFigure.Rectangle:
                case EDrawFigure.RectFilled:
                case EDrawFigure.Ellipse:
                case EDrawFigure.EllipseFilled:
                    Cursor = Cursors.Cross;
                    break;
            }
        }

        private void CmdiDraw_MouseLeave(object sender, EventArgs e)
        {
            Cursor = Cursors.Arrow;
        }

    }
}