﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert die Funktionalität des Hauptfesnters der Demo
    /// </summary>
    public partial class CwlbMainWindow : Form, INotifyPropertyChanged
    {
        // Private Instnzdaten
        private CmdiBase _mdiBase;
        private CmdiCoordinateSystem _mdiCoordinateSystem;
        private Font _fontDemo;
        private Image _imgDemo;

        /// <summary>
        /// Titel der Anwendung
        /// </summary>
        public const string TITLE = "Learning Forms 05";

        public static CwlbMainWindow MainWindow { get; private set; }

        /// <summary>
        /// Ereignis für Änderung einer Eigenschaft.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Liefert oder definiert die Schriftart für die Demo.
        /// </summary>
        public Font DemoFont 
        {
            get
            {
                return _fontDemo;
            }
            set
            {
                if (_fontDemo != null)
                {
                    _fontDemo.Dispose();
                }
                _fontDemo = value;
            }
        }

        /// <summary>
        /// Liefert oder definiert das Bild, das für die Demo verwendet wird.
        /// </summary>
        public Image DemoImage
        {
            get
            {
                return _imgDemo;
            }
            set
            {
                if (_imgDemo != null)
                {
                    _imgDemo.Dispose();
                }
                _imgDemo = value;
                OnPropertyChanged("DemoImage");
            }
        }


        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbMainWindow.
        /// </summary>
        public CwlbMainWindow()
        {
            InitializeComponent();

            // Singleton Zugriff initialisieren
            MainWindow = this;

            // Anwendungsereignisse registrieren
            Application.Idle += new EventHandler(Application_Idle);
            Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(Application_ThreadException);

            // Font für Demos erstellen
            DemoFont = new Font(this.Font.FontFamily, 12);
            // Bild für Demo laden
            DemoImage = Properties.Resources.Grille;
        }

        /// <summary>
        /// Löst das Ereignis PropertyChanged aus.
        /// </summary>
        /// <param name="strPropertyChanged">Name der Eigenschaft, die geändert wurde.</param>
        private void OnPropertyChanged(string strPropertyChanged)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(strPropertyChanged));
            }
        }

        /// <summary>
        /// Ereignisbehandlung für unerwartete Ausnahmen.
        /// </summary>
        void Application_ThreadException(object sender, System.Threading.ThreadExceptionEventArgs e)
        {
            MessageBox.Show(e.Exception.Message, TITLE, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        /// <summary>
        /// Behandlung der Leerlauferignisse.
        /// </summary>
        void Application_Idle(object sender, EventArgs e)
        {
            // Tastenstatusse zeigen
            if (Console.CapsLock)
            {
                _statusCaps.Text = "CAPS";
            }
            else
            {
                _statusCaps.Text = string.Empty;
            }

            if (Console.NumberLock)
            {
                _statusNum.Text = "Num";
            }
            else
            {
                _statusNum.Text = string.Empty;
            }

            // Prüfen, ob das MDI-Fenster für die Demo der Basisfunktionen vorhanden ist
            if (_mdiBase != null)
            {
                _mnuFileNewBase.Enabled = false;
                _tbtnFileNewBase.Enabled = false;
                
            }
            else
            {
                _mnuFileNewBase.Enabled = true;
                _tbtnFileNewBase.Enabled = true;
            }

            // Prüfen, ob das MDI-Fenster für die Demo des Koordinatensystems vorhanden ist
            if (_mdiCoordinateSystem != null)
            {
                _mnuFileNewCoordinateSystem.Enabled = false;
                _tbtnFileNewCoordinateSystem.Enabled = false;

            }
            else
            {
                _mnuFileNewCoordinateSystem.Enabled = true;
                _tbtnFileNewCoordinateSystem.Enabled = true;
            }

            // Fenster anordnungen kontrollieren
            if (MdiChildren.Count() == 0)
            {
                _mnuWindowHorizontal.Enabled = false;
                _mnuWindowVertical.Enabled = false;
                _mnuWindowDiagonal.Enabled = false;
            }
            else
            {
                _mnuWindowHorizontal.Enabled = true;
                _mnuWindowVertical.Enabled = true;
                _mnuWindowDiagonal.Enabled = true;
            }

            if (ActiveMdiChild is CmdiDraw)
            {
                // Zeichnungsmodus
                CmdiDraw objDraw = (CmdiDraw)ActiveMdiChild;
                if (objDraw.CurrentShape != objDraw.OldShape)
                {

                    // Optimierung nachführen
                    objDraw.OldShape = objDraw.CurrentShape;
                    foreach (ToolStripItem objItem in _toolMain.Items)
                    {
                        if (objItem.Tag != null && objItem.Tag.ToString() == "D")
                        {
                            ToolStripButton objButton = objItem as ToolStripButton;
                            if (objButton != null)
                            {
                                objButton.Checked = false;
                            }
                        }
                    }

                    switch (((CmdiDraw)ActiveMdiChild).CurrentShape)
                    {
                        case CmdiDraw.EDrawFigure.Show:
                            _tbtnDrawShow.Checked = true;
                            break;
                        case CmdiDraw.EDrawFigure.Line:
                            _tbtnDrawLine.Checked = true;
                            break;
                        case CmdiDraw.EDrawFigure.Rectangle:
                            _tbtnDrawRectangle.Checked = true;
                            break;
                        case CmdiDraw.EDrawFigure.RectFilled:
                            _tbtnDrawFilledRectangle.Checked = true;
                            break;
                        case CmdiDraw.EDrawFigure.Ellipse:
                            _tbtnDrawEllipse.Checked = true;
                            break;
                        case CmdiDraw.EDrawFigure.EllipseFilled:
                            _tbtnDrawFilledEllipse.Checked = true;
                            break;
                        default:
                            break;
                    }
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Fenster Zeichnen Grundlagen erstellen
        /// </summary>
        private void MenuFileNewBase_Click(object sender, EventArgs e)
        {
            _mdiBase = new CmdiBase();
            _mdiBase.MdiParent = this;
            _mdiBase.WindowState = FormWindowState.Maximized;
            _mdiBase.Show();
            _mdiBase.FormClosing += new FormClosingEventHandler(DemoBase_FormClosing);
            this.PropertyChanged += new PropertyChangedEventHandler(_mdiBase.MainWindow_PropertyChanged);
        }


        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Fenster Koordinatensystem
        /// </summary>
        private void MenuFileNewCoordinateSystem_Click(object sender, EventArgs e)
        {
            _mdiCoordinateSystem = new CmdiCoordinateSystem();
            _mdiCoordinateSystem.MdiParent = this;
            _mdiCoordinateSystem.WindowState = FormWindowState.Maximized;
            _mdiCoordinateSystem.Show();
            _mdiCoordinateSystem.FormClosing += new FormClosingEventHandler(DemoCoordinateSystem_FormClosing);
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl neue Zeichnung.
        /// </summary>
        private void MenuFileNewDraw_Click(object sender, EventArgs e)
        {
            CmdiDraw mdiDraw = new CmdiDraw();
            mdiDraw.MdiParent = this;
            mdiDraw.WindowState = FormWindowState.Maximized;
            mdiDraw.Show();
        }


        /// <summary>
        /// Ereignisbehandlung beim Schliessen der Demo des Koordinatensystems.
        /// </summary>
        void DemoCoordinateSystem_FormClosing(object sender, FormClosingEventArgs e)
        {
            _mdiCoordinateSystem.FormClosing -= new FormClosingEventHandler(DemoCoordinateSystem_FormClosing);
            _mdiCoordinateSystem = null;
        }


        /// <summary>
        /// Ereignisbehandlung beim Schliessen der Demo der Basisfunktionalitäten.
        /// </summary>
        void DemoBase_FormClosing(object sender, FormClosingEventArgs e)
        {
            _mdiBase.FormClosing -= new FormClosingEventHandler(DemoBase_FormClosing);
            this.PropertyChanged -= new PropertyChangedEventHandler(_mdiBase.MainWindow_PropertyChanged);
            _mdiBase = null;
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Anwendung beenden.
        /// </summary>
        private void MenuFileExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Ereignisbehandlung für Schriftauswahl.
        /// </summary>
        private void MenuExtrasFont_Click(object sender, EventArgs e)
        {
            // Fontdialog erzeugen und konfigurieren
            FontDialog dlgFont = new FontDialog();
            dlgFont.Font = DemoFont;
            dlgFont.ShowColor = false;

            // Dialogfeld anzeigen und auswerten
            if (dlgFont.ShowDialog() == DialogResult.OK)
            {
                DemoFont = dlgFont.Font;
            }
        }

        private void MenuExtrasImage_Click(object sender, EventArgs e)
        {
            // Datedialog erzeugen und konfigurieren
            OpenFileDialog dlgOpen = new OpenFileDialog();
            dlgOpen.Filter = "Bildformate|*.jpg;*.gif;*.bmp;*.emf;*.png;*.tiff";
            dlgOpen.Filter += "|File interchange format|*.jpg";
            dlgOpen.Filter += "|Bitmap format|*.bmp";
            dlgOpen.Filter += "|Graphics interchange format|*.gif";
            dlgOpen.Filter += "|Enhanced meta file|*.emf";
            dlgOpen.Filter += "|Portable network grafik|*.png";
            dlgOpen.Filter += "|Tagged image file format|*.tiff";
            dlgOpen.FilterIndex = 1;

            // Dialogfeld anzeigen und auswerten
            if (dlgOpen.ShowDialog() == DialogResult.OK)
            {
                // Versuche Datei zu öffnen
                try
                {
                    DemoImage = Image.FromFile(dlgOpen.FileName);
                }
                catch (Exception excObject)
                {
                    MessageBox.Show(string.Format("Die Datei konnte nicht korrekt eingelesen werden.{0}{1}", Environment.NewLine, excObject.Message),
                        TITLE,MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Fenster Horizontal ausrichten.
        /// </summary>
        private void MenuWindowHorizontal_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.TileHorizontal);
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Fenster vertikal ausrichten.
        /// </summary>
        private void MenuWindowVertical_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.TileVertical);
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Fenster diagonal ausrichten.
        /// </summary>
        private void MenuWindowDiagonal_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.Cascade);
        }

        /// <summary>
        /// Ereignisbehandlung für den Menübefehl Info.
        /// </summary>
        private void MenuInfo_Click(object sender, EventArgs e)
        {
            CdlgInfo dlgInfo = new CdlgInfo();
            dlgInfo.ShowDialog();
        }

        /// <summary>
        /// Ereignisbehandlung beim aktivieren einer MDI Kindfensters.
        /// </summary>
        private void CwlbMainWindow_MdiChildActivate(object sender, EventArgs e)
        {
            if (ActiveMdiChild is CmdiDraw)
            {
                foreach (ToolStripItem objItem in _toolMain.Items)
                {
                    if (objItem.Tag != null && objItem.Tag.ToString() == "D")
                    {
                        objItem.Visible = true;
                    }
                }
            }
            else
            {
                foreach (ToolStripItem objItem in _toolMain.Items)
                {
                    if (objItem.Tag != null && objItem.Tag.ToString() == "D")
                    {
                        objItem.Visible = false;
                    }
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Zeichnen des Inhalt der Schaltfläche Linienbreite in der Symbolleiste.
        /// </summary>
        private void ToolBarButtonDrawLineWidth_Paint(object sender, PaintEventArgs e)
        {
            // Breite des Menübalkens bestimmen
            int iCheckSize = (int)(_mnuMain.ImageScalingSize.Width * 1.5);
            // Dicke der Linie aus Eigenschaft Tag übernehmen
            Pen pen = new Pen(Color.Blue,
                Convert.ToSingle(((ToolStripItem)sender).Tag));
            // Linie zeichnen
            e.Graphics.DrawLine(pen,
                iCheckSize + 4,
                e.ClipRectangle.Height / 2,
                e.ClipRectangle.Width - 6,
                e.ClipRectangle.Height / 2);
            // Ressource freigeben
            pen.Dispose();
        }

        /// <summary>
        /// Ereignisbehandlung Klick Schaltfläche Zeigen in der Symbolleiste zeigen.
        /// </summary>
        private void _tbtnDrawShow_Click(object sender, EventArgs e)
        {
            if (ActiveMdiChild is CmdiDraw)
            {
                ((CmdiDraw)ActiveMdiChild).CurrentShape = CmdiDraw.EDrawFigure.Show;
            }
        }

        /// <summary>
        /// Ereignisbehandlung Klick Schaltfläche Linie in der Symbolleiste zeigen.
        /// </summary>
        private void _tbtnDrawLine_Click(object sender, EventArgs e)
        {
            if (ActiveMdiChild is CmdiDraw)
            {
                ((CmdiDraw)ActiveMdiChild).CurrentShape = CmdiDraw.EDrawFigure.Line;
            }
        }

        /// <summary>
        /// Ereignisbehandlung Klick Schaltfläche Rechteck in der Symbolleiste zeigen.
        /// </summary>
        private void _tbtnDrawRectangle_Click(object sender, EventArgs e)
        {
            if (ActiveMdiChild is CmdiDraw)
            {
                ((CmdiDraw)ActiveMdiChild).CurrentShape = CmdiDraw.EDrawFigure.Rectangle;
            }
        }

        /// <summary>
        /// Ereignisbehandlung Klick Schaltfläche gefülltes Rechteck in der Symbolleiste zeigen.
        /// </summary>
        private void _tbtnDrawFilledRectangle_Click(object sender, EventArgs e)
        {
            if (ActiveMdiChild is CmdiDraw)
            {
                ((CmdiDraw)ActiveMdiChild).CurrentShape = CmdiDraw.EDrawFigure.RectFilled;
            }
        }

        /// <summary>
        /// Ereignisbehandlung Klick Schaltfläche Ellipse in der Symbolleiste zeigen.
        /// </summary>
        private void _tbtnDrawEllipse_Click(object sender, EventArgs e)
        {
            if (ActiveMdiChild is CmdiDraw)
            {
                ((CmdiDraw)ActiveMdiChild).CurrentShape = CmdiDraw.EDrawFigure.Ellipse;
            }
        }

        /// <summary>
        /// Ereignisbehandlung Klick Schaltfläche gefüllte Ellipse in der Symbolleiste zeigen.
        /// </summary>
        private void _tbtnDrawFilledEllipse_Click(object sender, EventArgs e)
        {
            if (ActiveMdiChild is CmdiDraw)
            {
                ((CmdiDraw)ActiveMdiChild).CurrentShape = CmdiDraw.EDrawFigure.EllipseFilled;
            }
        }
    }
}
