using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;

namespace WeroSoft.Samples
{

    /// <summary>
    /// Geometrische Linienfigur
    /// </summary>
    public class CwlbLineShape : CwlbLineBaseShape
    {


        /// <summary>
        /// Konstruktor
        /// </summary>
        /// <param name="ptStart">Startpunkt der Linie.</param>
        /// <param name="ptEnd">Endpunkt der Linie.</param>
        /// <param name="iLineWidth">Liniendicke.</param>
        /// <param name="colLine">Linienfarbe.</param>
        /// <param name="enmDashStyle">Linientyp.</param>
        public CwlbLineShape(Point ptStart, Point ptEnd, int iLineWidth, Color colLine, DashStyle enmDashStyle)
        {
            _ptP1 = ptStart;
            _ptP2 = ptEnd;
            _iLineWidth = iLineWidth;
            _colLine = colLine;
            _enmDashStyle = enmDashStyle;
        }

        public override bool IsMarker(Point ptMove)
        {

            // Prfe 1. Punkt
            if ((Math.Abs(ptMove.X - _ptP1.X) < 5) &&
                (Math.Abs(ptMove.Y - _ptP1.Y) < 5))
            {
                return true;
            }

            // Prfe 2. Punkt
            if ((Math.Abs(ptMove.X - _ptP2.X) < 5) &&
                (Math.Abs(ptMove.Y - _ptP2.Y) < 5))
            {
                return true;
            }

            // Kein Punkt 
            return false;
        }


        /// <summary>
        /// Zeichnet eine Figur
        /// </summary>
        /// <param name="objGraphics">Grphics Objekt</param>
        /// <param name="bMarked">Flag, zur kennzeichnung der Markierung</param>
        public override void Draw(Graphics objGraphics, bool bMarked)
        {

            // Erstelle Pen
            using (Pen objPen = new Pen(_colLine, _iLineWidth))
            {

                // Linienszil setzen
                objPen.DashStyle = _enmDashStyle;

                // Zeichne Linie
                objGraphics.DrawLine(objPen, _ptP1, _ptP2);

                // Prfe Markierung
                if (bMarked)
                {
                    ShowMarker(objGraphics);
                }
            }
        }

        /// <summary>
        /// Zeichnet die Markierungspunkte
        /// </summary>
        /// <param name="objGraphics">Grphics Objekt</param>
        public override void ShowMarker(Graphics objGraphics)
        {

            // Zeichne Markierung
            DrawMarker(objGraphics, _ptP1);
            DrawMarker(objGraphics, _ptP2);
        }

        /// <summary>
        /// Prft ob eine Figur Selektiert werden kann
        /// </summary>
        /// <param name="ptHitTest">Selektionspunkt</param>
        /// <returns>Wahr, wenn die Figur betroffen ist.</returns>
        public override bool Select(Point ptHitTest)
        {

            // lokale Daten
            GraphicsPath pathFigure = new GraphicsPath();
            bool bHit = false;
            Point[] aptLine = new Point[4];
            aptLine[0] = new Point(_ptP1.X - 10, _ptP1.Y - 10);
            aptLine[1] = new Point(_ptP1.X + 10, _ptP1.Y + 10);
            aptLine[2] = new Point(_ptP2.X - 10, _ptP2.Y - 10);
            aptLine[3] = new Point(_ptP2.X + 10, _ptP2.Y + 10);

            // Pfad herstellen
            pathFigure.FillMode = FillMode.Winding;
            pathFigure.StartFigure();
            pathFigure.AddPolygon(aptLine);
            pathFigure.CloseFigure();

            // Prfe Punkt
            bHit = pathFigure.IsVisible(ptHitTest);

            // Ressourcen freigeben
            pathFigure.Dispose();

            // Wertrckgabe
            return bHit;
        }
    }
}
