using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;

namespace WeroSoft.Samples
{
    /// <summary>
    /// 
    /// </summary>
    public class CwlbRectangleShape : CwlbClosedShape
    {

        public Rectangle Rect
        {
            get
            {
                int iWidth = Math.Abs(_ptP1.X - _ptP2.X);
                int iHeight = Math.Abs(_ptP1.Y - _ptP2.Y);
                return new Rectangle(_ptP1.X, _ptP1.Y, iWidth, iHeight);
            }
        }

        /// <summary>
        /// Konstruktor
        /// </summary>
        /// <param name="ptStart">Startpunkt des Rechtecks.</param>
        /// <param name="ptEnd">Endpunkt des Rechtecks.</param>
        /// <param name="iLineWidth">Liniendicke.</param>
        /// <param name="colLine">Linienfarbe.</param>
        /// <param name="enmDashStyle">Linientyp.</param>
        public CwlbRectangleShape(int iX, int iY, int iWidth, int iHeight, int iLineWidth, Color colLine, DashStyle enmDashStyle)
        {

            // Parameter bernehmen
            _ptP1 = new Point(iX, iY);
            _ptP2 = new Point(iX + iWidth, iY + iHeight);
            _iLineWidth = iLineWidth;
            _colLine = colLine;
            _enmDashStyle = enmDashStyle;
        }

        /// <summary>
        /// Konstruktor
        /// </summary>
        /// <param name="ptStart">Startpunkt des Rechtecks.</param>
        /// <param name="ptEnd">Endpunkt des Rechtecks.</param>
        /// <param name="iLineWidth">Liniendicke.</param>
        /// <param name="colLine">Linienfarbe.</param>
        /// <param name="enmDashStyle">Linientyp.</param>
        public CwlbRectangleShape(int iX, int iY, int iWidth, int iHeight, int iLineWidth, Color colLine, DashStyle enmDashStyle, Color colArea)
            : this(iX, iY, iWidth, iHeight, iLineWidth, colLine, enmDashStyle)
        {

            // Parameter bernehmen
            _colArea = colArea;
            _bFill = true;
        }

        /// <summary>
        /// Prft ob die Mausposition einem Marker entspricht
        /// </summary>
        /// <param name="ptMove">Punkt der gerade mit der Maus gezeigt wird</param>
        /// <returns>Wahr, wenn die Maus auf einem Marker ist</returns>
        public override bool IsMarker(Point ptMove)
        {

            // Prfe 1. Punkt
            if ((Math.Abs(ptMove.X - _ptP1.X) < 5) &&
                (Math.Abs(ptMove.Y - _ptP1.Y) < 5))
            {
                return true;
            }

            // Prfe 2. Punkt
            if ((Math.Abs(ptMove.X - _ptP2.X) < 5) &&
                (Math.Abs(ptMove.Y - _ptP2.Y) < 5))
            {
                return true;
            }

            // Prfe 3. Punkt
            if ((Math.Abs(ptMove.X - _ptP2.X) < 5) &&
                (Math.Abs(ptMove.Y - _ptP1.Y) < 5))
            {
                return true;
            }

            // Prfe 4. Punkt
            if ((Math.Abs(ptMove.X - _ptP1.X) < 5) &&
                (Math.Abs(ptMove.Y - _ptP2.Y) < 5))
            {
                return true;
            }

            // Kein Punkt 
            return false;
        }


        /// <summary>
        /// Zeichnet eine Figur
        /// </summary>
        /// <param name="objGraphics">Grphics Objekt</param>
        /// <param name="bMarked">Flag, zur kennzeichnung der Markierung</param>
        public override void Draw(Graphics objGraphics, bool bMarked)
        {

            // Erstelle Pen
            Pen objPen = new Pen(_colLine, _iLineWidth);
            Brush objBrush = new SolidBrush(_colArea);

            // Linienszil setzen
            objPen.DashStyle = _enmDashStyle;

            // Zeichne Linie
            if (_bFill)
            {
                objGraphics.FillRectangle(objBrush, this.Rect);
            }
            objGraphics.DrawRectangle(objPen, this.Rect);

            // Prfe Markierung
            if (bMarked)
            {
                ShowMarker(objGraphics);
            }

            // Ressourcen freigeben
            objPen.Dispose();
            objBrush.Dispose();
        }

        /// <summary>
        /// Zeichnet die Markierungspunkte
        /// </summary>
        /// <param name="objGraphics">Grphics Objekt</param>
        public override void ShowMarker(Graphics objGraphics)
        {

            // Zeichne Markierung
            DrawMarker(objGraphics, _ptP1);
            DrawMarker(objGraphics, _ptP2);
            DrawMarker(objGraphics, new Point(_ptP1.X, _ptP2.Y));
            DrawMarker(objGraphics, new Point(_ptP2.X, _ptP1.Y));
        }

        /// <summary>
        /// Prft ob eine Figur Selektiert werden kann
        /// </summary>
        /// <param name="ptHitTest">Selektionspunkt</param>
        /// <returns>Wahr, wenn die Figur betroffen ist.</returns>
        public override bool Select(Point ptHitTest)
        {

            // lokale Daten
            GraphicsPath pathFigure = new GraphicsPath();
            bool bHit = false;

            // Pfad herstellen
            pathFigure.FillMode = FillMode.Winding;
            pathFigure.StartFigure();
            pathFigure.AddRectangle(this.Rect);
            pathFigure.CloseFigure();

            // Prfe Punkt
            bHit = pathFigure.IsVisible(ptHitTest);

            // Ressourcen freigeben
            pathFigure.Dispose();

            // Wertrckgabe
            return bHit;
        }
    }
}
