﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert die Interaktion des Hauptfensters.
    /// </summary>
    public partial class CwlbMainForm : Form
    {
        /// <summary>
        /// Definiert den Titel der Anwendung
        /// </summary>
        public const string TITLE = "Learning Windows Forms 06";

        /// <summary>
        /// MDI Fenster Demo Drag und Drop.
        /// </summary>
        private CmdiDragDrop _mdiDragDrop;

        /// <summary>
        /// MDI Fenster Demo Clipboard.
        /// </summary>
        private CmdiClipboard _mdiClipboard;

        /// <summary>
        /// MDI Fenster Demo Datenbindung.
        /// </summary>
        private CmdiDatabinding _mdiDatabinding;

        /// <summary>
        /// MDI Fenster Demo parallele Programmierung mit Forms.
        /// </summary>
        private CmdiParallel _mdiParallel;

        /// <summary>
        /// MDI Fenster Demo Lokalisierung.
        /// </summary>
        private CmdiLocalization _mdiLocalization;

        /// <summary>
        /// MDI Fenster Demo eigenes Steuerelement.
        /// </summary>
        private CmdiOwnControl _mdiControl;

        /// <summary>
        /// Liefert oder definiert das Hauptfenster der Anwendugn als Singleton.
        /// </summary>
        public static CwlbMainForm MainWindow { get; private set; }

        /// <summary>
        /// Merker, ob die Seite für die Sprachdemo automatisch neu geladen werden soll.
        /// </summary>
        public static bool ReloadLanguagePage { get; set; }


        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwwlbMainForm.
        /// </summary>
        public CwlbMainForm()
        {
            // Singleton initialisierung
            MainWindow = this;

            InitializeComponent();

            // Leerlaufhandling aktivieren
            Application.Idle += new EventHandler(Application_Idle);
            Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(Application_ThreadException);
        }

        /// <summary>
        /// Ereignisbehandlung für unbehandelte Ausnahmen.
        /// </summary>
        void Application_ThreadException(object sender, System.Threading.ThreadExceptionEventArgs e)
        {
            MessageBox.Show(e.Exception.Message, TITLE, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        /// <summary>
        /// Ereignisbehandlung für das Leerlaufereignis.
        /// </summary>
        void Application_Idle(object sender, EventArgs e)
        {
            // Drag und Drop Demo kontrollieren
            if (_mdiDragDrop == null)
            {
                _mnuFileDragDrop.Enabled = true;
            }
            else
            {
                _mnuFileDragDrop.Enabled = false;
            }
            // Clipboard Demo kontrollieren
            if (_mdiClipboard == null)
            {
                _mnuFileClipboard.Enabled = true;
            }
            else
            {
                _mnuFileClipboard.Enabled = false;
            }
            // Demo Datenbindung kontrollieren
            if (_mdiDatabinding == null)
            {
                _mnuFileDatabinding.Enabled = true;
            }
            else
            {
                _mnuFileDatabinding.Enabled = false;
            }
            // Demo parallele Programmierung kontrollieren
            if (_mdiParallel == null)
            {
                _mnuFileParallel.Enabled = true;
            }
            else
            {
                _mnuFileParallel.Enabled = false;
            }
            // Demo Lokalisierung kontrollieren
            if (_mdiLocalization == null)
            {
                _mnuFileLocalization.Enabled = true;
            }
            else
            {
                _mnuFileLocalization.Enabled = false;
            }
            // Demo eigenes Steuerelement kontrollieren
            if (_mdiControl == null)
            {
                _mnuFileControl.Enabled = true;
            }
            else
            {
                _mnuFileControl.Enabled = false;
            }
        }

        /// <summary>
        /// ereignisbehandlung für das Laden des Hauptfensters.
        /// </summary>
        private void CwlbMainForm_Load(object sender, EventArgs e)
        {
            // Prüfen, ob die Sprachdemo automatisch gealaden werden soll.
            if (ReloadLanguagePage)
            {
                _mnuFileLocalization.PerformClick();
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Schliessen des Hauptfensters.
        /// </summary>
        private void CwlbMainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Eventhandler deregistrieren
            Application.Idle -= new EventHandler(Application_Idle);
            Application.ThreadException -= new System.Threading.ThreadExceptionEventHandler(Application_ThreadException);

            // Einstellungen speichern
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Ereignisbehandlung für das Starten der Drag und Drop Demo.
        /// </summary>
        private void MenuFileDragDrop_Click(object sender, EventArgs e)
        {
            _mdiDragDrop = new CmdiDragDrop();
            _mdiDragDrop.MdiParent = this;
            _mdiDragDrop.FormClosing += new FormClosingEventHandler(DragDrop_FormClosing);
            _mdiDragDrop.Show();
            _mdiDragDrop.WindowState = FormWindowState.Maximized;
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen des MDI-Clientfensters für die Demo von Drag&Drop.
        /// </summary>
        private void DragDrop_FormClosing(object sender, FormClosingEventArgs e)
        {
            _mdiDragDrop.FormClosing -= new FormClosingEventHandler(DragDrop_FormClosing);
            _mdiDragDrop = null;
        }

        /// <summary>
        /// Ereignisbehandlung für das Starten der Demo der Zwischenablage.
        /// </summary>
        private void MenuFileClipboard_Click(object sender, EventArgs e)
        {
            _mdiClipboard = new CmdiClipboard();
            _mdiClipboard.MdiParent = this;
            _mdiClipboard.FormClosing += new FormClosingEventHandler(Clipboard_FormClosing);
            _mdiClipboard.Show();
            _mdiClipboard.WindowState = FormWindowState.Maximized;
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen des MDI-Clientfensters für die Demo der Zwischenablage.
        /// </summary>
        private void Clipboard_FormClosing(object sender, FormClosingEventArgs e)
        {
            _mdiClipboard.FormClosing -= new FormClosingEventHandler(Clipboard_FormClosing);
            _mdiClipboard = null;
        }

        /// <summary>
        /// Ereignisbehandlung für das Starten der Demo Datenbindung.
        /// </summary>
        private void MenuFileDatabinding_Click(object sender, EventArgs e)
        {
            _mdiDatabinding = new CmdiDatabinding();
            _mdiDatabinding.MdiParent = this;
            _mdiDatabinding.FormClosing += new FormClosingEventHandler(Databinding_FormClosing);
            _mdiDatabinding.Show();
            _mdiDatabinding.WindowState = FormWindowState.Maximized;
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen des MDI-Clientfensters für die Demo der Datenbindung.
        /// </summary>
        private void Databinding_FormClosing(object sender, FormClosingEventArgs e)
        {
            _mdiDatabinding.FormClosing -= new FormClosingEventHandler(Databinding_FormClosing);
            _mdiDatabinding = null;
        }


        /// <summary>
        /// Ereignisbehandlung für das Starten der Demo parallelen Programmierung.
        /// </summary>
        private void MenuFileParallel_Click(object sender, EventArgs e)
        {
            _mdiParallel = new CmdiParallel();
            _mdiParallel.MdiParent = this;
            _mdiParallel.FormClosing += new FormClosingEventHandler(Parallel_FormClosing);
            _mdiParallel.Show();
            _mdiParallel.WindowState = FormWindowState.Maximized;
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen des MDI-Clientfensters für die Demo der parallelen Programmierung.
        /// </summary>
        private void Parallel_FormClosing(object sender, FormClosingEventArgs e)
        {
            _mdiParallel.FormClosing -= new FormClosingEventHandler(Parallel_FormClosing);
            _mdiParallel = null;
        }

        /// <summary>
        /// Ereignisbehandlung für das Starten der Demo Lokalisierung.
        /// </summary>
        private void MenuFileLocalization_Click(object sender, EventArgs e)
        {
            _mdiLocalization = new CmdiLocalization();
            _mdiLocalization.MdiParent = this;
            _mdiLocalization.FormClosing += new FormClosingEventHandler(Localization_FormClosing);
            _mdiLocalization.Show();
            _mdiLocalization.WindowState = FormWindowState.Maximized;
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen des MDI-Clientfensters für die Demo der Lokalisierung.
        /// </summary>
        private void Localization_FormClosing(object sender, FormClosingEventArgs e)
        {
            _mdiLocalization.FormClosing -= new FormClosingEventHandler(Localization_FormClosing);
            _mdiLocalization = null;
        }

        /// <summary>
        /// Ereignisbehandlung für das Starten der Demo eigenes Steuerelement.
        /// </summary>
        private void MenuFileControl_Click(object sender, EventArgs e)
        {
            _mdiControl = new CmdiOwnControl();
            _mdiControl.MdiParent = this;
            _mdiControl.FormClosing += new FormClosingEventHandler(Control_FormClosing);
            _mdiControl.Show();
            _mdiControl.WindowState = FormWindowState.Maximized;
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen des MDI-Clientfensters für die Demo der eigenen Steuerelemente.
        /// </summary>
        private void Control_FormClosing(object sender, FormClosingEventArgs e)
        {
            _mdiControl.FormClosing -= new FormClosingEventHandler(Control_FormClosing);
            _mdiControl = null;
        }

        /// <summary>
        /// Ereignisbehandlung für das Starten des Dialogfeldes Demo Validierung.
        /// </summary>
        private void _mnuFileValidation_Click(object sender, EventArgs e)
        {
            // Dialog erzeugen und anzeigen
            CdlgValidation dlgValidation = new CdlgValidation();
            dlgValidation.ShowDialog();
        }

        /// <summary>
        /// Ereignisbehandlugn für das Starten der Demo Keine Rückmeldung.
        /// </summary>
        private void _mnuFileNoResponse_Click(object sender, EventArgs e)
        {
            // Rückfrage und Anweisung der Demo
            string strCheck =
                "Diese Demo friert die Anwendung während einer Minute ein. " +
                "Innerhalb dieser Minute ist die Ausführung der Anwendung nur mit " +
                "Gewalt über den Taskmanager abbrechber. Nach einer Minute " +
                "ist die Anwendung wieder normal verfügbar. Für Ihr System besteht " +
                "keinerlei Gefahr.\r\n\r\nBetätigen Sie die Schaltfläche <Ja>, wenn Sie " +
                "die Demo starten wollen, andernfalls die Schaltfläche <Nein>";
            if (MessageBox.Show(strCheck, CwlbMainForm.TITLE, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                Thread.Sleep(60000);
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Beenden der Demo.
        /// </summary>
        private void MenuFileExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Ereignisbehandlung für das horizontale Anordnen der MDI-Clientfenster.
        /// </summary>
        private void MenuWindowHorizontal_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.TileHorizontal);
        }

        /// <summary>
        /// Ereignisbehandlung für das vertikale Anordnen der MDI-Clientfenster.
        /// </summary>
        private void MenuWindowVertical_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.TileVertical);
        }

        /// <summary>
        /// Ereignisbehandlung für das diagonale Anordnen der MDI-Clientfenster.
        /// </summary>
        private void MenuWindowDiagonal_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.Cascade);
        }

        /// <summary>
        /// Ereignisbehandlung für das Anzeigen der Info.
        /// </summary>
        private void MenuInfo_Click(object sender, EventArgs e)
        {
            CdlgInfo dlgInfo = new CdlgInfo();
            dlgInfo.ShowDialog();
        }
    }
}
