using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Demo zu Validierung eines Dialogfeldes.
    /// </summary>
    public partial class CdlgValidation : Form
    {
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CDlgValidation.
        /// </summary>
        public CdlgValidation()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung fr das Validierung des Textfeldes Name
        /// </summary>
        private void Name_Validating(object sender, CancelEventArgs e)
        {
            // Prfkriterium
            if (_tbName.Text.Length < 3)
            {
                // Prfung fehlgeschlagen
                _errorProvider.SetError(_tbName,
                    "Bitte geben Sie einen gltigen Namen ein.");
                e.Cancel = true;
            }
            else
            {
                // Prfung OK
                _errorProvider.SetError(_tbName, "");
            }
        }

        /// <summary>
        /// Ereignisbehandlung Validierung des DateTimePicker-Steuerelements Geburtstag
        /// </summary>
        private void Birthday_Validating(object sender, CancelEventArgs e)
        {
            // Prfkriterium
            if (_dtBirthday.Value < DateTime.Now - new TimeSpan(36500, 0, 0, 0) ||
                _dtBirthday.Value > DateTime.Now)
            {
                // Prfung fehlgeschlagen
                _errorProvider.SetError(_dtBirthday,
                    "Das kann unmglich ihr Geburtstag sein.");
                e.Cancel = true;
            }
            else
            {
                // Prfung OK
                _errorProvider.SetError(_dtBirthday, "");
            }
        }

        /// <summary>
        /// Ereignisbehandlung Validierung des Textfeldes fr Email-Adresse
        /// </summary>
        private void Email_Validating(object sender, CancelEventArgs e)
        {
            // Erstelle regulren Ausdruck
            Regex rxExpression = new Regex(@"\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*");

            // Testen des Ausdrucks
            Match rxMatch = rxExpression.Match(_tbEmail.Text);
            if (!rxMatch.Success)
            {
                // Prfung fehlgeschlagen
                _errorProvider.SetError(_tbEmail,
                    "Diese E-Mail-Adresse scheint eher unwahrscheinlich.");
                e.Cancel = true;
            }
            else
            {
                // Prfung OK
                _errorProvider.SetError(_tbEmail, "");
            }
        }

        /// <summary>
        /// Ereignisbehandlung Verlassen des Textfeldes E-Mail-Adresse
        /// </summary>
        /// <remarks>Dieser EventHandler prft explizit.</remarks>
        private void Email_Leave(object sender, EventArgs e)
        {
            // Prfe noch einmal alle Daten
            if (this.ValidateChildren())
            {
                // Prfung OK
                _btnOK.Enabled = true;
            }
            else
            {
                // Prfung fehlgeschlagen
                _btnOK.Enabled = false;
            }
        }

        /// <summary>
        /// Ereignisbehandlung CheckBox Explizite Validierung wurde bettigt
        /// </summary>
        private void ExplizitValidation_CheckedChanged(object sender, EventArgs e)
        {
            // Prfe Status
            if (_chkIndirectValidation.Checked)
            {
                // Explizite Validierung gewnscht
                this.AutoValidate = AutoValidate.Disable;
            }
            else
            {
                // Implizite Validierung gewnscht
                this.AutoValidate = AutoValidate.EnablePreventFocusChange;
            }
        }
    }
}