using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Klasse fr die Darstellung eines Balls.
    /// </summary>
    public sealed class CwlbBall : IDisposable
    {
        // Instanzdaten
        private PointF _ptCoordinate;
        private float _fDeltaX;
        private float _fDeltaY;
        private Color _colSurface;
        private float _fSize;
        private Brush _objBrush;
        private bool _bDisposed = false;
        private RectangleF _rectBall;
        private static Random _objRandom = new Random(DateTime.Now.Millisecond);

        #region Konstruktionn und Destruktion
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbBall.
        /// </summary>
        public CwlbBall(Size sizeBounds, int iMaLightWidth)
        {
            // Lokale Daten
            _colSurface = Color.Blue;

            // Startposition bestimmen
            _ptCoordinate = new Point(sizeBounds.Width / 2, sizeBounds.Height / 2);

            // Daten per Zufall bestimmen
            _fDeltaX = _objRandom.Next(1, 25);
            if (_objRandom.Next() % 2 == 0)
            {
                _fDeltaX *= -1;
            }
            _fDeltaY = _objRandom.Next(1, 25);
            if (_objRandom.Next() % 2 == 0)
            {
                _fDeltaY *= -1;
            }
            // Grsse bestimmen
            _fSize = _objRandom.Next(5, iMaLightWidth);

            // Erzeuge den Pen
            _objBrush = new SolidBrush(Color.FromArgb(
                _objRandom.Next(0, 255),
                _objRandom.Next(0, 255),
                _objRandom.Next(0, 255)));
        }

        /// <summary>
        /// Implementierung der Schnittstelle IDisposable
        /// </summary>
        public void Dispose()
        {
            // Kontrolle des Disposings
            if (_bDisposed)
            {
                // Bereits zerstrt, keine Aktion
                return;
            }
            else
            {
                // Noch zu zerstren
                _objBrush.Dispose();
                _objBrush = null;
                _bDisposed = true;
            }
        }
        #endregion

        /// <summary>
        /// Berechnet die nchste Position des Balls
        /// </summary>
        /// <param name="rectBounds">
        /// Grenzen des Rechtecks innerhalb deren sich der Ball bewegen darf.
        /// </param>
        public void Move(Size sizeBounds)
        {
            // Prfen der neuen Koordinate
            // linker Rand
            if (_ptCoordinate.X - _fSize + _fDeltaX < 0.0f)
            {
                if (_fDeltaX < 0) _fDeltaX *= -1;
            }
            // oberer Rand
            if (_ptCoordinate.Y - _fSize + _fDeltaY < 0.0f)
            {
                if (_fDeltaY < 0) _fDeltaY *= -1.0f;
            }
            // recher Rand
            if (_ptCoordinate.X + _fSize + _fDeltaX > sizeBounds.Width)
            {
                if (_fDeltaX > 0) _fDeltaX *= -1.0f;
            }
            // unterer Rand
            if (_ptCoordinate.Y + _fSize + _fDeltaY > sizeBounds.Height)
            {
                if (_fDeltaY > 0) _fDeltaY *= -1.0f;
            }

            // Neuer Standord berechnen
            _ptCoordinate.X += _fDeltaX;
            _ptCoordinate.Y += _fDeltaY;

            // Ausgaberechteck berechnen
            _rectBall = new RectangleF(
                _ptCoordinate.X - _fSize,
                _ptCoordinate.Y - _fSize,
                _fSize * 2,
                _fSize * 2);
        }

        /// <summary>
        /// Zeichnet den Ball auf der Fensteroberflche
        /// </summary>
        /// <param name="objGraphics">GDI+ Objekt.</param>
        public void Draw(Graphics objGraphics)
        {
            // Zeichne den Ball
            objGraphics.FillEllipse(_objBrush, _rectBall);
            objGraphics.DrawEllipse(Pens.Black, _rectBall);
        }
    }
}
