﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Drawing.Imaging;
using System.Drawing.Text;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die Demo für die Zwischenablage.
    /// </summary>
    public partial class CmdiClipboard : Form
    {
        // Daten die mit der Zwischenablage gezeigt werden
        private string _strData;
        private Image _imgData;

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CmdiClipboard.
        /// </summary>
        public CmdiClipboard()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlungbeim Laden des Fensters.
        /// </summary>
        private void CmdiClipboard_Load(object sender, EventArgs e)
        {
            // Standardformate abfragen
            foreach(FieldInfo objInfo in typeof(DataFormats).GetFields(BindingFlags.Static | BindingFlags.Public))
            {
                int i = _clbFormat.Items.Add(objInfo.GetValue(null).ToString());
            }

            // Eigenes Format hinzufügen
            _clbFormat.Items.Add(typeof(CwlbClipboardHelper).Name);
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche Text wurde betätigt.
        /// </summary>
        private void ButtonText_Click(object sender, EventArgs e)
        {
            Clipboard.SetText("Demonstration der Zwischenablage.");
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche Bild in Zwischenablage einfügen wurde betätigt.
        /// </summary>
        private void ButtonSetOwnBitmap_Click(object sender, EventArgs e)
        {
            Clipboard.SetImage(Properties.Resources.Smiley);
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche eigene Daten in Zwischenablage einfügen wurde betätigt.
        /// </summary>
        private void ButtonOwnData_Click(object sender, EventArgs e)
        {
            // Datenobjekt erstellen und mit eigenen Daten initialisieren
            DataObject objData = new DataObject();
            CwlbClipboardHelper objOwnData = new CwlbClipboardHelper();
            objData.SetText(objOwnData.ToString());
            objData.SetImage(Properties.Resources.Smiley);
            objData.SetData(typeof(CwlbClipboardHelper).FullName, true, objOwnData);
            // Daten an Zwischenablage übergeben
            Clipboard.SetDataObject(objData);
        }

        /// <summary>
        /// Ereignisbehandlung Schaltfläche Löschen wurde betätigt.
        /// </summary>
        private void ButtonClear_Click(object sender, EventArgs e)
        {
            Clipboard.Clear();
        }

        /// <summary>
        /// Daten aus der Zwischenablage lesen.
        /// </summary>
        private void ButtonFromClipboard_Click(object sender, EventArgs e)
        {
            // Ausgewähltes Format verwenden
            int iIndex = _clbFormat.SelectedIndex;
            if (!_clbFormat.GetItemChecked(iIndex))
            {
                MessageBox.Show("Dieses Format ist nicht im Clipboard vorhanden", CwlbMainForm.TITLE);
                return;
            }

            // Format auslesen
            string strFormat = _clbFormat.SelectedItem.ToString();

            // Inhalt zurücksetzen
            _strData = null;
            _imgData = null;

            // Format auswerten und übernehmen
            switch (strFormat)
            {
                default:
                    string strOut = string.Format("Ihr ausgewähltes Format ist: '{0}'." +
                        "{1}Die Demo unterstützt nur Text oder Bildformate.", strFormat, Environment.NewLine);
                    MessageBox.Show(strOut, CwlbMainForm.TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    break;
                case "System.String":
                case "UnicodeText":
                case "Text":
                case "HTML Format":
                case "Rich Text Format":
                    _strData = Clipboard.GetData(strFormat) as string;
                    break;
                case "PNG":
                case "GIF":
                case "System.Drawing.Bitmap":
                case "Bitmap":
                case "DeviceIndependentBitmap":
                    _imgData = Clipboard.GetData(DataFormats.Bitmap) as Image;
                    break;
                case "EnhancedMetafile":
                    _imgData = Clipboard.GetData(DataFormats.EnhancedMetafile) as Metafile;
                    break;
                case "MetaFilePict":
                    _imgData = Clipboard.GetData(DataFormats.MetafilePict) as Metafile;
                    break;
                case "CwlbClipboardHelper":
                    IDataObject objData = Clipboard.GetDataObject();
                    object obj = objData.GetData(typeof(CwlbClipboardHelper).FullName);
                    _strData = Clipboard.GetData(typeof(CwlbClipboardHelper).FullName).ToString();
                    break;
            }

            // Fläche neu zeichnen
            _pnlShow.Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung zum aktualisieren der Anzeige der Formate für die Zwischenablage
        /// </summary>
        private void _timerRefresh_Tick(object sender, EventArgs e)
        {
            RefreshFormats();            
        }

        /// <summary>
        /// Ereneuert die Anzeige der enthaltenen Datenformate in der Zwischenablage.
        /// </summary>
        private void RefreshFormats()
        {
            // Lokale Daten          
            string strFormat;
            int iIndex;

            // Check bei allen Elementen ausschalten
            for( iIndex = 0; iIndex < _clbFormat.Items.Count; iIndex++)
            {
                _clbFormat.SetItemChecked(iIndex, false);
            }

            // Standardformate abfragen
            foreach(FieldInfo objInfo in typeof(DataFormats).GetFields(BindingFlags.Static | BindingFlags.Public))
            {
                strFormat = objInfo.GetValue(null).ToString();
                if (Clipboard.ContainsData(strFormat))
                {
                    // Prüfen, ob das Element schon in der Liste ist
                    iIndex = _clbFormat.FindStringExact(strFormat);
                    _clbFormat.SetItemChecked(iIndex, true);
                }
            }

            // Eigenes Format prüfen
            if (Clipboard.ContainsData(typeof(CwlbClipboardHelper).FullName))
            {
                iIndex = _clbFormat.FindStringExact(typeof(CwlbClipboardHelper).Name);
                _clbFormat.SetItemChecked(iIndex, true);
            }

            iIndex = _clbFormat.SelectedIndex;
            if (iIndex == -1)
            {
                _btnFromClipboard.Enabled = false;
            }
            else
            {
                _btnFromClipboard.Enabled = true;
            }
        }

        /// <summary>
        /// Ereignisbahndlung für das Zeichnen der Ausgabefläche.
        /// </summary>
        private void PanelShow_Paint(object sender, PaintEventArgs e)
        {
            using (Font font = new Font("Comic Sans MS", 18.0f))
            {
                e.Graphics.TextRenderingHint = TextRenderingHint.ClearTypeGridFit;
                if (_strData != null)
                {
                    // Textdaten werden prioritär ausgegeben
                    e.Graphics.DrawString(_strData, font, Brushes.Navy, _pnlShow.ClientRectangle);
                }
                else if (_imgData != null)
                {
                    // Bilddaten ausgeben
                    e.Graphics.DrawImage(_imgData, _pnlShow.ClientRectangle, new Rectangle(0, 0, _imgData.Width, _imgData.Height), GraphicsUnit.Pixel);
                }
                else
                {
                    // Weder Bilddatennoch Textdaten, Ersatztext anzeigen
                    e.Graphics.DrawString("Oops! Keine Daten!", font, Brushes.Navy, _pnlShow.ClientRectangle);
                }
            }
        }
    }
}
