﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Imaging;
using System.Drawing.Text;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die Demo für Drag & Drop in Windows Forms.
    /// </summary>
    public partial class CmdiDragDrop : Form
    {
        // Daten die mit D&D in die Anwendung gezogen wurden
        private string _strData;
        private Image _imgData;

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CmdiDragDrop.
        /// </summary>
        public CmdiDragDrop()
        {
            InitializeComponent();

            // Initialisiern der Daten
            _panelLeft.Tag = new CwlbDragDropHelper
            {
                Title = "Drag&Drop Helper",
                LastDrop = DateTime.Now
            };
        }

        /// <summary>
        /// Ereignisbehandlung beim Laden des Formulars.
        /// </summary>
        private void CmdiDragDrop_Load(object sender, EventArgs e)
        {
            ShowData(_panelLeft, _panelLeft.Tag as CwlbDragDropHelper);
            _panelLeft.BackColor = Color.DarkGreen;
            _panelRight.BackColor = Color.Red;
        }

        /// <summary>
        /// Ereignisbehandlung für Drag&Drop von Dateinamen aus dem Datei-Explorer.
        /// </summary>
        private void ListBoxFiles_DragDrop(object sender, DragEventArgs e)
        {
            // Dateinamen aus den D&D-Informationen übernehmen
            string[] astrFileData = (string[])e.Data.GetData(DataFormats.FileDrop);
            if (astrFileData.Length > 0)
            {
                _lbFiles.Items.Clear();
                foreach (string strFileName in astrFileData)
                {
                    _lbFiles.Items.Add(Path.GetFileName(strFileName));
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für Drag&Drop von Dateinamen aus dem Datei-Explorer.
        /// </summary>
        private void ListBoxFiles_DragOver(object sender, DragEventArgs e)
        {
            string[] astrFileData = (string[])e.Data.GetData(DataFormats.FileDrop);
            if (astrFileData.Length > 0)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        /// <summary>
        /// Ereignisbehandlung bei Bewegung der Maus auf einem der beiden Panel unten links in der Demo.
        /// </summary>
        private void PanelDragDrop_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button.HasFlag(MouseButtons.Left) && ModifierKeys.HasFlag(Keys.Control) && ModifierKeys.HasFlag(Keys.Shift))
            {
                // Beginne D&D-Operation, wenn die Maus mit linker Maustaste ausgeführt wird,
                // und gleichzeitig die Tasten Ctrl und Shift betätigt werden
                DataObject objData = new DataObject();
                objData.SetData(typeof(CwlbDragDropHelper), ((Control)sender).Tag);         // Eigenes Datenformat
                objData.SetText(((Control)sender).Tag.ToString(), TextDataFormat.Text);     // Textformat
                objData.SetImage(Properties.Resources.Smiley);                              // Bildformat
                DragDropEffects enmDoneEffect = DoDragDrop(objData, DragDropEffects.Copy | DragDropEffects.Move);
                if (enmDoneEffect == DragDropEffects.Move)
                {
                    MessageBox.Show("Daten wurden verschoben.", CwlbMainForm.TITLE);
                }
            }
            else if (e.Button.HasFlag(MouseButtons.Left))
            {
                // Beginne D&D-Operation wenn die Maus mit linker Maustaste ausgeführt wird
                DoDragDrop(((Control)sender).Tag, DragDropEffects.Move);
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Ziehen der Daten mit der Maus zwischen den Panels unten links.
        /// </summary>
        private void PanelDragDrop_DragOver(object sender, DragEventArgs e)
        {
            // Einfache Prüfung über Daten
            CwlbDragDropHelper objData = e.Data.GetData(typeof(CwlbDragDropHelper)) as CwlbDragDropHelper;
            if (objData != null)
            {
                e.Effect = DragDropEffects.Move;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Fallenlassen der Daten auf das rechte Panel.
        /// </summary>
        private void PanelRight_DragDrop(object sender, DragEventArgs e)
        {
            CwlbDragDropHelper objData = e.Data.GetData(typeof(CwlbDragDropHelper)) as CwlbDragDropHelper;
            if (objData != null)
            {

                // Daten verändert
                objData.LastDrop = DateTime.Now;
                objData.AmountDrops++;

                // Ziel und Quelle vertauschen
                _panelRight.Tag = objData;
                _panelRight.BackColor = Color.DarkGreen;
                _panelRight.AllowDrop = false;
                _panelLeft.Tag = null;
                _panelLeft.BackColor = Color.Red;
                _panelLeft.AllowDrop = true;

                e.Effect = DragDropEffects.Move;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Fallenlassen der Daten auf das linke Panel.
        /// </summary>
        private void PanelLeft_DragDrop(object sender, DragEventArgs e)
        {
            CwlbDragDropHelper objData = e.Data.GetData(typeof(CwlbDragDropHelper)) as CwlbDragDropHelper;
            if (objData != null)
            {
                // Daten verändert
                objData.LastDrop = DateTime.Now;
                objData.AmountDrops++;

                // Ziel und Quelle vertauschen
                _panelLeft.Tag = objData;
                _panelLeft.BackColor = Color.DarkGreen;
                _panelLeft.AllowDrop = false;
                _panelRight.Tag = null;
                _panelRight.BackColor = Color.Red;
                _panelRight.AllowDrop = true;
            }
        }

        /// <summary>
        /// Zeigt die Informationen im D&D-Panel unten an.
        /// </summary>
        private void ShowData(Control objControl, CwlbDragDropHelper objData)
        {
            string strOut = string.Format("Die Information {0} wurde am {1} zum {2}. Mal verschoben.", objData.Title, objData.LastDrop, objData.AmountDrops, Environment.NewLine);
            using (Font font = new Font("Comic Sans MS", 10.0f))
            using (Graphics objGraphics = objControl.CreateGraphics())
            {
                objGraphics.TextRenderingHint = TextRenderingHint.ClearTypeGridFit;
                objGraphics.DrawString(strOut, font, Brushes.Yellow, objControl.ClientRectangle);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Neuzeichnen der Panel unten links in der Demo.
        /// </summary>
        private void Panel_Paint(object sender, PaintEventArgs e)
        {
            if (sender is Control && ((Control)sender).Tag != null)
            {
                ShowData(sender as Control, ((Control)sender).Tag as CwlbDragDropHelper);
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Ziehen der Daten auf das Panel rechts.
        /// </summary>
        private void PanelTypedTarget_DragOver(object sender, DragEventArgs e)
        {
            // Prüfen, ob Formate angezeigt werden
            string[] astrFormats = e.Data.GetFormats();
            if (astrFormats.Length > 0)
            {
                e.Effect = DragDropEffects.Copy;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Fallenlassen von typisierten Daten auf der Fläche rechts.
        /// </summary>
        private void PanelTypedTarget_DragDrop(object sender, DragEventArgs e)
        {
            // Lokale Daten
            string strFormat = null;

            // Dem Absender signalisieren, dass wir nichts tun. Das ist 
            // notwendig, weil die Methode mehrere entsprechende Ausgänge hat.
            e.Effect = DragDropEffects.None;

            // Prüfen der akzeptierten Formate
            string[] astrFormats = e.Data.GetFormats();
            if (astrFormats.Length == 0)        // Keine Formate gefunden, sofort abbrechen
            {
                return;
            }
            else if (astrFormats.Length == 1)   // Nur ein Format gefunden, dieses übernehmen
            {
                strFormat = astrFormats[0];
            }
            else                                // Mehrere Formate gefunden, Auswahldialogfeld anzeigen
            {
                CdlgFormatChooser dlgFormat = new CdlgFormatChooser();
                dlgFormat._lbFormat.Items.AddRange(astrFormats);
                if (dlgFormat.ShowDialog() == DialogResult.OK)
                {
                    // Ausgewähltes Format übernehmen
                    strFormat = dlgFormat._lbFormat.SelectedItem.ToString();
                }
            }

            // Kein Format ausgewählt, Behandlung beenden
            if (string.IsNullOrEmpty(strFormat))
            {
                return;
            }

            // Daten übernehmen
            _strData = null;
            _imgData = null;
            switch (strFormat)
            {
                default:
                    string strOut = string.Format("Ihr ausgewähltes Format ist: '{0}'.{1}Die Demo unterstützt nur Text oder Bildformate.", strFormat, Environment.NewLine);
                    MessageBox.Show(strOut, CwlbMainForm.TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    break;
                case "System.String":
                case "UnicodeText":
                case "Text":
                case "HTML Format":
                case "Rich Text Format":
                    _strData = e.Data.GetData(strFormat) as string;
                    break;
                case "PNG":
                case "GIF":
                case "System.Drawing.Bitmap":
                case "Bitmap":
                    _imgData = e.Data.GetData(DataFormats.Bitmap) as Image;
                    break;
                case "EnhancedMetafile":
                    _imgData = e.Data.GetData(DataFormats.EnhancedMetafile) as Metafile;
                    break;
                case "MetaFilePict":
                    _imgData = e.Data.GetData(DataFormats.MetafilePict) as Metafile;
                    break;
                case "WeroSoft.Samples.CwlbDragDropHelper":
                    _strData = e.Data.GetData(typeof(CwlbDragDropHelper)).ToString();
                    break;
            }

            // Dem Absender signalisieren dass wir kopiert haben
            e.Effect = DragDropEffects.Copy;
            // Fläche neu zeichnen
            _panelTypedTarget.Invalidate();
        }

        /// <summary>
        /// Ereignisbehandlung zum Zeichnen der Demofläche rechts.
        /// </summary>
        private void PanelTypedTarget_Paint(object sender, PaintEventArgs e)
        {
            if (_strData != null)
            {
                // Textdaten werden prioritär ausgegeben
                e.Graphics.DrawString(_strData, SystemFonts.DialogFont, Brushes.Navy, _panelTypedTarget.ClientRectangle);
            }
            else if (_imgData != null)
            {
                // Bilddaten ausgeben
                e.Graphics.DrawImage(_imgData, _panelTypedTarget.ClientRectangle, new Rectangle(0, 0, _imgData.Width, _imgData.Height), GraphicsUnit.Pixel);
            }
            else
            {
                // Weder Bilddatennoch Textdaten, Ersatztext anzeigen
                e.Graphics.DrawString("Oops! Keine Daten!", SystemFonts.DialogFont, Brushes.Navy, _panelTypedTarget.ClientRectangle);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Verändern der Grösse des Panels rechts
        /// </summary>
        private void PanelTypedTarget_SizeChanged(object sender, EventArgs e)
        {
            Invalidate();
        }
    }
}
