﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Globalization;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert die Demo für die Anwendung der Loaklisierung von Forms Anwendungen.
    /// </summary>
    public partial class CmdiLocalization : Form
    {
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CmdiLocalization.
        /// </summary>
        public CmdiLocalization()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Prüft ob der Sprachwechsel einen Neustart der Anwendung verursacht.
        /// </summary>
        /// <param name="objNewCulture"></param>
        private static void CheckLanguageChange(CultureInfo objNewCulture)
        {
            // Gewünschte Sprache definieren
            if (!objNewCulture.CompareInfo.Equals(Thread.CurrentThread.CurrentUICulture.CompareInfo))
            {
                if (MessageBox.Show(Properties.Resources.LanguageCheck, CwlbMainForm.TITLE, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    Program.CurrentUICultureInfo = objNewCulture;
                    Program.CurrentState = EwlbRunState.Run;
                    CwlbMainForm.ReloadLanguagePage = true;
                    CwlbMainForm.MainWindow.Close();
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Laden der Seite
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CmdiLocalization_Load(object sender, EventArgs e)
        {
            // Label initialisieren
            _lblTitle.Text = Properties.Resources.Title;
            _lblDefaultLanguage.Text = Properties.Resources.DefaultLanguageLabel;
            _lblThreadLanguage.Text = Properties.Resources.ThreadLanguageLabel;
            _lblResourceLanguage.Text = Properties.Resources.ResourceLanguageLabel;
            _lblCurrency.Text = Properties.Resources.CurrencyLabel;
            _lblNumber.Text = Properties.Resources.NumberLabel;
            _lblDateTime.Text = Properties.Resources.DateTimeLabel;
            _lblDemoText.Text = Properties.Resources.DemoTextLabel;

            _tabThread.Text = Properties.Resources.TabPageThread;
            _tabResource.Text = Properties.Resources.TabPageResources;
            
            // Werte initialisieren
            _tbDefaultLanguage.Text = CultureInfo.InstalledUICulture.ToString();
            _tbThreadLanguage.Text = Thread.CurrentThread.CurrentCulture.ToString();
            _tbResourceLanguage.Text = Thread.CurrentThread.CurrentUICulture.ToString();
            _tbCurrency.Text = Thread.CurrentThread.CurrentCulture.NumberFormat.CurrencySymbol;
            _tbNumber.Text = string.Format("{0:N2}", 567834.98);
            _tbDateTime.Text = DateTime.Now.ToString();
            _tbDemoText.Text = Properties.Resources.DemoText;

            // Flagge initialisieren
            _picCountry.Image = Properties.Resources.Flag;

            // Anzeige CulturInfos initialisieren
            _pgrThread.SelectedObject = Thread.CurrentThread.CurrentCulture;
            _pgrResource.SelectedObject = Thread.CurrentThread.CurrentUICulture;

            // Einstellung Menü
            _mnuLanguage_de_CH.Checked = false;
            _mnuLanguage_de_DE.Checked = false;
            _mnuLanguage_en_US.Checked = false;
            _mnuLanguage_fr.Checked = false;

            switch (Program.CurrentUICultureInfo.ToString())
            {
                case "de-CH":
                    _mnuLanguage_de_CH.Checked = true;
                    break;
                case "de-DE":
                    _mnuLanguage_de_DE.Checked = true;
                    break;
                case "en-US":
                    _mnuLanguage_en_US.Checked = true;
                    break;
                case "fr":
                    _mnuLanguage_fr.Checked = true;
                    break;
                default:
                    break;
            }
        }


        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Sprachwahl im Menu.
        /// </summary>
        private void MenuLanguage_Click(object sender, EventArgs e)
        {
            ToolStripDropDownItem objItem = sender as ToolStripDropDownItem;
            if (objItem != null)
            {
                CultureInfo objNewCulture = new System.Globalization.CultureInfo(objItem.Tag.ToString());
                CheckLanguageChange(objNewCulture);
            }
        }
    }
}
