﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert die Demo für die Anwendung eigener Steuerelemente.
    /// </summary>
    public partial class CmdiOwnControl : Form
    {
        // Konstante Daten
        private const string LABEL_CONTENT = "CPU Privileged Time {0:F1} [%] - CPU User Time {1:F1} [%]";

        // Instanzdaten
        private int iCount = 0;

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CmdiOwnControl.
        /// </summary>
        public CmdiOwnControl()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung für das Laden der eigenen Controls
        /// </summary>
        private void CmdiOwnControl_Load(object sender, EventArgs e)
        {
            // Lokale Daten
            List<PerformanceCounter> cobjPerformanceCounter = new List<PerformanceCounter>();
            int iCPUCount = -1;

            // Ausnahmebehandlung
            try
            {
                // Jedes Steuerelement des Typs CwlbLightChain initalisieren
                foreach (CwlbLightChain objControl in this.Controls
                    .OfType<CwlbLightChain>()
                    .OrderBy( objControl => objControl.InstanceNumber))
                {
                    // Initialisieren der Steuerelemente
                    objControl.PercentageLight = 0;
                    objControl.PercentageHalfLight = 0;

                    // Prüfen, ob die Maschine über soviele Prozessoren
                    // verfügt, wie gebraucht werden
                    if (iCPUCount >= Environment.ProcessorCount)
                    {
                        objControl.Enabled = false;
                        continue;
                    }

                    // Performancecounter instanziieren
                    objControl.Tag = new List<PerformanceCounter>() {
                        new PerformanceCounter( objControl.ComponentName,
                            "% Privileged Time", objControl.InstanceName, true),
                        new PerformanceCounter(objControl.ComponentName, 
                            "% User Time", objControl.InstanceName, true)
                    };

                    // Anzahl Konfigurierte CPU's inkrementieren
                    iCPUCount++;
                }

                // Starte Timer
                _timerCPU.Start();
            }
            catch
            {
                this._lblCpuValue.Text = "Der Benutzer hat nicht" +
                    " genügend Rechte CPU-Leistungsdaten zu lesen!";
                this._lblCpuValue.ForeColor = Color.Red;
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Schliessen der Demo.
        /// </summary>
        private void CmdiOwnControl_FormClosing(object sender, FormClosingEventArgs e)
        {
            // lokale Daten
            List<PerformanceCounter> cobjPerformanceCounter;
            // Timer stoppen
            _timerCPU.Stop();
            // Ressourcen freigeben
            foreach (CwlbLightChain objControl in this.Controls.OfType<CwlbLightChain>())
            {
                cobjPerformanceCounter = objControl.Tag as List<PerformanceCounter>;
                if (cobjPerformanceCounter != null)
                {
                    cobjPerformanceCounter[0].Dispose();
                    cobjPerformanceCounter[1].Dispose();
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für den Ablauf des Timerintervalls
        /// </summary>
        private void _ctrCpuUserTime_Tick(object sender, EventArgs e)
        {
            // lokale Daten
            List<PerformanceCounter> cobjPerformanceCounter;
            float fCpuPrivilegedTime;
            float fCpuUserTime;

            // Prüfen, ob Anzeige erfolgen soll
            iCount++;
            if (_rbLow.Checked)
            {
                if (iCount % 10 == 0)
                {
                    iCount = 0;
                }
                else
                {
                    return;
                }
            }
            else if (_rbMid.Checked)
            {
                if (iCount % 5 == 0)
                {
                    iCount = 0;
                }
                else
                {
                    return;
                }
            }

            // Alle Steuerelemente des Typs CwlbLightChain aktualisieren
            foreach (CwlbLightChain objControl in this.Controls.OfType<CwlbLightChain>())
            {
                cobjPerformanceCounter = objControl.Tag as List<PerformanceCounter>;
                if (cobjPerformanceCounter != null)
                {
                    fCpuPrivilegedTime = cobjPerformanceCounter[0].NextValue();
                    fCpuUserTime = cobjPerformanceCounter[1].NextValue();

                    // set light processor
                    objControl.PercentageLight = fCpuPrivilegedTime;
                    objControl.PercentageHalfLight = fCpuUserTime;
                    if (objControl.InstanceName == "_Total")
                    {
                        _lblCpuValue.Text = string.Format(
                            "CPU Privileged Time {0:F1} [%] - CPU User Time {1:F1} [%]",
                            fCpuPrivilegedTime, fCpuUserTime);
                    }
                    objControl.RefreshGraphics();
                }
            }
        }
    }
}
