﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using System.Globalization;
using System.Threading;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Repräsentiert die Demo für die erweiterte Nutzung der Windows Forms-Technologien.
    /// </summary>
    static class Program
    {
        /// <summary>
        /// Liefert oder definiert die aktuell eingestellte Sprache für das Auflösen von Ressourcen.
        /// </summary>
        public static CultureInfo CurrentUICultureInfo { get; set; }

        /// <summary>
        /// Definiert den aktuellen Zustand der Anwendung. Dieser Zustand wird für die Steuerung
        /// des Neustarts bei dynamischer Spracheinstellung verwendet.
        /// </summary>
        public static EwlbRunState CurrentState { get; set; }

        /// <summary>
        /// Der Eintrittspunkt in die Anwendung.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // Sprache initialisieren
            CurrentUICultureInfo = Thread.CurrentThread.CurrentUICulture;

            // Standardinitialisierungen (generierte)
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            // Solange der Zustand Run ist, das Programm immer wieder starten
            while (CurrentState == EwlbRunState.Run)
            {
                // Voreinstellung für das Abbrechen der Verarbeitung bei Beendigung.
                CurrentState = EwlbRunState.Stop;
                // Spracheinstellung vornehmen
                if (CurrentUICultureInfo != Thread.CurrentThread.CurrentUICulture)
                {
                    // Nur die Ressourcensprache umstellen
                    if (!CurrentUICultureInfo.IsNeutralCulture)
                    {
                        Thread.CurrentThread.CurrentCulture = CurrentUICultureInfo;
                    }
                    Thread.CurrentThread.CurrentUICulture = CurrentUICultureInfo;
                }
                Application.Run(new CwlbMainForm());
            }
        }
    }
}
