using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Demo Klasse fr eine Lichterkette
    /// </summary>
    public partial class CwlbLightChain : Control
    {
        private const string CATEGORY_BEHAVIOR = "Behavior";
        private const string CATEGORY_APPEARANCE = "Appearance";
        private const string CATEGORY_DATA = "Data";

        #region Properties
        /// <summary>
        /// Liefert oder definiert den Namen der Komponente.
        /// </summary>
        [Category(CATEGORY_DATA)]
        [Description("Liefert oder definiert den Namen der Komponente. Diese Eigenschaft " + 
            "hat keine interne Verwendung, sondern dient der Aufnahme benutzerspezifischer Daten.")]
        public string ComponentName { get; set; }

        /// <summary>
        /// Liefert oder definiert den Namen des Werts der Komponente.
        /// </summary>
        [Category(CATEGORY_DATA)]
        [Description("Liefert oder definiert den Namen des Werts der Komponente. Diese Eigenschaft " +
            "hat keine interne Verwendung, sondern dient der Aufnahme benutzerspezifischer Daten.")]
        public string ValueName { get; set; }

        /// <summary>
        /// Liefert oder definiert den Namen der Instanz der Komponente.
        /// </summary>
        [Category(CATEGORY_DATA)]
        [Description("Liefert oder definiert den Namen der Instanz der Komponente. Diese Eigenschaft " +
            "hat keine interne Verwendung, sondern dient der Aufnahme benutzerspezifischer Daten.")]
        public string InstanceName { get; set; }

        /// <summary>
        /// Liefert oder definiert eine laufende Nummer der Instanz der Komponente.
        /// </summary>
        [Category(CATEGORY_DATA)]
        [Description("Liefert oder definiert eine laufende Nummer der Instanz der Komponente. Diese Eigenschaft " +
            "hat keine interne Verwendung, sondern dient der Aufnahme benutzerspezifischer Daten.")]
        public int InstanceNumber { get; set; }

        /// <summary>
        /// Liefert oder definiert die Anzahl der Lampen.
        /// </summary>
        [Category(CATEGORY_BEHAVIOR)]
        [DefaultValue(100)]
        [Description("Liefert oder definiert die Anzahl der Lampen in der gesamten Breite")]
        public int CountLights { get; set; }

        /// <summary>Liefert oder definiert die horizontale Ausdehnung einer Lampe.</summary>
        /// <remarks>Diese Eigenschaft wird ignoriert, wenn DynamicSizeing true ist.</remarks>
        [Category(CATEGORY_BEHAVIOR)]
        [DefaultValue(4.0f)]
        [Description("Liefert oder definiert die horizontale Ausdehnung")]
        public float LightWidth { get; set; }

        /// <summary>Liefert oder definiert die vertikale Ausdehnung einer Lampe.</summary>
        /// <remarks>Diese Eigenschaft wird ignoriert, wenn DynamicSizeing true ist.</remarks>
        [Category(CATEGORY_BEHAVIOR)]
        [DefaultValue(20.0f)]
        [Description("Liefert oder definiert die vertikale Ausdehnung")]
        public float LightHeight { get; set; }

        /// <summary>Liefert oder definiert die prozentuale Anzahl der Lichter,
        /// die die volle Leuchtstrke aufweisen</summary>
        [Category(CATEGORY_BEHAVIOR)]
        [DefaultValue(10.0)]
        [Description("Liefert oder definiert die prozentuale Anzahl" +
            " der Lichter, die die volle Leuchtstrke aufweisen.")]
        public double PercentageLight { get; set; }

        /// <summary>Liefert oder definiert die prozentuale Anzahl der Lichter,
        /// die die halbe Leuchtstrke aufweisen</summary>
        [Category(CATEGORY_BEHAVIOR)]
        [DefaultValue(10.0)]
        [Description("Liefert oder definiert die prozentuale Anzahl" +
            " der Lichter, die die halbe Leuchtstrke aufweisen.")]
        public double PercentageHalfLight { get; set; }

        /// <summary>Liefert oder definiert die Randfarbe der dunklen
        /// Lichter.</summary>
        [Category(CATEGORY_APPEARANCE)]
        [Description("Liefert oder definiert die Randfarbe" +
            " der Lichter mit dunkler Leuchtstrke.")]
        public Color ColorNormal { get; set; }

        /// <summary>Liefert oder definiert die Randfarbe der Lichter
        /// mit halber Leuchtstrke.</summary>
        [Category(CATEGORY_APPEARANCE)]
        [Description("Liefert oder definiert die Randfarbe" +
            " der Lichter mit halber Leuchtstrke.")]
        public Color ColorHalfLight { get; set; }

        /// <summary>Liefert oder definiert die Randfarbe der Lichter
        /// mit voller Leuchtstrke.</summary>
        [Category(CATEGORY_APPEARANCE)]
        [Description("Liefert oder definiert die Randfarbe" +
            " der Lichter mit voller Leuchtstrke.")]
        public Color ColorLight { get; set; }

        /// <summary>Liefert oder definiert die Farbe in der Mitte der
        /// Lichter mit dunkler Leuchtstrke.</summary>
        [Category(CATEGORY_APPEARANCE)]
        [Description("Liefert oder definiert die Farbe in der Mitte" +
            " der Lichter mit dunkler Leuchtstrke.")]
        public Color ColorNormalCenter { get; set; }

        /// <summary>Liefert oder definiert die Farbe in der Mitte der
        /// Lichter mit halber Leuchtstrke.</summary>
        [Category(CATEGORY_APPEARANCE)]
        [Description("Liefert oder definiert die Farbe in der Mitte" +
            " der Lichter mit halber Leuchtstrke.")]
        public Color ColorHalfLightCenter { get; set; }

        /// <summary>Liefert oder definiert die Farbe in der Mitte der
        /// Lichter mit voller Leuchtstrke.</summary>
        [Category(CATEGORY_APPEARANCE)]
        [Description("Liefert oder definiert die Farbe in der Mitte" +
            " der Lichter mit voller Leuchtstrke.")]
        public Color ColorLightCenter { get; set; }

        /// <summary>Liefert oder definiert true, wenn dynamische
        /// Grenanpassung erlaubt ist.</summary>
        [Category(CATEGORY_BEHAVIOR)]
        [DefaultValue(true)]
        [Description("Liefert oder definiert true, wenn dynamische" +
            " Grenanpassung erlaubt ist")]
        public bool DynamicSizeing { get; set; }
        #endregion

        #region Construction / Destruction
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbLightChain
        /// </summary>
        public CwlbLightChain()
        {
            // Definieren von Standradwerten fr Farben
            CountLights = 100;
            PercentageLight = 10.0;
            PercentageHalfLight = 10.0;
            LightHeight = 20.0f;
            LightWidth = 4.0f;
            ColorNormal = Color.Green;
            ColorHalfLight = Color.DarkOrange;
            ColorLight = Color.Red;
            ColorNormalCenter = Color.LightGreen;
            ColorHalfLightCenter = Color.LightYellow;
            ColorLightCenter = Color.White;
            DynamicSizeing = true;

            // Initialisierung
            InitializeComponent();
            // Doppelpufferung einschalten
            this.DoubleBuffered = true;
        }
        #endregion

        #region Several supporting methods
        /// <summary>
        /// Methode zur kommerziellen Rundung
        /// </summary>
        private double RoundCommercial(double dValue, int iDigits)
        {
            int dSign = Math.Sign(dValue);
            dValue = Math.Abs(dValue);
            double dFactor = Math.Pow(10, iDigits);
            dValue *= dFactor;
            dValue += 0.5;
            return dSign * Math.Floor(dValue) / dFactor;
        }
        #endregion

        #region Paint methods
        /// <summary>
        /// Neuzeichnen der grafischen Ansicht
        /// </summary>
        public void RefreshGraphics()
        {
            // Graphics-Objekt herstellen und zeichnen
            using (Graphics objGraphics = this.CreateGraphics())
            {
                Refresh(objGraphics);
            }
        }

        /// <summary>
        /// Neuzeichnen der grafischen Ansicht
        /// </summary>
        /// <param name="objGraphics">Grafikkontext</param>
        public void Refresh(Graphics objGraphics)
        {
            // Lokale Daten
            Color clrUsage;
            Color clrCenterUsage;
            float fXPos = 0;
            int iCount = 0;

            // Brechnen der Verteilung der Lichter 
            int iLights = (int)RoundCommercial(
                (double)CountLights / 100.0 * PercentageLight,
                0);
            int iHalfLights = (int)RoundCommercial(
                (double)CountLights / 100.0 * PercentageHalfLight,
                0);

            // Dynamische Grssenberechnung
            if (DynamicSizeing)
            {
                // X- und Y-Ausdehnung berechnen
                LightWidth = ((float)this.Width / (float)CountLights);
                LightHeight = this.Height;
            }

            // Schleife durch Elemente
            while (iCount < CountLights)
            {

                // Prfe Farbbenutzung
                if (iCount < iLights)
                {
                    // Volle Lichtstrke
                    clrUsage = ColorLight;
                    clrCenterUsage = ColorLightCenter;
                }
                else
                {
                    if (iCount < iHalfLights + iLights)
                    {
                        // Halbe Lichtstrke
                        clrUsage = ColorHalfLight;
                        clrCenterUsage = ColorHalfLightCenter;
                    }
                    else
                    {
                        // Dunkle Lichtstrke
                        clrUsage = ColorNormal;
                        clrCenterUsage = ColorNormalCenter;
                    }
                }

                // Erzeuge Pfadobjekt
                using (GraphicsPath graphPath = new GraphicsPath())
                {
                    fXPos = iCount * LightWidth;
                    graphPath.AddRectangle(new RectangleF(fXPos, 0.0f, LightWidth, LightHeight));
                    using (PathGradientBrush brush = new PathGradientBrush(graphPath))
                    {
                        brush.CenterPoint = new PointF(fXPos + LightWidth * 0.6f, LightHeight * 0.3f);
                        brush.CenterColor = clrCenterUsage;
                        brush.SurroundColors = new Color[] { clrUsage };
                        objGraphics.FillRectangle(brush, fXPos, 0.0f, LightWidth, LightHeight - 1.0f);
                    }
                }

                // Anzahl inkrementieren
                iCount++;
            }
        }

        /// <summary>
        /// Ereignishandler Zeichnen der Grafik
        /// </summary>
        private void CLightChain_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            Refresh(e.Graphics);
        }
        #endregion
    }
}
