﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Windows;
using System.Windows.Threading;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionslogik für App.xaml
    /// </summary>
    public partial class App : Application
    {
        /// <summary>
        /// Wird beim Starten der Anwendung aufgerufen (vor der Darstellung des ersten Fensters)
        /// </summary>
        protected override void OnStartup(StartupEventArgs e)
        {
            // Registrieren der Ausnahmebehandlung für den Dispatcher der Anwendung
            this.DispatcherUnhandledException += new DispatcherUnhandledExceptionEventHandler(App_DispatcherUnhandledException);
            base.OnStartup(e);
        }

        /// <summary>
        /// Ereignisbehandlung für eine unbehandelte Ausnahme im Dispatcher der Anwendung
        /// </summary>
        void App_DispatcherUnhandledException(object sender, DispatcherUnhandledExceptionEventArgs e)
        {
            // Prüfen der anzuzeigenden Ausnahme
            Exception excShow = e.Exception.InnerException != null ? e.Exception.InnerException : e.Exception;

            // Verarbeitung der Ausnahme
            if (MessageBox.Show(excShow.Message, MainWindow.Title, MessageBoxButton.YesNo, MessageBoxImage.Error) == MessageBoxResult.Yes)
            {
                // Die Ausnahme als behandelt markieren
                e.Handled = true;
            }
            else
            {
                // Die Ausnahme weiterhin als unbehandelt markieren.
                // Achtung, diese Auswahl führt zur Beendigung der Anwendung.
                e.Handled = false;
            }
        }

        /// <summary>
        /// Wird im Lebenszyklus bei jeder Aktivierung der Anwendung aufgerufen (Anwendung erhält den Fokus).
        /// </summary>
        protected override void OnActivated(EventArgs e)
        {
            base.OnActivated(e);
        }

        /// <summary>
        /// Wird im Lebenszyklus bei jeder Deaktivierung der Anwendung aufgerufen (Anwendung verliert den Fokus)
        /// </summary>
        protected override void OnDeactivated(EventArgs e)
        {
            base.OnDeactivated(e);
        }

        /// <summary>
        /// Wird beim Beenden der Anwendung aufgerufen (Nach dem Schließen der Fenster).
        /// </summary>
        protected override void OnExit(ExitEventArgs e)
        {
            base.OnExit(e);
        }

        /// <summary>
        /// Wird beim Beenden der Sitzung des Benutzers (Abmelden) aufgerufen.
        /// </summary>
        protected override void OnSessionEnding(SessionEndingCancelEventArgs e)
        {
            base.OnSessionEnding(e);
        }
    }
}
