﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Collections.ObjectModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaction logic for CdlgEventDemo.xaml
    /// </summary>
    public partial class CdlgEventDemo : Window
    {
        /// <summary>
        /// Instanziert eine neue Instanz der Klasse CdlgEventDemo.
        /// </summary>
        public CdlgEventDemo()
        {
            AddData(".ctor", strToolTip:"Vor InitializeComponent()", strSearchName:"Ctor");
            InitializeComponent();
            AddData(".ctor", strToolTip: "Nach InitializeComponent()", strSearchName: "Ctor");
        }

        /// <summary>
        /// Fügt ein Ereignisobjekt in die Log-Liste ein.
        /// </summary>
        /// <param name="strMethod">Methodenname.</param>
        /// <param name="strType">Typ des Elements (Bubbeling, Tunneling ...)</param>
        /// <param name="strSender">Ansender des Ereignisses.</param>
        /// <param name="strSource">Quelle des Ereignisses.</param>
        /// <param name="strOriginalSource">Abgeänderte Quelle des Ereignisses.</param>
        /// <param name="strToolTip">Tooltip des Ereignisses.</param>
        /// <param name="strSearchName">Suchname des Ereignisses</param>
        public void AddData(string strMethod, string strType = null, string strSender = null, string strSource = null, string strOriginalSource = null, string strToolTip = null, string strSearchName = null)
        {
            CwlbEventData objData = new CwlbEventData();
            objData.Method = strMethod;
            objData.Type = strType;
            objData.Sender = strSender;
            objData.Source = strSource;
            objData.OriginalSource = strOriginalSource;
            objData.ToolTip = strToolTip;
            if (string.IsNullOrEmpty(strSearchName))
            {
                if (strMethod.StartsWith("Preview"))
                {
                    objData.SearchName = strMethod.Substring(7);
                }
                else
                {
                    objData.SearchName = strMethod;
                }
            }
            else
            {
                objData.SearchName = strSearchName;
            }
            ((MainWindow)Application.Current.MainWindow).Data.Add(objData);
        }

        /// <summary>
        /// Ereignisbehandlung für die Initialisierung.
        /// </summary>
        public void InitializedHandler(object sender, EventArgs e)
        {
            string strSender;
            if (sender is FrameworkElement)
            {
                strSender = ((FrameworkElement)sender).Name;
            }
            else
            {
                strSender = sender.GetType().Name;
            }
            AddData("Initialized", "CLR", strSender);
        }

        /// <summary>
        /// Generelle Ereignisbehandlung für alle gerouteten Ereignisse.
        /// </summary>
        public void GeneralHandler(object sender, RoutedEventArgs e)
        {
            // Lokale Daten
            string strSender;

            // Bestimmen des Namens des Senders
            if (sender is FrameworkElement)
            {
                strSender = ((FrameworkElement)sender).Name;
            }
            else
            {
                strSender = sender.GetType().Name;
            }
            AddData(e.RoutedEvent.Name, e.RoutedEvent.RoutingStrategy.ToString(), strSender, e.Source.GetType().Name, e.OriginalSource.GetType().Name);
        }
    }
}
