﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Diagnostics;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaction logic for CdlgEventDemo.xaml
    /// </summary>
    public partial class CdlgEventLog : Window
    {
        /// <summary>
        /// Referenz zum Hauptfenster.
        /// </summary>
        public MainWindow MainWindow { get; set; }

        /// <summary>
        /// Liste der CheckBox-Steuerelemente
        /// </summary>
        private List<CheckBox> _cobjChecker = new List<CheckBox>();

        /// <summary>
        /// Instanziert eine neue Instanz der Klasse CdlgEventLog.
        /// </summary>
        public CdlgEventLog()
        {
            InitializeComponent();

            // Liste der Checkboxen einfüllen
            _cobjChecker.Add(_chkPreview);
            _cobjChecker.Add(_chkCtor);
            _cobjChecker.Add(_chkInitialized);
            _cobjChecker.Add(_chkLoaded);
            _cobjChecker.Add(_chkUnloaded);
            _cobjChecker.Add(_chkMouseEnter);
            _cobjChecker.Add(_chkMouseDown);
            _cobjChecker.Add(_chkMouseUp);
            _cobjChecker.Add(_chkMouseLeave);
            _cobjChecker.Add(_chkMouseMove);
            _cobjChecker.Add(_chkClick);
            _cobjChecker.Add(_chkKeyDown);
            _cobjChecker.Add(_chkKeyUp);
            _cobjChecker.Add(_chkTextInput);
            _cobjChecker.Add(_chkGotFocus);
            _cobjChecker.Add(_chkLostFocus);
            _cobjChecker.Add(_chkGotKeyboardFocus);
            _cobjChecker.Add(_chkLostKeyboardFocus);
        }

        /// <summary>
        /// Ereignisbehandlung für das Löschen der aufgenommenen Daten.
        /// </summary>
        private void ButtonClear_Click(object sender, RoutedEventArgs e)
        {
            MainWindow.Data.Clear();
        }

        /// <summary>
        /// Filtermethode für das Anzeigen der ausgesuchten Daten.
        /// </summary>
        private void CollectionViewSource_Filter(object sender, FilterEventArgs e)
        {
            // Ereignisargument für die Behandlung umwandeln
            CwlbEventData objItem = e.Item as CwlbEventData;
            Trace.WriteLine(objItem.ToString());

            // Preview Element filtern
            if (!_chkPreview.IsChecked.GetValueOrDefault() && objItem.Method.StartsWith("Preview"))
            {
                Trace.WriteLine("Preview ausgeschaltet");
                e.Accepted = false;
                return;
            }

            // Toggle filtern
            if (_tglButton.IsChecked.GetValueOrDefault() && objItem.Sender != "Textbox")
            {
                Trace.WriteLine("Nicht akzeptiert");
                e.Accepted = false;
                return;
            }

            // Einzelfilter
            CheckBox objCheck = _cobjChecker.FirstOrDefault(objSearch => objSearch.Tag.ToString() == objItem.SearchName);
            if (objCheck != null && objCheck.IsChecked.GetValueOrDefault())
            {
                Trace.WriteLine("Akzeptiert");
                e.Accepted = true;
            }
            else
            {
                Trace.WriteLine("Nicht akzeptiert");
                e.Accepted = false;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Einschalten aller Möglichkeiten.
        /// </summary>
        private void All_Click(object sender, RoutedEventArgs e)
        {
            _cobjChecker.ForEach(objCheck => objCheck.IsChecked = true);
        }

        /// <summary>
        /// Ereignisbehandlung für das Ausschalten aller Möglichkeiten.
        /// </summary>
        private void None_Click(object sender, RoutedEventArgs e)
        {
            _cobjChecker.ForEach(objCheck => objCheck.IsChecked = false);
        }

        /// <summary>
        /// Ereignisbehandlung beim schliessen des Dialoges
        /// </summary>
        private void ButtonClose_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
    }
}
