﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionslogik für CpnlMediaElement.xaml
    /// </summary>
    public partial class CpnlMediaElement : UserControl
    {
        // Timer objekt für die Nachführung der 
        private DispatcherTimer _timerMedia = new DispatcherTimer(DispatcherPriority.Background);
        // Hilfsvariable für die Kontrolle der Trackerpositionierung
        private bool _bMouseInTrackerDown;
        // Hilfsvariable für die Steuerung der Stopfunktion
        private bool _bIgnoreStop = false;

        /// <summary>
        /// Initialisiert eine neune Instanz der Klasse CpnlMediaElement.
        /// </summary>
        public CpnlMediaElement()
        {
            InitializeComponent();
            _timerMedia.Interval = TimeSpan.FromMilliseconds(500);
            _timerMedia.Tick += new EventHandler(TimerMedia_Tick);
        }

        /// <summary>
        /// Ereignisbehenaldung beim Laden der Demo.
        /// </summary>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            _btnStart.IsEnabled = false;
        }

        /// <summary>
        /// Ereignisbehandlung für den internen Timer zur Nachführung des Sliders.
        /// </summary>
        void TimerMedia_Tick(object sender, EventArgs e)
        {
            // Die automatische Nachführung wird nur gemacht, 
            // wenn nicht mit dern Maus am Tracker geschoben wird.
            if (!_bMouseInTrackerDown)
            {
                // Prüfen, ob das Medium ein Zeit Information hat
                if (_mediaPlayer.NaturalDuration.HasTimeSpan)
                {
                    TimeSpan tsPosition = _mediaPlayer.Position;
                    _sldPosition.Value = (int)tsPosition.TotalMilliseconds;
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Ziehen einer Information auf das Element.
        /// </summary>
        private void MediaDragOver(object sender, DragEventArgs e)
        {
            // Prüfen, ob es sich um eine Datei-Explorer Liste handelt
            string[] astrFileData = (string[])e.Data.GetData("FileDrop");
            if (astrFileData.Length == 1)
            {
                e.Effects = System.Windows.DragDropEffects.Copy;
            }
            else
            {
                e.Effects = System.Windows.DragDropEffects.None;
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Ablegen der Information auf dem Medienelement.
        /// </summary>
        private void MediaDrop(object sender, DragEventArgs e)
        {
            // Dateinamen aus den DD-Informationen übernehmen
            string[] astrFileData = (string[])e.Data.GetData("FileDrop");
            if (astrFileData.Length == 1)
            {
                _mediaPlayer.Visibility = System.Windows.Visibility.Visible;
                _mediaPlayer.Source = new Uri(astrFileData[0]);
                _statusText.Content = astrFileData[0];
                StartPlayingMedia();
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Öffnen des Mediums
        /// </summary>
        private void MediaOpened(object sender, RoutedEventArgs e)
        {
            // Prüfen, ob das Medium ein Zeit Information hat
            if (_mediaPlayer.NaturalDuration.HasTimeSpan)
            {
                // Zeitachsenanzeige konfigurieren
                _sldPosition.Maximum = (int)_mediaPlayer.NaturalDuration.TimeSpan.TotalMilliseconds;
                // Anzeigen des Ersatztexts oder des Medienelements
                if (_mediaPlayer.HasVideo)
                {
                    _mediaPlayer.Visibility = System.Windows.Visibility.Visible;
                    _tbMusic.Visibility = System.Windows.Visibility.Hidden;
                }
                else
                {
                    _mediaPlayer.Visibility = System.Windows.Visibility.Hidden;
                    _tbMusic.Visibility = System.Windows.Visibility.Visible;
                }

                _bIgnoreStop = false;
            }
            else
            {
                _bIgnoreStop = true;
                _timerMedia.Stop();
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Beenden des Mediums.
        /// </summary>
        private void MediaEnded(object sender, RoutedEventArgs e)
        {
            if (!_bIgnoreStop)
            {
                _timerMedia.Stop();
                _mediaPlayer.Stop();
                ResetControls();
            }
        }

        /// <summary>
        /// Ereignisbehandlung im Fall einer fehlerbehafteten Nutzung des Medienelements.
        /// </summary>
        private void MediaFailed(object sender, ExceptionRoutedEventArgs e)
        {
            MessageBox.Show(string.Format(
                "{0}{1}{2}",
                "Fehler bei der Verarbeitung der Datei. Prüfen Sie, ob das verwendete Medienformat unterstützt wird.",
                Environment.NewLine,
                e.ErrorException.Message));
            _timerMedia.Stop();
        }

        /// <summary>
        /// Started das Abspielen der Multimediadatei
        /// </summary>
        private void StartPlayingMedia()
        {
            _mediaPlayer.Play();
            _timerMedia.Start();
            _btnStart.IsEnabled = false;
            _btnStop.IsEnabled = true;
        }

        /// <summary>
        /// Ereignisbehandlung beim Beginn des Veschiebens des Sliders für die Positionierung.
        /// </summary>
        private void Position_MouseDown(object sender, MouseButtonEventArgs e)
        {
            _bMouseInTrackerDown = true;
            _mediaPlayer.Pause();
        }

        /// <summary>
        /// Ereignisbehandlung beim Beenden des Veschiebens des Sliders für die Positionierung.
        /// </summary>
        private void Position_MouseUp(object sender, MouseButtonEventArgs e)
        {
            _bMouseInTrackerDown = false;
            _mediaPlayer.Play();
        }

        /// <summary>
        /// Ereignisbehandlung beim Verschieben des Sliders für die Positionierung.
        /// </summary>
        private void Position_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (_bMouseInTrackerDown)
            {
                // Prüfen, ob das Medium ein Zeit Information hat
                if (_mediaPlayer.NaturalDuration.HasTimeSpan)
                {
                    TimeSpan tsPosition = TimeSpan.FromMilliseconds(_sldPosition.Value);
                    _mediaPlayer.Position = tsPosition;
                }
            }
        }

        /// <summary>
        /// Ereignisbahndlung bei der Überprüfung ob das Abspielen des Mediums erlaubt ist.
        /// </summary>
        private void CanExecutePlay(object sender, CanExecuteRoutedEventArgs e)
        {
            if (_mediaPlayer != null && _mediaPlayer.Source != null)
            {
                e.CanExecute = true;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Abspielen des Mediums.
        /// </summary>
        private void ExecutePlay(object sender, ExecutedRoutedEventArgs e)
        {
            _mediaPlayer.Play();
            _btnStart.IsEnabled = false;
            _btnPause.IsEnabled = true;
            _btnStop.IsEnabled = true;
            _sldPosition.IsEnabled = true;
            _timerMedia.Start();
            if (_mediaPlayer.HasVideo)
            {
                _mediaPlayer.Visibility = System.Windows.Visibility.Visible;
            }
            else if (_mediaPlayer.HasAudio)
            {
                _tbMusic.Visibility = System.Windows.Visibility.Visible;
            }
        }

        /// <summary>
        /// Ereignisbehanldung bei der Überprüfung ob das Abspielen des Mediums pausiert werden kann.
        /// </summary>
        private void CanExecutePause(object sender, CanExecuteRoutedEventArgs e)
        {
            if (_mediaPlayer != null && _mediaPlayer.CanPause)
            {
                e.CanExecute = true;
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Pausieren der Wiedergabe des Mediums.
        /// </summary>
        private void ExecutePause(object sender, ExecutedRoutedEventArgs e)
        {
            _mediaPlayer.Pause();
            _btnStart.IsEnabled = true;
            _btnPause.IsEnabled = false;
            _btnStop.IsEnabled = false;
            _sldPosition.IsEnabled = false;
            _tbMusic.Visibility = System.Windows.Visibility.Hidden;
        }

        /// <summary>
        /// Ereignisbehanldung bei der Überprüfung ob das Abspielen des Mediums gestoppt werden kann.
        /// </summary>
        private void CanExecuteStop(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }

        /// <summary>
        /// Ereignisbehandlung beim Stoppen der Wiedergabe des Mediums.
        /// </summary>
        private void ExecuteStop(object sender, ExecutedRoutedEventArgs e)
        {
            _mediaPlayer.Stop();
            _timerMedia.Stop();
            _btnStart.IsEnabled = true;
            _btnPause.IsEnabled = false;
            _btnStop.IsEnabled = false;
            _mediaPlayer.Visibility = System.Windows.Visibility.Hidden;
            _tbMusic.Visibility = System.Windows.Visibility.Hidden;
        }

        /// <summary>
        /// Schaltet alle weiteren Bedienelemente aus.
        /// </summary>
        private void ResetControls()
        {
            _btnStart.IsEnabled = false;
            _btnPause.IsEnabled = false;
            _btnStop.IsEnabled = false;
            _sldPosition.IsEnabled = false;
            _sldPosition.Value = 0;
            _tbMusic.Visibility = System.Windows.Visibility.Hidden;
            _mediaPlayer.Visibility = System.Windows.Visibility.Hidden;
        }
    }
}
