﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionslogik für CpnlWebBrowser.xaml
    /// </summary>
    public partial class CpnlWebBrowser : UserControl, INotifyPropertyChanged
    {
        /// <summary>
        /// Wird ausgelöst, wenn eine Eigenschaft der Klasse ändert.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Definiert die Breite der Textbox.
        /// </summary>
        private double _dUrlTextBoxWidth;

        /// <summary>
        /// Liefert oder definiert die Breite der TextBox für die Eingabe der Url.
        /// </summary>
        public double UrlTextBoxWidth
        {
            get
            {
                return _dUrlTextBoxWidth;
            }
            private set
            {
                _dUrlTextBoxWidth = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("UrlTextBoxWidth"));
                }
            }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CpnlWebBrowser.
        /// </summary>
        public CpnlWebBrowser()
        {
            // Generierte Elemente initialisieren
            InitializeComponent();
            // Ereignisbehandlung für die Veränderung der Größe des Elements registrieren
            this.SizeChanged += new SizeChangedEventHandler(CpnlWebBrowser_SizeChanged);
        }

        /// <summary>
        /// Ereignisbehandlung für die Änderung der Größe.
        /// </summary>
        void CpnlWebBrowser_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            UrlTextBoxWidth = this.ActualWidth - 150.0;
        }

        private void ExecuteGoToPage(object sender, ExecutedRoutedEventArgs e)
        {
            _browserDemo.Navigate(_tbUrl.Text);
        }

        /// <summary>
        /// Ereignisbehandlung für die Prüfung der Ausführung eines Befehls GoToPage.
        /// </summary>
        private void GoToPage_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }

        /// <summary>
        /// Ereignisbehandlung für die Prüfung der Ausführung des Befehls rückwärts navigieren.
        /// </summary>
        private void CanExecuteBrowseBack(object sender, CanExecuteRoutedEventArgs e)
        {
            if (_browserDemo != null)
            {
                e.CanExecute = _browserDemo.CanGoBack;
            }
        }

        /// <summary>
        /// Führt die ausgelöste Aktion für das vorwärts Navigieren aus.
        /// </summary>
        private void ExecuteBrowseBack(object sender, ExecutedRoutedEventArgs e)
        {
            _browserDemo.GoBack();
        }

        /// <summary>
        /// Ereignisbehandlung für die Prüfung der Ausführung des Befehls vorwärts navigieren.
        /// </summary>
        private void CanExecuteBrowseForeward(object sender, CanExecuteRoutedEventArgs e)
        {
            if (_browserDemo != null)
            {
                e.CanExecute = _browserDemo.CanGoForward;
            }
        }

        /// <summary>
        /// Führt die ausgelöste Aktion für das rückwärts Navigieren aus.
        /// </summary>
        private void ExecuteBrowseForeward(object sender, ExecutedRoutedEventArgs e)
        {
            _browserDemo.GoForward();
        }

        /// <summary>
        /// Ereignisbehandlung für die Prüfung der Ausführung des Befehls Gehe zu Seite.
        /// </summary>
        private void CanExecuteGoToPage(object sender, CanExecuteRoutedEventArgs e)
        {
            if (_tbUrl.Text.Length > 10)
            {
                e.CanExecute = true;
            }
        }
    }
}
