﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Threading;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionslogik für CpnlMenu.xaml
    /// </summary>
    public partial class CpnlMenu : UserControl, INotifyPropertyChanged
    {
        /// <summary>
        /// Wird ausgelöst, wenn eine Eigenschaft ändert.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Liefert die Zeit.
        /// </summary>
        public DateTime Time {get; private set; }

        /// <summary>
        /// Liefert eine Auflistung der Bildelemente
        /// </summary>
        public ObservableCollection<CwlbImages> Images { get; private set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CpnlMenu.
        /// </summary>
        public CpnlMenu()
        {
            // Anlegen der Auflistung
            Images = new ObservableCollection<CwlbImages>();
            // Generierte Initialisierung
            InitializeComponent();
            // Bilder hinzufügen
            Images.Add(new CwlbImages { ImageText = "Schnittstelle", ImageSource = "/Images/Interface.png" });
            Images.Add(new CwlbImages { ImageText = "Klasse", ImageSource = "/Images/Class.png" });
            Images.Add(new CwlbImages { ImageText = "Ereignis", ImageSource = "/Images/Event.png" });
            Images.Add(new CwlbImages { ImageText = "Eigenschaft", ImageSource = "/Images/Property.png" });
            Images.Add(new CwlbImages { ImageText = "Methode", ImageSource = "/Images/Method.png" });
        }

        /// <summary>
        /// Ereignisbehandlung beim Laden des Steuerelements
        /// </summary>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            DispatcherTimer timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromMilliseconds(400);
            timer.Tick += new EventHandler(timer_Tick);
            timer.Start();
        }

        /// <summary>
        /// Ereignisbehandlung beim erreichen des Timerticks.
        /// </summary>
        void timer_Tick(object sender, EventArgs e)
        {
            Time = DateTime.Now;
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs("Time"));
            }
        }
    }
}
