﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using System.Xml;
using System.Windows.Markup;
using Microsoft.Win32;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaction logic for CpnlXamlPad.xaml
    /// </summary>
    public partial class CpnlXamlPad : UserControl, IwlbDynamicXaml
    {
        // Instance Data
        private UIElement _objUIElement = null;

        /// <summary>
        /// Wird ausgelöst, wenn der XAML-Code compiliert wurde
        /// </summary>
        public event EventHandler<CwlbCompiledEventArgs> Compiled;

        /// <summary>
        /// Liefert einen Wert der definiert ob der Code modifiziert wurde oder nicht.
        /// </summary>
        public bool Modified { get; set; }

        /// <summary>
        /// Liefert oder definiert den Dateinamen des Inhalts
        /// </summary>
        public string FileName { get; set; }

        /// <summary>
        /// Liefert das Befehlsobjekt für die Kompilation des XAML-Codes.
        /// </summary>
        public static readonly RoutedCommand CommandCompile = new RoutedUICommand("XAML kompilieren", "Compile", typeof(CwlbMainWindow), new InputGestureCollection { new KeyGesture(Key.R, ModifierKeys.Control, "Ctrl+R")});

        /// <summary>
        /// Liefert das Befehlsobjekt fpr das Löschen des compilierten XAML-Codes.
        /// </summary>
        public static readonly RoutedCommand CommandDeleteCompiled = new RoutedUICommand("XAML löschen", "DeleteCompiled", typeof(CwlbMainWindow), new InputGestureCollection { new KeyGesture(Key.D, ModifierKeys.Control, "Ctrl+D")});

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CpnlXamlPad.
        /// </summary>
        public CpnlXamlPad()
        {
            InitializeComponent();

            // Instruktion hinzufügen
            _ctlParent.Child = new CpnlInstruction();
        }

        /// <summary>
        /// Ereignisbehandlung bei der Initialisierung des Steuerelements.
        /// </summary>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            _tbXaml.Focus();
        }

        /// <summary>
        /// Definiert den Fokus auf dem Editor.
        /// </summary>
        public new void Focus()
        {
            _tbXaml.Focus();
        }

        /// <summary>
        /// Ereignisbehandlung beim Laden des Fensters.
        /// </summary>
        void Control_Loaded(object sender, RoutedEventArgs e)
        {
            // Raise event
            if (Compiled != null)
            {
                Compiled(this, new CwlbCompiledEventArgs(_objUIElement));
            }
        }

        /// <summary>
        /// Ereignisbehandlung bei der Betätigung der Schaltfläche Kompilieren.
        /// </summary>
        private void Execute_Compile(object sender, ExecutedRoutedEventArgs e)
        {
            // Exception handled
            try
            {
                // Prüfe ob das Steuerelement bereits dfiniert ist
                if (_objUIElement != null && _objUIElement is FrameworkElement)
                {
                    ((FrameworkElement)_objUIElement).Loaded -= new RoutedEventHandler(Control_Loaded);
                }

                // Create XAML
                StringReader strReader = new StringReader(_tbXaml.Text);
                XmlReader xmlReader = XmlReader.Create(strReader);
                _objUIElement = (UIElement)XamlReader.Load(xmlReader);

                // Check successfully creation
                if (_objUIElement != null)
                {
                    if (_objUIElement is FrameworkElement)
                    {
                        ((FrameworkElement)_objUIElement).Loaded += new RoutedEventHandler(Control_Loaded);
                    }
                    _ctlParent.Child = _objUIElement;

                    // Write message to output area
                    _tbErrors.Text = "Erfolgreich kompiliert und eingefügt.";
                    _tbErrors.Foreground = Brushes.DarkGreen;
                }
            }
            catch (Exception exc)
            {
                string strResult = string.Format("Fehler bei der Umsetzung von XAML:{0}{1}",
                    Environment.NewLine,
                    exc.Message);
                strResult = strResult.Replace(". ", ".\r\n\t");
                _tbErrors.Text = strResult;
                _tbErrors.Foreground = Brushes.Red;
            }
        }

        /// <summary>
        ///  Ausführung des Befehls Neu.
        /// </summary>
        private void Execute_New(object sender, RoutedEventArgs e)
        {
            _tbXaml.Clear();

            StringBuilder objInitCode = new StringBuilder();
            objInitCode.AppendLine("<Grid xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"");
            objInitCode.AppendLine("      xmlns:x=\"http://schemas.microsoft.com/winfx/2006/xaml\">");
            objInitCode.AppendLine();
            objInitCode.AppendLine("    <Hier den XAML-Code einbringen>");
            objInitCode.AppendLine("</Grid>");

            // Put string into textbox
            _tbXaml.Text = objInitCode.ToString();
        }

        /// <summary>
        /// Ausführung des Befehls Datei öffnen.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Execute_Open(object sender, ExecutedRoutedEventArgs e)
        {
            OpenFileDialog dlgOpen = new OpenFileDialog();

            // Initialize
            dlgOpen.Filter = "xaml-Dateien (*.xaml)|*.xaml|Alle Dateien (*.*)|*.*";
            dlgOpen.FilterIndex = 1;

            if (dlgOpen.ShowDialog().GetValueOrDefault())
            {
                LoadFile(dlgOpen.FileName);
            }
        }

        /// <summary>
        /// Lädt die definierte Datei in den Editor.
        /// </summary>
        /// <param name="strFileName">Dateiname der zu ladenden Datei.</param>
        public void LoadFile(string strFileName)
        {
            TextReader objReader = new StreamReader(strFileName);
            _tbXaml.Text = objReader.ReadToEnd();
            FileName = strFileName;
            objReader.Close();

            // Definiere die Datei als unverändert-
            Modified = false;

            // Lösche den Inhalt der Kompilationsausgabe
            _tbErrors.Text = string.Empty;
        }

        /// <summary>
        /// Ausführen des Befehls Speichern.
        /// </summary>
        private void Execute_Save(object sender, RoutedEventArgs e)
        {
            // Check Filename
            if (string.IsNullOrEmpty(FileName))
            {
                Execute_SaveAs(sender, e);
            }
            else
            {
                SaveFile(FileName);
            }
        }

        /// <summary>
        /// Ausführen des Befehls Speichern unter.
        /// </summary>
        private void Execute_SaveAs(object sender, RoutedEventArgs e)
        {
            SaveFileDialog dlgSave = new SaveFileDialog();

            // Initialize
            dlgSave.Filter = "xaml-Dateien (*.xaml)|*.xaml|Alle Dateien (*.*)|*.*";
            dlgSave.FilterIndex = 1;

            if (dlgSave.ShowDialog() == true)
            {
                SaveFile(dlgSave.FileName);
            }
        }

        /// <summary>
        /// Speichert den Inhalt des Editors der definierten Datei.
        /// </summary>
        /// <param name="strFileName">Dateiname der zu speichernden Datei.</param>
        private void SaveFile(string strFileName)
        {
            TextWriter objWriter = new StreamWriter(strFileName);
            objWriter.Write(_tbXaml.Text);
            objWriter.Close();

            // Set the filename
            FileName = strFileName;
            Modified = false;
        }

        /// <summary>
        /// Ausführen des Befehls Löschen
        /// </summary>
        private void Execute_DeleteCompiled(object sender, ExecutedRoutedEventArgs e)
        {
            _ctlParent.Child = new CpnlInstruction();
        }

        /// <summary>
        /// Ereignisbehandlung für die Prüfung der Ausführbarkeit eines Befehls.
        /// </summary>
        private void CanExecuteAlways(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }
    }
}
