﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.IO;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert einen Container für Systeminformationen.
    /// </summary>
    public abstract class CwlbContainer : CwlbSystemInfo
    {
        /// <summary>
        /// Liefert eine Liste der enthaltenen Elemente des Containers.
        /// </summary>
        public ObservableCollection<CwlbSystemInfo> Content { get; private set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbContainer.
        /// </summary>
        public CwlbContainer()
        {
            // Erstelle die Liste.
            Content = new ObservableCollection<CwlbSystemInfo>();
        }

        /// <summary>
        /// Befüllt die Liste in den angegebenen Container.
        /// </summary>
        /// <param name="objDirectoryInfo">Verzeichnisobjekt dessen Inhalt gelesen werden soll.</param>
        /// <param name="objContent">Container der befüllt werden soll.</param>
        protected void Fill(DirectoryInfo objDirectoryInfo, ObservableCollection<CwlbSystemInfo> objContent)
        {
            // Ausnahmebehandlung, damit Probleme mit den Zugriffsrechten umgangen werden können.
            try
            {
                // Zuerst werden die Verezichnisse eingelesen (gefiltert und sortiert nach Namen)
                foreach (DirectoryInfo objDirectory in objDirectoryInfo
                    .GetDirectories()
                    .Where(objItem => objItem.Attributes.HasFlag(FileAttributes.Directory) || 
                        objItem.Attributes.HasFlag(FileAttributes.Hidden) ||
                        objItem.Attributes.HasFlag(FileAttributes.System))
                    .OrderBy( objItem => objItem.Name))
                {
                    objContent.Add(new CwlbDirectoryInfo(objDirectory));
                }

                // Anschliessend werden die Dateien eingelesen (sortiert nach Namen)
                foreach (FileInfo objFile in objDirectoryInfo.GetFiles()
                    .OrderBy(objItem => objItem.Name))
                {
                    objContent.Add(new CwlbFileInfo(objFile));
                }
            }
            catch
            {
            }
        }

        /// <summary>
        /// Erweitert das aktuelle Objekt mit dem Inhalt der nächsten Stufe.
        /// </summary>
        public override void Expand()
        {
            // Den gesamten Inhalt der aktuellen Stufe auflisten
            foreach (CwlbSystemInfo objInfo in Content)
            {
                // Sofern es sich um einen Container handelt diesen 
                // mit der nächsten Stufe befüllen
                if (objInfo is CwlbDirectoryInfo)
                {
                    ((CwlbContainer)objInfo).Content.Clear();
                    Fill(((CwlbDirectoryInfo)objInfo).Directory, ((CwlbDirectoryInfo)objInfo).Content);
                }
            }
        }
    }
}
