﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.IO;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert ein Ansichtsobjekt für ein Verzeichnis.
    /// </summary>
    public class CwlbDirectoryInfo : CwlbContainer
    {
        /// <summary>
        /// Liefert die originale Verzeichnisinformation des Betriebssystem.
        /// </summary>
        public DirectoryInfo Directory { get; private set; }

        /// <summary>
        /// Definiert true, wenn es sich um eine Hidden- oder Systemdatei handelt.
        /// </summary>
        public override bool IsSystemOrHidden
        {
            get
            {
                return Directory.Attributes.HasFlag(FileAttributes.Hidden) || Directory.Attributes.HasFlag(FileAttributes.System);
            }
        }

        /// <summary>
        /// Liefert den Inhalt eines Tooltipps.
        /// </summary>
        public override string ToolTip
        {
            get { return null; }
        }

        /// <summary>
        /// Definiert den Ansichtsnamen des Verzeichnisobjekts.
        /// </summary>
        public override string ViewName
        {
            get { return Directory.Name; }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbDirectoryInfo.
        /// </summary>
        /// <param name="objFileInfo">Originale Verzeichnisinformation des Betriebssystems.</param>
        public CwlbDirectoryInfo(DirectoryInfo objDirectory)
        {
            Directory = objDirectory;
        }
    }
}
