﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.IO;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert ein Laufwerksobjekt.
    /// </summary>
    public class CwlbDriveInfo : CwlbContainer
    {
        /// <summary>
        /// Liefert die originale Laufwerksinformation des Betriebssystems.
        /// </summary>
        public DriveInfo DriveInfo { get; private set; }

        /// <summary>
        /// Liefert den Ansichtsnamen des Objekts.
        /// </summary>
        public override string ViewName
        {
            get
            {
                // Prüfen, ob auf die Daten des Laufwerks zugegriffen werden kann
                if (DriveInfo.IsReady)
                {
                    return string.Format(
                                "{0} ({1}) / {2}",
                                DriveInfo.Name,
                                DriveInfo.VolumeLabel,
                                DriveInfo.DriveFormat);
                }
                else
                {
                    // Der Name des Laufwerks kann in jedem Fall zurückgegeben werden
                    return DriveInfo.Name;
                }
            }
        }

        /// <summary>
        /// Liefert true, wenn das Laufwerk bereits ist.
        /// </summary>
        public override bool IsSystemOrHidden
        {
            get
            {
                return ! DriveInfo.IsReady;
            }
        }

        /// <summary>
        /// Liefert den Inhalt eines Tooltipps.
        /// </summary>
        public override string ToolTip
        {
            get { return string.Format("Freier Speicher: {0:N0} MB{2}Größe gesamt: {1:N0} MB",
                    DriveInfo.TotalFreeSpace / 1024 / 1024, 
                    DriveInfo.TotalSize / 1024 / 1024,
                    Environment.NewLine); }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbDriveInfo.
        /// </summary>
        /// <param name="objFileInfo">Originale Laufwerksinformation des Betriebssystems.</param>
        public CwlbDriveInfo(DriveInfo objDriveInfo)
        {
            DriveInfo = objDriveInfo;

            // Laden des Stammverzeichnisses
            if (objDriveInfo.IsReady)
            {
                Fill(DriveInfo.RootDirectory, Content);
            }
        }
    }
}
