﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert einen Viewtyp für eine Dateiinformation.
    /// </summary>
    public class CwlbFileInfo : CwlbSystemInfo
    {
        /// <summary>
        /// Liefert die originale Dateiinformation des Betriebssystems.
        /// </summary>
        public FileInfo FileInfo { get; protected set; }

        /// <summary>
        /// Definiert den Ansichtsnamen des Dateiobjekts.
        /// </summary>
        public override string ViewName
        {
            get { return FileInfo.Name; }
        }

        /// <summary>
        /// Definiert true, wenn es sich um eine Hidden- oder Systemdatei handelt.
        /// </summary>
        public override bool IsSystemOrHidden
        {
            get
            {
                return FileInfo.Attributes.HasFlag(FileAttributes.Hidden) || FileInfo.Attributes.HasFlag(FileAttributes.System);
            }
        }

        /// <summary>
        /// Liefert den Inhalt eines Tooltipps.
        /// </summary>
        public override string ToolTip
        {
            get
            {
                if (FileInfo.Length <= 1024)
                {
                    return string.Format("Größe der Datei ist {0:N0} Bytes", FileInfo.Length);
                }
                else if (FileInfo.Length < 1024 * 1024)
                {
                    return string.Format("Größe der Datei ist {0:N0} KB", FileInfo.Length / 1024);
                }
                else
                {
                    return string.Format("Größe der Datei ist {0:N0} MB", FileInfo.Length / 1024 / 1024);
                }
            }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbFileInfo.
        /// </summary>
        /// <param name="objFileInfo">Originale Dateiinformation des Betriebssystems.</param>
        public CwlbFileInfo(FileInfo objFileInfo)
        {
            FileInfo = objFileInfo;
        }

        /// <summary>
        /// Erweitert den Knoten.
        /// </summary>
        public override sealed void Expand()
        {
            // Hier gibt es nichts zu tun, ausser der Implementierung der Methode.
        }
    }
}
