﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Data;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert einen Multibinding Converter für drei Textfelder.
    /// </summary>
    public class CwlbMultiBindingConverter : IMultiValueConverter
    {
        /// <summary>
        /// Konvertiert drei Textangaben unterberücksichtigung eines Filters.
        /// </summary>
        /// <param name="aobjValues">Werte, die Konvertiert werden sollen.</param>
        /// <param name="objTargetType">Notwendiger Zieltyp für die Konvertierung.</param>
        /// <param name="objParameter">Parameter für die Formatierung.</param>
        /// <param name="objCulture">Zu berücksichtigende Sprache für die Konvertierung.</param>
        /// <returns>Liefert einen aus den einzelnen Daten zusammengefügten Text.</returns>
        public object Convert(object[] aobjValues, Type objTargetType, object objParameter, System.Globalization.CultureInfo objCulture)
        {
            return string.Format(objParameter.ToString(), aobjValues[0], aobjValues[1], aobjValues[2]);
        }

        /// <summary>
        /// Konvertiert die Eingabe zurück in Einzelwerte.
        /// </summary>
        /// <param name="objValue">Eingabewert für die Konvertierung.</param>
        /// <param name="objTargetType">Zieltypen für die Konvertierung.</param>
        /// <param name="objParameter">Parameter für die Formatierung.</param>
        /// <param name="objCulture">Zu berücksichtigende Sprache für die Konvertierung.</param>
        /// <returns></returns>
        public object[] ConvertBack(object objValue, Type[] objTargetType, object objParameter, System.Globalization.CultureInfo objCulture)
        {
            string[] astrData = objValue.ToString().Split(':', '-');

            int iIndexColon = objValue.ToString().IndexOf(':');
            int iIndexHyphen = objValue.ToString().IndexOf('-');
            switch (astrData.Length)
            {
                case 1:
                    return new object[] { astrData[0].Trim(), "<Eingabe>", "<Eingabe>" };
                case 2:
                    if (iIndexColon > iIndexHyphen)
                    {
                        return new object[] { astrData[0].Trim(), astrData[1].Trim(), "<Eingabe>" };
                    }
                    else
                    {
                        return new object[] { astrData[0].Trim(), "<Eingabe>", astrData[1].Trim() };
                    }
                case 3:
                    if (iIndexColon > iIndexHyphen)
                    {
                        return new object[] { astrData[0].Trim(), astrData[1].Trim(), astrData[2].Trim() };
                    }
                    else
                    {
                        return new object[] { astrData[0].Trim(), astrData[2].Trim(), astrData[1].Trim() };
                    }
                default:
                    return new object[] { "<Eingabe>", "<Eingabe>", "<Eingabe>" };
            }
        }
    }
}
