﻿using WeroSoft.Samples;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;

namespace WeroSoft.Samples
{
    
    
    /// <summary>
    ///This is a test class for CwlbInMemoryDataTest and is intended
    ///to contain all CwlbInMemoryDataTest Unit Tests
    ///</summary>
    [TestClass()]
    public class CwlbDataTest
    {
        int iEvent;

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        /// <summary>
        ///A test for GetContacts
        ///</summary>
        [TestMethod()]
        public void GetContactsTest()
        {
            // Erstellen der Daten
            List<CwlbContact> cobjData = CwlbInMemoryData.Data.GetContacts();
            // Prüfen der Liste
            Assert.IsNotNull(cobjData, "Die Datenliste darf nicht null sein.");
            Assert.AreNotEqual<int>(0, cobjData.Count, "Die Liste darf nicht leer sein.");
        }


        /// <summary>
        /// Prüft ob ein Personenkontakt zur Liste hinzugefügt werden kann.
        /// </summary>
        [TestMethod()]
        public void AddContactTest()
        {
            // Erstellen der Daten
            List<CwlbContact> cobjData = CwlbInMemoryData.Data.GetContacts();
            Assert.IsNotNull(cobjData, "Die Datenliste darf nicht null sein.");
            // Element hinzufügen
            iEvent = 0;
            CwlbInMemoryData.Data.EventContactAdded += new EventHandler<CwlbCustomerEventArgs>(Data_EventContactAdded);
            CwlbInMemoryData.Data.AddContact(new CwlbPersonContact { ID = "1000", Name = "TestName", FirstName = "TestVorname" });
            CwlbInMemoryData.Data.EventContactAdded -= new EventHandler<CwlbCustomerEventArgs>(Data_EventContactAdded);
            Assert.AreEqual<int>(1, iEvent, "Das Ereignis für die Erfassung wurde nciht ausgelöst.");

            // Erneutes hinzufügen muss eine Ausnahme auslösen
            try
            {
                CwlbInMemoryData.Data.AddContact(new CwlbPersonContact { ID = "1000", Name = "TestName", FirstName = "TestVorname" });
                Assert.Fail("Es wurde keine Ausnahme bei der Doppelerfassung ausgelöst");
            }
            catch (InvalidOperationException)
            {
            }
            catch
            {
                Assert.Fail("Es wurde eine falsche Ausnahme bei der Doppelerfassung ausgelöst");
            }

            // Auslösen mit einem null Argumen
            try
            {
                CwlbInMemoryData.Data.AddContact(null);
                Assert.Fail("Es wurde keine Ausnahme beim Erfassen eines leeren Kontakts ausgelöst.");
            }
            catch (ArgumentNullException)
            {
            }
            catch
            {
                Assert.Fail("Es wurde die falsche Ausnahme beim erfassen eines leeren Kontakts ausgelöst");
            }
        }

        /// <summary>
        /// Überprüfen der Ereignisargumente.
        /// </summary>
        private void Data_EventContactAdded(object sender, CwlbCustomerEventArgs e)
        {
            iEvent++;
            Assert.IsNotNull(e.Contact, "Der Contaktinhalt beim Ereignis ContactAdded darf nicht null sein.");
            Assert.AreEqual<string>("1000", e.Contact.ID, "Die ID des ergänzenten Elements ist nicht korrekt");
            Assert.AreEqual<string>("TestName", e.Contact.Name, "Der Name des ergänzenten Elements ist nicht korrekt");
            Assert.AreEqual<string>("TestVorname", ((CwlbPersonContact)e.Contact).FirstName, "Der Vorname des ergänzenten Elements ist nicht korrekt");
        }
    }
}
