﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Reflection;
using System.Windows.Threading;
using System.ComponentModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interktionslogik für CwlbClipboard.xaml
    /// </summary>
    public partial class CwlbClipboardDemo : Page, INotifyPropertyChanged
    {
        /// <summary>
        /// Flag für die Kennzeichnung des Einschaltzustandes der Schaltfläche "Daten empfangen"
        /// </summary>
        bool _bSelectedItemEnabled;

        /// <summary>
        /// Ereignis für die Veränderung von Eigenschaften.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        /// <summary>
        /// Liefert oder definiert den Zustand der Schaltfläche "Daten empfangen"
        /// </summary>
        public bool IsSelectedItemEnabled
        {
            get { return _bSelectedItemEnabled; }
            set { 
                _bSelectedItemEnabled = value;
                PropertyChanged(this, new PropertyChangedEventArgs("IsSelectedItemEnabled"));
            }
        }

        /// <summary>
        /// Liste der Formate in der Zwischenablage
        /// </summary>
        public ObservableCollection<CwlbClipboardFormat> ClipboardFormats { get; private set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbClipboardDemo.
        /// </summary>
        public CwlbClipboardDemo()
        {
            ClipboardFormats = new ObservableCollection<CwlbClipboardFormat>();
            foreach (FieldInfo objFieldInfo in typeof(DataFormats).GetFields())
            {
                ClipboardFormats.Add(new CwlbClipboardFormat { Text = objFieldInfo.Name });
            }

            InitializeComponent();
            RefreshFormats();

            DispatcherTimer objTimer = new DispatcherTimer();
            objTimer.Interval = TimeSpan.FromMilliseconds(500);
            objTimer.Tick += new EventHandler(objTimer_Tick);
            objTimer.Start();
        }

        /// <summary>
        /// Ereignisbehandlung des Zeitgeberintervalls.
        /// </summary>
        void objTimer_Tick(object sender, EventArgs e)
        {
            RefreshFormats();
        }

        /// <summary>
        /// Ereignisbehandlung für das Betätigen der Schaltfläche "Text".
        /// </summary>
        private void Button_Text_Click(object sender, RoutedEventArgs e)
        {
            Clipboard.SetText("Demonstration der Zwischenablage.");
        }

        /// <summary>
        /// Ereignisbehandlung für das Betätigen der Schaltfläche "Abbildung".
        /// </summary>
        private void Button_Image_Click(object sender, RoutedEventArgs e)
        {
            Uri uri = new Uri(
                "pack://application:,,,/LearningWpf_05;component/Resources/smiley.png", 
                UriKind.RelativeOrAbsolute);
            BitmapSource bmpSource = new BitmapImage(uri);
            Clipboard.SetImage(bmpSource);
        }

        /// <summary>
        /// Ereignisbehandlung für das Betätigen der Schaltfläche "Eigene Daten.
        /// </summary>
        private void Button_OwnData_Click(object sender, RoutedEventArgs e)
        {
            // Datenobjekt erstellen und mit eigenen Daten initialisieren
            DataObject objData = new DataObject();
            CwlbClipboardHelper objOwnData = new CwlbClipboardHelper();
            objData.SetText(objOwnData.ToString());
            Uri uri = new Uri(
                "pack://application:,,,/LearningWpf_05;component/Resources/smiley.png",
                UriKind.RelativeOrAbsolute);
            BitmapSource bmpSource = new BitmapImage(uri);
            objData.SetImage(bmpSource);
            objData.SetData(typeof(CwlbClipboardHelper).FullName, objOwnData);
            // Daten an Zwischenablage übergeben
            Clipboard.SetDataObject(objData);
        }

        /// <summary>
        /// Ereignisbehandlung für das Betätigen der Schaltfläche "Löschen".
        /// </summary>
        private void Button_Clear_Click(object sender, RoutedEventArgs e)
        {
            Clipboard.Clear();
            _borderResult.Child = null;
        }

        /// <summary>
        /// Erneuert die Anzeige der enthaltenen Datenformate in der Zwischenablage.
        /// </summary>
        private void RefreshFormats()
        {
            // Zurücksetzen des Flags für das Einschalten der SChaltfläche übernehmen
            IsSelectedItemEnabled = false;

            // Check bei allen Elementen ausschalten
            foreach(CwlbClipboardFormat objFormat in ClipboardFormats)
            {
                if (Clipboard.ContainsData(objFormat.Text))
                {
                    objFormat.IsCurrentInClipboard = true;
                    if (IsSelectedItemEnabled == false)
                    {
                        if (ReferenceEquals(_lbFormats.SelectedItem, objFormat))
                        {
                            IsSelectedItemEnabled = true;
                        }
                    }
                }
                else
                {
                    objFormat.IsCurrentInClipboard = false;
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Betätigen der Schaltfläche "Daten empfangen".
        /// </summary>
        private void Button_Receive_Click(object sender, RoutedEventArgs e)
        {
            // Lokale Daten;
            Image objImage = null;

            // Format auslesen
            string strFormat = _lbFormats.SelectedItem.ToString();

            // Format auswerten und übernehmen
            switch (strFormat)
            {
                default:
                    string strOut = string.Format("Ihr ausgewähltes Format ist: '{0}'." +
                        "{1}Die Demo kennt dieses Format nicht.", strFormat, Environment.NewLine);
                    MessageBox.Show(strOut, "Demo der Zwichenablage", 
                        MessageBoxButton.OK, MessageBoxImage.Information);
                    break;
                case "System.String":
                case "UnicodeText":
                case "Text":
                case "OemText":
                case "HTML Format":
                case "Rich Text Format":
                    TextBlock objTextBlock = new TextBlock();
                    objTextBlock.TextWrapping = TextWrapping.Wrap;
                    objTextBlock.Text = Clipboard.GetData(strFormat) as string;
                    _borderResult.Child = objTextBlock;
                    break;
                case "PNG":
                case "GIF":
                case "System.Drawing.Bitmap":
                case "Bitmap":
                case "DeviceIndependentBitmap":
                case "EnhancedMetafile":
                case "MetaFilePict":
                    objImage = new Image();
                    objImage.Source = Clipboard.GetImage();
                    _borderResult.Child = objImage;
                    break;
            }
        }
    }
}
