﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionslogik für CwlbDragDropDemo.xaml    
    /// </summary>
    public partial class CwlbDragDropDemo : Page
    {
        public CwlbDragDropDemo()
        {
            InitializeComponent();

            _objLeft.Tag = new CwlbDragDropHelper();
            _objLeft.Text = ((CwlbDragDropHelper)_objLeft.Tag).Text;
        }

        /// <summary>
        /// Ereignisbehandlung für die Übernahme von Dateinamen aus dem Datei-Explorer.
        /// </summary>
        private void ListBoxFiles_DragOver(object sender, DragEventArgs e)
        {
            if ( e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                string[] astrFileData = (string[])e.Data.GetData(DataFormats.FileDrop);
                if (astrFileData.Length > 0)
                {
                    e.Effects = DragDropEffects.Copy;
                    return;
                }
            }

            // In allen andern Fällen wird die Übernahme abgelehnt
            e.Effects = DragDropEffects.None;
            e.Handled = true;
        }

        /// <summary>
        /// Ereignisbehandlung für die Übernahme von Dateinamen aus dem Datei-Explorer.
        /// </summary>
        private void ListBoxFiles_Drop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                // Dateinamen aus den D&D-Informationen übernehmen
                string[] astrFileData = (string[])e.Data.GetData(DataFormats.FileDrop);
                if (astrFileData.Length > 0)
                {
                    _lbFiles.Items.Clear();
                    foreach (string strFileName in astrFileData)
                    {
                        _lbFiles.Items.Add(new CwlbFileInfo { FullName = strFileName });
                    }
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung zum Starten der Drag & Drop-Operation auf dem farbigen Rechteck.
        /// </summary>
        private void DD_Demo_MouseMove(object sender, MouseEventArgs e)
        {
            TextBlock objTextBlock = sender as TextBlock;
            if (objTextBlock != null)
            {
                if (e.LeftButton == MouseButtonState.Pressed && objTextBlock.Tag != null)
                {
                    DragDrop.DoDragDrop(objTextBlock, objTextBlock.Tag, DragDropEffects.Move);
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim ziehen einer Information auf die Rechtecke der Drag & Drop Demo.
        /// </summary>
        private void DemoObject_DragEnter(object sender, DragEventArgs e)
        {
            if ( e.Data.GetDataPresent(typeof(CwlbDragDropHelper)))
            {
                e.Effects = DragDropEffects.Move;
            }
            else
            {
                e.Effects = DragDropEffects.None;
                e.Handled = true;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die Drop-Operation des linken farbigen Rechtecks.
        /// </summary>
        private void LeftObject_Drop(object sender, DragEventArgs e)
        {
            CwlbDragDropHelper objData = e.Data.GetData(typeof(CwlbDragDropHelper)) as CwlbDragDropHelper;
            if (objData != null)
            {
                // Daten verändert
                objData.LastUsed = DateTime.Now;
                objData.AmountDrops++;

                // Ziel und Quelle vertauschen
                _objLeft.Tag = objData;
                _objLeft.Text = objData.Text;
                _objLeft.Background = Brushes.DarkGreen;
                _objLeft.AllowDrop = false;
                _objRight.Tag = null;
                _objRight.Text = null;
                _objRight.Background = Brushes.Red;
                _objRight.AllowDrop = true;

                e.Effects = DragDropEffects.Move;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die Drop-Operation des rechten farbigen Rechtecks.
        /// </summary>
        private void RightObject_Drop(object sender, DragEventArgs e)
        {
            CwlbDragDropHelper objData = e.Data.GetData(typeof(CwlbDragDropHelper)) as CwlbDragDropHelper;
            if (objData != null)
            {

                // Daten verändert
                objData.LastUsed = DateTime.Now;
                objData.AmountDrops++;

                // Ziel und Quelle vertauschen
                _objRight.Tag = objData;
                _objRight.Text = objData.Text;
                _objRight.Background = Brushes.DarkGreen;
                _objRight.AllowDrop = false;
                _objLeft.Tag = null;
                _objLeft.Text = null;
                _objLeft.Background = Brushes.Red;
                _objLeft.AllowDrop = true;

                e.Effects = DragDropEffects.Move;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Ziehen der Daten auf das Panel rechts.
        /// </summary>
        private void PanelTypedTarget_DragOver(object sender, DragEventArgs e)
        {
            // Prüfen, ob Formate angezeigt werden
            string[] astrFormats = e.Data.GetFormats();
            if (astrFormats.Length > 0)
            {
                e.Effects = DragDropEffects.Copy;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Fallenlassen von typisierten Daten auf der Fläche rechts.
        /// </summary>
        private void PanelTypedTarget_Drop(object sender, DragEventArgs e)
        {
            // Lokale Daten
            string strFormat = null;
            TextBlock objTextBlock = null;
            System.Windows.Controls.Image objImage = null;

            // Dem Absender signalisieren, dass wir nichts tun. Das ist 
            // notwendig, weil die Methode mehrere entsprechende Ausgänge hat.
            e.Effects = DragDropEffects.None;

            // Prüfen der akzeptierten Formate
            string[] astrFormats = e.Data.GetFormats();
            if (astrFormats.Length == 0)        // Keine Formate gefunden, sofort abbrechen
            {
                return;
            }
            else if (astrFormats.Length == 1)   // Nur ein Format gefunden, dieses übernehmen
            {
                strFormat = astrFormats[0];
            }
            else                                // Mehrere Formate gefunden, Auswahldialogfeld anzeigen
            {
                try
                {
                    this.IsEnabled = false;
                    CdlgFormatChooser dlgFormat = new CdlgFormatChooser();
                    dlgFormat.WindowStartupLocation = WindowStartupLocation.CenterOwner;
                    foreach (string strFormatItem in astrFormats)
                    {
                        dlgFormat._lbFormats.Items.Add(strFormatItem);
                    }
                    if (dlgFormat.ShowDialog().GetValueOrDefault())
                    {
                        // Ausgewähltes Format übernehmen
                        strFormat = dlgFormat._lbFormats.SelectedItem.ToString();
                    }
                }
                finally
                {
                    this.IsEnabled = true;
                }
            }

            // Kein Format ausgewählt, Behandlung beenden
            if (string.IsNullOrEmpty(strFormat))
            {
                return;
            }

            // Daten übernehmen
            switch (strFormat)
            {
                default:
                    string strOut = string.Format(
                        "Ihr ausgewähltes Format ist: '{0}'.{1}Dieses Format wird von der Demo nicht unterstützt.", 
                        strFormat, Environment.NewLine);
                    MessageBox.Show(strOut, "Demo Drag & Drop", MessageBoxButton.OK, MessageBoxImage.Information);
                    break;
                case "System.String":
                case "UnicodeText":
                case "Text":
                case "HTML Format":
                case "Rich Text Format":
                    objTextBlock = new TextBlock();
                    objTextBlock.Text = e.Data.GetData(strFormat) as string;
                    _dropContainer.Children.Clear();
                    _dropContainer.Children.Add(objTextBlock);
                    break;
                case "FileDrop":
                    string [] strFileName = e.Data.GetData(DataFormats.FileDrop) as string[];
                    switch(System.IO.Path.GetExtension(strFileName[0]).ToLower())
                    {
                        case ".jpg":
                        case ".png":
                        case ".tiff":
                        case ".gif":
                        case ".bmp":
                            BitmapImage objBitmapImage = new BitmapImage();
                            objBitmapImage.BeginInit();
                            objBitmapImage.UriSource = new Uri("file:///" + strFileName[0].Replace("\\","/"));
                            objBitmapImage.EndInit();
                            objImage = new System.Windows.Controls.Image();
                            objImage.Source = objBitmapImage;
                            _dropContainer.Children.Clear();
                            _dropContainer.Children.Add(objImage);
                            break;
                        case ".txt":
                            break;
                        default:
                            break;
                    }
                    break;

                case "WeroSoft.Samples.CwlbDragDropHelper":
                    CwlbDragDropHelper objHelper = e.Data.GetData(typeof(CwlbDragDropHelper)) as CwlbDragDropHelper;
                    if (objHelper != null )
                    {
                        objTextBlock = new TextBlock();
                        objTextBlock.Text = objHelper.Text;
                        objTextBlock.TextWrapping = TextWrapping.Wrap;
                        _dropContainer.Children.Clear();
                        _dropContainer.Children.Add(objTextBlock);
                        break;
                    }
                    break;
            }

            // Dem Absender signalisieren dass wir kopiert haben
            e.Effects = DragDropEffects.Copy;
        }
    }
}
