﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media.Imaging;
using System.ComponentModel;
using System.Windows;
using System.Windows.Media;
using System.IO;
using System.Runtime.InteropServices;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Allgemeine Hilfsklasse des Beispiels mit C#-Erweiterungsmethoden.
    /// </summary>
    public static class CwlbCommonHelper
    {
        /// <summary>
        /// Interop zur Methode DeleteObject um über WIN32-Handles zu löschen.
        /// </summary>
        /// <param name="hObject">Handle der gelöscht werden soll.</param>
        /// <returns>True wenn das Löschen erfolgreich war.</returns>
        [DllImport("gdi32.dll")]
        private static extern bool DeleteObject(IntPtr hObject);

        /// <summary>
        /// Konvertiert ein GDI+ Bild in eine WPF BitmapSource.
        /// </summary>
        /// <param name="source">GDI+ Bild das konvertiert werden soll.</param>
        /// <returns>Liefert ein Objekt der WPF Klasse BitmapSource.</returns>
        public static BitmapSource ToBitmapSource(this System.Drawing.Image source) 
        {
            System.Drawing.Bitmap bitmap = new System.Drawing.Bitmap(source);
            var bitSrc = bitmap.ToBitmapSource(); 
            bitmap.Dispose(); 
            bitmap = null; 
            return bitSrc;
        }

        /// <summary>
        /// Konvertiert ein GDI+ Bitmap in eine WPF BitmapSource.
        /// </summary>
        /// <param name="source">GDI+ Bitmap das konvertiert werden soll.</param>
        /// <returns>Liefert ein Objekt der WPF Klasse BitmapSource.</returns>
        public static BitmapSource ToBitmapSource(this System.Drawing.Bitmap source)
        {
            BitmapSource bitSrc = null; 
            var hBitmap = source.GetHbitmap();
            try 
            {
                bitSrc = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(
                    hBitmap,
                    IntPtr.Zero,
                    Int32Rect.Empty,
                    BitmapSizeOptions.FromEmptyOptions());
            }
            catch (Win32Exception)
            {
                bitSrc = null;
            }
            finally 
            {
               DeleteObject(hBitmap);
            }

            return bitSrc;
        }
    }
}
