﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionslogik von CwlbNumericUpDown.xaml
    /// </summary>
    public partial class CwlbNumericUpDown : UserControl
    {
        /// <summary>
        /// Liefert oder definiert den eingegeben Wert.
        /// </summary>
        public double Value
        {
            get { return (double)GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }

        /// <summary>
        /// Definition der Abhängigkeitseigenschaft für Value.
        /// </summary>
        public static readonly DependencyProperty ValueProperty =
            DependencyProperty.Register("Value", typeof(double), typeof(CwlbNumericUpDown), new UIPropertyMetadata(0.0));

        /// <summary>
        /// Liefert oder definiert das Increment.
        /// </summary>
        public double Increment
        {
            get { return (double)GetValue(IncrementProperty); }
            set { SetValue(IncrementProperty, value); }
        }

        /// <summary>
        /// Definition der Abhängigkeitseigenschaft für Increment.
        /// </summary>
        public static readonly DependencyProperty IncrementProperty =
            DependencyProperty.Register("Increment", typeof(double), typeof(CwlbNumericUpDown), new UIPropertyMetadata(0.1));

        /// <summary>
        /// Liefert oder definiert den minimal einzugebenden Wert.
        /// </summary>
        public double Minimum
        {
            get { return (double)GetValue(MinimumProperty); }
            set { SetValue(MinimumProperty, value); }
        }

        /// <summary>
        /// Definition der Abhängigkeitseigenschaft für Minimum.
        /// </summary>
        public static readonly DependencyProperty MinimumProperty =
            DependencyProperty.Register("Minimum", typeof(double), typeof(CwlbNumericUpDown), new UIPropertyMetadata(0.0));

        /// <summary>
        /// Liefert oder definiert den maximalen einzugebenden Wert.
        /// </summary>
        public double Maximum
        {
            get { return (double)GetValue(MaximumProperty); }
            set { SetValue(MaximumProperty, value); }
        }

        /// <summary>
        /// Definition der Abhängigkeitseigenschaft für Maximum.
        /// </summary>
        public static readonly DependencyProperty MaximumProperty =
            DependencyProperty.Register("Maximum", typeof(double), typeof(CwlbNumericUpDown), new UIPropertyMetadata(100.0));

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbNumericUpDown.
        /// </summary>
        public CwlbNumericUpDown()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Taste Wert vergrössern.
        /// </summary>
        private void Button_Up_Click(object sender, RoutedEventArgs e)
        {
            Value += Increment;
            if (Value > Maximum)
            {
                Value = Maximum;
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Taste Wert verkleinern.
        /// </summary>
        private void Button_Down_Click(object sender, RoutedEventArgs e)
        {
            Value -= Increment;
            if (Value < Minimum)
            {
                Value = Minimum;
            }
        }

        /// <summary>
        /// Ereignisbehandlung der Tastatureingaben für das Steuerelement.
        /// </summary>
        private void TextBox_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            // Einschränkung auf numerische Tasten
            if (e.Key >= Key.D0 && e.Key <= Key.D9 && Keyboard.Modifiers == ModifierKeys.None)
            {
                return;
            }
            // Einschränkung auf numerische Tasten der Zehnertastatur
            if (e.Key >= Key.NumPad0 && e.Key <= Key.NumPad9 && Keyboard.Modifiers == ModifierKeys.None)
            {
                return;
            }
            // Einschränkung auf Vorzeichen
            if (e.Key == Key.OemMinus || e.Key == Key.Subtract || e.Key == Key.OemPlus || e.Key == Key.Add)
            {
                return;
            }
            // Einschränkung auf Dezimalstelle
            if (e.Key == Key.Decimal || e.Key == Key.OemPeriod)
            {
                return;
            }
            // Einschränkung auf Backspace und Löschen
            if (e.Key == Key.Back || e.Key == Key.Delete)
            {
                return;
            }
            // Einschränkung auf links - rechts Navigation
            if (e.Key == Key.Left || e.Key == Key.Right)
            {
                return;
            }

            // Alles andere ist ungültig
            e.Handled = true;
        }
    }
}
