﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Text.RegularExpressions;
using System.ComponentModel.DataAnnotations;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Hilfsklasse für die Demonstration von Validierungen in der WPF
    /// </summary>
    public class CwlbPerson : INotifyPropertyChanged, IDataErrorInfo
    {
        /// <summary>
        /// Ereignis für die Realisierung der Schnittstelle INotifyPropertyChanged.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        // Name der Person.
        private string _strName;

        /// <summary>
        /// Liefert oder definiert den Namen der Person.
        /// </summary>
        public string Name
        {
            get { return _strName; }
            set { 
                _strName = value;
                if (string.IsNullOrWhiteSpace(_strName))
                {
                    throw new ApplicationException("Der Name darf nicht leer sein.");
                }
                PropertyChanged(this, new PropertyChangedEventArgs("Name"));
            }
        }

        // Vorname der Person
        private string _strFirstName;

        /// <summary>
        /// Liefert oder definiertd en Vornamen der Person.
        /// </summary>
        public string FirstName
        {
            get { return _strFirstName; }
            set
            {
                _strFirstName = value;
                if (string.IsNullOrWhiteSpace(_strFirstName))
                {
                    throw new ApplicationException("Der Vorname darf nicht leer sein.");
                }
                PropertyChanged(this, new PropertyChangedEventArgs("Name"));
            }
        }

        // Wohnort der Person.
        private string _strCity;

        /// <summary>
        /// Liefert oder definiert den Wohnort der Person.
        /// </summary>
        public string City
        {
            get { return _strCity; }
            set { 
                _strCity = value;
                if (string.IsNullOrWhiteSpace(_strCity))
                {
                    throw new ApplicationException("Der Ort darf nicht leer sein.");
                }
                PropertyChanged(this, new PropertyChangedEventArgs("City"));
            }
        }

        // Emailadresse der Person.
        private string _strEmail;

        /// <summary>
        /// Liefert oder definiert die Emailadresse der Person.
        /// </summary>
        public string Email
        {
            get { return _strEmail; }
            set { 
                _strEmail = value;
                PropertyChanged(this, new PropertyChangedEventArgs("Email"));
            }
        }

        // Telefonnummer der Person
        private string _strPhoneNumber;

        /// <summary>
        /// Liefert oder definiert die Telefonnummer der Person.
        /// </summary>
        public string PhoneNumber
        {
            get { return _strPhoneNumber; }
            set { 
                _strPhoneNumber = value;
                PropertyChanged(this, new PropertyChangedEventArgs("PhoneNumber"));
            }
        }

        /// <summary>
        /// Realisiert die Eigenschaft Error der Schnittstelle IDataErrorInfo.
        /// </summary>
        public string Error
        {
            // Bleibt im Beispiel ungenutzt.
            get { return string.Empty; }
        }

        /// <summary>
        /// Realisiert den Indexer der Schnittstelle IDataErrorInfo.
        /// </summary>
        /// <param name="strPropertyName">Name der zu prüfenden Eigenschaft.</param>
        /// <returns>Liefert einen Fehlertext, oder eine leere Zeichenkette, sofern kein Fehler vorliegt.</returns>
        public string this[string strPropertyName]
        {
            get {

                switch (strPropertyName)
                {
                    case "PhoneNumber":
                        if (string.IsNullOrEmpty(PhoneNumber))
                        {
                            return "Die Telefonnummer muss eingegeben werden";
                        }
                        else
                        {
                            return string.Empty;
                        }

                    default:
                        return string.Empty;
                }
            }
        }
    }
}
