﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Data;
using System.Windows.Input;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Konverter für die Suche eines Menübefehls aus einer View.
    /// </summary>
    public class CwlbCommandConverter : IValueConverter
    {
        /// <summary>
        /// Methode für das Konvertieren (Ursprung der Methode ist IValueConverter).
        /// </summary>
        /// <param name="objValue">Quellenobjekt aus dem das gesuchte Element konvertiert werden soll.</param>
        /// <param name="objTargetType">Zieltyp für die Konvertierung.</param>
        /// <param name="objParameter">Definition des gesuchten Elements.</param>
        /// <param name="objCulture">Definition der zu verwendenden Kultur.</param>
        /// <returns>Liefert ein Befehls View Objekt oder null, wenn der entsprechende Befehl nicht gefunden wurde..</returns>
        public object Convert(object objValue, Type objTargetType, object objParameter, System.Globalization.CultureInfo objCulture)
        {
            // Konvertieren der Quelle in ein streng typisiertes Objekt.
            IList<CwlbCommandViewModel> cobjList = objValue as IList<CwlbCommandViewModel>;

            // Suchen des Elements in der Liste
            CwlbCommandViewModel objCommand = cobjList.FirstOrDefault(
                objSearch => StringComparer.CurrentCultureIgnoreCase.Compare(
                    ((CwlbDelegatingCommand)objSearch.Command).Name,
                    (string)objParameter) == 0) as CwlbCommandViewModel;

            // Prüfen des Resultats und entsprechende Rückgabe
            if (objCommand == null)
            {
                return null;
            }
            else
            {
                return objCommand.Command;
            }
        }

        /// <summary>
        /// Methode für die Rückwärtskonvertierung (Ursprung der Methode ist IValueConverter). Diese Methode wird in der Awendung
        /// nicht benötigt.
        /// </summary>
        /// <param name="objValue">Quellenobjekt aus dem das gesuchte Element konvertiert werden soll.</param>
        /// <param name="objTargetType">Zieltyp für die Konvertierung.</param>
        /// <param name="objParameter">Definition des gesuchten Elements.</param>
        /// <param name="objCulture">Definition der zu verwendenden Kultur.</param>
        /// <returns>Diese Methode hat keinen Rückgabewert. Sie lösst immer eine Ausnahem des Typs NotImplementeedException aus.</returns>
        public object ConvertBack(object objValue, Type objTargetType, object objParameter, System.Globalization.CultureInfo objCulture)
        {
            throw new NotImplementedException("Der Konverter 'CwlbCommandConverter' implementiert dies gewünschte Methode nicht.");
        }
    }
}
