﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Input;
using System.Windows.Data;
using System.Windows.Controls;
using System.Diagnostics;
using System.ComponentModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert eine Übersetzung eines weitergeleiteten Ereignisses in ein Viewmodel
    /// </summary>
    public class CwlbEventCommandMapping
    {
        /// <summary>
        /// Eigentliches weitergeleitetes Ereignis
        /// </summary>
        internal RoutedEvent RoutedEvent;

        /// <summary>
        /// Befehl in den das weitergeleitete Ereignis übersetzt werden soll.
        /// </summary>
        internal ICommand Command { get; set; }

        /// <summary>
        /// Name des weitergeleiteten Ereignisses.
        /// </summary>
        public string EventName { get; set; }

        public string ElementType { get; set; }

        /// <summary>
        /// Name des Viewmedells in dem der zu übersetzende Befehl gesucht werden soll.
        /// </summary>
        public string ViewModelName { get; set; }

        /// <summary>
        /// Name des Befehls, in den das weitergeleitete Ereignis übersetzt werden soll.
        /// </summary>
        public string CommandName { get; set; }

        /// <summary>
        /// Führt die eigentliche Übersetzung aus.
        /// </summary>
        /// <param name="objSender">Absender des Ereignisses.</param>
        /// <param name="objEventArgs">Argumente des Ereignisses.</param>
        internal void Mapping(object objSender, RoutedEventArgs objEventArgs)
        {
            Trace.WriteLine( string.Format(
                "Mapping SenderTyp: {0}, Senderkontext: {1}, Argtyp: {2}",
                    objSender.GetType().Name, 
                    (objSender as FrameworkElement) == null ? "N.A." :
                    (objSender as FrameworkElement).DataContext == null ? "Kein DC" : (objSender as FrameworkElement).DataContext.GetType().GetGenericTypeDefinition().Name,
                    objEventArgs.GetType().Name));

            CwlbCommandPanelViewModel objCurrentPanel;
            ICollectionView cobjView = CollectionViewSource.GetDefaultView(CwlbMainViewModel.Current.Views);
            if (cobjView == null)
            {
                throw new InvalidOperationException("Das Hauptobjekt für die Views ist nicht mehr vorhanden.");
            }
            objCurrentPanel = cobjView.CurrentItem as CwlbCommandPanelViewModel;
            if (objCurrentPanel == null )
            {
                // Keine View vorhanden.
                return;
            }

            CwlbCommandViewModel delegatedCommand = objCurrentPanel.Commands.FirstOrDefault(
                objSearch => ((CwlbDelegatingCommand)objSearch.Command).Name == CommandName);
            if (delegatedCommand != null)
            {
                Command = delegatedCommand.Command;
            }
            else
            {
                // Prüfen, ob für das Hauptmodell gearbeitet werden soll
                if (ViewModelName == typeof(CwlbMainViewModel).Name)
                {
                    Command = CwlbMainViewModel.Current.Commands.FirstOrDefault(
                        objSearch => ((CwlbDelegatingCommand)objSearch.Command).Name == CommandName).Command;
                }
                else if (ViewModelName == typeof(CwlbContactListViewModel).Name)
                {
                    ListView lvData = objSender as ListView;
                    if (lvData != null)
                    {
                        Command = ((CwlbCommandPanelViewModel)lvData.SelectedItem).Commands.FirstOrDefault(
                            objSearch => ((CwlbDelegatingCommand)objSearch.Command).Name == CommandName).Command;
                    }
                }
                else
                {
                    throw new InvalidOperationException("Ooops!");
                }
            }

            // Ausführen des Befehls
            if (Command != null)
            {
                Command.Execute(objEventArgs);
            }
            else
            {
                throw new InvalidOperationException("Der Befehl für die Übersetzung des weitergeleiteten Ereginisses wurde nicht gefunden.");
            }
        }
    }
}
