﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert Methoden für Inhaltsüberprüfung.
    /// </summary>
    public static class CwlbValidation
    {
        /// <summary>
        /// Prüft ob eine Zeichnfolge leer ist.
        /// </summary>
        /// <param name="strPropertyName">Name der Eigenschaft deren Inhalt geprüft werden soll.</param>
        /// <param name="strContent">Inhalt der Eigenschaft.</param>
        /// <returns>Liefert einen lokalisierten Fehlertext.</returns>
        public static string ValidateEmptyString(string strPropertyName, string strContent)
        {
            if (string.IsNullOrWhiteSpace(strContent))
            {
                return string.Format(Properties.MVVMDemo.PropertyEmptyNotAllowed, strPropertyName);
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// Prüft ob eine Email addresse korrekt ist.
        /// </summary>
        /// <param name="strContent">Inhalt der geprüft werden soll.</param>
        /// <returns>Liefert einen lokalisierten Fehlertext.</returns>
        public static string ValidateEmail(string strContent)
        {
            if ( string.IsNullOrWhiteSpace(strContent) || ! Regex.IsMatch(strContent, @"\w+([-+.']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*"))
            {
                return Properties.MVVMDemo.PropertyEmailNotValid;
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// Prüft das angegebene Datum auf die Gültigkeit.
        /// </summary>
        /// <param name="dtBirthdate">Datum das geprüft werden soll.</param>
        /// <returns>Liefert einen lokalisierten Fehlertext.</returns>
        public static string ValidateBirthdate(DateTime dtBirthdate)
        {
            // Prüfe auf Zukunft
            if (dtBirthdate > DateTime.Now)
            {
                return Properties.MVVMDemo.PropertyBirthdateNotValid;
            }
            else if (dtBirthdate < new DateTime(1900, 1, 1))
            {
                return Properties.MVVMDemo.PropertyBirthdateNotValid;
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// Prüft ob alle enthaltenen Zeichen rein numerisch sind.
        /// </summary>
        /// <param name="_strPhone">Enthaltene zeichen</param>
        /// <returns>Liefert einen lokalisierten Fehlertext.</returns>
        internal static string ValidateNumericOnly(string strPhone)
        {
            if (string.IsNullOrWhiteSpace(strPhone))
            {
                return string.Format(Properties.MVVMDemo.PropertyEmptyNotAllowed, Properties.MVVMDemo.LabelPhone.Replace("_", ""));
            }

            foreach(char chCurrent in strPhone)
            {
                if (char.IsNumber(chCurrent) || char.IsSeparator(chCurrent))
                {
                    continue;
                }
                else
                {
                    return Properties.MVVMDemo.PropertyPhoneNumberInvalid;
                }
            }
            return null;
        }
    }
}
