﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die Daten einer juristischen Person.
    /// </summary>
    public class CwlbCompanyContact : CwlbContact
    {
        /// <summary>
        /// Erstellt ein Objekt einer natürlichen Person mit einer Identifikation.
        /// </summary>
        /// <returns></returns>
        public static CwlbCompanyContact Create()
        {
            return new CwlbCompanyContact { ID = Guid.NewGuid().ToString() };
        }

        /// <summary>
        /// Liefert oder definiert die Adresse der Website.
        /// </summary>
        public string WebSite { get; set; }

        /// <summary>
        /// Liefert oder definiert die Steuernummer der Firma.
        /// </summary>
        public string TaxIdentifier { get; set; }
        protected const string _strTaxIdentifier = "TaxIdentifier";

        /// <summary>
        /// Definiert eine Methode, die eine Liste der zu überprüfenden Eigenschaften enthält.
        /// </summary>
        /// <returns>Array mit den zu überprüfenden Eigenschaftsnamen.</returns>
        protected override string[] GetValidatedProperties()
        {
            return new string[]
            {
                _strID,
                _strName,
                _strAddress,
                _strEmail,
                _strPhone,
                _strTaxIdentifier
            };
        }

        /// <summary>
        /// Liefert den Fehlertext für eine definierte Eigenschaft.
        /// </summary>
        /// <param name="strPropertyName">Eigenschaft die überprüft werden soll.</param>
        /// <returns>Fehlertext.</returns>
        protected override string GetValidationErrorText(string strPropertyName)
        {
            // Basisklasse anfragen.
            string strBaseClass = base.GetValidationErrorText(strPropertyName);
            if (!string.IsNullOrWhiteSpace(strBaseClass))
            {
                return strBaseClass;
            }

            // Ist nicht in der Basisklasse vorhanden und wird hier geprüft.
            switch (strPropertyName)
            {
                default:
                    return null;
                case _strTaxIdentifier:
                    return CwlbValidation.ValidateEmptyString(_strTaxIdentifier, TaxIdentifier);
            }
        }
    }
}
