﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die gemeinsamen Daten von Person und Firma.
    /// </summary>
    public abstract class CwlbContact : IDataErrorInfo
    {
        /// <summary>
        /// Liefert oder definiert die eindeutige Identifiaktion eines Kontakts.
        /// </summary>
        public string ID { get; set; }
        protected const string _strID = "ID";

        /// <summary>
        /// Liefert oder definiert den Namen des Kontakts.
        /// </summary>
        public string Name { get; set; }
        protected const string _strName = "Name";

        /// <summary>
        /// Liefert oder definiert die komplette Anschrift des Kontakts.
        /// </summary>
        public string Address { get; set; }
        protected const string _strAddress = "Address";

        /// <summary>
        /// Liefert oder definiert die Email-Adresse des Kontakts.
        /// </summary>
        public string Email { get; set; }
        protected const string _strEmail = "Email";

        /// <summary>
        /// Liefert oder definiert die Telefonnummer des Kontakts
        /// </summary>
        public string Phone { get; set; }
        protected const string _strPhone = "Phone";

        /// <summary>
        /// Liefert oder definiert true, wenn der Kontakt ein Kunde ist.
        /// </summary>
        public bool IsCustomer { get; set; }

        /// <summary>
        /// Liefert oder definiert den Gesmatumsatz, der mit dem Kontakt gemacht wurde.
        /// </summary>
        public double TotalSales { get; set; }

        /// <summary>
        /// Liefert true, wenn keine Fehler in den Daten vorhanden sind.
        /// </summary>
        public bool IsValid
        {
            get
            {
                foreach (string property in GetValidatedProperties())
                {
                    if (GetValidationErrorText(property) != null)
                    {
                        return false;
                    }
                }

                return true;
            }
        }

        /// <summary>
        /// Definiert eine Methode, die eine Liste der zu überprüfenden Eigenschaften enthält.
        /// </summary>
        /// <returns>Array mit den zu überprüfenden Eigenschaftsnamen.</returns>
        protected abstract string[] GetValidatedProperties();

        /// <summary>
        /// Liefert einen Fehler für das ganze Objekt.
        /// </summary>
        string IDataErrorInfo.Error
        {
            get {
                // Diese Möglichkeit wird hier nicht genutzt.
                return null; 
            }
        }

        /// <summary>
        /// Liefert einen Fehlertext für das definiert Element.
        /// </summary>
        /// <param name="strPropertyName">Name der zu überprüfenden Eigenschaft.</param>
        /// <returns>Fehlertext oder null wenn fehlerfreier Zustand.</returns>
        string IDataErrorInfo.this[string strPropertyName]
        {
            get
            {
                return GetValidationErrorText(strPropertyName);
            }
        }

        /// <summary>
        /// Liefert einen Fehlertext für die definierte Eigenschaft.
        /// </summary>
        /// <param name="strPropertyName">Name der zu überprüfenden Eigenschaft.</param>
        /// <returns>Fehlertext.</returns>
        protected virtual string GetValidationErrorText(string strPropertyName)
        {
            switch (strPropertyName)
            {
                default:
                    return null;
                case _strID:
                    return CwlbValidation.ValidateEmptyString(_strID, ID);
                case _strName:
                    return CwlbValidation.ValidateEmptyString(_strName, Name);
                case _strAddress:
                    return CwlbValidation.ValidateEmptyString(_strAddress, Address);
                case _strEmail:
                    return CwlbValidation.ValidateEmail(Email);
                case _strPhone:
                    return CwlbValidation.ValidateNumericOnly(Phone);
            }
        }

        /// <summary>
        /// Liefert true, wenn der Kontakt gelöscht werden kann, andernfalls false.
        /// </summary>
        public bool CanDelete
        {
            get
            {
                // Kunden können nicht gelöscht werden.
                if (IsCustomer)
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
        }
    }
}
