﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die Daten einer natürlichen Person.
    /// </summary>
    public class CwlbPersonContact : CwlbContact
    {
        /// <summary>
        /// Erstellt ein Objekt einer natürlichen Person mit einer Identifikation.
        /// </summary>
        /// <returns></returns>
        public static CwlbPersonContact Create()
        {
            return new CwlbPersonContact 
            {
                ID = Guid.NewGuid().ToString(),
                Birthdate = new DateTime(1980,1,1)
            };
        }

        /// <summary>
        /// Liefert oder definiert dîe Anrede der Person.
        /// </summary>
        public string Title { get; set; }

        /// <summary>
        /// Liefert oder definiert den Vornamen einer Person.
        /// </summary>
        public string FirstName { get; set; }

        /// <summary>
        /// Liefert oder definiert das Geburtsdatum der Person.
        /// </summary>
        public DateTime Birthdate { get; set; }
        protected const string _strBirthdate = "Birthdate";

        /// <summary>
        /// Liefert oder definiert die Telefonnummer des mobilen Anschlusses.
        /// </summary>
        public string MobilePhone { get; set; }

        /// <summary>
        /// Definiert eine Methode, die eine Liste der zu überprüfenden Eigenschaften enthält.
        /// </summary>
        /// <returns>Array mit den zu überprüfenden Eigenschaftsnamen.</returns>
        protected override string[] GetValidatedProperties()
        {
            return new string[]
            {
                _strID,
                _strName,
                _strAddress,
                _strEmail,
                _strPhone,
                _strBirthdate
            };
        }

        /// <summary>
        /// Liefert den Fehlertext für eine definierte Eigenschaft.
        /// </summary>
        /// <param name="strPropertyName">Eigenschaft die überprüft werden soll.</param>
        /// <returns>Fehlertext.</returns>
        protected override string GetValidationErrorText(string strPropertyName)
        {
            // Basisklasse anfragen.
            string strBaseClass = base.GetValidationErrorText(strPropertyName);
            if (!string.IsNullOrWhiteSpace(strBaseClass))
            {
                return strBaseClass;
            }

            // Ist nicht in der Basisklasse vorhanden und wird hier geprüft.
            switch (strPropertyName)
            {
                default:
                    return null;
                case _strBirthdate:
                    return CwlbValidation.ValidateBirthdate(Birthdate);
            }
        }
    }
}
