﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using System.Collections.ObjectModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Stelle eine Basisklasse für einen Arbeistbereich in einer Benutzeroberfläche bereit.
    /// Die Arbeitsfläche kann unterstützt die Handhabung von Befehlen.
    /// </summary>
    public abstract class CwlbCommandPanelViewModel : CwlbViewModelBase
    {
        /// <summary>
        /// Liste der verwendeten Befehle.
        /// </summary>
        protected List<CwlbCommandViewModel> _cobjCommands = new List<CwlbCommandViewModel>();

        /// <summary>
        /// Liste der verwendeten Befehle als schreibgeschützte Liste.
        /// </summary>
        private ReadOnlyCollection<CwlbCommandViewModel> _cobjCommandViews;

        /// <summary>
        /// Liefert die Liste der Befehle.
        /// </summary>
        public ReadOnlyCollection<CwlbCommandViewModel> Commands
        {
            get
            {
                if (_cobjCommandViews == null)
                {
                    _cobjCommandViews = new ReadOnlyCollection<CwlbCommandViewModel>(_cobjCommands);
                }
                return _cobjCommandViews;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public CwlbCommandPanelViewModel()
        {
            // Registrieren des Befehls für das Schliessen der View
            _cobjCommands.Add(new CwlbCommandViewModel(
                Properties.MVVMDemo.CmdDisplayViewNameClose,
                new CwlbDelegatingCommand(action => this.OnClose(), GwlbMVVM.COMMAND_CLOSE)));
        }

        /// <summary>
        /// Ereignis das genutzt wird, wenn die Arbeistfläche geschlossen wird.
        /// </summary>
        public event EventHandler EventClose = delegate { };

        /// <summary>
        /// Löst das Ereignis für das Schliessen der Arbeitsfläche aus
        /// </summary>
        void OnClose()
        {
            EventClose(this, EventArgs.Empty);
        }
    }
}
