﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WeroSoft.Samples
{
    /// <summary>
    /// View model eines Kontakts für eine natürliche Person.
    /// </summary>
    public class CwlbContactCompanyViewModel : CwlbContactViewModel
    {
        /// <summary>
        /// Interner Wrappereigenschaft um den gespeicherten Kontakt in der Basisklasse 
        /// streng typisiert anzusprechen.
        /// </summary>
        private CwlbCompanyContact Company
        {
            get
            {
                return Contact as CwlbCompanyContact;
            }
        }

        /// <summary>
        /// Liefert oder definiert das Titel Label.
        /// </summary>
        public string LabelPageTitle { get; set; }

        /// <summary>
        /// Liefert oder definiert das Label für die Adresse der Webseite.
        /// </summary>
        public string LabelWebSite { get; set; }

        /// <summary>
        /// Liefert oder definiert die Steuernummer der Unternehmung.
        /// </summary>
        public string LabelTaxIdentifier { get; set; }

        /// <summary>
        /// Definiert den Feldnamen der Webseite der Firma.
        /// </summary>
        protected const string _strWebSite = "WebSite";

        /// <summary>
        /// Liefert oder definiert die URL der Webseite der Firma.
        /// </summary>
        public string WebSite
        {
            get
            {
                return Company.WebSite;
            }
            set
            {
                if (value == Company.WebSite)
                {
                    return;
                }

                Company.WebSite = value;
                base.OnPropertyChanged(_strWebSite);
            }
        }

        /// <summary>
        /// Definiert den Feldnamen die der Steuernummer der Firma.
        /// </summary>
        protected const string _strTaxIdentifier = "TaxIdentifier";

        /// <summary>
        /// Liefert oder definiert die Steuernummer der Firma.
        /// </summary>
        public string TaxIdentifier
        {
            get
            {
                return Company.TaxIdentifier;
            }
            set
            {
                if (value == Company.TaxIdentifier)
                {
                    return;
                }

                Company.TaxIdentifier = value;
                base.OnPropertyChanged(_strTaxIdentifier);
            }
        }

        /// <summary>
        /// Liefert oder definiert den Namen der View.
        /// </summary>
        /// <remarks>
        /// Der Viewname wird für die Beschriftung der Registerkarte verwendet und 
        /// soll nicht mit dem ViewTitle (der Überschrift in der View) verwechselt werden.
        /// </remarks>
        public override string ViewName
        {
            get
            {
                return Company.Name;
            }
        }

        /// <summary>
        /// Instanziert eine neue Instanz der Klasse CwlbContactPersonViewModel.
        /// </summary>
        /// <param name="objContact"></param>
        /// <param name="objData"></param>
        public CwlbContactCompanyViewModel(CwlbContact objContact, CwlbInMemoryData objData)
            : base(objContact, objData)
        {
            LabelPageTitle = Properties.MVVMDemo.LabelPageTitleContactCompany;
            LabelWebSite = Properties.MVVMDemo.LabelWebSite;
            LabelTaxIdentifier = Properties.MVVMDemo.LabelTaxIdentifier;
        }
    }
}
