﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Repräsentiert das Viewmodel der Liste aller Kontakte.
    /// </summary>
    public class CwlbContactListViewModel : CwlbCommandPanelViewModel
    {
        // Referenz auf die verwendeten Daten
        private readonly CwlbInMemoryData Data;

        /// <summary>
        /// Liste der angezeigten Elemente des Viewmodels.
        /// </summary>
        public ObservableCollection<CwlbCommandPanelViewModel> Contacts { get; private set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbContactListViewModel.
        /// </summary>
        /// <param name="objData">Zu verwendende Daten.</param>
        public CwlbContactListViewModel(CwlbInMemoryData objData)
        {
            // Definieren des Viewnamens und der Daten
            ViewName = Properties.MVVMDemo.ContactListViewName;
            Data = objData;

            // Registriere den Befehl für das Bearbeiten

            // Registrieren der Ereignisbehandlungen für die Befehle Kontakt hinzufügen und löschen
            Data.EventContactAdded += new EventHandler<CwlbCustomerEventArgs>(ContactAdded);
            Data.EventContactDeleted += new EventHandler<CwlbCustomerEventArgs>(ContactDeleted);

            // Erstellen des Dateninhalts der View
            CreateDataContent();
        }

        /// <summary>
        /// Erstellt den Dateninhalt des Modells
        /// </summary>
        private void CreateDataContent()
        {
            // Erstellen von ViewModel-Objekten für jedes Datenelement in den Daten
            List<CwlbCommandPanelViewModel> _cobjContacts = Data.GetContacts()
                .Select(objContact => CwlbContactViewModelFactory.CreateView(objContact, Data)).ToList();

            // Erstellen einer beobachtbaren Auflistung für das UI.
            Contacts = new ObservableCollection<CwlbCommandPanelViewModel>(_cobjContacts);
        }

        /// <summary>
        /// Ereignisbehandlung beim Hinzufügen eines Kontakts in die View.
        /// </summary>
        private void ContactAdded(object sender, CwlbCustomerEventArgs e)
        {
            var viewModel = CwlbContactViewModelFactory.CreateView(e.Contact, Data); //.(e.NewCustomer, _customerRepository);
            this.Contacts.Add(viewModel);
        }

        /// <summary>
        /// Ereignisbehandlung beim Löschen eines Kontakts in die View.
        /// </summary>
        void ContactDeleted(object sender, CwlbCustomerEventArgs e)
        {
            // Lokale Daten
            CwlbContactViewModel objPanelDelete = Contacts.OfType<CwlbContactViewModel>().FirstOrDefault(objSearch => objSearch.ID == e.Contact.ID);

            // Löschen des Befehls löschen
            if (objPanelDelete != null)
            {
                Contacts.Remove(objPanelDelete);
            }
        }

        /// <summary>
        /// Aufräumen des Objekts.
        /// </summary>
        public override void Dispose()
        {
            // Löschen der enthaltenen Liste
            Contacts.Clear();
            //Contacts.CollectionChanged -= new System.Collections.Specialized.NotifyCollectionChangedEventHandler(Contacts_CollectionChanged);

            // Registrierung für das Hinzufügen von Kontakten in die View aufheben.
            Data.EventContactAdded -= new EventHandler<CwlbCustomerEventArgs>(ContactAdded);
            Data.EventContactDeleted -= new EventHandler<CwlbCustomerEventArgs>(ContactDeleted);
        }
    }
}
