﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WeroSoft.Samples
{
    /// <summary>
    /// View model eines Kontakts für eine natürliche Person.
    /// </summary>
    public class CwlbContactPersonViewModel : CwlbContactViewModel
    {
        /// <summary>
        /// Interner Wrappereigenschaft um den gespeicherten Kontakt in der Basisklasse 
        /// streng typisiert anzusprechen.
        /// </summary>
        private CwlbPersonContact Person
        {
            get
            {
                return Contact as CwlbPersonContact;
            }
        }

        /// <summary>
        /// Liefert oder definiert das Titel Label.
        /// </summary>
        public string LabelPageTitle { get; set; }

        /// <summary>
        /// Liefert oder definiert das Label des Titels der Person.
        /// </summary>
        public string LabelTitle { get; set; }

        /// <summary>
        /// Liefert oder definiert das Label des Vornamens der Person.
        /// </summary>
        public string LabelFirstName { get; set; }

        /// <summary>
        /// Liefert oder definiert das Label des Geburtsdatums der Person.
        /// </summary>
        public string LabelBirthdate { get; set; }

        /// <summary>
        /// Liefetr oder definiert das Label für die Telefonnummer des mobilen Anschlusses.
        /// </summary>
        public string LabelMobile { get; set; }

        /// <summary>
        /// Definiert den Feldnamen des Titels des Personenkontakts.
        /// </summary>
        protected const string _strTitle = "Title";

        /// <summary>
        /// Liefert oder definiert den Titel des Personenkontakts.
        /// </summary>
        public string Title
        {
            get
            {
                return Person.Title;
            }
            set
            {
                if (value == Person.Title)
                {
                    return;
                }

                Person.Title = value;
                base.OnPropertyChanged(_strTitle);
            }
        }

        /// <summary>
        /// Definiert den Feldnamen des Vornamens des Personenkontakts.
        /// </summary>
        protected const string _strFirstName = "FirstName";

        /// <summary>
        /// Liefert oder definiert den Vornamen einer Person
        /// </summary>
        public string FirstName
        {
            get
            {
                return Person.FirstName;
            }
            set
            {
                if (value == Person.FirstName)
                {
                    return;
                }

                Person.FirstName = value;
                base.OnPropertyChanged(_strFirstName);
            }
        }

        /// <summary>
        /// Definiert den Feldnamen des Geburtsdatums des Kontakts.
        /// </summary>
        protected const string _strBirthdate = "Birthdate";

        /// <summary>
        /// Liefert oder definiert den Geburtsdatum der Person.
        /// </summary>
        public DateTime Birthdate
        {
            get
            {
                return Person.Birthdate;
            }
            set
            {
                if (value == Person.Birthdate)
                {
                    return;
                }

                Person.Birthdate = value;
                base.OnPropertyChanged(_strBirthdate);
            }
        }

        /// <summary>
        /// Definiert den Feldnamen die Telefonnummer des mobilen Anschlusses der Person.
        /// </summary>
        protected const string _strMobilePhone = "MobilePhone";

        /// <summary>
        /// Liefert oder definiert die Telefonnummer des mobilen Anschlusses.
        /// </summary>
        public string MobilePhone
        {
            get
            {
                return Person.MobilePhone;
            }
            set
            {
                if (value == Person.MobilePhone)
                {
                    return;
                }

                Person.MobilePhone = value;
                base.OnPropertyChanged(_strMobilePhone);
            }
        }


        /// <summary>
        /// Liefert oder definiert den Namen der View.
        /// </summary>
        /// <remarks>
        /// Der Viewname wird für die Beschriftung der Registerkarte verwendet und 
        /// soll nicht mit dem ViewTitle (der Überschrift in der View) verwechselt werden.
        /// </remarks>
        public override string ViewName
        {
            get
            {
                if ( string.IsNullOrWhiteSpace(Person.FirstName))
                {
                    return Person.Name;
                }
                else
                {
                    return string.Format("{1} {0}", Person.Name, Person.FirstName);
                }
            }
        }

        /// <summary>
        /// Instanziert eine neue Instanz der Klasse CwlbContactPersonViewModel.
        /// </summary>
        /// <param name="objContact"></param>
        /// <param name="objData"></param>
        public CwlbContactPersonViewModel(CwlbContact objContact, CwlbInMemoryData objData)
            : base(objContact, objData)
        {
            LabelPageTitle = Properties.MVVMDemo.LabelPageTitleContactPerson;
            LabelTitle = Properties.MVVMDemo.LabelTitle;
            LabelFirstName = Properties.MVVMDemo.LabelFirstName;
            LabelBirthdate = Properties.MVVMDemo.LabelBirthdate;
            LabelMobile = Properties.MVVMDemo.LabelMobile;
        }
    }
}
