﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Stellt eine abstrakte Basisklasse für die Gemeinsamkeiten von Kontakt zu Person und
    /// Firma zur Verfügung.
    /// </summary>
    public abstract class CwlbContactViewModel : CwlbCommandPanelViewModel, IDataErrorInfo
    {
        // Daten
        CwlbInMemoryData _objData;

        /// <summary>
        /// Liefert oder definiert den Kontakt, der mit diesem Objekt verwaltet wird.
        /// </summary>
        public CwlbContact Contact;

        /// <summary>
        /// Liefert einen Wert der aussagt, ob der Kontakt neu ist oder nicht.
        /// </summary>
        private bool IsNewContact
        {
            get 
            {
                // Prüfe direkt in der Datenbank, ob der Kontakt schon vorhanden ist.
                return !_objData.ContainsContact(Contact);
            }
        }

        /// <summary>
        /// Liefert oder definiert das Label des Namens eines Kontakts.
        /// </summary>
        public virtual string LabelName { get; set; }

        /// <summary>
        /// Liefert oder definiert das Label der Adresse des Kontakts.
        /// </summary>
        public virtual string LabelAddress { get; set; }

        /// <summary>
        /// Liefert oder definiert das Label der Emailadresse des Kontakts.
        /// </summary>
        public virtual string LabelEmail { get; set; }

        /// <summary>
        /// Liefert oder definiert das Label des Festnetzanschlusses des Kontakts.
        /// </summary>
        public virtual string LabelPhone { get; set; }

        /// <summary>
        /// Liefert oder definiert das Label der Eigenschaft Kunde des Kontakts.
        /// </summary>
        public string LabelIsCustomer { get; set; }

        /// <summary>
        /// Liefert oder definiert das Label der Eigenschaft Total Verkäfe mit dem Kontakt.
        /// </summary>
        public string LabelTotalSales { get; set; }

        /// <summary>
        /// Definiert den Feldnamen der Identifikation des Kontakts.
        /// </summary>
        protected const string _strID = "ID";

        /// <summary>
        /// Liefert oder definiert die Identifiaktion des Kontakts
        /// </summary>
        public string ID
        {
            get
            {
                return Contact.ID;
            }
            set
            {
                if (value == Contact.ID)
                {
                    return;
                }

                Contact.ID = value;
                base.OnPropertyChanged(_strID);
            }
        }

        /// <summary>
        /// Definiert den Feldnamen des Namens des Kontakts.
        /// </summary>
        protected const string _strName = "Name";

        /// <summary>
        /// Liefert oder definiert den Namen des Kontakts.
        /// </summary>
        public string Name
        {
            get
            {
                return Contact.Name;
            }
            set
            {
                if (value == Contact.Name)
                {
                    return;
                }

                Contact.Name = value;
                base.OnPropertyChanged(_strName);
            }
        }

        /// <summary>
        /// Definiert den Feldnamen der Adresse des Kontakts.
        /// </summary>
        protected const string _strAddress = "Address";

        /// <summary>
        /// Liefert oder definiert den Vornamen des Kontakts.
        /// </summary>
        public string Address
        {
            get
            {
                return Contact.Address;
            }
            set
            {
                if (value == Contact.Address)
                {
                    return;
                }

                Contact.Address = value;
                base.OnPropertyChanged(_strAddress);
            }
        }

        /// <summary>
        /// Definiert den Feldnamen der Emailadresse des Kontakts.
        /// </summary>
        protected const string _strEmail = "Email";

        /// <summary>
        /// Liefert oder definiert die Email-Adresse des Kontakts.
        /// </summary>
        public string Email
        {
            get
            {
                return Contact.Email;
            }
            set
            {
                if (value == Contact.Email)
                {
                    return;
                }

                Contact.Email = value;
                base.OnPropertyChanged(_strEmail);
            }
        }

        /// <summary>
        /// Definiert den Feldnamen der Telefonnummer des Kontakts.
        /// </summary>
        protected const string _strPhone = "Phone";

        /// <summary>
        /// Liefert oder definiert die Telefonnummer des Kontakts.
        /// </summary>
        public string Phone
        {
            get
            {
                return Contact.Phone;
            }
            set
            {
                if (value == Contact.Phone)
                {
                    return;
                }

                Contact.Phone = value;
                base.OnPropertyChanged(_strPhone);
            }
        }

        /// <summary>
        /// Definiert den Feldnamen der Kundenmarkierung des Kontakts.
        /// </summary>
        protected const string _strIsCustomer = "IsCustomer";

        /// <summary>
        /// Liefert oder definiert true, wenn es sich um einen Kundenkontakt handelt.
        /// </summary>
        public bool IsCustomer
        {
            get
            {
                return Contact.IsCustomer;
            }
            set
            {
                if (value == Contact.IsCustomer)
                {
                    return;
                }

                Contact.IsCustomer = value;
                base.OnPropertyChanged(_strIsCustomer);
                base.OnPropertyChanged(_strIsCustomerText);
            }
        }


        /// <summary>
        /// Definiert den Feldnamen der Kundenmarkierung wie sie in der Benutzeroberfläche verwendet wird.
        /// </summary>
        protected const string _strIsCustomerText = "IsCustomerText";

        /// <summary>
        /// Liefert einen lokalisierten Text, ob der Kontakt ein Kunde repräsentiert oder nicht.
        /// </summary>
        public string IsCustomerText
        {
            get
            {
                if (IsCustomer)
                {
                    return Properties.MVVMDemo.Yes;
                }
                else
                {
                    return Properties.MVVMDemo.No;
                }
            }
        }

        /// <summary>
        /// Definiert den Feldnamen des Totals der Verkäufe des Kontakts.
        /// </summary>
        protected const string _strTotalSales = "TotalSales";

        /// <summary>
        /// Liefert oder definiert das Total der Verkäufe mit dem Kontakt.
        /// </summary>
        public double TotalSales
        {
            get
            {
                return Contact.TotalSales;
            }
            set
            {
                if (value == Contact.TotalSales)
                {
                    return;
                }

                Contact.TotalSales = value;
                base.OnPropertyChanged(_strTotalSales);
            }
        }

        /// <summary>
        /// Liefert den Ansichtsnamen des Kontakts.
        /// </summary>
        public override string ViewName
        {
            get
            {
                return base.ViewName;
            }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz des Typs CwlbContactViewModel.
        /// </summary>
        /// <param name="objContact">Kontaktobjekt das mit der Ansicht verwaltet wird.</param>
        /// <param name="objData">Datensammlung.</param>
        public CwlbContactViewModel(CwlbContact objContact, CwlbInMemoryData objData)
        {
            Contact = objContact;
            _objData = objData;

            // Erstellen der Label aus den Ressourcen
            LabelName = Properties.MVVMDemo.LabelName;
            LabelAddress = Properties.MVVMDemo.LabelAddress;
            LabelEmail = Properties.MVVMDemo.LabelEmail;
            LabelPhone = Properties.MVVMDemo.LabelPhone;
            LabelIsCustomer = Properties.MVVMDemo.LabelIsCustomer;
            LabelTotalSales = Properties.MVVMDemo.LabelTotalSales;

            // Registrieren des Befehls für das Speichern
            _cobjCommands.Add(new CwlbCommandViewModel(
                Properties.MVVMDemo.CmdDisplayViewNameSave, 
                new CwlbDelegatingCommand(
                    objParam => this.Save(), 
                    objParam => this.CanSave,
                    GwlbMVVM.COMMAND_SAVE)));

            // Registrieren des Befehls für das Löschen
            _cobjCommands.Add(new CwlbCommandViewModel(
                Properties.MVVMDemo.CmdDisplayViewNameDelete,
                new CwlbDelegatingCommand(
                    objEventArgs => this.Delete(objEventArgs as RoutedEventArgs),
                    objEventArgs => this.CanDelete(objEventArgs as RoutedEventArgs),
                    GwlbMVVM.COMMAND_DELETE)));

            // Registrieren des Befehls für das Bearbeiten
            _cobjCommands.Add(new CwlbCommandViewModel(
                Properties.MVVMDemo.CmdDisplayViewNameEdit,
                new CwlbDelegatingCommand(
                    objEventArgs => this.Edit(objEventArgs as RoutedEventArgs),
                    GwlbMVVM.COMMAND_EDIT)));
        }

        /// <summary>
        /// Speichert den angegebenen Kontakt.
        /// </summary>
        public void Save()
        {
            if (!Contact.IsValid)
            {
                throw new InvalidOperationException("Kann nicht speichern");
            }

            if (this.IsNewContact)
            {
                _objData.AddContact(Contact);
            }

            base.OnPropertyChanged("ViewName");
        }

        /// <summary>
        /// Prüft ob ein Kontakt gelöscht werden kann.
        /// </summary>
        /// <param name="objParam"></param>
        /// <returns></returns>
        public bool CanDelete(RoutedEventArgs objParam)
        {
            return Contact.CanDelete;
        }

        /// <summary>
        /// Löschen eines Kontakts
        /// </summary>
        /// <param name="objParam">Argumente für das Löschen des Kontakts.</param>
        public void Delete(RoutedEventArgs objParam)
        {
            // Prüfen, ob ein Tastenereignis eingetroffen ist
            KeyEventArgs objKeyEventArgs = objParam as KeyEventArgs;
            if (objKeyEventArgs != null && objKeyEventArgs.Key != Key.Delete)
            {
                // Alle andern Ereignisse werden nicht unterstützt
                return;
            }

            // Sicherheitsabfrage zum Löschen
            if (MessageBox.Show(
                string.Format("Wollen Sie den Kontakt '{0}' wirlich löschen?", ViewName),
                "Löschen bestätigen",
                MessageBoxButton.YesNo,
                MessageBoxImage.Question) == MessageBoxResult.No)
            {
                return;
            }

            // Löschen des Kontakts
            _objData.DeleteContact(Contact);
        }

        /// <summary>
        /// Bearbeiten eines Kontaks
        /// </summary>
        /// <param name="objParam">Parameter für das Bearbeiten.</param>
        public void Edit(RoutedEventArgs objParam)
        {
            // Prüfen, ob die Ansicht schon offen ist
            if (CwlbMainViewModel.Current.Views.OfType<CwlbContactViewModel>().FirstOrDefault(objSearch => objSearch.ID == this.ID) == null)
            {
                CwlbMainViewModel.Current.Views.Add(this);
            }
            CwlbMainViewModel.Current.SetActiveView(this);
        }

        /// <summary>
        /// Liefert einen Wert, der aussagt, ob der Kontakt gespeichert werden kann oder nicht.
        /// </summary>
        public bool CanSave
        {
            get {
                return Contact.IsValid;
            }
        }

        /// <summary>
        /// Liefert eine Fehlermeldung für das Objekt. 
        /// </summary>
        string IDataErrorInfo.Error
        {
            get 
            {
                return (Contact as IDataErrorInfo).Error;
            }
        }

        /// <summary>
        /// Liefert eine Fehlermeldung für eine definierte Eigenschaft.
        /// </summary>
        /// <param name="propertyName"></param>
        /// <returns></returns>
        string IDataErrorInfo.this[string propertyName]
        {
            get
            {
                string error = null;
                error = (Contact as IDataErrorInfo)[propertyName];
                CommandManager.InvalidateRequerySuggested();
                return error;
            }
        }
    }
}
