﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Diagnostics;

namespace WeroSoft.Samples
{
    public abstract class CwlbViewModelBase : INotifyPropertyChanged, IDisposable
    {
        /// <summary>
        /// Wird ausgelöst, wenn eine Eigenschaft der Klasse geändert wurde.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        /// <summary>
        /// Liefert oder definiert den Anzeigenamen einer View.
        /// </summary>
        public virtual string ViewName { get; protected set; }

        /// <summary>
        /// Löst das Ereignis <see cref="PropertyChanged"/> aus.
        /// </summary>
        /// <param name="strPropertyName">The property that has a new value.</param>
        protected virtual void OnPropertyChanged(string strPropertyName)
        {
            VerifyPropertyName(strPropertyName);
            PropertyChanged(this, new PropertyChangedEventArgs(strPropertyName));
        }

        /// <summary>
        /// Warnt einen Entwickler, wenn der übergebene Name nicht einem
        /// Namen einer Eigenschaft entspricht. Diese Methode exisitiert
        /// im Releasbuild nicht.
        /// </summary>
        [Conditional("DEBUG"), DebuggerStepThrough]
        private void VerifyPropertyName(string strPropertyName)
        {
            if (TypeDescriptor.GetProperties(this)[strPropertyName] == null)
            {
                throw new InvalidOperationException(string.Format("Die Eigenschaft mit Namen {0} existiert nicht", strPropertyName));
            }
        }

        /// <summary>
        /// Wird aufgerufen wenn das objekt vom Modell entfernt wird.
        /// </summary>
        public virtual void Dispose()
        {
        }
    }
}
