﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WeroSoft.Samples.CustomUserControls
{
    /// <summary>
    /// Definiert eine Canvas mit zentriertem Koordinatensystem, so dass alle vier Quadranten des
    /// kartesischen Koordinatensystems sichtbar sind.
    /// </summary>
    public class CwlbMathCanvas : Canvas
    {
        /// <summary>
        /// Liefert oder definiert true, wenn die Hauptachsen gezeichnet werden sollen.
        /// </summary>
        public bool ShowAxis
        {
            get { return (bool)GetValue(ShowAxisProperty); }
            set { SetValue(ShowAxisProperty, value); }
        }

        /// <summary>
        /// Definition der Abhängigkeitseigenschaft für ShowAxis.
        /// </summary>
        public static readonly DependencyProperty ShowAxisProperty =
            DependencyProperty.Register("ShowAxis", typeof(bool), typeof(CwlbMathCanvas), new UIPropertyMetadata(true));

        /// <summary>
        /// Liefert oder definiert true, wenn das Hilfsgitter gezeichnet werden soll.
        /// </summary>
        public bool ShowGrid
        {
            get { return (bool)GetValue(ShowGridProperty); }
            set { SetValue(ShowGridProperty, value); }
        }

        /// <summary>
        /// Definition der Abhängigkeitseigenschaft für ShowGrid.
        /// </summary>
        public static readonly DependencyProperty ShowGridProperty =
            DependencyProperty.Register("ShowGrid", typeof(bool), typeof(CwlbMathCanvas), new UIPropertyMetadata(true));

        /// <summary>
        /// Liefert oder definiert die horizontale Distanz zwischen zwei Hilfslinien.
        /// </summary>
        /// <remarks>
        /// Der Standardwert ist 50.
        /// </remarks>
        public double GridX
        {
            get { return (double)GetValue(GridXProperty); }
            set { SetValue(GridXProperty, value); }
        }

        /// <summary>
        /// Definition der Abhängigkeitseigenschaft für GridX.
        /// </summary>
        public static readonly DependencyProperty GridXProperty =
            DependencyProperty.Register("GridX", typeof(double), typeof(CwlbMathCanvas), new UIPropertyMetadata(50.0));

        /// <summary>
        /// Liefert oder definiert die vertikale Distanz zwischen zwei Hilfslinien.
        /// </summary>
        public double GridY
        {
            get { return (double)GetValue(GridYProperty); }
            set { SetValue(GridYProperty, value); }
        }

        /// <summary>
        /// Definition der Abhängigkeitseigenschaft für GridY.
        /// </summary>
        public static readonly DependencyProperty GridYProperty =
            DependencyProperty.Register("GridY", typeof(double), typeof(CwlbMathCanvas), new UIPropertyMetadata(50.0));

        /// <summary>
        /// Liefert oder definiert die positive Breite des Grids (Halbe Gesamtbreite).
        /// </summary>
        public double GridWidth
        {
            get { return (double)GetValue(GridWidthProperty); }
            set { SetValue(GridWidthProperty, value); }
        }

        /// <summary>
        /// Definition der Abhängigkeitseigenschaft für GridWidth.
        /// </summary>
        public static readonly DependencyProperty GridWidthProperty =
            DependencyProperty.Register("GridWidth", typeof(double), typeof(CwlbMathCanvas), new UIPropertyMetadata(Math.PI * 3000));


        /// <summary>
        /// Liefert oder definiert die positive Höhe des Grids (Halbe Gesamthöhe).
        /// </summary>
        public double GridHeigh
        {
            get { return (double)GetValue(GridHeighProperty); }
            set { SetValue(GridHeighProperty, value); }
        }

        /// <summary>
        /// Definition der Abhängigkeitseigenschaft für GridHeight.
        /// </summary>
        public static readonly DependencyProperty GridHeighProperty =
            DependencyProperty.Register("GridHeigh", typeof(double), typeof(CwlbMathCanvas), new UIPropertyMetadata(Math.PI * 3000));

        /// <summary>
        /// Liefert oder definiert die Farbe der Hauptachsen.
        /// </summary>
        public Brush AxisBrush
        {
            get { return (Brush)GetValue(AxisBrushProperty); }
            set { SetValue(AxisBrushProperty, value); }
        }

        /// <summary>
        /// Definition der Abhängigkeitseigenschaft für AxisBrush.
        /// </summary>
        public static readonly DependencyProperty AxisBrushProperty =
            DependencyProperty.Register("AxisBrush", typeof(Brush), typeof(CwlbMathCanvas), new UIPropertyMetadata(Brushes.Black));


        /// <summary>
        /// Liefert oder definiert die Farbe des Gitters.
        /// </summary>
        public Brush GridBrush
        {
            get { return (Brush)GetValue(GridBrushProperty); }
            set { SetValue(GridBrushProperty, value); }
        }

        /// <summary>
        /// Definition der Abhängigkeitseigenschaft für GridBrush.
        /// </summary>
        public static readonly DependencyProperty GridBrushProperty =
            DependencyProperty.Register("GridBrush", typeof(Brush), typeof(CwlbMathCanvas), new UIPropertyMetadata(Brushes.DarkGray));

        /// <summary>
        /// Initialisiert die klassenspezifischen Daten des Typs CwlbMathCanvas.
        /// </summary>
        static CwlbMathCanvas()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(CwlbMathCanvas), new FrameworkPropertyMetadata(typeof(CwlbMathCanvas)));
        }

        /// <summary>
        /// Erstellt eine Linie auf der Canvals
        /// </summary>
        /// <param name="dX1">X-Koordinate des Startpunkts.</param>
        /// <param name="dY1">Y-Koordinate des Startpunkts.</param>
        /// <param name="dX2">X-Koordinate des Endpunkts.</param>
        /// <param name="dY2">Y-Koordinate des Endpunkts.</param>
        /// <param name="brush">Farbe der Linie.</param>
        /// <param name="dLineThickness">Thickness of the line.</param>
        /// <returns>Liefert das Linienobjekt, das noch nicht in die Canvas gebunden ist.</returns>
        private Line CreateLine(double dX1, double dY1, double dX2, double dY2, Brush brush, double dLineThickness = 1.0)
        {
            Line line = new Line();
            line.X1 = dX1;
            line.Y1 = dY1;
            line.X2 = dX2;
            line.Y2 = dY2;
            line.Stroke = brush;
            line.StrokeThickness = dLineThickness;
            return line;
        }

        /// <summary>
        /// Erstellt die Achsen, wenn diese konfiguriert sind.
        /// </summary>
        public override void EndInit()
        {
            // Aufrufen der Basisklassenmethode
            base.EndInit();

            // BEMERKUNG:
            // Die untenstehenden Visuals werden in die Liste der enthaltenen Objekte ganz vorne
            // eingefügt. Durch das wird erreicht, dass die Elemente vor den eigentlichen Nutz-
            // elementen gezeichnet werden und enstprechend optisch hinter diesen liegen.

            // Achsen einfügen wenn gewünscht
            if (ShowAxis)
            {
                // X-Hauptachse
                Children.Insert(0, CreateLine(-GridWidth, 0, GridWidth, 0, AxisBrush, 2));
                // Y-Hauptachse
                Children.Insert(0, CreateLine(0, GridHeigh, 0, -GridHeigh, AxisBrush, 2));
            }

            // Hilfsgitter einfügen wenn gewünscht
            if (ShowGrid)
            {
                // Vertikale Unterteilung rechts
                for (double dXOffset = GridX; dXOffset < GridWidth; dXOffset += GridX)
                {
                    Children.Insert(0, CreateLine(dXOffset, GridHeigh, dXOffset, -GridHeigh, GridBrush));
                    Children.Insert(0, CreateLine(-dXOffset, GridHeigh, -dXOffset, -GridHeigh, GridBrush));
                }
                // Horizontale Unterteilung
                for (double dYOffset = GridY; dYOffset < GridHeigh; dYOffset += GridY)
                {
                    Children.Insert(0, CreateLine(-GridWidth, dYOffset, GridWidth, dYOffset, GridBrush));
                    Children.Insert(0, CreateLine(-GridWidth, -dYOffset, GridWidth, -dYOffset, GridBrush));
                }
            }
        }

        /// <summary>
        /// Bestimmt die Position der Elemente auf dem Grid.
        /// </summary>
        /// <param name="sizeArrange">Finale Grösse, die die Canvas haben wird.</param>
        /// <returns>Liefert die Grösse der Figur.</returns>
        protected override Size ArrangeOverride(Size sizeArrange)
        {
            // Lokale Daten
            double x = 0.0;
            double y = 0.0;

            // Bestimmen des Mittelpunkts.
            Point ptCenter = new Point(sizeArrange.Width / 2, sizeArrange.Height / 2);

            // Verschieben der Punkte mit der Transformationsmatrix
            foreach (UIElement objContainedElement in base.InternalChildren)
            {
                if (objContainedElement == null)
                {
                    continue;
                }

                // Spiegeln der Y-Achse. 
                // ACHTUNG:
                // Diese Spiegelung wäre 100% wirksam. Das hiesst, dass auch Texte nun Kopf stehen
                // würden. Deshalb werden entsprechende Textelemente ausgeschlossen.
                if (!(objContainedElement is TextBlock))
                {
                    objContainedElement.RenderTransform = new MatrixTransform(1, 0, 0, -1, 0, 0);
                }

                // Ausgabe des Elements berechnen (mit Berücksichtigung der relativen Positionierung auf der Canvas)
                double dLeft = Canvas.GetLeft(objContainedElement);
                if (double.IsNaN(dLeft))
                {
                    dLeft = 0.0;
                }
                double dTop = Canvas.GetTop(objContainedElement);
                if (double.IsNaN(dTop))
                {
                    dTop = 0.0;
                }
                objContainedElement.Arrange(
                    new Rect(
                        new Point(ptCenter.X + x + dLeft, ptCenter.Y + y - dTop),
                        objContainedElement.DesiredSize));
            }
            return sizeArrange;
        }
    }
}

