﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WeroSoft.Samples.UserControls
{
    /// <summary>
    /// Interaktionslogik für CwlbLabledTextBox.xaml
    /// </summary>
    public partial class CwlbLabledTextBox : UserControl
    {
        /// <summary>
        /// Definiert den Abstand der zwischen Beschriftung und Eingabefeld benutzt wird
        /// </summary>
        private const double MARGIN = 5;

        /// <summary>
        /// Liefert oder definiert die Position des Label-Steuerelements relativ zum Eingabefeld.
        /// </summary>
        public Dock LabelPosition
        {
            get { return (Dock)GetValue(LabelPositionProperty); }
            set { SetValue(LabelPositionProperty, value); }
        }

        // Using a DependencyProperty as the backing store for LabelPosition.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty LabelPositionProperty =
            DependencyProperty.Register("LabelPosition", typeof(Dock), typeof(CwlbLabledTextBox), new UIPropertyMetadata(Dock.Left));


        /// <summary>
        /// Liefert oder definiert den Anzeigetext der Beschriftung.
        /// </summary>
        public string LabelText
        {
            get { return (string)GetValue(LabelTextProperty); }
            set { SetValue(LabelTextProperty, value); }
        }

        // Using a DependencyProperty as the backing store for LabelText.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty LabelTextProperty =
            DependencyProperty.Register("LabelText", typeof(string), typeof(CwlbLabledTextBox), new UIPropertyMetadata(string.Empty));


        /// <summary>
        /// Liefert oder definiert den Inhalt des Eingabefeldes.
        /// </summary>
        public string Text
        {
            get { return (string)GetValue(TextProperty); }
            set { SetValue(TextProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Text.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TextProperty =
            DependencyProperty.Register("Text", typeof(string), typeof(CwlbLabledTextBox), new UIPropertyMetadata(string.Empty));

        
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbLabledTextBox.
        /// </summary>
        public CwlbLabledTextBox()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Kontrolliert die Ausgabegröße des Steuerelements
        /// </summary>
        /// <param name="sizeArrangeBounds">Die Größe des Platzes, der dieses Element benutzen kann.</param>
        /// <returns>Die Größe des Platzes, der dieses Element benutzt.</returns>
        protected override Size ArrangeOverride(Size sizeArrangeBounds)
        {
            switch (LabelPosition)
            {
                case Dock.Bottom:
                    _tbControlEntry.Margin = new Thickness(0,0,0,MARGIN);
                    break;
                case Dock.Left:
                    _tbControlEntry.Margin = new Thickness(MARGIN, 0, 0, 0);
                    break;
                case Dock.Right:
                    _tbControlEntry.Margin = new Thickness(0, 0, MARGIN, 0);
                    break;
                case Dock.Top:
                    _tbControlEntry.Margin = new Thickness(0, MARGIN, 0, 0);
                    break;
            }

            return base.ArrangeOverride(sizeArrangeBounds);
        }
    }
}
