﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Reflection;
using System.Collections;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die Funktionalität der Demoseite für Cookies.
    /// </summary>
    public partial class CwlbCookies : System.Web.UI.Page
    {
        // Instanzdaten
        private HtmlTable _objTable = new HtmlTable();

        /// <summary>
        /// Ereignisbehandlung für das Laden der Seite.
        /// </summary>
        protected void Page_Load(object sender, EventArgs e)
        {
            // Cookies anzeigen
            if (!IsPostBack)
            {
                this.Refresh();
            }
        }

        /// <summary>
        /// Ereignishandler: Schaltfläche Erstelle Cookie 
        /// </summary>
        protected void _btnCreateCookie_Click(object sender, EventArgs e)
        {
            // Cookie direkt erstellen (7 Tage gültig)
            Response.Cookies["UserNameDirect"].Value = this.User.Identity.Name;
            Response.Cookies["UserNameDirect"].Expires = DateTime.Now.AddDays(7);

            // Cookie indirekt erstellen (3 Tage gültig)
            HttpCookie objCookie = new HttpCookie("UserNameIndirect");
            objCookie.Value = this.User.Identity.Name;
            objCookie.Expires = DateTime.Now.AddDays(3);
            Response.Cookies.Add(objCookie);

            // Cookie mit mehreren Schlüsseln (direkt)
            Response.Cookies["UserInfoDirect"]["Name"] = this.User.Identity.Name;
            Response.Cookies["UserInfoDirect"]["LastVisit"] = DateTime.Now.ToString("u");
            Response.Cookies["UserInfoDirect"]["Language"] = this.Request.UserLanguages[0];
            Response.Cookies["UserInfoDirect"].Expires = DateTime.Now.AddDays(7);

            // Cookie mit mehreren Schlüsseln (indirekt)
            objCookie = new HttpCookie("UserInfoIndirect");
            objCookie.Values["Name"] = this.User.Identity.Name;
            objCookie.Values["LastVisit"] = DateTime.Now.ToString("u");
            objCookie.Values["Language"] = this.Request.UserLanguages[0];
            objCookie.Expires = DateTime.Now.AddDays(3);
            Response.Cookies.Add(objCookie);
            Response.Redirect(Request.Path);
        }

        /// <summary>
        /// Ereignishandler: Schaltfläche Verändere Cookie 
        /// </summary>
        protected void _btnModifyCookie_Click(object sender, EventArgs e)
        {
            // Prüfe, ob ein Cookie ausgewählt wurde
            if (_lbCookies.SelectedItem != null)
            {
                // Ergänze / Korrigiere LastVisit
                HttpCookie objCookie = Request.Cookies[_lbCookies.SelectedItem.Text];
                objCookie.Values["LastVisit"] = DateTime.Now.ToString("u");
                objCookie.Expires = DateTime.Now.AddDays(7);
                Response.Cookies.Add(objCookie);
                Response.Redirect(Request.Path);
            }
        }

        /// <summary>
        /// Ereignishandler: Schaltfläche Lösche Cookie
        /// </summary>
        protected void _btnDeleteCookie_Click(object sender, EventArgs e)
        {
            // Lösche Cookie
            if (_lbCookies.SelectedItem != null)
            {
                HttpCookie objCookie = Request.Cookies[_lbCookies.SelectedItem.Text];
                objCookie.Expires = DateTime.Now.AddDays(-1);
                Response.Cookies.Add(objCookie);
                Response.Redirect(Request.Path);
            }
        }

        /// <summary>
        /// Ereignishandler: Selektion in der Liste erfolgt
        /// </summary>
        protected void _lbCookies_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Lese Cookie
            if (_lbCookies.SelectedItem != null)
            {
                // Auslesen der selektion
                HttpCookie objCookie = Request.Cookies[_lbCookies.SelectedItem.Text];

                // Zeige alle enthaltenen Werte
                foreach (string strKey in objCookie.Values.AllKeys)
                {
                    if (strKey == null)
                    {
                        GwlbApplication.WriteValue(_objTable, "[No Key]:", objCookie[strKey]);
                    }
                    else
                    {
                        GwlbApplication.WriteValue(_objTable, strKey + ": ", objCookie[strKey]);
                    }
                }
                GwlbApplication.WriteValue(_objTable, "Domäne: ", objCookie.Domain);
                GwlbApplication.WriteValue(_objTable, "Pfad: ", objCookie.Path);

                // Platzhalter neu definieren
                _placeHolder.Controls.Clear();
                _placeHolder.Controls.Add(_objTable);
            }
        }

        /// <summary>
        /// Stellt die Liste der Cookies neu zusammen
        /// </summary>
        private void Refresh()
        {
            _lbCookies.Items.Clear();
            foreach (string strCookieName in Request.Cookies)
            {
                _lbCookies.Items.Add(strCookieName);
            }
        }
    }
}