﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Reflection;
using System.Collections;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die Funktionalität der Demoseite für den Viewstate.
    /// </summary>
    public partial class CwlbViewState : System.Web.UI.Page
    {
        private const string SEQ_NUMBER = "ViewStateSequenceNumber";
        private const string OWN_CONTENT = "OwnContent";

        // Instanzdaten
        private HtmlTable _objTable = new HtmlTable();
        private Hashtable _objHashtable;

        /// <summary>
        /// Ereignisbehandlung für das Laden der Seite.
        /// </summary>
        protected void Page_Load(object sender, EventArgs e)
        {
            // Prüfe erstmalige Durchführung
            if (!IsPostBack)
            {
                // Hashtable erstellen und in den ViewState einfügen
                _objHashtable = new Hashtable();
                _objHashtable[SEQ_NUMBER] = 0;
                ViewState.Add(OWN_CONTENT, _objHashtable);
                // Liste ausgeben
                RefreshView();
            }
            else
            {
                // Daten vom Viewstate wiederherstellen
                _objHashtable = (Hashtable)ViewState[OWN_CONTENT];
            }
        }

        /// <summary>
        /// Ereignishandler: Schaltfläche Aktualisieren 
        /// </summary>
        protected void _btnRefresh_Click(object sender, EventArgs e)
        {
            // Sequenznummer erhöhen
            _objHashtable[SEQ_NUMBER] =
                ((int)_objHashtable[SEQ_NUMBER]) + 1;

            // Einen neuen Eintrag erstellen
            _objHashtable[string.Format("Eigene Daten {0:000}",
                (int)_objHashtable[SEQ_NUMBER])] =
                DateTime.Now.ToString("HH:mm:ss.fff");

            // Liste ausgeben
            RefreshView();
        }

        /// <summary>
        /// Die Ansicht der Daten neu zusammenstellen
        /// </summary>
        private void RefreshView()
        {
            // Tabelle zusammenstellen
            foreach (DictionaryEntry objEntry in _objHashtable.Cast<DictionaryEntry>().OrderBy( objItem => objItem.Key.ToString()))
            {
                GwlbApplication.WriteValue(_objTable, objEntry.Key.ToString(), objEntry.Value.ToString());
            }

            // Anzahl Elemente ausgeben
            _lblCount.Text = "Anzahl Einträge: " + _objHashtable.Count.ToString();

            // Platzhalter neu definieren
            _placeHolder.Controls.Clear();
            _placeHolder.Controls.Add(_objTable);
        }
    }
}