﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Demo für das TreeView-Steuerelement.
    /// </summary>
    public partial class WebTreeView : System.Web.UI.Page
    {
        /// <summary>
        /// Lade ab definiertem Eintrag Dateisystemeinträge in die TreeView
        /// </summary>
        /// <remarks>
        /// Diese Methode arbeitet rekursiv
        /// </remarks>
        /// <param name="tnCollection">Knoten, an den die Einträge verbunden werden</param>
        /// <param name="strPath">Aktueller Pfad, der dem Knoten verbunden wird</param>
        /// <returns>Liefert die Anzahl Subknoten</returns>
        private int AddTreeViewEntries(TreeNodeCollection tnCollection, string strPath,
            string strFullApplicationPath)
        {
            // Lokale Daten
            int iSubNodes = 0;
            TreeNode tnData;
            string strRelativePath;

            // Bearbeite alle Unterverzeichnisse
            foreach (string strDirectory in Directory.GetDirectories(strPath))
            {
                // Erzeuge Knoten
                tnData = new TreeNode();
                // Definiere Name
                tnData.Text = Path.GetFileName(strDirectory);
                tnData.Value = Path.GetFileName(strDirectory);
                // Rekursiver Aufruf
                if (AddTreeViewEntries(tnData.ChildNodes,
                    strDirectory,
                    strFullApplicationPath) > 0)
                {

                    // Ergänze Knoten im TreeView nur, wenn Subknoten vorhanden sind
                    tnCollection.Add(tnData);
                    iSubNodes++;
                }
            }

            // Ergänze Dateieinträge im Knoten
            foreach (string strFile in Directory.GetFiles(strPath))
            {
                // Erzeuge Knoten
                tnData = new TreeNode();
                // Definiere Name
                tnData.Text = Path.GetFileName(strFile);
                tnData.Value = Path.GetFileName(strFile);
                // URL ergänzen
                strRelativePath = strFile.Substring(Page.MapPath("~").Length);
                tnData.NavigateUrl = strFullApplicationPath + strRelativePath;
                tnData.Target = "_blank";
                // ToolTip ergänzen
                FileInfo objFileInfo = new FileInfo(strFile);
                tnData.ToolTip = "Bytes: " + objFileInfo.Length.ToString() + "\r\n" +
                    "Erstellungsdatum: " + objFileInfo.CreationTime.ToString("g") + "\r\n" +
                    "Letzte Veränderung: " + objFileInfo.LastWriteTime.ToString("g");
                // Ergänze Knoten in die TreeView
                tnCollection.Add(tnData);
                iSubNodes++;
            }

            // Rückgabe der Anzahl Subknoten
            return iSubNodes;
        }

        /// <summary>
        /// Ereignisbehandlung beim Laden der Seite.
        /// </summary>
        protected void Page_Load(object sender, EventArgs e)
        {
            // Definiert den Titel und den offenen Menüeintrag
            ((Demo)Master).SetDemoTitle("Web - TreeView");
            ((Demo)Master).KeepNodeOpen("Web/Polynominal");

            if (!IsPostBack)
            {
                // Initialisierung des TreeView
                _tvDemo.ImageSet = TreeViewImageSet.XPFileExplorer;
                _tvDemo.ExpandDepth = 1;

                // Ergänze einen Root-Knoten in der TreeView
                string strFullApplicationPath = Request.Url.Scheme +
                    "://" +
                    Request.Url.Authority +
                    Request.ApplicationPath;
                TreeNode tnData = new TreeNode(Request.Url.ToString());
                tnData.NavigateUrl = strFullApplicationPath;
                tnData.Target = "_blank";
                _tvDemo.Nodes.Add(tnData);

                // Lade Dateisystemeinträge in den Baum
                AddTreeViewEntries(tnData.ChildNodes, Page.MapPath("~"), strFullApplicationPath);
            }
        }

        /// <summary>
        /// Ereignisbehandlung bei Änderung der Auswahl eines Knotens.
        /// </summary>
        protected void _tvDemo_SelectedNodeChanged(object sender, EventArgs e)
        {

        }
    }
}