﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Collections;
using System.Web.Caching;

namespace WeroSoft.Samples.Cache
{
    /// <summary>
    /// Definiert die Funktionalität der Demoseite für die Objektzwischenspeicherung.
    /// </summary>
    public partial class CachedObejcts : System.Web.UI.Page
    {
        // Konstante Daten der Klasse
        private const string SEQ_NUMBER = "CacheSequenceNumber";

        // Liste der Daten die aus dem Zwischenspeicher gelöscht wurden.
        private static Dictionary<string, string> _cobjListRemovedFromCache = new Dictionary<string, string>();

        // Instanzdaten
        private HtmlTable objTable = new HtmlTable();
        private HtmlTable objTableDeleted = new HtmlTable();

        /// <summary>
        /// Ereignisbehandlung für das Laden der Seite.
        /// </summary>
        protected void Page_Load(object sender, EventArgs e)
        {
            // Sequenznummer erhöhen
            if (Cache[SEQ_NUMBER] == null)
            {
                // Allererste Durchführung
                Cache[SEQ_NUMBER] = 0;
            }

            if (!IsPostBack)
            {
                // Liste ausgeben
                RefreshView();
            }
        }

        /// <summary>
        /// Behandlung des Ereignisses für das Betätigen der Schaltfläche Aktualisieren.
        /// </summary>
        protected void _btnRefresh_Click(object sender, EventArgs e)
        {
            // Sequenznummer erhöhen
            Cache[SEQ_NUMBER] = ((int)Cache[SEQ_NUMBER]) + 1;

            // Einen neuen Eintrag erstellen
            Cache.Add(
                string.Format("Datenelement {0:000} erzeugt um",   // Schlüssel Zwischenspeicher
                    (int)Cache[SEQ_NUMBER]),
                DateTime.Now.ToString("HH:mm:ss.fff"),             // Daten Zwischenspeicher
                null,                                              // Keine Abhängigkeiten
                DateTime.Now.AddSeconds(30),                       // Verweildauer im Zwischenspeicher
                System.Web.Caching.Cache.NoSlidingExpiration,      // Verlängerung der Verweildauer nach Zugriff
                CacheItemPriority.High,                            // Priorität des Elements im Zwischenspeicher
                new CacheItemRemovedCallback(OnRemovedItem));      // Rückrufmethode beim Löschen

            // Liste ausgeben
            RefreshView();
        }

        /// <summary>
        /// Die Ansicht der Daten neu zusammenstellen
        /// </summary>
        private void RefreshView()
        {
            // Tabelle zusammenstellen
            foreach (DictionaryEntry objData in Cache.Cast<DictionaryEntry>().OrderBy(objItem => objItem.Key.ToString()))
            {
                GwlbApplication.WriteValue(objTable, objData.Key.ToString(), objData.Value.ToString());
            }

            // Tabelle der gelöschten Elemente zusammenstellen
            foreach (KeyValuePair<string, string> objData in _cobjListRemovedFromCache.OrderBy(objItem => objItem.Value.ToString()))
            {
                GwlbApplication.WriteValue(objTableDeleted, objData.Key.ToString(), objData.Value.ToString());
            }

            // Anzahl Elemente ausgeben
            _lblCount.Text = "Anzahl Einträge: " + (Cache.Count - 1).ToString();

            // Platzhalter neu definieren
            _placeHolder.Controls.Clear();
            _placeHolder.Controls.Add(objTable);
            _placeHolderDeleted.Controls.Clear();
            _placeHolderDeleted.Controls.Add(objTableDeleted);
        }

        /// <summary>
        /// Rückrufmethode die vom Zwischenspeicher nach dem Löschen eines Elements aufgerufen wird.
        /// </summary>
        /// <param name="strKey">Schlüssel des Cacheobjekts das gelöscht wird.</param>
        /// <param name="objValue">Objekt selber das gelöscht wird.</param>
        /// <param name="objReason">Grund für das Löschen eines Objekts</param>
        private void OnRemovedItem(string strKey, object objValue, CacheItemRemovedReason objReason)
        {
            // Gelöschtes Element zwecks Visualisierung in eigene Liste übernehmen.
            _cobjListRemovedFromCache.Add(
                strKey,
                string.Format(
                    "Element {0} wurde um {1} aus dem Zwischenspeicher entfernt. Grund: {2}",
                    objValue.ToString(),
                    DateTime.Now.ToLongTimeString(),
                    objReason.ToString()
                )
           );
        }
    }
}