﻿using System;
using System.Web;
using System.Threading;
using System.Diagnostics;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Asynchron arbeitender Handler.
    /// </summary>
    public class AsyncHandlerDemo : IHttpAsyncHandler
    {
        /// <summary>
        /// Definiert ob der Handler in einen Pool gelegt werden kann. Definieren Sie diesen Wert mit false,
        /// wenn Sie im Handler Statusinformationen führen.
        /// </summary>
        public bool IsReusable
        {
            get { return false; }
        }

        /// <summary>
        /// Führt den Handler synchron aus.
        /// </summary>
        /// <param name="objContext">HTTP-Kontext der Anforderung.</param>
        public void ProcessRequest(HttpContext objContext)
        {
            // Diese Methode wird in der asynchronen Verarbeitung nicht benutzt
            throw new NotImplementedException();
        }

        /// <summary>
        /// Initialisiert die asynchrone Verarbeitung.
        /// </summary>
        /// <param name="objContext">HTTP-Kontext der Anforderung.</param>
        /// <param name="objAsyncCallBack">Delegat für den Rückruf.</param>
        /// <param name="objExtraData">Spezifische Daten der Verarbeitung</param>
        /// <returns></returns>
        public IAsyncResult BeginProcessRequest(HttpContext objContext, AsyncCallback objAsyncCallBack, object objExtraData)
        {
            Stopwatch _objStopwatch = Stopwatch.StartNew();
            WriteStartDocument(objContext);
            Write(objContext, "Starte asynchrone Verarbeitung ...", _objStopwatch);
            CwlbAsyncHandlerOperation objOperation = new CwlbAsyncHandlerOperation(objAsyncCallBack, objContext, _objStopwatch);
            objOperation.StartAsync();
            Write(objContext, "Asynchrone Verarbeitung in Auftrag gegeben ...", _objStopwatch);
            return objOperation;
        }

        /// <summary>
        /// Beendet die asynchrone Verarbeitung.
        /// </summary>
        /// <param name="objResult">Spezifische Daten der Verarbeitung.</param>
        public void EndProcessRequest(IAsyncResult objResult)
        {
            Write(((CwlbAsyncHandlerOperation)objResult).HttpContext, "Asynchrone Verarbeitung beendet.", objResult.AsyncState as Stopwatch);
            WriteEndDocument(((CwlbAsyncHandlerOperation)objResult).HttpContext);
            ((Stopwatch)objResult.AsyncState).Stop();
        }

        /// <summary>
        /// Vorbereiten der Dokumentenausgabe.
        /// </summary>
        /// <param name="objContext">HTTP-Kontext der Anforderung.</param>
        public static void WriteStartDocument(HttpContext objContext)
        {
            objContext.Response.Write("<html><head><title>Resultat asynchrone Verarbeitung</title></head><body style=\"font-family:Calibri\">");
        }

        /// <summary>
        /// Schribt eine Text in die Antwort.
        /// </summary>
        /// <param name="objContext">HTTP-Kontext der Anforderung.</param>
        /// <param name="strOutput">Text der ausgegeben werden soll.</param>
        /// <param name="objStopwatch">Stopwatch used to create output.</param>
        public static void Write(HttpContext objContext, string strOutput, Stopwatch objStopwatch)
        {
            objContext.Response.Write(
                string.Format("<p>{0} - Thread: {1:x4} - {2}</p>", 
                    objStopwatch.Elapsed, 
                    Thread.CurrentThread.ManagedThreadId,
                    strOutput));
        }

        /// <summary>
        /// Beenden der Dokumentenausgabe.
        /// </summary>
        /// <param name="objContext">HTTP-Kontext der Anforderung.</param>
        public static void WriteEndDocument(HttpContext objContext)
        {
            objContext.Response.Write("</body></html>");
        }
    }
}
