﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Diagnostics;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionscode für die Demo der asynchronen Verarbeitung einer Seite.
    /// </summary>
    public partial class AsyncPageDemo : System.Web.UI.Page
    {
        /// <summary>
        /// Ereignisbehandlung beim Laden der Seite.
        /// </summary>
        protected void Page_Load(object sender, EventArgs e)
        {
            AddOnPreRenderCompleteAsync(
                new BeginEventHandler(BeginProcessRequest),
                new EndEventHandler(EndProcessRequest));
        }

        /// <summary>
        /// Initialisiert die asynchrone Verarbeitung.
        /// </summary>
        /// <param name="objContext">HTTP-Kontext der Anforderung.</param>
        /// <param name="objAsyncCallBack">Delegat für den Rückruf.</param>
        /// <param name="objExtraData">Spezifische Daten der Verarbeitung</param>
        /// <returns></returns>
        public IAsyncResult BeginProcessRequest(object sender, EventArgs e, AsyncCallback objAsyncCallBack, object objExtraData)
        {
            Stopwatch _objStopwatch = Stopwatch.StartNew();
            AsyncHandlerDemo.WriteStartDocument(Context);
            AsyncHandlerDemo.Write(Context, "Starte asynchrone Verarbeitung ...", _objStopwatch);
            CwlbAsyncHandlerOperation objOperation = new CwlbAsyncHandlerOperation(objAsyncCallBack, Context, _objStopwatch);
            objOperation.StartAsync();
            AsyncHandlerDemo.Write(Context, "Asynchrone Verarbeitung in Auftrag gegeben ...", _objStopwatch);
            return objOperation;
        }

        /// <summary>
        /// Beendet die asynchrone Verarbeitung.
        /// </summary>
        /// <param name="objResult">Spezifische Daten der Verarbeitung.</param>
        public void EndProcessRequest(IAsyncResult objResult)
        {
            AsyncHandlerDemo.Write(((CwlbAsyncHandlerOperation)objResult).HttpContext, "Asynchrone Verarbeitung beendet.", objResult.AsyncState as Stopwatch);
            AsyncHandlerDemo.WriteEndDocument(((CwlbAsyncHandlerOperation)objResult).HttpContext);
            ((Stopwatch)objResult.AsyncState).Stop();
        }
    }
}