﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Threading;
using System.Diagnostics;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Klasse für die Handhabung einer asynchronen Operation.
    /// </summary>
    public class CwlbAsyncHandlerOperation : IAsyncResult
    {
        // Referenz auf den Delegaten für den Callback der asynchronen Operation
        private AsyncCallback _objCallback;

        // Spezifische Daten für die Verarbeitung
        private object _objState;

        /// <summary>
        /// Liefert oder definiert true, wenn die Verarbeitung beendet wurde.
        /// </summary>
        public bool Completed { get; private set; }

        /// <summary>
        /// Liefert oder definiert den HTTP-Kontext der Verarbeitung.
        /// </summary>
        public HttpContext HttpContext { get; private set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbAsyncHandlerOperation.
        /// </summary>
        /// <param name="objCallback">Delegate von IIS für den Callback.</param>
        /// <param name="objContext">HTTP-Kontext für der Verarbeitung.</param>
        /// <param name="objState">Spezifische Daten der Verarbeitung.</param>
        public CwlbAsyncHandlerOperation(AsyncCallback objCallback, HttpContext objContext, object objState)
        {
            _objCallback = objCallback;
            _objState = objState;
            HttpContext = objContext;
            Completed = false;
        }

        /// <summary>
        /// Liefert true, wenn die Operation beendet wurde.
        /// </summary>
        public bool IsCompleted
        {
            get { return Completed; }
        }

        /// <summary>
        /// Liefert das Resultat der asynchronen Verarbeitung.
        /// </summary>
        public object AsyncState
        {
            get { return _objState; }
        }

        /// <summary>
        /// Liefert einen WaitHandle (hir nicht gebraucht).
        /// </summary>
        public System.Threading.WaitHandle AsyncWaitHandle
        {
            get { return null; }
        }

        /// <summary>
        /// Liefert immer false, da keine synchrone Verarbeitung unterstützt wird.
        /// </summary>
        public bool CompletedSynchronously
        {
            get { return false; }
        }

        /// <summary>
        /// Started die Verarbeitung asynchron.
        /// </summary>
        public void StartAsync()
        {
            ThreadPool.QueueUserWorkItem(new WaitCallback(DoWork), _objState);
        }

        /// <summary>
        /// Eigentliche Arbeitsmethode.
        /// </summary>
        /// <param name="objState">Spezifische Daten der Verarbeitung.</param>
        public void DoWork(object objState)
        {
            AsyncHandlerDemo.Write(HttpContext, "Beginne Verarbeitung ...", objState as Stopwatch);
            Thread.Sleep(5000);
            AsyncHandlerDemo.Write(HttpContext, "Beende Verarbeitung ...", objState as Stopwatch);
            Completed = true;
            _objCallback(this);
        }
    }
}