﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace WeroSoft.Samples
{
    public static class CwlbGraficsFactory
    {
        /// <summary>
        /// Interne Methode für das Herstellen der Zeichnung
        /// </summary>
        /// <param name="fXMin">Beginn der X-Achse</param>
        /// <param name="fXMax">Ende der X-Achse</param>
        /// <param name="fFrequency">Multiplikator der Frequenz</param>
        /// <param name="fAmplitude">Multiplikator der Amplitude</param>
        /// <param name="colBackground">Farbe des Hintergrundes.</param>
        /// <param name="colLine">Farbe der Linie</param>
        /// <returns>Liefert das fertige Bild</returns>
        public static Bitmap CreateImage(float fXMin, float fXMax, float fFrequency, float fAmplitude, System.Drawing.Color colBackground, System.Drawing.Color colLine)
        {
            // Erstelle das Bitmap
            int iWidth = 500;
            int iHeight = 350;
            Bitmap objGraph = new Bitmap(iWidth, iHeight);
            Graphics objGraphics = Graphics.FromImage(objGraph);
            Pen penCurve = new Pen(colLine, 2);

            // Initialisieren des Bitmap
            objGraphics.Clear(colBackground);

            // Zeichnen der Achsen
            objGraphics.DrawLine(Pens.Blue, 0, 175, 500, 175);
            objGraphics.DrawLine(Pens.Blue, 250, 0, 250, 375);

            // Speichern der Einstellungen
            GraphicsState objState = objGraphics.Save();

            // Orientierung der Graphik einstellen.
            // Y-Achse spiegeln und Nullpunkt verschieben.
            Matrix mtxDraw = new Matrix(1, 0, 0, -1, 250, 175);
            objGraphics.Transform = mtxDraw;
            objGraphics.SmoothingMode = SmoothingMode.HighQuality;

            // Kurve zeichnen
            float fXDelta = 500.0f / (Math.Abs(fXMax) + Math.Abs(fXMin));
            float fXLast = fXMin * fXDelta;
            float fXCurrent = 0.0f;
            float fYLast = 0.0f;
            float fYCurrent = 0.0f;
            for (float fX = fXMin; fX <= fXMax; fX += 0.025f)
            {
                // Berechne neuen Y-Wert
                fXCurrent = fX * fXDelta;
                fYCurrent = fAmplitude * (float)Math.Sin(fX * fFrequency);

                // Zeichne Linie
                objGraphics.DrawLine(penCurve, fXLast, fYLast, fXCurrent, fYCurrent);
                // Aktueller Punkt ist neu letzter Punkt
                fXLast = fXCurrent;
                fYLast = fYCurrent;
            }

            // Weiderherstellen des Graphics-State
            objGraphics.Restore(objState);

            // Rahmen zeichnen
            objGraphics.DrawRectangle(Pens.Black, 0, 0, iWidth - 2, iHeight - 2);
            objGraphics.DrawLine(Pens.Gray, 0, iHeight - 1, iWidth - 1, iHeight - 1);
            objGraphics.DrawLine(Pens.Gray, iWidth - 1, 0, iWidth - 1, iHeight - 1);

            // Ressourcen freigeben und Abschluss
            penCurve.Dispose();
            objGraphics.Dispose();
            return objGraph;
        }
    }
}