﻿using System;
using System.Web;
using System.Reflection;
using System.IO;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Zeichnet den Verlauf der Benutzung der Anwendung auf.
    /// </summary>
    public class CwlbTrafficLog : IHttpModule
    {
        FileStream _objBaseStream;
        StreamWriter _objWriter;
        public static object LockFileWriting = new object();

        static int iValue = 0;

        /// <summary>
        /// Gibt die externen Ressourcen frei.
        /// </summary>
        public void Dispose()
        {
            lock (LockFileWriting)
            {
                if (_objWriter != null)
                {
                    _objWriter.Dispose();
                    _objWriter = null;
                    _objBaseStream.Dispose();
                    _objBaseStream = null;
                }
            }
        }

        /// <summary>
        /// Initialisierung des Moduls der Anwendung
        /// </summary>
        /// <param name="objAppContext"></param>
        public void Init(HttpApplication objAppContext)
        {
            try
            {
                iValue++;

                lock (LockFileWriting)
                {
                    if (_objWriter == null)
                    {
                        string strLogFile = objAppContext.Context.Server.MapPath("~");
                        strLogFile = Path.Combine(strLogFile, "App_Data", "TrafficLog.log");
                        _objBaseStream = new FileStream(strLogFile, FileMode.OpenOrCreate, FileAccess.Write, FileShare.ReadWrite);
                        _objWriter = new StreamWriter(_objBaseStream);
                    }
                }

                // Folge einer normalen Verarbeitung
                objAppContext.BeginRequest += new EventHandler(AppContext_BeginRequest);
                objAppContext.AuthenticateRequest += new EventHandler(AppContext_AuthenticateRequest);
                objAppContext.PostAuthenticateRequest += new EventHandler(AppContext_PostAuthenticateRequest);
                objAppContext.AuthorizeRequest += new EventHandler(AppContext_AuthorizeRequest);
                objAppContext.PostAuthorizeRequest += new EventHandler(AppContext_PostAuthorizeRequest);
                objAppContext.ResolveRequestCache += new EventHandler(AppContext_ResolveRequestCache);
                objAppContext.PostResolveRequestCache += new EventHandler(AppContext_PostResolveRequestCache);
                objAppContext.PostMapRequestHandler += new EventHandler(AppContext_PostMapRequestHandler);
                objAppContext.AcquireRequestState += new EventHandler(AppContext_AcquireRequestState);
                objAppContext.PostAcquireRequestState += new EventHandler(AppContext_PostAcquireRequestState);
                objAppContext.PreRequestHandlerExecute += new EventHandler(AppContext_PreRequestHandlerExecute);
                objAppContext.PostRequestHandlerExecute += new EventHandler(AppContext_PostRequestHandlerExecute);
                objAppContext.ReleaseRequestState += new EventHandler(AppContext_ReleaseRequestState);
                objAppContext.PostReleaseRequestState += new EventHandler(AppContext_PostReleaseRequestState);
                objAppContext.UpdateRequestCache += new EventHandler(AppContext_UpdateRequestCache);
                objAppContext.PostUpdateRequestCache += new EventHandler(AppContext_PostUpdateRequestCache);
                objAppContext.EndRequest += new EventHandler(AppContext_EndRequest);
                objAppContext.PreSendRequestHeaders += new EventHandler(AppContext_PreSendRequestHeaders);
                objAppContext.PreSendRequestContent += new EventHandler(AppContext_PreSendRequestContent);

                // Aussergwöhnliche Verarbeitungen
                objAppContext.Disposed += new EventHandler(AppContext_Disposed);
                objAppContext.Error += new EventHandler(AppContext_Error);

                // Verarbeitungen, die nur im integrierten Modus unterstützt sind
                // objApplicationContext.LogRequest += new EventHandler(AppContext_LogRequest);
                // objApplicationContext.MapRequestHandler += new EventHandler(AppContext_MapRequestHandler);
                // objApplicationContext.PostLogRequest += new EventHandler(AppContext_PostLogRequest);
            }
            catch
            {
                // Alle Fehler schlucken.
            }
        }

        #region Routing der Handler auf die Ausgabe
        void AppContext_UpdateRequestCache(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_ResolveRequestCache(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_ReleaseRequestState(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_PreSendRequestHeaders(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_PreSendRequestContent(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_PreRequestHandlerExecute(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_PostUpdateRequestCache(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_PostResolveRequestCache(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_PostRequestHandlerExecute(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_PostReleaseRequestState(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_PostMapRequestHandler(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_PostLogRequest(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_PostAuthorizeRequest(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_PostAuthenticateRequest(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_PostAcquireRequestState(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_MapRequestHandler(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_LogRequest(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_Error(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_EndRequest(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_Disposed(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_BeginRequest(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_AuthorizeRequest(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_AuthenticateRequest(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }

        void AppContext_AcquireRequestState(object sender, EventArgs e)
        {
            WriteLog(sender, e, MethodInfo.GetCurrentMethod().Name);
        }
        #endregion

        private void WriteLog(object sender, EventArgs e, string strMethodName)
        {
            lock (LockFileWriting)
            {
                try
                {
                    HttpApplication objApplication = sender as HttpApplication;
                    HttpContext objContext = objApplication.Context;

                    string strLog = string.Format("{0}; {1}; {2}; {3}; {4}; {5};",
                        DateTime.Now.ToString("yyyy.HH.ss HH:mm:ss.fff"),
                        objContext.Request.HttpMethod,
                        strMethodName,
                        objContext.User == null ? string.Empty : objContext.User.Identity.Name,
                        objContext.Request.Url,
                        objContext.Request.TotalBytes.ToString());

                    _objWriter.BaseStream.Seek(0, SeekOrigin.End);
                    _objWriter.WriteLine(strLog);
                    _objWriter.Flush();
                }
                catch
                {
                }
            }
        }
    }
}
