﻿using System;
using System.Web;
using System.Drawing;

namespace WeroSoft.Samples
{
    /// <summary>
    /// HTTP-Handler für die Erstellung einer Grafik.
    /// </summary>
    /// <remarks>
    /// You will need to configure this handler in the web.config file of your 
    /// web and register it with IIS before being able to use it. For more information
    /// see the following link: http://go.microsoft.com/?linkid=8101007
    /// </remarks>
    public class GraphicsFactory : IHttpHandler
    {
        /// <summary>
        /// Definiert ob der Handler in einen Pool gelegt werden kann. Definieren Sie diesen Wert mit false,
        /// wenn Sie im Handler Statusinformationen führen.
        /// </summary>
        public bool IsReusable
        {
            get { return true; }
        }

        /// <summary>
        /// Führt den Handler aus.
        /// </summary>
        /// <param name="objContext">HTTP-Kontext der Anforderung.</param>
        public void ProcessRequest(HttpContext objContext)
        {
            HttpRequest objRequest = objContext.Request;
            HttpResponse objResponse = objContext.Response;

            // Vorbereiten der Antwort
            objResponse.Clear();
            objResponse.ContentType = "image/jpeg";

            // Aufbau der Daten
            try
            {
                // Lesen der Daten
                float fXMin = Convert.ToSingle(objRequest.QueryString["XMin"]);
                float fXMax = Convert.ToSingle(objRequest.QueryString["XMax"]);
                float fFrequency = Convert.ToSingle(objRequest.QueryString["Freq"]);
                float fAmplitude = Convert.ToSingle(objRequest.QueryString["Ampl"]);

                // Herstellen des Bildes
                Bitmap objBmp = CwlbGraficsFactory.CreateImage(fXMin, fXMax, fFrequency, fAmplitude, Color.Wheat, Color.Black);
                objBmp.Save(objResponse.OutputStream, System.Drawing.Imaging.ImageFormat.Jpeg);
            }
            catch
            {
                string strFileError = objContext.Server.MapPath("~/Images") + "\\GraphError.jpg";
                System.Drawing.Image imgError = System.Drawing.Image.FromFile(strFileError);
                imgError.Save(objResponse.OutputStream, System.Drawing.Imaging.ImageFormat.Jpeg);
            }
        }
    }
}
