﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Globalization;
using System.Threading;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionscode für die Demo der Seite LocalizedPage.
    /// </summary>
    public partial class LocalizedPage : System.Web.UI.Page
    {
        // Schlüssel für die Speicherung der aktuell verwendeten Kultur in der Sitzung
        private const string UI_CULTURE = "UICulture";

        /// <summary>
        /// Initialisierung der Kultur für die Seitenbearbeitung
        /// </summary>
        protected override void InitializeCulture()
        {
            string strCulture = null;
            // Lesen der Sprache in der Sitzung
            if (Session[UI_CULTURE] != null)
            {
                strCulture = Session[UI_CULTURE].ToString();
            }
            else
            {
                strCulture = Request.UserLanguages[0];
                Session[UI_CULTURE] = strCulture;
            }
            if (strCulture.Length > 3)
            {
                // Definiere Sprache
                Thread.CurrentThread.CurrentCulture = CultureInfo.CreateSpecificCulture(strCulture);
                Thread.CurrentThread.CurrentUICulture = new CultureInfo(strCulture);
            }

            base.InitializeCulture();
        }

        /// <summary>
        /// Ereignisbehandlung beim Laden der Seite.
        /// </summary>
        protected void Page_Load(object sender, EventArgs e)
        {
            // Verwendung einer projektspezifischen Ressource
            _lblProjectResource.Text = Properties.Resources.NormalClrResource;

            // Verwendung einer globalen Ressource
            _lblGlobalResource.Text = Resources.DemoGlobalResources.DemoTextLabelImperative;

            // Initialisieren der Demodaten
            if (!IsPostBack)
            {
                _ddlLanguageSelection.SelectedValue =  Session[UI_CULTURE].ToString();
            }
        }

        /// <summary>
        /// Ereignisbehandlung für die Auswahl einer Sprache.
        /// </summary>
        protected void LanguageSelection_SelectedIndexChanged(object sender, EventArgs e)
        {
            Session[UI_CULTURE] = _ddlLanguageSelection.SelectedValue;
        }
    }
}