﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionscode für die Demo eines Composite Webserver-Steuerelements.
    /// </summary>
    public class AddressCompositeDefinition : CompositeControl, INamingContainer
    {
        /// <summary>
        /// Wird ausgelöst, wenn die Schaltfläche Speichern betätigt wurde.
        /// </summary>
        public event EventHandler Save = delegate { };

        /// <summary>
        /// Definition der Identifikation für das enthaltene Steuerelement Name.
        /// </summary>
        private const string ID_NAME = "_tbName";

        /// <summary>
        /// Liefert oder definiert den Inhalt des Steuerelements Name.
        /// </summary>
        public string Name
        {
            get
            {
                TextBox tbItem = (TextBox)FindControl(ID_NAME);
                return tbItem.Text;
            }
            set
            {
                TextBox tbItem = (TextBox)FindControl(ID_NAME);
                tbItem.Text = value;
            }
        }

        /// <summary>
        /// Definition der Identifikation für das enthaltene Steuerelement Vorname.
        /// </summary>
        private const string ID_FIRST_NAME = "_tbFirstName";

        /// <summary>
        /// Liefert oder definiert den Inhalt des Steuerelements Vorname.
        /// </summary>
        public string FirstName
        {
            get
            {
                TextBox tbItem = (TextBox)FindControl(ID_FIRST_NAME);
                return tbItem.Text;
            }
            set
            {
                TextBox tbItem = (TextBox)FindControl(ID_FIRST_NAME);
                tbItem.Text = value;
            }
        }

        /// <summary>
        /// Definition der Identifikation für das enthaltene Steuerelement Strasse.
        /// </summary>
        private const string ID_STREET = "_tbStreet";

        /// <summary>
        /// Liefert oder definiert den Inhalt des Steuerelements Strasse.
        /// </summary>
        public string Street
        {
            get
            {
                TextBox tbItem = (TextBox)FindControl(ID_STREET);
                return tbItem.Text;
            }
            set
            {
                TextBox tbItem = (TextBox)FindControl(ID_STREET);
                tbItem.Text = value;
            }
        }

        /// <summary>
        /// Definition der Identifikation für das enthaltene Steuerelement Stadt.
        /// </summary>
        private const string ID_CITY = "_tbCity";

        /// <summary>
        /// Liefert oder definiert den Inhalt des Steuerelements Stadt.
        /// </summary>
        public string City
        {
            get
            {
                TextBox tbItem = (TextBox)FindControl(ID_CITY);
                return tbItem.Text;
            }
            set
            {
                TextBox tbItem = (TextBox)FindControl(ID_CITY);
                tbItem.Text = value;
            }
        }

        /// <summary>
        /// Erstellen des Verbunds für das zusammengesetzte Steuerelement.
        /// </summary>
        protected override void CreateChildControls()
        {
            Panel panel = new Panel();
            TextBox tbName = new TextBox();
            tbName.ID = ID_NAME;
            TextBox tbFirsdName = new TextBox();
            tbFirsdName.ID = ID_FIRST_NAME;
            TextBox tbStreet = new TextBox();
            tbStreet.ID = ID_STREET;
            TextBox tbCity = new TextBox();
            tbCity.ID = ID_CITY;
            Button btnSave = new Button();
            btnSave.Text = "Speichern";
            btnSave.Click += new EventHandler(ButtonSave_Click);

            Controls.Add(panel);
            Controls.Add(new LiteralControl("<ul style=\"list-style-type: none; list-style-position:outside;\">"));
            AddLabeledControl("Name:", tbName);
            AddLabeledControl("Vorname:", tbFirsdName);
            AddLabeledControl("Strasse / Nr.:", tbStreet);
            AddLabeledControl("PLZ / Ort:", tbCity);
            Controls.Add(new LiteralControl("<br/>"));
            Controls.Add(new LiteralControl("<br/>"));
            Controls.Add(new LiteralControl("<li>"));
            Controls.Add(btnSave);
            Controls.Add(new LiteralControl("</li>"));
            Controls.Add(new LiteralControl("</ul>"));
        }

        /// <summary>
        /// Ereignisbehandlung beim betätigen der Schaltfläche Speichern.
        /// </summary>
        private void ButtonSave_Click(object sender, EventArgs e)
        {
            Save(sender, e);
        }

        /// <summary>
        /// Hilfsmethode für die Erstellung der Elemente im Containment.
        /// </summary>
        private void AddLabeledControl(string p, TextBox tbName)
        {
            Label lblItem = new Label();
            lblItem.Text = p;
            lblItem.AssociatedControlID = tbName.ClientID;

            Controls.Add(new LiteralControl("<li>"));
            Controls.Add(lblItem);
            Controls.Add(new LiteralControl("</li>"));
            Controls.Add(new LiteralControl("<li>"));
            Controls.Add(tbName);
            Controls.Add(new LiteralControl("</li>"));
        }
    }
}
