﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Drawing;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionscode für das benutzerdefinierte Webserver-Steuerelement LabeledTextBox.
    /// </summary>
    [DefaultProperty("Text")]
    [ToolboxData("<{0}:LabeledTextBox runat=\"server\"></{0}:LabeledTextBox>")]
    [ToolboxBitmap(typeof(LabeledTextBox), @"LabeledTextBox.bmp")]
    public class LabeledTextBox : WebControl
    {
        /// <summary>
        /// Definiert den Namen der Eigenschaft LabelText.
        /// </summary>
        private const string LABEL_TEXT = "LabelText";

        /// <summary>
        /// Liefert oder definiert den Text der Beschriftung.
        /// </summary>
        [Bindable(true)]
        [Category("Appearance")]
        [Description("Liefert oder definiert den Text der Beschriftung.")]
        [DefaultValue("")]
        [Localizable(true)]
        public string LabelText
        {
            get
            {
                string strLabelText = (string)ViewState[LABEL_TEXT];
                return ((strLabelText == null) ? "Label:" : strLabelText);
            }

            set
            {
                ViewState[LABEL_TEXT] = value;
            }
        }

        /// <summary>
        /// Definiert den Namen der Eigenschaft Orientation.
        /// </summary>
        private const string ORIENTATION = "Orientation";

        /// <summary>
        /// Liefert oder definiert die Orientierung des Steuerelements.
        /// </summary>
        /// <remarks>
        /// Eine horizontale Orientierung ordnet die Beschriftung und den Inhalt auf der gleichen Zeile an. 
        /// Die vertikale Orientierung ordent die beiden Elemente untereinander an. Die Beschriftung zuerst.
        /// </remarks>
        [Bindable(true)]
        [Category("Appearance")]
        [Description("Eine horizontale Orientierung ordnet die Beschriftung und den Inhalt auf der gleichen Zeile an; eine vertikale Beschriftung übereinander.")]
        [DefaultValue(Orientation.Vertical)]
        [Localizable(false)]
        public Orientation Orientation
        {
            get
            {
                object objOrientation = ViewState[ORIENTATION];
                return ((objOrientation == null) ? Orientation.Vertical : (Orientation)objOrientation);
            }

            set
            {
                ViewState[ORIENTATION] = value;
            }
        }

        /// <summary>
        /// Definiert den Namen der Eigenschaft Text.
        /// </summary>
        private const string TEXT = "Text";

        /// <summary>
        /// Liefert oder definiert den Inhalt der Textbox.
        /// </summary>
        [Bindable(true)]
        [Category("Appearance")]
        [Description("Liefert oder definiert den Eingabetext.")]
        [DefaultValue("")]
        [Localizable(false)]
        public string Text
        {
            get
            {
                string strText = (string)ViewState[TEXT];
                return ((strText == null) ? string.Empty : strText);
            }

            set
            {
                ViewState[TEXT] = value;
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Laden des Steuerelements.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnInit(EventArgs e)
        {
            base.OnLoad(e);
            string strText = Page.Request.Form[string.Format("{0}", ClientID)] as string;
            if (strText != null)
            {
                Text = strText;
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das Starten des Renderings
        /// </summary>
        /// <param name="writer"></param>
        public override void RenderBeginTag(HtmlTextWriter writer)
        {
            writer.RenderBeginTag("div");
        }

        /// <summary>
        /// Ereignisbehandlung für das Rendern des Inhalts des Steuerelements.
        /// </summary>
        /// <param name="output"></param>
        protected override void RenderContents(HtmlTextWriter output)
        {
            StringBuilder objRender = new StringBuilder();
            objRender.AppendFormat("<label id=\"{0}_{1}\" for=\"{0}\">{2}:</label>", ClientID, LABEL_TEXT, LabelText);
            if ( Orientation == Orientation.Vertical)
            {
                objRender.Append("<br />");
            }
            objRender.AppendFormat("<input name=\"{0}\" id=\"{0}\" type=\"text\" value=\"{1}\">", ClientID, Text);
            output.Write(objRender.ToString());
        }

        /// <summary>
        /// Ereignisbehandlung für das Beenden des Renderings
        /// </summary>
        /// <param name="writer"></param>
        public override void RenderEndTag(HtmlTextWriter writer)
        {
            writer.RenderEndTag();
        }
    }
}
