﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.ComponentModel.DataAnnotations;

namespace WeroSoft.Samples.Models
{
    /// <summary>
    /// Repräsentiert die Daten einer Person.
    /// </summary>
    public class CwlbPerson
    {
        /// <summary>
        /// Liefert oder definiert die Identifikation des Datenobjekts.
        /// </summary>
        public string Id { get; set; }

        /// <summary>
        /// Liefert oder definiert den Namen der Person.
        /// </summary>
        [Required(AllowEmptyStrings=false, ErrorMessage="Der Name muss eingegeben werden.")]
        [StringLength(40, ErrorMessage="Der eingegebene Name ist zu lang")]
        [System.ComponentModel.DisplayName("Name:")]
        public string Name { get; set; }

        /// <summary>
        /// Liefert oder definiert den Vornamen der Person.
        /// </summary>
        [System.ComponentModel.DisplayName("Vorname:")]
        public string FirstName { get; set; }

        /// <summary>
        /// Liefert oder definiert die Adresse der Person.
        /// </summary>
        [System.ComponentModel.DisplayName("Adresse:")]
        public string Address { get; set; }

        /// <summary>
        /// Liefert oder definiert die E-Mail-Adresse der Person.
        /// </summary>
        [RegularExpression(@"[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?", 
            ErrorMessage = "Die E-Mail-Adresse ist ungültig.")]
        [Required(AllowEmptyStrings = false, ErrorMessage = "Die E-Mail-Adresse muss eingegeben werden.")]
        [System.ComponentModel.DisplayName("E-Mail:")]
        public string Email { get; set; }

        /// <summary>
        /// Liefert oder definiert das Geburtsdatum der Person.
        /// </summary>
        [Range(typeof(DateTime), "1900.01.01", "2050.12.31", ErrorMessage="Das Datum muss zwischen 01.01.1900 und 13.12.2050 liegen.")]
        [System.ComponentModel.DisplayName("Geburtstag:")]
        public DateTime Birthdate { get; set; }
    }
}