﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Browser;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionscode für die Datei CwlbDemoMultiScaleImage.xaml
    /// </summary>
    public partial class CwlbDemoMultiScaleImage : UserControl
    {
        // Instanzdaten
        double _dZoom = 1;
        bool _bDragging = false;
        bool _bMouseDown = false;
        Point _ptLastMouseDown = new Point();
        Point _ptLastMouse = new Point();
        Point _ptLastMouseView = new Point();

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbDemoMultiScaleImage.
        /// </summary>
        public CwlbDemoMultiScaleImage()
        {
            InitializeComponent();

            // Uri des Bildes definieren
            string strUri = Application.Current.Host.Source.AbsoluteUri;
            strUri = string.Format("{0}/{1}", strUri.Substring(0, strUri.LastIndexOf("/")), "dzc_output.xml");
            // Multiscale Image definieren
            _msiDemo.Source = new DeepZoomImageTileSource(new System.Uri(strUri));
        }

        /// <summary>
        /// Stellt den aktuellen Zoom für das Multiscale-Bild ein.
        /// </summary>
        /// <param name="dNewZoom">Zoomfaktor.</param>
        /// <param name="ptViewPoint">Aktueller Punkt bei dem das Bild gezoomt werden soll.</param>
        private void Zoom(double dNewZoom, Point ptViewPoint)
        {
            // Beschränken des Zooms
            if (dNewZoom < 0.1)
            {
                dNewZoom = 0.1;
            }
            if (dNewZoom > 100)
            {
                dNewZoom = 100;
            }

            // Einstellen des Zooms auf dem Bild
            _msiDemo.ZoomAboutLogicalPoint(dNewZoom / _dZoom, ptViewPoint.X, ptViewPoint.Y);
            // Speichern des aktuellen Zooms
            _dZoom = dNewZoom;
        }

        /// <summary>
        /// Ereignisbehandlung für die Bewegung des Mausrads.
        /// </summary>
        private void MultiScaleImage_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            // Aktuellen Zoom übernehmen
            double dZoom = _dZoom;
            e.Handled = true;

            // Neuen Zoom aufgrund der Radbewegung berechnen
            if (e.Delta < 0)
            {
                dZoom /= 1.25;
            }
            else
            {
                dZoom *= 1.25;
            }

            // Einstellen des Zooms 
            Zoom(dZoom, _msiDemo.ElementToLogicalPoint(this._ptLastMouse));
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der linken Maustaste.
        /// </summary>
        private void MultiScaleImage_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            // Definieren der eigenen Mausdaten in Abhängigkeit vom Ereignis
            _ptLastMouseDown = e.GetPosition(_msiDemo);
            _ptLastMouseView = _msiDemo.ViewportOrigin;
            // Merken, dass die Maustaste betätigt ist
            _bMouseDown = true;
            // Mausbewegungen einfangen
            _msiDemo.CaptureMouse();
        }

        /// <summary>
        /// Ereignisbehandlung beim Bewegen der Maus.
        /// </summary>
        private void MultiScaleImage_MouseMove(object sender, MouseEventArgs e)
        {
            // Aktuelle Mausposition aus dem Ereignis übernehmen
            _ptLastMouse = e.GetPosition(_msiDemo);

            // Erstverarbeitung beim Bewegen der Maus
            if (_bMouseDown && !_bDragging)
            {
                // Merken der Mausbewegung
                _bDragging = true;
                // Neuen Zoom berechnen
                _dZoom = 1 / _msiDemo.ViewportWidth;
            }

            // Berechnung bei wiederholter Bewegung
            if (_bDragging)
            {
                Point ptNewView = _ptLastMouseView;
                ptNewView.X += (_ptLastMouseDown.X - _ptLastMouse.X) / _msiDemo.ActualWidth * _msiDemo.ViewportWidth;
                ptNewView.Y += (_ptLastMouseDown.Y - _ptLastMouse.Y) / _msiDemo.ActualWidth * _msiDemo.ViewportWidth;
                _msiDemo.ViewportOrigin = ptNewView;
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Loslassen der linken Maustaste.
        /// </summary>
        private void MultiScaleImage_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            // Merker für Status zurücksetzen
            _bDragging = false;
            _bMouseDown = false;
            // Verfolgung der Mausbewegungen freigeben
            _msiDemo.ReleaseMouseCapture();
        }

        /// <summary>
        /// Ereignisbehandlung nach dem Beenden der Bewegung des Bildes.
        /// </summary>
        private void MultiScaleImage_MotionFinished(object sender, RoutedEventArgs e)
        {
            _tbViewportOrigin.Text = string.Format("X: {0:N4} / Y: {1:N4}", _msiDemo.ViewportOrigin.X, _msiDemo.ViewportOrigin.Y);
            _tbViewportWidth.Text = string.Format("{0:N8}", _msiDemo.ViewportWidth);
            _tbZoom.Text = string.Format("{0:N8}", _dZoom);
        }
    }
}
