﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Browser;
using System.Windows.Interop;
using System.Text;
using System.ComponentModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Interaktionscode für die Datei CwlbDemoPlugInBrowser.xml.
    /// </summary>
    public partial class CwlbDemoPlugInBrowser : UserControl, INotifyPropertyChanged
    {
        /// <summary>
        /// Ereignis der Schnittstelle INotifyPropertyChanged.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        /// <summary>
        /// Liefert das Silverlight-PlugIn-Objekt.
        /// </summary>
        public SilverlightHost HostInfo
        {
            get
            {
                return App.Current.Host;
            }
        }

        /// <summary>
        /// Liefert die Parameter aus der Initialisierung des PlugIn.
        /// </summary>
        public string InitParams
        {
            get
            {
                return GetInitParams();
            }
        }

        /// <summary>
        /// Liefert die Browserinformationen.
        /// </summary>
        public BrowserInformation BrowserInfo
        {
            get
            {
                return HtmlPage.BrowserInformation;
            }
        }

        /// <summary>
        /// Liefert dsa HTML-Dokument.
        /// </summary>
        public HtmlDocument DocumentInfo
        {
            get
            {
                return HtmlPage.Document;
            }
        }

        /// <summary>
        /// Liefert die aktuelle Höhe der Silverlight-Anwendung.
        /// </summary>
        public double ContentActualHeight
        {
            get
            {
                return App.Current.Host.Content.ActualHeight;
            }
        }

        /// <summary>
        /// Liefert die aktuelle Breite der Silverlight-Anwendung.
        /// </summary>
        public double ContentActualWidth
        {
            get
            {
                return App.Current.Host.Content.ActualWidth;
            }
        }

        /// <summary>
        /// Liefert den aktuellen Zoomfaktor der Silverlight-Anwendung.
        /// </summary>
        public double ContentZoomFactor
        {
            get
            {
                return App.Current.Host.Content.ZoomFactor;
            }
        }

        /// <summary>
        /// Liefert true, wenn die Anwendung in der Vollbilddarstellung ist.
        /// </summary>
        public bool ContentIsFullScreen
        {
            get
            {
                return App.Current.Host.Content.IsFullScreen;
            }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbDemoPlugInBrowser.
        /// </summary>
        public CwlbDemoPlugInBrowser()
        {
            DataContext = this;
            InitializeComponent();

            // Registriert die Ereignisbehandlungen der Umgebung, damit intern auf Veränderungen
            // reagiert werden kann.
            App.Current.Host.Content.Resized += new EventHandler(Content_Resized);
            App.Current.Host.Content.FullScreenChanged += new EventHandler(Content_FullScreenChanged);
            App.Current.Host.Content.Zoomed += new EventHandler(Content_Zoomed);
        }

        /// <summary>
        /// Ereignisbehandlung beim Verändern des ZoomFaktors.
        /// </summary>
        void Content_Zoomed(object sender, EventArgs e)
        {
            PropertyChanged(this, new PropertyChangedEventArgs("ContentZoomFactor"));
        }

        /// <summary>
        /// Ereignisbehandlung beim Verändern der Vollbilddarstellung.
        /// </summary>
        void Content_FullScreenChanged(object sender, EventArgs e)
        {
            PropertyChanged(this, new PropertyChangedEventArgs("ContentActualHeight"));
            PropertyChanged(this, new PropertyChangedEventArgs("ContentActualWidth"));
            PropertyChanged(this, new PropertyChangedEventArgs("ContentIsFullScreen"));
        }

        /// <summary>
        /// Ereignisbehandlung beim Verändern der Fenstergrösse.
        /// </summary>
        void Content_Resized(object sender, EventArgs e)
        {
            PropertyChanged(this, new PropertyChangedEventArgs("ContentActualHeight"));
            PropertyChanged(this, new PropertyChangedEventArgs("ContentActualWidth"));
        }

        /// <summary>
        /// Stellt die Parameter der Konfiguration des PlugIn zu einem Text zusammen.
        /// </summary>
        /// <returns>Textuelle Darstellung der Parameter mit Zeilenumbruch.</returns>
        private string GetInitParams()
        {
            StringBuilder objBuilder = new StringBuilder();
            foreach (var objItem in App.Current.Host.InitParams)
            {
                objBuilder.AppendFormat("{0}: {1}\r\n", objItem.Key, objItem.Value);
            }
            if (objBuilder.Length == 0)
            {
                return "Keine Parameter übergeben.";
            }
            else
            {
                return objBuilder.ToString();
            }
        }

        /// <summary>
        /// Ereignisbehandlung für das betätigen der Toggle-Schaltfläche Vollbild ein.
        /// </summary>
        private void FullScreenOn(object sender, RoutedEventArgs e)
        {
            App.Current.Host.Content.IsFullScreen = true;
        }

        /// <summary>
        /// Ereignisbehandlung für das betätigen der Toggle-Schaltfläche Vollbild aus.
        /// </summary>
        private void FullScreenOff(object sender, RoutedEventArgs e)
        {
            App.Current.Host.Content.IsFullScreen = false;
        }
    }
}
